/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file ae_lib_interface.h
 *
 * @brief include all AE lib interfance parameters and pointer to function
 *
 *****************************************************************************/
#ifndef __AEI_LIB_INTERFACE_H__
#define __AEI_LIB_INTERFACE_H__

#include "aaa_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef void *AeLibHandle_t;
#define AEI_CONFIG_NUM      8
#define AEI_HDR_RATIO_NUM   3
#define AEI_METADATA_EX_NUM 4

/*****************************************************************************/
/**
 *          AeLibInstanceConfig_s
 *
 * @brief   Config structure to create a new ae lib instance
 *
 */
/*****************************************************************************/
typedef struct AeLibInstanceConfig_s
{
    AeLibHandle_t  hAeLibHandle;           /**< Ae Lib handle returned by AeiCreate */
} AeLibInstanceConfig_t;

/*****************************************************************************/
/**
 *          AeSemMode_t
 *
 * @brief   Ae sem mode, When choose AE_MODE_SCENE_EVALUATION mode, 
 *          there will be three type little mode to choose in this mode type.
 *
 */
/*****************************************************************************/
typedef enum AeSemMode_e
{     
    AE_SCENE_EVALUATION_FIX        = 0,        
    AE_SCENE_EVALUATION_ADAPTIVE   = 1,   
    AE_SCENE_METADATA              = 2,       
    AE_SCENE_EVALUATION_MAX
} AeSemMode_t;


/*****************************************************************************/
/**
 *          AeFlickerPeroid_t
 *
 * @brief   Ae flicker peroid value
 *
 */
/*****************************************************************************/
typedef enum AeFlickerPeroid_e
{
    AE_FLICKER_OFF              = 0,
    AE_FLICKER_50HZ             = 1,
    AE_FLICKER_60HZ             = 2,
    AE_FLICKER_CUSTOMER_DEFINE  = 3,
    AE_FLICKER_MAX
} AeFlickerPeroid_t;


/*****************************************************************************/
/**
 *          AeFlickerMode_t
 *
 * @brief   Ae flicker mode, there will be three type mode to choose
 *
 */
/*****************************************************************************/
typedef struct AeFlickerMode_s {
    AeFlickerPeroid_t  flickerPeriod;
    float32_t          userDefinedPeriodus;       
} AeFlickerMode_t;


/*****************************************************************************/
/**
 *          AeMode_t
 *
 * @brief   ae Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct AeMode_s {
    AeSemMode_t      semMode;
    AeFlickerMode_t  antiBandingMode;
} AeMode_t;


/*****************************************************************************/
/**
 *          AeConfig_t
 *
 * @brief   AE lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct AeConfig_s{
    float32_t     setPoint;
    float32_t     tolerance;
    float32_t     dampOver;
    float32_t     dampUnder;
    float32_t     dampUnderRatio;
    float32_t     dampUnderGain;
    float32_t     dampOverRatio;
    float32_t     dampOverGain;
    float32_t     motionFilter;
    float32_t     motionThreshold;
    float32_t     targetFilter;
    float32_t     lowlightLinearRepress[AEI_CONFIG_NUM];
    float32_t     lowlightLinearGain[AEI_CONFIG_NUM];
    uint32_t      lowlightLinearLevel;
    float32_t     lowlightHdrRepress[AEI_CONFIG_NUM];
    float32_t     lowlightHdrGain[AEI_CONFIG_NUM];
    uint32_t      lowlightHdrLevel;
    float32_t     wdrContrastMin;
    float32_t     wdrContrastMax;

    float32_t     expv2WindowWeight[AEI_EXPV2_GRID_NUM * AEI_EXPV2_GRID_NUM];
    
    //performance optimization
    PerformanceiOptiMode_t     performanceOptiMode;   
} AeConfig_t;


/*****************************************************************************/
/**
 *          AeAtrrParm_t
 *
 * @brief   AE ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AeAttrParm_s {
    uint8_t         ispBitwidth;
    AaaRect_t       hAaaRect;
#if defined ISP_AEV2_V4 || defined ISP_AEV2_V2 || defined ISP_AEV2
    uint16_t        expv2Grid;
#endif    
	float32_t       aeIncreasement;
	float32_t       gainIncreasement;
	float32_t       integTimeMax;
	float32_t       integTimeMin;
	float32_t       sensorGainMax;
	float32_t       sensorGainMin;
    float32_t       ispGainMax;
	float32_t       ispGainMin;
} AeAttrParm_t;


/*****************************************************************************/
/**
 *          AeInfo_t
 *
 * @brief   AE and AWB lib caluclation's statitistic value
 *
 */
/*****************************************************************************/
typedef struct AeInfo_s {
    float32_t         sensorGain; //totalgain
    float32_t         sensorIntergramTime;
    float32_t         ispDigtialGain;
    float32_t         aeGainR;
    float32_t         aeGainGr;
    float32_t         aeGainGb;
    float32_t         aeGainB; 

    AeStatistic_t     aeStatistic;
    bool              hdrEnable;
    float32_t         hdrRatio;
} AeInfo_t;


/*****************************************************************************/
/**
 *          AeResult_t
 *
 * @brief   AE lib calculation result
 *
 */
/*****************************************************************************/
typedef struct AeResult_s {
    float32_t   sensorTotalGain; //totalgain
    float32_t   sensorIntergramTime;
    float32_t   ispDigtialGain;
    int32_t     AeLibStatus;
    float32_t   hdrRatio[AEI_HDR_RATIO_NUM];  /* 0:S/VS, 1:L/S, 2:VS/VVS */
    float32_t   meanluma;
} AeResult_t;


/*****************************************************************************/
/**
 *          AeMetaDataInfo_t
 *
 * @brief   AE MetaData input value
 *
 */
/*****************************************************************************/
typedef struct AeMetaDataInfo_s {
    float32_t    metadataExposureTime[AEI_METADATA_EX_NUM];
    float32_t    metadataGain[AEI_METADATA_EX_NUM];
    float32_t    integTimeMax;
#ifdef ISP_DG
    float32_t    ispDgain;
#endif
} AeMetaDataInfo_t;


/*****************************************************************************/
/**
 *          AeMetaDataResult_t
 *
 * @brief   AE meta data result
 *
 */
/*****************************************************************************/
typedef struct AeMetaDataResult_s {
    float32_t   ExposureTotalGain;
} AeMetaDataResult_t;


typedef AAA_RESULT (*AeiCreate_t)             (AeLibInstanceConfig_t *pAeLibConfig, uint8_t ispId);
typedef AAA_RESULT (*AeiRelease_t)            (AeLibHandle_t handle, uint8_t ispId);
typedef AAA_RESULT (*AeiSetRoi_t)             (AeLibHandle_t handle, const AaaRoi_t *pRoi);
typedef AAA_RESULT (*AeiGetRoi_t)             (AeLibHandle_t handle, AaaRoi_t *pRoi);
typedef AAA_RESULT (*AeiSetMode_t)            (AeLibHandle_t handle, const AeMode_t *pAeMode);
typedef AAA_RESULT (*AeiGetMode_t)            (AeLibHandle_t handle, AeMode_t *pAeMode);
typedef AAA_RESULT (*AeiSetConfig_t)          (AeLibHandle_t handle, const AeConfig_t *pAeConfig);
typedef AAA_RESULT (*AeiGetConfig_t)          (AeLibHandle_t handle, AeConfig_t *pAeConfig);
typedef AAA_RESULT (*AeiSetAttribute_t)       (AeLibHandle_t handle, const AeAttrParm_t *pAeAttrParm);
typedef AAA_RESULT (*AeiExecute_t)            (AeLibHandle_t handle, AeInfo_t *pAeInfo, AeResult_t *pAeResult);
typedef AAA_RESULT (*AeiMetaDataExecute_t)    (AeLibHandle_t handle, AeMetaDataInfo_t *pAeMetaDataInfo, AeMetaDataResult_t *pAeResult);

/*****************************************************************************/
/**
 *          AeiLib_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef struct AeiLib_s
{
    const char           *pAeLibName;
    AeiCreate_t          pAeiCreate;                
    AeiRelease_t         pAeiRelease;             
    AeiSetRoi_t          pAeiSetRoi;
    AeiGetRoi_t          pAeiGetRoi;
    AeiSetMode_t         pAeiSetMode;
    AeiGetMode_t         pAeiGetMode;
    AeiSetConfig_t       pAeiSetConfig;
    AeiGetConfig_t       pAeiGetConfig;
    AeiSetAttribute_t    pAeiSetAttribute;
    AeiExecute_t         pAeiExecute;
    AeiMetaDataExecute_t pAeiMetaDataExecute;
} AeiLib_t;

#ifdef ES_3A
	/**AE**/
	typedef enum VENDOR_AE_CTRL_CMD_E {
		V_AE_ROI_SET,
		V_AE_ROI_GET,
		V_AE_MODE_SET,
		V_AE_MODE_GET,
		V_AE_CONFIG_SET,
		V_AE_CONFIG_GET,
		V_AE_ATTR_SET,
	} V_AE_CTRL_CMD_E;
	
	typedef struct VENDOR_ISP_AE_PARAM_S {
		//xiaoy todo:init params
		uint32_t			 SensorId;
		uint16_t			 u16BlackLevel;
		float				 f32Fps;
	}V_ISP_AE_PARAM_S;
	
	typedef struct VENDOR_ISP_AE_EXP_FUNC_S {
		int32_t (*pfn_ae_init)(int32_t s32Handle, const V_ISP_AE_PARAM_S *pstAeParam);
		int32_t (*pfn_ae_run)(int32_t s32Handle, const AeInfo_t *pstAeInfo, AeResult_t *pstAeResult, int32_t s32Rsv);
		int32_t (*pfn_ae_ctrl)(int32_t s32Handle, uint32_t u32Cmd, void *pValue);
		int32_t (*pfn_ae_exit)(int32_t s32Handle);
	} V_ISP_AE_EXP_FUNC_S;
	
	typedef struct VENDOR_ISP_AE_REGISTER_S {
		V_ISP_AE_EXP_FUNC_S stAeExpFunc;
	} V_ISP_AE_REGISTER_S;
#endif


#ifdef __cplusplus
}
#endif

#endif 

