/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file af_lib_interface.h
 *
 * @brief include all Af lib interfance parameters and pointer to function
 *
 *****************************************************************************/
#ifndef __AFI_LIB_INTERFACE_H__
#define __AFI_LIB_INTERFACE_H__

#include "aaa_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef void *AfLibHandle_t;

#define AFM_V1_WINDOW_NUM 3
#define AF_FILTERNUM      5
#define PD_FOCAL_NUM_MAX  48

/*****************************************************************************/
/**
 *          AfLibInstanceConfig_s
 *
 * @brief   Config structure to create a new Af lib instance
 *
 */
/*****************************************************************************/
typedef struct AfLibInstanceConfig_s {
    AfLibHandle_t  hAfLibHandle;           /**< Af Lib handle returned by AfiCreate */
} AfLibInstanceConfig_t;

/*****************************************************************************/
/**
 *          AfctrlPdafPdCalResult_t
 *
 * @brief   PDAF calculating result such as pd shift and so on
 *
 */
/*****************************************************************************/
typedef struct AfctrlPdafPdCalResult_s {
	int32_t   pdShift;              //final pd shift.
	float32_t correctedPd;        //correcte pd shift.
	int32_t   confidence;           //final confidence of pd_shift.
}AfctrlPdafPdCalResult_t;

/*****************************************************************************/
/**
 *          AfMode_t
 *
 * @brief   Af Mode parameters
 *
 */
/*****************************************************************************/
typedef enum AfMode_s {
    AFI_MODE_NORMAL  = 0,
    AFI_MODE_MAX
} AfMode_t;

/*****************************************************************************/
/**
 *          OtpCdaf_t
 *
 * @brief   af CDAF OTP focus parameter
 */
/*****************************************************************************/
typedef struct OtpCdaf_s {
    uint16_t minFocal;
    uint16_t maxFocal;
} OtpCdaf_t;

/*****************************************************************************/
/**
 *          OtpPdaf_t
 *
 * @brief   af PDAF OTP focus parameter
 */
/*****************************************************************************/
typedef struct OtpPdaf_s {
    int32_t       pdFocal[PD_FOCAL_NUM_MAX];
} OtpPdaf_t;

/*****************************************************************************/
/**
 *          AfOTPData_t
 *
 * @brief  All AF OTP value from sensor get
 *
 */
/*****************************************************************************/
typedef struct AfOTPData_s {
   uint32_t   otpVersion;
   bool       otpFocusEnable;  
   OtpCdaf_t  cdafOtp;
   OtpPdaf_t  pdafOtp;
} AfOTPData_t;

/*****************************************************************************/
/**
 *          AfConfig_t
 *
 * @brief   Af lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct AfConfig_s {
	// CDAF params 
    float32_t       weightWindow[AFM_V1_WINDOW_NUM];
    float32_t       cMotionThreshold;
    float32_t       focalStableThreshold;
    float32_t       cStableTolerance;      // [0, 1]
    uint8_t         cPointsOfCurve;         // [5, 100]
    float32_t       focalFilter[AF_FILTERNUM];
    float32_t       shapFilter[AF_FILTERNUM];
    uint16_t        maxFocal;
    uint16_t        minFocal;
    	
	// PDAF params
    float32_t       cMseTolerance;
    float32_t       cPdConfThreshold;
    int32_t         pdFocal[PD_FOCAL_NUM_MAX];
    int32_t         pdDistance;
    float32_t       pdShiftThreshold;
    uint8_t         pdStableCountMax;
} AfConfig_t;

/*****************************************************************************/
/**
 *          AfAtrrParm_t
 *
 * @brief   Af ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AfAttrParm_s {
    AfOTPData_t        afOtpConfig;
} AfAttrParm_t;

typedef enum AfCtrlAfStatisticType_e { 
    AFCTRL_AFMV1_STATISTIC_INVALID = 0,
    AFCTRL_AFMV1_STATISTIC         = 1,
    AFCTRL_AFMV1_1_STATISTIC       = 2,
} AfCtrlAfStatisticType_t;

/*****************************************************************************/
/**
 *          AfInfo_t
 *
 * @brief   AF lib caluclation's statitistic value
 *
 */
/*****************************************************************************/
typedef struct AfInfo_s {
    AaaRect_t                 aaaRect[AFM_V1_WINDOW_NUM];
    uint8_t                   pdafRoiIndex;
    uint32_t                  afV1Sharpness[AFM_V1_WINDOW_NUM];
    uint32_t                  afV1Luminance[AFM_V1_WINDOW_NUM];
	AfctrlPdafPdCalResult_t   pdafPdCalResult;
    AfCtrlAfStatisticType_t   afCtrlAfStatisticType;
    bool                      pdafEnable;
} AfInfo_t;

/*****************************************************************************/
/**
 *          AfResult_t
 *
 * @brief   AF lib calculation result
 *
 */
/*****************************************************************************/
typedef struct AfResult_s {
    uint32_t  lastLensPosition;
    uint32_t  lensPosition;
    float32_t sharpness;
    uint16_t  topFocal;
    int32_t   step;
    uint16_t  internalStatus;
    int32_t   AfLibStatus;
} AfResult_t;

typedef AAA_RESULT (*AfiCreate_t)             (AfLibInstanceConfig_t *pAfLibConfig, uint8_t ispId);
typedef AAA_RESULT (*AfiRelease_t)            (AfLibHandle_t handle, uint8_t ispId);
typedef AAA_RESULT (*AfiSetMode_t)            (AfLibHandle_t handle, const AfMode_t *pAfMode);
typedef AAA_RESULT (*AfiGetMode_t)            (AfLibHandle_t handle, AfMode_t *pAfMode);
typedef AAA_RESULT (*AfiSetConfig_t)          (AfLibHandle_t handle, const AfConfig_t *pAfConfig);
typedef AAA_RESULT (*AfiGetConfig_t)          (AfLibHandle_t handle, AfConfig_t *pAfConfig);
typedef AAA_RESULT (*AfiSetAttribute_t)       (AfLibHandle_t handle, const AfAttrParm_t *pAfAttrParm);
typedef AAA_RESULT (*AfiExecute_t)            (AfLibHandle_t handle, AfInfo_t *pAfInfo, AfResult_t *pAfResult);
typedef AAA_RESULT (*AfiReset_t)              (AfLibHandle_t handle);

/*****************************************************************************/
/**
 *          AfiLib_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef struct AfiLib_s
{
    const char          *pAfLibName;
    AfiCreate_t         pAfiCreate;                
    AfiRelease_t        pAfiRelease;             
    AfiSetMode_t        pAfiSetMode;
    AfiGetMode_t        pAfiGetMode;
    AfiSetConfig_t      pAfiSetConfig;
    AfiGetConfig_t      pAfiGetConfig;
    AfiSetAttribute_t   pAfiSetAttribute;
    AfiExecute_t        pAfiExecute;
    AfiReset_t          pAfiReset;
} AfiLib_t;

#ifdef __cplusplus
}
#endif

#endif 

