/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file af_ctrl_api.h
 *
 * @brief Include all aaa_ctrl Af's function and parameters
 *
 *****************************************************************************/

#ifndef __AF_CTRL_API_H__
#define __AF_CTRL_API_H__

#include "3ai/afi_api.h"
#include "aaa_ctrl_common.h"

#include <cameric_drv/cameric_isp_afm_drv_api.h>

#if defined ISP_PDAF
#include <cameric_drv/cameric_isp_pdaf_drv_api.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#define AF_CTRL_MOTION_THRESHOLD_MIN         0.0f
#define AF_CTRL_MOTION_THRESHOLD_MAX         1.0f

#define AF_CTRL_FOCAL_STABLE_THRESHOLD_MIN   0.0f
#define AF_CTRL_FOCAL_STABLE_THRESHOLD_MAX   1.0f

#define AF_CTRL_STABLE_TOLERANCE_MIN         0.0f
#define AF_CTRL_STABLE_TOLERANCE_MAX         1.0f

#define AF_CTRL_POINT_OFCURVE_MIN            3
#define AF_CTRL_POINT_OFCURVE_MAX            20

#define AF_CTRL_FILTER_MIN                   0.0f
#define AF_CTRL_FILTER_MAX                   1023.0f

#define AF_CTRL_WEIGHT_WINDOW_MIN            0.0f
#define AF_CTRL_WEIGHT_WINDOW_MAX            255.0f

#define AF_CTRL_FOCAL_POSITION_MIN           0.0f
#define AF_CTRL_FOCAL_POSITION_MAX           1023.0f

#define AF_CTRL_MSE_TOLERANCE_MIN            0.0f
#define AF_CTRL_MSE_TOLERANCE_MAX            1.0f

#define AF_CTRL_PD_CONF_THRESHOLD_MIN        0.0f
#define AF_CTRL_PD_CONF_THRESHOLD_MAX        1023.0f

#define AF_CTRL_PD_FOCAL_MIN                 -255.0f
#define AF_CTRL_PD_FOCAL_MAX                 255.0f

#define AF_CTRL_PD_DISTANCE_MIN              0.0f
#define AF_CTRL_PD_DISTANCE_MAX              255.0f

#define AF_CTRL_PD_SHIFT_THRES_MIN           0.0f
#define AF_CTRL_PD_SHIFT_THRES_MAX           1.0f

#define AF_CTRL_PD_STABLE_COUNT_MIN          1
#define AF_CTRL_PD_STABLE_COUNT_MAX          10

typedef struct AfCtrlContext_t *AfCtrlHandle_t;

/*****************************************************************************/
/**
 *          AfCtrlState_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef enum AfCtrlState_e {
    AF_CTRL_STATE_INVALID          = 0,
    AF_CTRL_STATE_INITIALIZED      = 1,
    AF_CTRL_STATE_STOPPED          = 2,                
    AF_CTRL_STATE_RUNNING          = 3,                
    AF_CTRL_STATE_RUNNING_SETTING  = 4,   
    AF_CTRL_STATE_TRACKING         = 5,       
    AF_CTRL_STATE_LOCK             = 6,    
    AF_CTRL_STATE_MAX
} AfCtrlState_t;

/*****************************************************************************/
/**
 *          AfCtrInstanceConfig_t
 *
 * @brief   Config structure to create a new Af lib instance
 *
 */
/*****************************************************************************/
typedef struct AfCtrInstanceConfig_s {
    int                 ispId;
    AfiLib_t            *pAfiLib;           /**< AF Lib struct interface */
    CamerIcDrvHandle_t  hCamerIc;
    IsiSensorHandle_t   hSensor;
    AfCtrlHandle_t      hAfCtrlHandle;      /**< AF ctrl handle which create by Af_CTRL_ctrl_create */
} AfCtrInstanceConfig_t;

/*****************************************************************************/
/**
 *          AfCtrlMode_t
 *
 * @brief   AF Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef enum AfCtrlMode_e {
    AF_CTRL_MODE_NORMAL  = 0,
    AF_CTRL_MODE_MAX
} AfCtrlMode_t;

/*****************************************************************************/
/**
 *          AfCtrlConfig_t
 *
 * @brief   Af lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct AfCtrlConfig_s {
	// CDAF params 
    float32_t    weightWindow[AFM_V1_WINDOW_NUM];
    float32_t    cMotionThreshold;
    float32_t    focalStableThreshold;
    float32_t    cStableTolerance;      // [0, 1]
    uint8_t      cPointsOfCurve;        // [5, 100]
    float32_t    focalFilter[AF_FILTERNUM];
    float32_t    shapFilter[AF_FILTERNUM];
    uint16_t     maxFocal;
    uint16_t     minFocal;
    
	// PDAF params
	float32_t    cMseTolerance;  //mse tolerance which 
	float32_t    cPdConfThreshold;
    int32_t      pdFocal[PD_FOCAL_NUM_MAX];
    int32_t      pdDistance;
    float32_t    pdShiftThreshold;
    uint8_t      pdStableCountMax;
} AfCtrlConfig_t;


/*****************************************************************************/
/**
 *          PdafOutStatistic_t
 *
 * @brief   PDAF statistic which need to calculate
 *
 */
/*****************************************************************************/
typedef struct PdafOutStatistic_s {
	int32_t     pdShift;                  //final pd shift.
	float32_t   correctedPd;              //correcte pd shift.
	uint16_t    confidence;               //final confidence of pd_shift.
} PdafOutStatistic_t;


/*****************************************************************************/
/**
 *          AfCtrlStatistic_t
 *
 * @brief   AF statistic which include CDAF and PDAF statistic
 *
 */
/*****************************************************************************/
typedef struct AfCtrlStatistic_s {
    CamerIcAfmMeasuringResult_t  staticCdaf;
    PdafOutStatistic_t           staticPdaf;
} AfCtrlStatistic_t;


/*****************************************************************************/
/**
 *          AfCtrlRegister
 *
 * @brief   register Af lib and get the Af lib's handle
 *
 * @param   pAfCtrConfig      Ctrl Af function's parameters
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AfCtrlRegister
( 
    AfCtrInstanceConfig_t  *pAfCtrConfig
);


/*****************************************************************************/
/**
 *          AfCtrlUnRegister
 *
 * @brief   unregister Af lib and release the Af lib's handle
 *
 * @param   handle       Handle to the Af lib intance
 * @param   ispId        isp id number
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlUnRegister
( 
    AfCtrlHandle_t handle,
    uint8_t        ispId
);


/*****************************************************************************/
/**
 *          AfCtrlSetMode
 *
 * @brief   Set Af's working mode
 *
 * @param   handle         Handle to the Af lib intance
 * @param   pAfCtrlMode    The pointer to current Working mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlSetMode
( 
    AfCtrlHandle_t      handle,
    const AfCtrlMode_t  *pAfCtrlMode
);


/*****************************************************************************/
/**
 *          AfCtrlGetMode
 *
 * @brief   Get Af's working mode
 *
 * @param   handle         Handle to the Af lib intance
 * @param   pAfCtrlMode    The pointer to current Working mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetMode
( 
    AfCtrlHandle_t  handle,
    AfCtrlMode_t    *pAfCtrlMode
);


/*****************************************************************************/
/**
 *          AfCtrlSetConfig
 *
 * @brief   Set Af's calibration or tunning parameters
 *
 * @param   handle               Handle to the Af lib intance
 * @param   hAfCtrlConfig        The config type to set
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlSetConfig
( 
    AfCtrlHandle_t        handle,
    const AfCtrlConfig_t  *pAfCtrlConfig
);


/*****************************************************************************/
/**
 *          AfCtrlGetConfig
 *
 * @brief   Get Af's calibration or tunning parameters
 *  
 * @param   handle              Handle to the Af lib intance
 * @param   hAfCtrlConfig       The config type to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetConfig
( 
    AfCtrlHandle_t  handle,
    AfCtrlConfig_t  *pAfCtrlConfig
);


/*****************************************************************************/
/**
 *          AfCtrlSetPdafRoiIndex
 *
 * @brief   Set pdaf roi index number which there will 25 roi, user choose one roi to pdaf
 *  
 * @param   handle         Handle to the Af lib intance
 * @param   roiIndex       The roi area's index
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlSetPdafRoiIndex
( 
    AfCtrlHandle_t  handle,
    const uint8_t   *roiIndex
);


/*****************************************************************************/
/**
 *          AfCtrlGetPdafRoiIndex
 *
 * @brief   Get pdaf roi index number which there will 25 roi, user choose one roi to pdaf
 *  
 * @param   handle         Handle to the Af lib intance
 * @param   roiIndex       The roi area's index
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetPdafRoiIndex
( 
    AfCtrlHandle_t  handle,
    uint8_t         *roiIndex
);


/*****************************************************************************/
/**
 *          AfCtrlGetStatus
 *
 * @brief   Get Af's status
 *  
 * @param   handle            Handle to the Af lib intance
 * @param   pAfCtrlState      The af ctrl state
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetStatus
( 
    AfCtrlHandle_t  handle,
    AfCtrlState_t   *pAfCtrlState
);


/*****************************************************************************/
/**
 *          AfCtrlStart
 *
 * @brief   modify the Ctrl Af's state to begin Af module
 *
 * @param   handle           Handle to the Af lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlStart
( 
    AfCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          AfCtrlStop
 *
 * @brief   modify the Ctrl Af's state to stop Af module
 *
 * @param   handle          Handle to the Af lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlStop
( 
    AfCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          CdafCtrlGetStatistic
 *
 * @brief   running Af lib module to get the result
 *
 * @param   handle            Handle to the Af lib intance
 * @param   CdafStatic        cdaf statistic value
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetStatisticAfmv1
( 
    AfCtrlHandle_t              handle,
    const CamerIcAfmMeasuringResult_t *cdafStatic
);


/*****************************************************************************/
/**
 *          PdafCtrlGetStatistic
 *
 * @brief   running Af lib module to get the result
 *
 * @param   handle            Handle to the Af lib intance
 * @param   PdafStatistic     The pointer to pdaf statistic value
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
#if defined ISP_PDAF
RESULT AfCtrlGetStatisticPDAF
( 
    AfCtrlHandle_t             handle,
    const CamericIspPdafEventData_t  *PdafStatistic
);


/*****************************************************************************/
/**
 *          AfCtrlGetPDAFstatistic
 *
 * @brief   get pdaf statistic to engine layer
 *
 * @param   handle             Handle to the Af lib intance
 * @param   pPdafOutStatistic  The pointer to pd out statistic structure
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlGetPDAFstatistic
( 
    AfCtrlHandle_t     handle,
    PdafOutStatistic_t *pPdafOutStatistic
);
#endif


/*****************************************************************************/
/**
 *          AfCtrlExcute
 *
 * @brief   running Af lib module to get the result
 *
 * @param   handle            Handle to the Af lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlExcute
( 
    AfCtrlHandle_t      handle
);


/*****************************************************************************/
/**
 *          AfCtrlReset
 *
 * @brief   reset Af lib to init status
 *
 * @param   handle            Handle to the Af lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AfCtrlReset
( 
    AfCtrlHandle_t  handle
);

#ifdef __cplusplus
}
#endif

#endif
