/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @file binder_hal_api.h
 *
 * @brief   binder api *
 *          binder with abstracts services from different hardware platforms.
 *
 *****************************************************************************/

#ifndef BINDER_HAL_API_H
#define BINDER_HAL_API_H


#include<hal/hal_api.h>
#include<hal/hal_common.h>
#include <ebase/builtins.h>
#include "binder_hal_ops.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief  binder context
 *
 *****************************************************************************/
typedef struct BinderCtx_s {
    uint8_t ispId;
    BinderHalApiOps_t* pHalApiOps;    //pointer to the hal api ops.
    void*   pPrivateCtx;               //pointer to the lower level context.It determines by the lower level.
    HalHandle_t*   hHal;
    BindOpsType    binderMode;        //indictate this is general or tile mode. 
}BinderCtx_t;

typedef void* BinderHandle_t;


/*****************************************************************************/
/**
 * @brief   call the bindered low level opem api.
 * @return  lower level context pointer. NULL on failure.
 *
 *****************************************************************************/
BinderHandle_t BinderHalOpen
(
    uint8_t devId,
   BinderTileCfg_t *pTileCfg,
    BinderWorkModeCfg_t *pModeCfg
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level close api.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalClose
(
    BinderHandle_t binderHandle
);


/*****************************************************************************/
/**
 * @brief   call the bindered low level write register api to write register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalWriteReg
(
    BinderHandle_t binderHandle,
    uint32_t addr,
    uint32_t val
);


/*****************************************************************************/
/**
 * @brief   call the bindered low level read register api to read register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t BinderHalReadReg
(
    BinderHandle_t binderHandle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level write register api to write register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalWriteBatchRegs
(
    BinderHandle_t binderHandle,
    uint32_t addr,
    uint32_t *pVal
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level write register api to write isp id register.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalWriteIdReg
(
    BinderHandle_t binderHandle,
    uint8_t  ispId,
    uint32_t addr,
    uint32_t val
);


/*****************************************************************************/
/**
 * @brief   call the bindered low level read register api to read isp id register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t BinderHalReadIdReg
(
    BinderHandle_t binderHandle,
    uint8_t  ispId,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level read register api to read isp id register.
 * @return  register value
 *
 *****************************************************************************/
uint32_t BinderHalReadIdTbl
(
    BinderHandle_t binderHandle,
    uint8_t  ispId,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   ioctl.
 * @return  RET_SUCCES
 *
 *****************************************************************************/
RESULT BinderHalIoctl
(
    BinderHandle_t binderHandle,
    uint32_t cmd, 
    void *args
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level connect irq api to create irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalConnectIrq
(
    BinderHandle_t binderHandle,
    BinderIrqCtx_t *pIrqCtx,
    uint32_t    int_src,
    osIsrFunc   IsrFunction,
    osDpcFunc   DpcFunction,
    void*       pContext
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level disconnect irq api to destroy irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalDisconnectIrq
(
    BinderHandle_t binderHandle,
    BinderIrqCtx_t *pIrqCtx
);

/*****************************************************************************/
/**
 * @brief   call the bindered low level disconnect irq api to destroy irq handler 
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalDisConnect
(
    BinderHandle_t binderHandle,
    BinderIrqCtx_t *pIrqCtx
);


/*****************************************************************************/
/**
 * @brief   call the bindered low level 
 *           Cmodel specific interface: Hal ISR and Event generator
 * thread.
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalEventGenerator
(
    BinderHandle_t binderHandle,
    HalIrqCtx_t *pIrqCtx,
    HalIsrSrc_t isr_src,
    uint32_t misValue
);

RESULT BinderHalAddRef(
     BinderHandle_t binderHandle
);

RESULT BinderHalDelRef(
     BinderHandle_t binderHandle
);

HalHandle_t BinderHalGetHalHandler(
     BinderHandle_t binderHandle
);

VirtualHandle_t BinderHalGetVirtualHandler(
     BinderHandle_t binderHandle
);

RESULT BinderHalGetBinderMode(
     BinderHandle_t binderHandle,
     BindOpsType *pBinderMode
);

RESULT BinderHalSetIcdev(
     BinderHandle_t binderHandle,
     void*  pIcdev
);
/*****************************************************************************/
/**
 * @brief   get the isp that work count
 * @param   binderHandle binder layer handle.
 * @param   pIspCnt isp count pointer
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalGetIspCnt(
     BinderHandle_t binderHandle,
     uint8_t*  pIspCnt
);

/*****************************************************************************/
/**
 * @brief   get the isp 3dnr reference frame buffer size
 * @param   binderHandle binder layer handle.
 * @param   pRefBufSize isp reference buffer size pointer
 * @return  RESULT of operation.
 *
 *****************************************************************************/
RESULT BinderHalGetRefBufSize(
     BinderHandle_t binderHandle,
     uint32_t       *pRefBufSize
);
#ifdef HAL_CMODEL
RESULT BinderHalBufTransfer(
     BinderHandle_t binderHandle,
     uint32_t trans_ctrl
);

RESULT BinderHalShdRegisterUpdate(
     BinderHandle_t binderHandle,
     uint8_t path
);

RESULT BinderHalCfgDma(
     BinderHandle_t binderHandle,
    HalCmodelBufConfig_t* pBufCfg
);

RESULT BinderHalCfgMi(
     BinderHandle_t binderHandle,
     uint8_t path,
    HalCmodelBufConfig_t* pBufCfg
);

RESULT BinderHalIspPipline(
     BinderHandle_t binderHandle
);
#endif

#ifdef __cplusplus
}
#endif
#endif
