/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef BLS_H
#define BLS_H

/**
 * @file bls.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup ABLS Auto Black Level Subtraction module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"

#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define BLS_CHANNEL_NUM 4
#define BLS_MAX ((1U << 24U) - 1)

#if defined ISP_CMPD_24BIT
#define CMPD_BLS_BIT_WIDTH 24
#elif defined ISP_CMPD_20BIT
#define CMPD_BLS_BIT_WIDTH 20
#endif

/*****************************************************************************/
/**
 *          ABlsHandle_t
 *
 * @brief   ABLS Module instance handle
 *
 *****************************************************************************/
typedef struct ABlsContext_s *ABlsHandle_t;         /**< handle to ABLS context */

/*****************************************************************************/
/**
 *          ABlsConfig_t
 *
 * @brief   ABLS Module configuration structure
 *
 *****************************************************************************/
typedef struct ABlsConfig_s
{
    uint32_t            bls[BLS_CHANNEL_NUM];
    uint8_t             inBit;
    ASensorMode_t       sensorMode;
    IsiSensorHandle_t   hIsiSensor;
    bool_t              imageInput;
} ABlsConfig_t;

typedef struct ABlsAutoParamTable_s {
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    uint32_t bls[AUTO_NUM][BLS_CHANNEL_NUM];
} ABlsAutoParamTable_t;

/*****************************************************************************/
/**
 *          ABlsInstanceConfig_t
 *
 * @brief   ABLS Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ABlsInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    ABlsHandle_t            hABls;              /**< handle returned by ABlsInit() */
} ABlsInstanceConfig_t;

/*****************************************************************************/
/**
 *          ABlsSetMode()
 *
 * @brief   The function set the Black Level Subtraction module mode
 *
 * @param   handle  Handle to bls
 * @param   mode    bls mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsSetMode
(
    ABlsHandle_t        handle,
    const ARunMode_t    mode
);

/*****************************************************************************/
/**
 *          ABlsGetMode()
 *
 * @brief   The function get bls mode
 *
 * @param   handle  Handle to bls
 * @param   mode    bls mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsGetMode
(
    ABlsHandle_t       handle,
    ARunMode_t         *mode
);

/*****************************************************************************/
/**
 *          ABlsInit()
 *
 * @brief   This function initializes the Black Level Subtraction module
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsInit
(
    ABlsInstanceConfig_t *pInstConfig
);

/*****************************************************************************/
/**
 *          ABlsRelease()
 *
 * @brief   This function releases/frees the Black Level Subtraction module
 *
 * @param   handle
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsRelease
(
    ABlsHandle_t handle
);

/*****************************************************************************/
/**
 *          ABlsSetConfigure()
 *
 * @brief   This function set the Black Level Subtraction module configuration
 *
 * @param   handle
 * @param   pConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsSetConfigure
(
    ABlsHandle_t handle,
    ABlsConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ABlsGetConfigure()
 *
 * @brief   This function get the Black Level Subtraction module configuration
 *
 * @param   handle
 * @param   pConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsGetConfigure
(
    ABlsHandle_t handle,
    ABlsConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ABlsGetStatus()
 *
 * @brief   This function get the Black Level Subtraction module status
 *
 * @param   handle
 * @param   pRunning
 * @param   pMode
 * @param   pCurrentParam
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsGetStatus
(
    ABlsHandle_t handle,
    bool_t       *pRunning,
    ARunMode_t   *pMode,
    ABlsConfig_t *pCurrentParam
);

/*****************************************************************************/
/**
 *          ABlsSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to bls
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ABlsSetAutoParamTable
(
    ABlsHandle_t handle,
    ABlsAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ABlsGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to bls
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ABlsGetAutoParamTable
(
    ABlsHandle_t handle,
    ABlsAutoParamTable_t *pParamTable
);


/*****************************************************************************/
/**
 *          ABlsOutSetConfigure()
 *
 * @brief   This function set the Black Level out Subtraction module configuration
 *
 * @param   handle
 * @param   pConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsOutSetConfigure
(
    ABlsHandle_t handle,
    ABlsConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ABlsOutGetConfigure()
 *
 * @brief   This function get the Black Level out Subtraction module configuration
 *
 * @param   handle
 * @param   pConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ABlsOutGetConfigure
(
    ABlsHandle_t handle,
    ABlsConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ABlsProcessFrame()
 *
 * @brief   bls process frame
 *
 * @param   handle  Handle to Cproc
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ABlsProcessFrame
(
    ABlsHandle_t    	handle,
    const float32_t		fGain
);

#ifdef __cplusplus
}
#endif

/* @} ABLS */

#endif /* ABLS_H*/
