/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/*! \page ch3 Buffer Management API
*
*   The Vivante ISP Buffer Management API is defined in the %buffer_management/include/buf_mgmt_api.h,
*   together with data enumerations and structures.
*
*
*/

#ifndef BUF_MGMT_API_H
#define BUF_MGMT_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <oslayer/oslayer.h>
#include "media_buffer_queue.h"
#include "media_buffer_queue_ex.h"


/** \defgroup Enumerations
* @{
*/

/*****************************************************************************/
/**
 * @brief   Enumeration type to configure the notification command of the callback function.
 *
 *****************************************************************************/
typedef enum BufMgmtNotifyCmd_e {

    BUFMGMT_EMPTY_BUF_ADDED     = 0, /**< Add empty buffer*/
    BUFMGMT_EMPTY_BUF_REMOVED,       /**< Remove empty buffer*/
    BUFMGMT_FULL_BUF_ADDED,          /**< Add full buffer*/
    BUFMGMT_FULL_BUF_REMOVED         /**< Remove full buffer*/
} BufMgmtNotifyCmd_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to configure the block type when reading the queue.
 *
 *****************************************************************************/
typedef enum BufMgmtQueBlockType_e {

    BUFMGMT_NONBLOCK_TYPE    =0,    /**< Non-block read queue*/
    BUFMGMT_TIMEOUT_TYPE,           /**< Timeout block read queue*/
    BUFMGMT_BLOCK_TYPE,             /**< Block read queue*/
    BUFMGMT_BLOCK_TYPE_MAX          /**< Max block type*/
}BufMgmtQueBlockType_t;

/** @} Enumerations*/


/** \defgroup Structures
* @{
*/
/*****************************************************************************/
/**
 * @brief   Structure to configure queue read parameters.
 *
 *****************************************************************************/
typedef struct BufMgmtQueOpConfig_s {
    BufMgmtQueBlockType_t blockType;    /**< Block type when reading the queue.*/
    uint32_t              waitTime;     /**< Wait time in units of msec. The wait time should be configured when setting the timeout block.*/
}BufMgmtQueOpConfig_t;


/*****************************************************************************/
/**
 * @brief   The notification callback function pointer.
 *
 *****************************************************************************/
typedef uint32_t (*BufMgmtBufNotifyCbFunc_t)(uint32_t parauInt32,void * paraVoid);


/*****************************************************************************/
/**
 * @brief   Structure to configure notification callback function.
 *
 *****************************************************************************/
typedef struct BufMgmtBufNotifyCb_s {
    BufMgmtBufNotifyCbFunc_t cbFunc;    /**< The notification callback function pointer.*/
    void  *pUserCtx;                    /**< The user context pointer.*/
}BufMgmtBufNotifyCb_t;

/*****************************************************************************/
/**
 * @brief   Structure to configure buffer management context.
 *
 *****************************************************************************/
typedef struct BufMgmtConfig_s
{
    uint32_t                bufQueLength;     /**< Number of empty queue and full queue length. */

    BufMgmtQueOpConfig_t    emptyQueOp;       /**< Empty queue read configurations. */
    BufMgmtQueOpConfig_t    fullQueOp;        /**< Full queue read configurations. */
    BufMgmtBufNotifyCb_t    emptyBufCb;       /**< Empty queue notification callback configurations. */
    BufMgmtBufNotifyCb_t    fullBufCb;        /**< Full queue notification callback configurations. */
} BufMgmtConfig_t;

/*****************************************************************************/
/**
 * @brief   Structure to configure buffer management pool.
 *
 *****************************************************************************/
typedef struct BufMgmtPoolConfig_s {
    BUFF_MODE            bufMode;          /**< The buffer mode */
    uint32_t             bufNum;           /**< The buffer number */
    uint32_t             bufSize;          /**< The buffer size */
    uint16_t             bufAlign;         /**< The buffer align value */
    uint32_t            *pBaseAddrList;    /**< The physical base address list*/
    uint32_t             metadataSize;     /**< The metadata size in media buffer*/
    bool_t               is_mapped;        /**< Indicates whether the Ipladdress value is valid*/
    void               **pIplAddrList;     /**< The ISP platform address(virtual) list*/
}BufMgmtPoolConfig_t;

/** @} Structures*/


/** \defgroup Functions
* @{
*/

/*****************************************************************************/
/**
 * @brief   Initialize the buffer management.
 *
 * @param   pConfig      Buffer management configuration parameters.
 * @param   pHandle      Pointer to the buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_OUTOFMEM       Out of memory
 * @retval  RET_WRONG_CONFIG   Given configuration is invalid
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT BufMgmtInit
(
    BufMgmtConfig_t     *pConfig,
    BufMgmtHandle_t     *pHandle
);


/*****************************************************************************/
/**
 * @brief   Deinitialize the buffer management.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT BufMgmtDeInit
(
    BufMgmtHandle_t bufMgmtHandle
);

/*****************************************************************************/
/**
 * @brief   Create the buffer management pool.
 *
 * @param   pConfig            Buffer management pool configuration parameters.
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_OUTOFMEM       Out of memory
 * @retval  RET_INVALID_PARM   Invalid parameter
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT BufMgmtCreatePool
(
    BufMgmtPoolConfig_t  *pConfig,
    BufMgmtHandle_t       bufMgmtHandle
);


/*****************************************************************************/
/**
 * @brief   Destroy the buffer management pool.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT BufMgmtDestroyPool
(
    BufMgmtHandle_t       bufMgmtHandle
);


/*****************************************************************************/
/**
 * @brief   Start the buffer management.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT BufMgmtStart
(
    BufMgmtHandle_t       bufMgmtHandle
);


/*****************************************************************************/
/**
 * @brief   Stop the buffer management.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT BufMgmtStop
(
    BufMgmtHandle_t       bufMgmtHandle
);


/*****************************************************************************/
/**
 * @brief   Register the empty buffer queue notification callback function.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   notifyFunc         Notification callback function configuration.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_WRONG_CONFIG   Given configuration is invalid
 *
 *****************************************************************************/
RESULT  BufMgmtRegisterEmptyBufNotifyCb
(
    BufMgmtHandle_t       bufMgmtHandle,
    BufMgmtBufNotifyCb_t  notifyFunc
);


/*****************************************************************************/
/**
 * @brief   Deregister the empty buffer queue notification callback function.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT  BufMgmtDeRegisterEmptyBufNotifyCb
(
    BufMgmtHandle_t       bufMgmtHandle
);


/*****************************************************************************/
/**
 * @brief   Register the full buffer queue notification callback function.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   notifyFunc         Notification callback function configuration.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_WRONG_CONFIG   Given configuration is invalid
 *
 *****************************************************************************/
RESULT  BufMgmtRegisterFullBufNotifyCb
(
    BufMgmtHandle_t       bufMgmtHandle,
    BufMgmtBufNotifyCb_t  notifyFunc
);


/*****************************************************************************/
/**
 * @brief   Deregister the full buffer queue notification callback function.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_WRONG_STATE    State machine in wrong state
 *
 *****************************************************************************/
RESULT  BufMgmtDeRegisterFullBufNotifyCb
(
    BufMgmtHandle_t       bufMgmtHandle
);

/*****************************************************************************/
/**
 * @brief   Request an empty buffer from empty buffer queue.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   pBuffer            The media buffer pointer.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_WRONG_CONFIG   Given configuration is invalid
 * @retval  RET_TIME_OUT       Wait time out
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT  BufMgmtRequestEmptyBuffer
(
    BufMgmtHandle_t       bufMgmtHandle,
    MediaBuffer_t       **pBuffer
);


/*****************************************************************************/
/**
 * @brief   Return an empty buffer to empty buffer queue.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   pBuffer            The media buffer pointer.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT  BufMgmtReturnEmptyBuffer
(
    BufMgmtHandle_t       bufMgmtHandle,
    MediaBuffer_t        *pBuffer
);


/*****************************************************************************/
/**
 * @brief   Get a full buffer from full buffer queue.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   pBuffer            The media buffer pointer.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_WRONG_CONFIG   Given configuration is invalid
 * @retval  RET_TIME_OUT       Wait time out
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT  BufMgmtGetFullBuffer
(
    BufMgmtHandle_t       bufMgmtHandle,
    MediaBuffer_t       **pBuffer
);


/*****************************************************************************/
/**
 * @brief   Put a full buffer into full buffer queue.
 *
 * @param   bufMgmtHandle      Buffer management handle.
 * @param   pBuffer            The media buffer pointer.
 *
 * @retval  RET_SUCCESS        Operation succeeded
 * @retval  RET_WRONG_HANDLE   Handle is wrong
 * @retval  RET_NULL_POINTER   The/one/all parameter(s) is a(are) NULL pointer
 * @retval  RET_WRONG_STATE    State machine in wrong state
 * @retval  RET_FAILURE        General failure
 *
 *****************************************************************************/
RESULT  BufMgmtPutFullBuffer
(
    BufMgmtHandle_t       bufMgmtHandle,
    MediaBuffer_t        *pBuffer
);

/** @} Functions*/

#endif /* BUF_MGMT_API_H */

