/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @file buf_mgmt_pool.h
 *
 * @brief
 *         Buffer management Pool interface
 *
 * <pre>
 *
 *   Principal Author: Joerg Detert
 *   Creation date:    Feb 28, 2008
 *
 * </pre>
 *
 *****************************************************************************/
/**
 * @mainpage Welcome to the MediaBufferPool Interface Documentation
 *
 * Doc-Id: xx-xxx-xxx-xxx (MediaBufferPool Implementation Spec)\n
 * Author: Klaus Kaiser
 *
 * @image html silicon_image_logo.gif
 * \n
 *
 *  This buffer service is the central instance for all multimedia modules, which
 *  transmit data through buffers. All buffers are grouped into buffer pools.
 *  Each buffer pool contains one or multiple buffers with the same size. The buffer
 *  service usually manages buffers with uncompressed data (e.g. YUV, RGB, PCM) or
 *  compressed data (Audio-PES, Video-PES) but could be used for any type of data.
 *  Beside the plain data each buffer holds sideband information, e.g. timestamp
 *  and frame resolution. The following picture shows the buffer pool structure.
 *
 * @image html MediaBufferPool.png
 * \n
 *
 * Allocation and management of memory is done through those Media Buffer Pools.
 * The Buffer Pool is created with a bounded size of a contiguous physical memory block,
 * separated into equally sized smaller blocks, the so called Media Buffer. Furthermore
 * it is possible during initialization to specify a random access or ring buffer like
 * access pattern, supporting different types of applications. For example hardware blocks,
 * like decoders, which accesses content in memory through a ring buffer scheme must be
 * supported. The Media Buffer Pool interface provides functionality to request empty
 * buffers, to put or release filled buffers and to request filled buffers. This is the
 * core functionality made available to connected Media Modules.
 *
 *
 * - @subpage media_buffer_queue
 *
 */

#ifndef MEDIA_BUFFER_POOL_H
#define MEDIA_BUFFER_POOL_H


#if defined (__cplusplus)
extern "C" {
#endif

#include <ebase/types.h>
#include <ebase/dct_assert.h>
#include <common/return_codes.h>
#include <common/buf_defs.h>
#include <oslayer/oslayer.h>

/**
 * Definition of Media Buffer Pool Flags.
 */

/** Buffer pool acts like a ring buffer, that means no random access */
#define BUFPOOL_RINGBUFFER    0x00000001U


/**
 * @brief   By now 3 users can register a callback.
 *          These should be 1 consumer, 1 producer and 1 control instance.
 */
#define MAX_NUM_REGISTERED_CB   3

/**
 * @brief   Parameter id, used when calling @ref BufMgmtPoolSetParameter.
 */
typedef int32_t    BufMgmtPoolParamId_t;


/**
 * @brief   Enumerations to indicate which callback to call
 */
typedef enum BufMgmtPoolEvent_e {
    BUFMGMT_POOL_BUF_INVALID       = 0x0000,
    BUFMGMT_POOL_FULLBUF_ADD,
    BUFMGMT_POOL_EMPTYBUF_ADD,
    BUFMGMT_POOL_FULLBUF_REMOVE,
    HIGH_WATERMARK_CRITICAL_REGION_ENTERED  = 0x0010,//TODO: for remove bufferpool/ add
    HIGH_WATERMARK_CRITICAL_REGION_LEFT     = 0x0020,
    LOW_WATERMARK_CRITICAL_REGION_ENTERED   = 0x0030,
    LOW_WATERMARK_CRITICAL_REGION_LEFT      = 0x0040
}BufMgmtPoolEvent_t;

/**
 * @brief   Callback function that can be registered to receive
 *          Media Buffer Pool events.
 */
typedef void (*BufMgmtPoolCbNotify_t)
(
    int32_t              event,
    void*                pUserContext,
    const MediaBuffer_t* pMediaBuffer    //!< Just for information purposes, may be used to handle chained ancillary buffers.
);


/**
 * @brief Structure holding all information to notify a registered user
 */
typedef struct BufMgmtPoolNotify_s
{
    BufMgmtPoolCbNotify_t  fpCallback;      /**< Notification callback */
    void                  *pUserContext;    /**< Pointer to user context to pass to callback */
} BufMgmtPoolNotify_t;

//TODO: for remove bufferpool
/**
 * @brief MediaBufPoolMemory is used to pass memory to the Media Buffer Pool.
 */
typedef struct BufMgmtPoolMemory_s
{
    void       *pMetaDataMemory; /**< Memory for administrative and extra meta data structures (for MediaBuffer and/or ScmiBuffer),
                                    must be of size BufMgmtPoolInternalConfig_t.metaDataMemSize as reported by BufMgmtPoolGetSize. */
    uint32_t    pBufferMemory;   /**< Actual memory where the data is going to be placed,
                                    must be of size BufMgmtPoolInternalConfig_t.bufMemSize as reported by BufMgmtPoolGetSize.
                                    Furthermore, must be aligned as specified by BufMgmtPoolInternalConfig_t.bufAlign. */
    BufIdentity    *pHalBufList;
    uint32_t        total_frame;
    BUFF_MODE       buf_work_mode;
} BufMgmtPoolMemory_t;


/**
 * @brief MediaBufPool pool object, used to maintain a number of
 *        MediaBuffer_t elements. Each MediaBuffer_t can be automatically
 *        associated to a memory chunk. The pool has to be initialized via
 *        BufMgmtPoolCreate() and released via BufMgmtPoolDestroy().
 */
typedef struct BufMgmtPool_s
{
    void                *pOwner;

    void*                pPoolDataMem;          /**< Base address of buffer pool. Note that */
    uint32_t             poolDataSize;
    void*                pMetaDataBase;         /**< Base address of meta data buffer pool for Media Buffers.
                                                     These meta data structures are linked to the Media Buffers. */

    uint32_t             metaDataSize;          /**< Size of meta data associated with media buffer. */

    uint16_t             bufNum;                /**< Number of buffers in buffer pool. */
    uint32_t             bufSize;               /**< Size of each buffer in bytes. */
    uint32_t             poolSize;              /**< Size of complete buffer pool in bytes. */
    MediaBuffer_t       *pBufArray;             /**< Array used to manage each buffer in the pool */
    //uint32_t             flags;               < Buffer Pool flags. 
    uint16_t             freeBufNum;            /**< Resources count: Number of free buffers */
    uint32_t             fillLevel;             /**< How many buffers are filled with data */
    uint32_t             index;                 /**< Pointer to current index in buffer array (internal use) */
    BufMgmtPoolNotify_t  notify[MAX_NUM_REGISTERED_CB];   /**< Array with info about users registered for notification */

    BUFF_MODE            bufMode;               /**<The memory type of this media buffer */
    osMutex              poolMutex;
} BufMgmtPool_t;


typedef struct BufMgmtPoolDataSConfig_s
{
    uint32_t    bufNum;
    uint32_t    metaDataSize;
} BufMgmtPoolDataSConfig_t;

/**
 * @brief The MediaBufPoolConfig is used to calculate the amount of memory needed to use
 *        pool. The call to @ref BufMgmtPoolGetSize will fill the sizes of the needed memory.
 *        The user is then responsible to allocate the memory and pass the structure to the
 *        @ref BufMgmtPoolCreate function.
 *
 */
typedef struct BufMgmtPoolInternalConfig_s
{
    void       *pOwner;
    /* Input parameters */
    uint32_t    bufSize;              /**< Size of memory chunk a buffer represents */
    uint16_t    bufNum;               /**< Initial number of buffers */
    uint32_t    metaDataSize;         /**< Size of extra meta data strutucture attached to each Media buffer (pointed to by MediaBuffer_t.pMetaData) */
    void       *pPoolDataMem;         /**< Memory for administrative and extra meta data structures (for MediaBuffer),
                                            must be of size BufMgmtPoolInternalConfig_t.metaDataMemSize as reported by BufMgmtPoolGetSize. */
    uint32_t    poolDataSize;
    uint32_t   *pBufMemList;
  //  uint32_t    flags;              < Configuration flags 

    uint16_t    bufAlign;             /**< Needed alignment of each memory chunk the buffers represent */

    BUFF_MODE   bufMode;              /**< The memory type of this media buffer */
    bool_t      is_mapped;            /**< Whether to map ISP platform virtual address.*/
    void      **pIplMemList;          /**< The memory type of this media buffer */

} BufMgmtPoolInternalConfig_t;


/**
 * @brief   Get a free buffer from the MediaBufferPool.
 *
 * @param   pBufPool Pointer to the MediaBufferPool.
 *
 *****************************************************************************/
MediaBuffer_t* BufMgmtPoolGetBuffer(BufMgmtPool_t* pBufPool);


/**
 * @brief   Inform MediaBufferPool that buffer has been filled.
 *
 * @param   pBuf  Full buffer to add.
 *
 *****************************************************************************/
void BufMgmtPoolBufferFilled(BufMgmtPool_t* pBufPool, MediaBuffer_t* pBuf);


/**
 * @brief   Free a buffer from the bufferpool.
 *
 * @param   pBuf    Buffer to free
 *
 *****************************************************************************/
void BufMgmtPoolFreeBuffer(BufMgmtPool_t* pBufPool, MediaBuffer_t* pBuf);


/**
 * @brief   Calculate the size of the memory the MediaBufferPool needs
 *          for the metadata the internal structure and the array of buffers.
 *          The caller has to assure to fill the structure with the corresponding
 *          information need. He has to fill bufSize, metaDataSize, bufNum,
 *          bufAlign and maxBufNum. With these information the function can
 *          calculate the amount of memory need.
 *          When the function returns the values for bufPoolMemSize and
 *          bufMemSize are filled.
 *
 * @param   pPoolConfig     Configuration parameters for the media buffer pool.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolGetSize
(
    BufMgmtPoolDataSConfig_t* pConfig,
    uint32_t *pDataSize
);


/**
 * @brief   Create a buffer pool.
 *
 * @param   pBufPool        Pointer to buffer pool object to be created.
 * @param   pPoolConfig     Pointer to config params for buffer pool.
 * @param   poolMemory      Pool memory information.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolCreate
(
    BufMgmtPoolInternalConfig_t  *pPoolConfig,
    BufMgmtPool_t                *pBufPool
);



/*****************************************************************************/
/**
 * @brief   Destroy a buffer pool.
 *
 * @param   pBufPool        Pointer to buffer pool object to be destroyed.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolDestroy
(
    BufMgmtPool_t  *pBufPool
);



/*****************************************************************************/
/**
 * @brief   Reset a buffer pool.
 *
 * @param   pBufPool        Pointer to buffer pool object to be reset.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolReset
(
    BufMgmtPool_t  *pBufPool
);


/* ***************************************************************************/
/**
 * @brief   Register a callback to get informed about MediaBufPool status.
 *
 * @param   pBufPool        Pointer to buffer pool object.
 * @param   fpCallback      Function to be triggered in case of event.
 * @param   pUserContext    Pointer passed in each call of the callback.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolRegisterCb
(
    BufMgmtPool_t          *pBufPool,
    BufMgmtPoolCbNotify_t   fpCallback,
    void                   *pUserContext
);


/****************************************************************************/
/**
 * @brief   Deregister a callback from MediaBufPool
 *
 * @param   pBufPool    Pointer to buffer pool object.
 * @param   fpCallback  Function to be deregistered.
 *
 * @return  Status of operation.
 *****************************************************************************/
RESULT BufMgmtPoolDeregisterCb
(
    BufMgmtPool_t          *pBufPool,
    BufMgmtPoolCbNotify_t   fpCallback
);

//TODO: add for remove bufferpool/
/****************************************************************************/
/**
 * @brief   Set parameter of MediaBufferPool.
 *
 * @param   pBufPool    Pointer to buffer pool object.
 * @param   paramId     Id of parameter to be set.
 *
 * @return  Status of operation.
 *****************************************************************************/
extern RESULT BufMgmtPoolSetParameter(BufMgmtPool_t                 *pBufPool,
                                               BufMgmtPoolParamId_t  paramId,
                                               int32_t               paramValue);


/****************************************************************************/
/**
 * @brief   Get parameter of MediaBufferPool.
 *
 * @param   pBufPool    Pointer to buffer pool object.
 * @param   paramId     Id of parameter to get.
 * @param   pParamValue Address to receive parameter value.
 *
 * @return  Status of operation.
 *****************************************************************************/
extern RESULT BufMgmtPoolGetParameter(BufMgmtPool_t                 *pBufPool,
                                               BufMgmtPoolParamId_t  paramId,
                                               int32_t*              pParamValue);


#if defined (__cplusplus)
}
#endif


#endif  /* MEDIA_BUFFER_POOL_H*/

