/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond DNR3_V3_1
 *
 * @defgroup cam_device_3dnr_v3_1 CamDevice 3DNR V3.1 Definitions
 * @{
 *
 */

#ifndef CAMDEV_3DNR_V3_1_API_H
#define CAMDEV_3DNR_V3_1_API_H

#ifdef __cplusplus
extern "C"
{
#endif


#include "cam_device_common.h"

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR motion configuration.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrMotion_s
{
    bool_t motionDilateEn;  /**< 3DNR enable/disable dilate option of motion mask */
    bool_t motionErodeEn;   /**< 3DNR enable/disable erode option of motion mask */
}CamDevice3DnrMotion_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR work mode.
 *
 *****************************************************************************/
typedef enum CamDevice3DnrWorkMode_e
{
    CAMDEV_3DNR_DISABLE = 0,    /**< Disable filter */
    CAMDEV_3DNR_NLM_MODE,         /**<Enable spatial filter*/
    CAMDEV_3DNR_TNR_MODE,         /**<Enable  temporal filter*/
    CAMDEV_3DNR_TNR_NLM_MODE,     /**<Enable temporal and spatial filter*/
    CAMDEV_3DNR_MODE_MAX,     /**<Enable temporal and spatial filter*/
} CamDevice3DnrWorkMode_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR noise model structure.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrNoiseModelConfig_s
{
    float32_t noisemodelA;   /**<Noise model A range:[0, 1000]*/
    float32_t noisemodelB;   /**<Noise model B range:[0, 1000]*/
    uint16_t fixCurveStart;    /**< Fix curve start */

}CamDevice3DnrNoiseModelConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR current configuration.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrManualConfig_s
{
    uint8_t dilateH;      /**<[3, 7]*/
    float32_t filterLen;      /**<[0, 1024]*/
    float32_t filterLen2;     /**<[0, 1024]*/
    float32_t noiseLevel;     /**<[0, 1024]*/

    uint8_t preWeight;      /**<[0, 32]*/
    uint8_t rangeH;       /**<[3, 7]*/
    uint8_t rangeV;       /**<[3, 7]*/
    uint8_t sadWeight;    /**<[0, 16]*/
    uint8_t strength;     /**<[0, 128]*/
    uint16_t thrMotionSlope; /**<[1, 1024]*/

    float32_t blendMotion;    /**<[0, 100]*/
    float32_t blendSlope;     /**<[0, 32]*/
    float32_t blendStatic;    /**<[0, 100]*/
    float32_t sigma;          /**<[0.1, 100]*/

    CamDevice3DnrNoiseModelConfig_t noiseMode;  /**< Noise mode */
}CamDevice3DnrManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR auto configuration.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrAutoConfig_s
{
    uint8_t autoLevel;  /**< 3DNR auto level */

    float32_t gains[CAMDEV_ISO_STRENGTH_NUM];   /**<  3DNR gains */
    float32_t filterLen[CAMDEV_ISO_STRENGTH_NUM];   /**< IIR filter length of reference frame */
    float32_t filterLen2[CAMDEV_ISO_STRENGTH_NUM];  /**< IIR filter length of motion frame */
    float32_t noiseLevel[CAMDEV_ISO_STRENGTH_NUM];  /**< A threshold to classify static/moving pixel */
    uint16_t thrMotionSlope[CAMDEV_ISO_STRENGTH_NUM];  /**< The threshold of gap between static and 100% moving. */
    uint8_t sadWeight[CAMDEV_ISO_STRENGTH_NUM];  /**< Set weight of motion difference(SAD+mean). Final
                                                             motion detection is a weighted sum of two types
                                                             of motion difference */

    bool_t motionDilateEn[CAMDEV_ISO_STRENGTH_NUM];  /**< Enable/disable dilate option of motion mask */
    bool_t motionErodeEn[CAMDEV_ISO_STRENGTH_NUM];  /**< Enable/disable erode option of motion mask */
    bool_t gammaFeBeEnable[CAMDEV_ISO_STRENGTH_NUM];  /**< Set weight of previous motion in update factor calculation. */

    uint8_t strength[CAMDEV_ISO_STRENGTH_NUM];  /**< General strength. */
    uint8_t rangeH[CAMDEV_ISO_STRENGTH_NUM];  /**< Set motion detection window size in horizontal direction. */
    uint8_t rangeV[CAMDEV_ISO_STRENGTH_NUM];  /**< Set motion detection window size in vertical direction. */
    uint8_t dilateH[CAMDEV_ISO_STRENGTH_NUM];  /**< Set motion dilation window size in horizontal direction. */
    uint8_t preWeight[CAMDEV_ISO_STRENGTH_NUM];  /**< Set weight of previous motion in update factor calculation. */

    float32_t sigma[CAMDEV_ISO_STRENGTH_NUM];  /**< Sigma to generate Gaussian weight table. */
    float32_t blendStatic[CAMDEV_ISO_STRENGTH_NUM];  /**< Weight of spatial NR result in final output for static pixels. */
    float32_t blendMotion[CAMDEV_ISO_STRENGTH_NUM];  /**< Weight of spatial NR result in final output for 100% moving pixels. */
    float32_t blendSlope[CAMDEV_ISO_STRENGTH_NUM];  /**< Merge slope. Larger values mean more NLM image weight. */
} CamDevice3DnrAutoConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR configuration.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrConfig_s {
    CamDeviceConfigMode_t configMode;       /**< The run mode: 0--manual, 1--auto */
    CamDevice3DnrAutoConfig_t autoCfg;      /**< 3DNR auto configuration*/
    CamDevice3DnrManualConfig_t manualCfg;  /**< 3DNR current configuration*/
}CamDevice3DnrConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice 3DNR status structure.
 *
 *****************************************************************************/
typedef struct CamDevice3DnrStatus_s
{
    bool_t enable;    /**< 3DNR enable status*/
    bool_t preGammaEn;                      /**< Enable mask of noise model(a,b) */
    CamDevice3DnrMotion_t motion;  /**< 3DNR motion configuration */
    CamDevice3DnrWorkMode_t workMode;  /**< 3DNR work mode */
    CamDeviceConfigMode_t currentMode;       /**< The run mode: 0--manual, 1--auto */
    CamDevice3DnrManualConfig_t currentCfg;  /**< 3DNR current configuration*/
} CamDevice3DnrStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets 3DNR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to 3DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    Module is not available
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrSetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDevice3DnrConfig_t     *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets 3DNR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to 3DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    Module is not available
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDevice3DnrConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function sets 3DNR motion configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to 3DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrMotionSetConfig
(
    CamDeviceHandle_t            hCamDevice,
    const CamDevice3DnrMotion_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief  This function gets 3DNR motion configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to 3DNR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrMotionGetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDevice3DnrMotion_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function enables 3DNR GammaFebe.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGammaFeBeEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables 3DNR GammaFebe.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGammaFeBeDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function sets the 3DNR work mode.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   workMode            3DNR work mode
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrSetWorkMode
(
    CamDeviceHandle_t hCamDevice,
    CamDevice3DnrWorkMode_t workMode
);

/*****************************************************************************/
/**
 * @brief   This function gets the 3DNR work mode.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   workMode            Pointer to 3DNR work mode
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGetWorkMode
(
    CamDeviceHandle_t hCamDevice,
    CamDevice3DnrWorkMode_t *workMode
);

/*****************************************************************************/
/**
 * @brief   This function gets 3DNR status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pStatus             Pointer to 3DNR status

 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDevice3DnrStatus_t *pStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets 3DNR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets 3DNR version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to 3DNR version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDevice3DnrGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t         *pVersion
);

#ifdef __cplusplus
}
#endif

/* @} cam_device_3dnr_v3_1 */
/* @endcond */

#endif /* CAMDEV_3DNR_V3_1_API_H */

