/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond AFM_V1
 *
 * @defgroup cam_device_afm_v1 CamDevice AFM V1 Definitions
 * @{
 *
 */

#ifndef CAMDEV_AFM_V1_API_H
#define CAMDEV_AFM_V1_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   Enumeration type to identify the AF measuring window.
 *
 *****************************************************************************/
typedef enum CamDeviceAfmWindowId_e
{
    CAMDEV_AFM_WINDOW_INVALID  = 0,    /**< Lower border (only for an internal evaluation) */
    CAMDEV_AFM_WINDOW_A        = 1,    /**< Window A (1st window) */
    CAMDEV_AFM_WINDOW_B        = 2,    /**< Window B (2nd window) */
    CAMDEV_AFM_WINDOW_C        = 3,    /**< Window C (3rd window) */
    CAMDEV_AFM_WINDOW_MAX,             /**< Upper border (only for an internal evaluation) */
} CamDeviceAfmWindowId_t;


/*****************************************************************************/
/**
 * @brief   CamDevice AFM measure results structure.
 *
*****************************************************************************/
typedef struct CamDeviceAfmMeasureResult_s
{
    uint32_t    sharpnessA;         /**< Sharpness of window A */
    uint32_t    sharpnessB;         /**< Sharpness of window B */
    uint32_t    sharpnessC;         /**< Sharpness of window C */

    uint32_t    luminanceA;         /**< Luminance of window A */
    uint32_t    luminanceB;         /**< Luminance of window B */
    uint32_t    luminanceC;         /**< Luminance of window C */

    uint32_t    pixelCntA;         /**< Pixel counts of window A */
    uint32_t    pixelCntB;         /**< Pixel counts of window B */
    uint32_t    pixelCntC;         /**< Pixel counts of window C */
} CamDeviceAfmMeasureResult_t;

/******************************************************************************/
/**
 * @brief   CamDevice AFM window configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceAfmWindowConfig_s
{
    CamDeviceWindow_t  window;    /**< AFM window */
    CamDeviceAfmWindowId_t id;     /**< AFM window index */
} CamDeviceAfmWindowConfig_t;


/******************************************************************************/
/**
 * @brief   CamDevice AFM status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceAfmStatus_s {
    bool_t enable;    /**< AFM enable status */
} CamDeviceAfmStatus_t;

/*****************************************************************************/
/**
 *
 * @brief   This function sets the threshold in the AFM module.
 *
 * @param   hCamDevice     Handle to the CamDevice instance
 * @param   threshold      Threshold value
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmSetThreshold
(
    CamDeviceHandle_t  hCamDevice,
    const uint32_t     threshold

);

/*****************************************************************************/
/**
 *
 * @brief   This function gets the threshold in the AFM module.
 *
 * @param   hCamDevice     Handle to the CamDevice instance
 * @param   pThreshold     Threshold value pointer
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmGetThreshold
(
    CamDeviceHandle_t  hCamDevice,
    uint32_t          *pThreshold
);

/*****************************************************************************/
/**
 *
 * @brief   This function gets the AFM statistic result.
 *
 * @param   hCamDevice       Handle to the CamDevice instance
 * @param   pResult          Measure results pointer
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmGetStatistics
(
    CamDeviceHandle_t              hCamDevice,
    CamDeviceAfmMeasureResult_t   *pResult
);

/*****************************************************************************/
/**
 *
 * @brief   This function sets the AFM statistics window
 *
 * @param   hCamDevice   Handle to the CamDevice instance
 * @param   pWindow      Pointer to window configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmSetMeasureWindow
(
    CamDeviceHandle_t            hCamDevice,
    CamDeviceAfmWindowConfig_t  *pWindow
);

/*****************************************************************************/
/**
 *
 * @brief   This function gets the AFM statistics window
 *
 * @param   hCamDevice   Handle to the CamDevice instance
 * @param   pWindow      Pointer to window configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmGetMeasureWindow
(
    CamDeviceHandle_t                 hCamDevice,
    CamDeviceAfmWindowConfig_t       *pWindow
);

/*****************************************************************************/
/**
 * @brief   This function enables AFM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmEnable
(
    CamDeviceHandle_t  hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables AFM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmDisable
(
    CamDeviceHandle_t  hCamDevice
);

/*****************************************************************************/
/**
 *
 * @brief   This function gets AFM status.
 *
 * @param   hCamDevice     Handle to the CamDevice instance
 * @param   pStatus        Pointer to AFM status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmGetStatus
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceAfmStatus_t     *pStatus
);

/*****************************************************************************/
/**
 * @brief   This function gets AFM version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to AFM version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t         *pVersion
);


/*****************************************************************************/
/**
 * @brief   This function resets AFM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAfmReset
(
    CamDeviceHandle_t             hCamDevice
);

/* @} cam_device_afm_v1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /*CAMDEV_AFM_V1_API_H*/

