/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_CCM_V1_1_API_H
#define CAMDEV_CCM_V1_1_API_H

#include "cam_device_common.h"

/**
 * @cond CCM_V1_1
 * 
 * @defgroup cam_device_ccm_v1_1 CamDevice CCM V1.1 Definitions
 * @{
 *
 */

#define CAMDEV_CC_MATRIX_SIZE       9      /**< Color correction matrix size 3x3*/
#define CAMDEV_CC_COLOR_CHANNEL_NUM 3      /**< Color channel number 3*/


#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   CamDevice CCM manual configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceCcmManualConfig_s {
    float32_t ccMatrix[CAMDEV_CC_MATRIX_SIZE];          /**< Color correction matrix coefficient*/
    float32_t ccOffset[CAMDEV_CC_COLOR_CHANNEL_NUM];    /**< Color offset coefficient*/
}CamDeviceCcmManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice CCM configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceCcmConfig_s {
    CamDeviceCcmManualConfig_t manualCfg;   /**< CCM manual configuration*/
    CamDeviceConfigMode_t      configMode; 
}CamDeviceCcmConfig_t;


/******************************************************************************/
/**
 * @brief   CamDevice CCM status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceCcmStatus_s {
    bool_t enable;              /**< CCM enable status*/
    CamDeviceConfigMode_t currentMode;       /**< The run mode: 0--manual, 1--auto */
    CamDeviceCcmManualConfig_t currentCfg;   /**< CCM current configuration*/
}CamDeviceCcmStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets CCM configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCcmCfg             Pointer to CCM configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmSetConfig
(
    CamDeviceHandle_t     hCamDevice,
    const CamDeviceCcmConfig_t *pCcmCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets CCM configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCcmCfg             Pointer to CCM configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmGetConfig
(
    CamDeviceHandle_t     hCamDevice,
    CamDeviceCcmConfig_t *pCcmCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables CCM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmEnable
(
	CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables CCM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmDisable
(
	CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function gets CCM status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCcmStatus          Pointer to CCM status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmGetStatus
(
	CamDeviceHandle_t     hCamDevice,
	CamDeviceCcmStatus_t *pCcmStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets CCM.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets CCM version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to CCM version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCcmGetVersion
(
	CamDeviceHandle_t hCamDevice,
	uint32_t         *pVersion
);

/* @} cam_device_ccm_v1_1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_CCM_V1_1_API_H
