/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond CPROC
 * 
 * @defgroup cam_device_cproc CamDevice CPROC Definitions
 * @{
 *
 */

#ifndef CAMDEV_CPROC_API_H
#define CAMDEV_CPROC_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef ES_API
typedef enum CamDeviceColorGamutType_e{
    CAMDEV_COLOR_GAMUT_INVALID  = 0,
    CAMDEV_COLOR_GAMUT_BT601    = 1,//bt601 
    CAMDEV_COLOR_GAMUT_BT709    = 2,//BT709
    CAMDEV_COLOR_GAMUT_BT2020   = 3,//BT2020
    CAMDEV_COLOR_GAMUT_USER     = 4,
    CAMDEV_COLOR_GAMUT_MAX
}CamDeviceColorGamutType_t;

typedef struct CamDeviceCsm3x3FloatMatrix_s
{
    float fCoeff[9];               /**< array of 3x3 float values */
} CamDeviceCsm3x3FloatMatrix_t;

typedef struct CamDeviceCsmConfig_s{
    CamDeviceColorGamutType_t colorGamut;
    bool_t                    bLimitedRangeEn;
    CamDeviceCsm3x3FloatMatrix_t csmMatrix;
}CamDeviceCsmConfig_t;
#endif


/******************************************************************************/
/**
 * @brief   CamDevice CPROC pixel clipping range type enumeration for chroma and luma.
 *
 *****************************************************************************/
typedef enum CamDeviceCprocYuvRangeType_e {
    CAMDEV_CPROC_YUV_RANGE_INVALID       = 0,    /**< Invalid range tpye */
    CAMDEV_CPROC_YUV_RANGE_BT601         = 1,    /**< Y/U/V clipping range 16..240 according to ITU-R BT.601 standard */
    CAMDEV_CPROC_YUV_RANGE_FULL_RANGE    = 2,    /**< Full Y/U/V clipping range 0..255 */
    CAMDEV_CPROC_YUV_RANGE_MAX                   /**< Range type max */
}CamDeviceCprocYuvRangeType_t;
/*******************************************/


/******************************************************************************/
/**
 * @brief   CamDevice CPROC YUV range configuration structure.
 *
 *****************************************************************************/
typedef struct CamDeviceCprocYuvRange_s {
    CamDeviceCprocYuvRangeType_t yuvRange;        /**< The clipping range for chrominance pixel output*/
}CamDeviceCprocYuvRangeS_t;

/******************************************************************************/
/**
 * @brief   CamDevice CPROC auto configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceCprocAutoConfig_s {
    uint8_t autoLevel;                          /**< The auto configuration level */

    float32_t gains[CAMDEV_ISO_STRENGTH_NUM];       /**< CPROC gains */
    float32_t contrast[CAMDEV_ISO_STRENGTH_NUM];    /**< CPROC contrast adjust value */
    float32_t bright[CAMDEV_ISO_STRENGTH_NUM];      /**< CPROC brightness adjust value */
    float32_t saturation[CAMDEV_ISO_STRENGTH_NUM];  /**< CPROC saturation adjust value */
    float32_t hue[CAMDEV_ISO_STRENGTH_NUM];         /**< CPROC rotation in HSV domain */
}CamDeviceCprocAutoConfig_t;


/******************************************************************************/
/**
 * @brief   CamDevice CPROC current configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceCprocManualConfig_s {
    float32_t contrast;     /**< CPROC contrast adjust value [0, 1.999]*/
    float32_t bright;		/**< CPROC brightness adjust value [-127, 127]*/
    float32_t saturation;	/**< CPROC saturation adjust value [0, 1.999]*/
    float32_t hue;			/**< CPROC rotation in HSV domain [-90, 90]*/
#ifdef ES_API
    bool_t bCtModeEn;   //0:contrast affect brightness
#endif	
}CamDeviceCprocManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice CPROC configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceCprocConfig_s {
    CamDeviceConfigMode_t configMode;           /**< The run mode: 0--manual, 1--auto */
    CamDeviceCprocAutoConfig_t autoCfg;         /**< CPROC auto configuration*/
    CamDeviceCprocManualConfig_t manualCfg;     /**< CPROC auto configuration*/
}CamDeviceCprocConfig_t;


/******************************************************************************/
/**
 * @brief   CamDevice CPROC status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceCprocStatus_s {
    bool_t enable;              /**< CPROC enable status*/
    CamDeviceConfigMode_t currentMode;           /**< The run mode: 0--manual, 1--auto */
    CamDeviceCprocManualConfig_t currentCfg;     /**< CPROC current configuration*/
}CamDeviceCprocStatus_t;

/*******************************************/


/*****************************************************************************/
/**
 * @brief   This function sets CPROC configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCprocCfg           Pointer to CPROC configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocSetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceCprocConfig_t *pCprocCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets CPROC configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCprocCfg           Pointer to CPROC configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocGetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceCprocConfig_t *pCprocCfg
);

#ifdef ES_API
RESULT EsCamDeviceCsmSetConfig
(
    CamDeviceHandle_t          hCamDevice,
    CamDeviceCsmConfig_t       *pstCsmCfg
);
RESULT EsCamDeviceCsmGetConfig
(
    CamDeviceHandle_t          hCamDevice,
    CamDeviceCsmConfig_t       *pstCsmCfg
);
#endif

/*****************************************************************************/
/**
 * @brief   This function sets CPROC pixel clipping range.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pYuvRange           Pointer to CPROC pixel clipping range
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocSetRange
(
    CamDeviceHandle_t          hCamDevice,
	CamDeviceCprocYuvRangeS_t *pYuvRange
);

/*****************************************************************************/
/**
 * @brief   This function gets CPROC pixel clipping range.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pYuvRange           Pointer to CPROC pixel clipping range
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocGetRange
(
    CamDeviceHandle_t          hCamDevice,
	CamDeviceCprocYuvRangeS_t *pYuvRange
);


/*****************************************************************************/
/**
 * @brief   This function enables CPROC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocEnable
(
	CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables CPROC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocDisable
(
	CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function gets CPROC status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pCprocStatus        Pointer to CPROC status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocGetStatus
(
	CamDeviceHandle_t       hCamDevice,
	CamDeviceCprocStatus_t *pCprocStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets CPROC.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocReset
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets CPROC version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to CPROC version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceCprocGetVersion
(
	CamDeviceHandle_t hCamDevice,
	uint32_t         *pVersion
);

#ifdef __cplusplus
}
#endif

/* @} cam_device_cproc */
/* @endcond */

#endif   // CAMDEV_CPROC_API_H
