/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_DPF_API_H
#define CAMDEV_DPF_API_H

#include "cam_device_common.h"

/**
 * @cond DPF_V1
 *
 * @defgroup cam_device_dpf_v1 CamDevice DPF Definitions
 * @{
 *
 */

#define CAMDEV_DPF_NOISE_CURVE_SIZE 17        /**< Bin number of noise level curve */


#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   CamDevice DPF auto configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceDpfAutoConfig_s {
    uint8_t autoLevel;    /**< DPF auto configuration level */

    float32_t gains[CAMDEV_ISO_STRENGTH_NUM];      /**< Sensor gain */
    float32_t gradient[CAMDEV_ISO_STRENGTH_NUM];   /**< Gradient. Larger value means more denoise strength */
    float32_t offset[CAMDEV_ISO_STRENGTH_NUM];     /**< Offset. Larger value means more denoise strength */
    float32_t min[CAMDEV_ISO_STRENGTH_NUM];        /**< Strength clip */
    float32_t div[CAMDEV_ISO_STRENGTH_NUM];        /**< Div. Larger value means more denoise strength */
    float32_t sigmaG[CAMDEV_ISO_STRENGTH_NUM];     /**< Gaussian sigma for green.Larger values mean more neighbor pixel weight */
    float32_t sigmaRb[CAMDEV_ISO_STRENGTH_NUM];    /**< Gaussian sigma for Red blue. Larger values mean more neighbor pixel weight */
    float32_t noiseCurve[CAMDEV_ISO_STRENGTH_NUM][CAMDEV_DPF_NOISE_CURVE_SIZE];    /**< NUV noise curve. Smaller values mean more denoise strength */
}CamDeviceDpfAutoConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice DPF current configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceDpfManualConfig_s {
    float32_t div;        /**< Div. Larger values mean more denoise strength */
    float32_t gradient;   /**< Gradient. Larger values mean more denoise strength */
    float32_t min;        /**< Strength clip */
    float32_t noiseCurve[CAMDEV_DPF_NOISE_CURVE_SIZE];   /**< NUV noise curve. Smaller values mean more denoise strength */
    float32_t offset;    /**< Offset. Larger values mean more denoise strength */
    float32_t sigmaG;    /**< Gaussian sigma for green. Larger values mean more neighbor pixel weight */
    float32_t sigmaRb;   /**< Gaussian sigma for Red blue. Larger values mean more neighbor pixel weight */

    float32_t gain;       /**< Exposure gain. It's Only used for r-simulator image load */
}CamDeviceDpfManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice DPF configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceDpfConfig_s {
    CamDeviceConfigMode_t mode;             /**< DPF mode configuration */
    CamDeviceDpfAutoConfig_t autoCfg;       /**< DPF auto configuration */
    CamDeviceDpfManualConfig_t manualCfg;   /**< DPF manual configuration */
}CamDeviceDpfConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice DPF status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceDpfStatus_s {
    bool_t enable;                        /**< DPF enable status */
    CamDeviceConfigMode_t currentMode;             /**< DPF mode configuration */
    CamDeviceDpfManualConfig_t currentCfg;   /**< DPF current configuration */
}CamDeviceDpfStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets DPF configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pDpfCfg             Pointer to DPF configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfSetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceDpfConfig_t *pDpfCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets DPF configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pDpfCfg             Pointer to DPF configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceDpfConfig_t *pDpfCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables DPF RAW output.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfRawOutputEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables DPF  RAW output.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfRawOutputDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function enables DPF.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables DPF.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets DPF status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pDpfStatus          Pointer to DPF status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceDpfStatus_t *pDpfStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets DPF.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfReset
(
    CamDeviceHandle_t      hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets DPF version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to DPF version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceDpfGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_dpf_v1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_DPF_API_H
