/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond HDR_V2_1
 *
 * @defgroup cam_device_hdr_v2_1 CamDevice HDR V2.1 Definitions
 * @{
 *
 */

#ifndef CAMDEV_HDR_V2_1_API_H
#define CAMDEV_HDR_V2_1_API_H

#include "cam_device_common.h"


#ifdef __cplusplus
extern "C"
{
#endif

#define CAMDEV_HDR_EXTEND_BIT_NUM              2  /**< Extend bit */
#define CAMDEV_HDR_RATIO_FRAME_NUM             2  /**< Two sets of exposure merge */
#define CAMDEV_HDR_COLOR_CHANNEL_NUM           3  /**< Three color channels, r/g/b */
#define CAMDEV_HDR_COMB_CURVE_NUM              4  /**< Four HDR combination curves */
#define CAMDEV_HDR_TRANS_VALUE_NUM             2  /**< Trans start and end value */


/******************************************************************************/
/**
 * @brief   CamDevice HDR bypass select.
 *
 *****************************************************************************/
typedef enum CamDeviceHdrBypassSelect_s
{
    CAMDEV_HDR_BYPASS_SELECT_L = 0,   /**< Long frame */
    CAMDEV_HDR_BYPASS_SELECT_S,       /**< Short frame */
    CAMDEV_HDR_BYPASS_SELECT_VS       /**< Very short frame */
} CamDeviceHdrBypassSelect_t;

/******************************************************************************/
/**
 * @brief   CamDevice HDR configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceHdrConfig_s
{
    float32_t                           ratio[CAMDEV_HDR_RATIO_FRAME_NUM];    /**< HDR ratio */
    float32_t                           transRange[CAMDEV_HDR_COMB_CURVE_NUM][CAMDEV_HDR_TRANS_VALUE_NUM];    /**< Transform range */
    uint8_t                         colorWeight[CAMDEV_HDR_COLOR_CHANNEL_NUM];  /**< [R weight, G weight, B weight] */
    int8_t                          extendBit[CAMDEV_HDR_EXTEND_BIT_NUM];  /**< [SHORT_LS_EXTEND_BIT, SHORT_VS_EXTEND_BIT] */
} CamDeviceHdrConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice HDR status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceHdrStatus_s
{
    bool_t      hdrEnable;    /**< HDR enable status */
    CamDeviceConfigMode_t currentMode;       /**< The run mode: 0--manual, 1--auto */
    CamDeviceHdrConfig_t currentCfg;   /**< HDR current configuration*/
} CamDeviceHdrStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets HDR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to HDR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrSetConfig
(
    CamDeviceHandle_t     hCamDevice,
    CamDeviceHdrConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets HDR configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to HDR configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrGetConfig
(
    CamDeviceHandle_t     hCamDevice,
    CamDeviceHdrConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function enables HDR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrEnable
(
    CamDeviceHandle_t       hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables HDR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrDisable
(
    CamDeviceHandle_t       hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function gets HDR status
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pStatus             Pointer to HDR status

 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrGetStatus
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceHdrStatus_t     *pStatus
);

/*****************************************************************************/
/**
 * @brief   This function resets HDR.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets HDR version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to HDR version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHdrGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/*****************************************************************************/
/**
 * @brief   (For debug only) This function disables HDR combination mode and sets the HDR bypass frame.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   bypassFrame         The selected bypass frame
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceSetBypassSelectEnable
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceHdrBypassSelect_t  bypassFrame
);


/* @} cam_device_hdr_v2_1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /* CAMDEV_HDR_V2_1_API_H */

