/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_WDRV4_API_H
#define CAMDEV_WDRV4_API_H

#include "cam_device_common.h"

/**
 * @cond WDR_V4
 * 
 * @defgroup cam_device_wdr_v4 CamDevice WDR V4 Definitions
 * @{
 *
 */
#define CAMDEV_WDR_GAMMA_DOWN_CURVE_SIZE   20   /**< Bin number of gamma_down curve */
#define CAMDEV_WDR_GAMMA_PRE_CURVE_SIZE    20   /**< Bin number of gamma_pre curve */
#define CAMDEV_WDR_GAMMA_UP_CURVE_SIZE     20   /**< Bin number of gamma_up curve */
#define CAMDEV_WDR_ENTROPY_LUT_SIZE        20   /**< Bin number of entropy lut */

#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   CamDevice WDR V4 auto configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceWdrAutoConfig_s {
    uint8_t autoLevel;                                /**< WDR V4 auto configuration level */

    float32_t gains[CAMDEV_ISO_STRENGTH_NUM];             /**< WDR V4 gain */
    uint8_t strength[CAMDEV_ISO_STRENGTH_NUM];        /**< WDR V4 strength */
    uint8_t highStrength[CAMDEV_ISO_STRENGTH_NUM];    /**< WDR V4 high strength */
    uint8_t lowStrength[CAMDEV_ISO_STRENGTH_NUM];     /**< WDR V4 low strength */
    uint8_t globalStrength[CAMDEV_ISO_STRENGTH_NUM];  /**< WDR V4 global strength */
    int16_t contrast[CAMDEV_ISO_STRENGTH_NUM];        /**< WDR V4 contrast */
    uint8_t flatStrength[CAMDEV_ISO_STRENGTH_NUM];    /**< WDR V4 flat area strength */
    uint8_t flatThr[CAMDEV_ISO_STRENGTH_NUM];         /**< WDR V4 flat area threshold */
    uint16_t entropySlope[CAMDEV_ISO_STRENGTH_NUM];    /**< WDR V4 factor for all pixel based on image slope */
    uint16_t entropyBase[CAMDEV_ISO_STRENGTH_NUM];     /**< WDR V4 factor for all pixel based on image base */
}CamDeviceWdrAutoConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice WDR V4 current configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceWdrManualConfig_s {
    int16_t contrast;             /**< WDR V4 contrast */
    uint8_t globalStrength;       /**< WDR V4 global strength */
    uint8_t highStrength;         /**< WDR V4 high strength */
    uint8_t lowStrength;          /**< WDR V4 low strength */
    uint8_t strength;             /**< WDR V4 strength */

    uint8_t flatStrength;         /**< WDR V4 flat area strength */
    uint8_t flatThr;              /**< WDR V4 flat area threshold */

    int16_t entropyBase;          /**< WDR V4 factor for all pixel based on image base */
    int16_t entropySlope;         /**< WDR V4 factor for all pixel based on image slope */
    uint32_t entropy[CAMDEV_WDR_ENTROPY_LUT_SIZE]; /**< WDR V4 factor for all pixel based on image entropy */

    uint32_t gammaDown[CAMDEV_WDR_GAMMA_DOWN_CURVE_SIZE];   /**< WDR V4 knee points of gamma down */
    uint32_t gammaPre[CAMDEV_WDR_GAMMA_PRE_CURVE_SIZE];     /**< WDR V4 knee points of gamma pre */
    uint32_t gammaUp[CAMDEV_WDR_GAMMA_UP_CURVE_SIZE];       /**< WDR V4 knee points of gamma up */
}CamDeviceWdrManualConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice WDR V4 configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceWdrConfig_s {
    CamDeviceConfigMode_t configMode;      /**< WDR V4 mode configuration */
    CamDeviceWdrAutoConfig_t autoCfg;      /**< WDR V4 auto configuration */
    CamDeviceWdrManualConfig_t manualCfg;  /**< WDR V4 manual configuration */
}CamDeviceWdrConfig_t;

/******************************************************************************/
/**
 * @brief   CamDevice WDR V4 status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceWdrStatus_s {
    bool_t enable;               /**< WDR V4 enable status */
    CamDeviceConfigMode_t currentMode;      /**< WDR V4 mode configuration */
    CamDeviceWdrManualConfig_t currentCfg;  /**< WDR V4 current configuration */
}CamDeviceWdrStatus_t;


/*****************************************************************************/
/**
 * @brief   This function sets WDR V4 configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWdrCfg             Pointer to WDR V4 configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrSetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceWdrConfig_t *pWdrCfg
);

/*****************************************************************************/
/**
 * @brief   This function gets WDR V4 configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWdrCfg             Pointer to WDR V4 configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrGetConfig
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceWdrConfig_t *pWdrCfg
);

/*****************************************************************************/
/**
 * @brief   This function enables WDR V4.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrEnable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables WDR V4.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrDisable
(
    CamDeviceHandle_t hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function resets WDR V4.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrReset
(
    CamDeviceHandle_t             hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets WDR V4 status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pWdrStatus          Pointer to WDR V4 status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrGetStatus
(
    CamDeviceHandle_t hCamDevice,
    CamDeviceWdrStatus_t *pWdrStatus
);

/*****************************************************************************/
/**
 * @brief   This function gets WDR V4 version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to WDR V4 version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceWdrGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/* @} cam_device_wdr_v4 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_WDRV4_API_H
