/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    DNR3_V3_1
 *
 * @file    cam_engine_3dnr_v3_1_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine ISP.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_3dnr_api CamEngine 3DNR V3.1 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_3DNR_V3_1_API_H
#define CAM_ENGINE_3DNR_V3_1_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include <common/return_codes.h>
#include "cam_engine_common.h"


/*****************************************************************************/
/**
 * @brief   3Dnr3 mode.
 *
 *****************************************************************************/
typedef enum CamEngine3Dnr3Mode_e
{
    CAM_ENGINE_3DNRV3_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_3DNRV3_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_3DNRV3_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_3DNRV3_MODE_MAX
} CamEngine3Dnr3Mode_t;

/******************************************************************************/
/**
 * @brief   Cam Engine 3DNR motion configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngine3Dnr3Motion_s
{
    bool_t motionDilateEn;  /**< 3DNR enable/disable dilate option of motion mask */
    bool_t motionErodeEn;   /**< 3DNR enable/disable erode option of motion mask */
}CamEngine3Dnr3Motion_t;

/******************************************************************************/
/**
 * @brief   Cam Engine 3DNR work mode structure.
 *
 *****************************************************************************/
typedef enum CamEngine3Dnr3WorkMode_e
{
    CAM_ENGINE_3DNR_DISABLE = 0,
    CAM_ENGINE_3DNR_NLM_MODE,         /**<enable spatial filter*/
    CAM_ENGINE_3DNR_TNR_MODE,         /**<enable  temporal filter*/
    CAM_ENGINE_3DNR_TNR_NLM_MODE,     /**<enable temporal and spatial filter*/
    CAM_ENGINE_3DNR_MODE_MAX,     /**<enable temporal and spatial filter*/
} CamEngine3Dnr3WorkMode_t;

/******************************************************************************/
/**
 * @brief   Cam Engine 3DNR noise model structure.
 *
 *****************************************************************************/
typedef struct CamEngine3Dnr3NoiseModelConfig_s
{
    float32_t noisemodelA;   /**<Noise model A range:[0, 1000]*/
    float32_t noisemodelB;   /**<Noise model B range:[0, 1000]*/
    uint16_t fixCurveStart; /**< Fix curve start */

}CamEngine3Dnr3NoiseModelConfig_t;

typedef struct CamEngine3Dnr3LumaConfig_s
{
    float32_t lumaPixelSlop;
    float32_t lumaPixelSlopShift;
    float32_t lumaPixelSlopMin;
    float32_t lumaPixelValShift;
}CamEngine3Dnr3LumaConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine 3DNR manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngine3Dnr3Config_s
{
    uint8_t dilateH;     /**<[3, 7]*/
    float32_t filterLen;      /**<[0, 1024]*/
    float32_t filterLen2;     /**<[0, 1024]*/
    float32_t noiseLevel;     /**<[0, 1024]*/

    uint8_t preWeight;    /**<[0, 32]*/
    uint8_t rangeH;       /**<[3, 7]*/
    uint8_t rangeV;       /**<[3, 7]*/
    uint8_t sadWeight;    /**<[0, 16]*/
    uint8_t strength;     /**<[0, 128]*/
    uint16_t thrMotionSlope; /**<[1, 1024]*/

    float32_t blendMotion;    /**<[0, 100]*/
    float32_t blendSlope;     /**<[0, 32]*/
    float32_t blendStatic;    /**<[0, 100]*/
    float32_t sigma;          /**<[0.1, 100]*/

}CamEngine3Dnr3Config_t;

typedef struct CamEngine3Dnr3Status_s
{
    bool_t preGammaEn;
    CamEngine3Dnr3Config_t config;
    CamEngine3Dnr3Motion_t motion;
    CamEngine3Dnr3WorkMode_t workMode;
    CamEngine3Dnr3NoiseModelConfig_t noiseModel;
} CamEngine3Dnr3Status_t;

/******************************************************************************/
/**
 * @brief   Cam Engine 3DNR auto configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngine3Dnr3AutoConfig_s
{
    uint8_t autoLevel;      /**< 3DNR auto level */

    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];   /**<  3DNR gains */
    float32_t filterLen[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< IIR filter length of reference frame */
    float32_t filterLen2[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< IIR filter length of motion frame */
    float32_t noiseLevel[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< A threshold to classify static/moving pixel */
    uint16_t thrMotionSlope[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< The threshold of gap between static and 100% moving. */
    uint8_t sadWeight[CAM_ENGINE_ISO_STRENGTH_NUM]; /**< Set weight of motion difference(SAD+mean). Final
                                                             motion detection is a weighted sum of two types
                                                             of motion difference */

    bool_t motionDilateEn[CAM_ENGINE_ISO_STRENGTH_NUM]; /**< Enable/disable dilate option of motion mask */
    bool_t motionErodeEn[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< Enable/disable dilate option of motion mask */
    bool_t gammaFeBeEnable[CAM_ENGINE_ISO_STRENGTH_NUM];    /**< Set weight of previous motion in update factor calculation. */

    uint8_t strength[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< General strength. */
    uint8_t rangeH[CAM_ENGINE_ISO_STRENGTH_NUM];    /**< Set motion detection window size in horizontal direction. */
    uint8_t rangeV[CAM_ENGINE_ISO_STRENGTH_NUM];    /**< Set motion detection window size in vertical direction. */
    uint8_t dilateH[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< Set motion dilation window size in horizontal direction. */
    uint8_t preWeight[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< Set weight of previous motion in update factor calculation. */

    float32_t sigma[CAM_ENGINE_ISO_STRENGTH_NUM];       /**< Sigma to generate Gaussian weight table. */
    float32_t blendStatic[CAM_ENGINE_ISO_STRENGTH_NUM]; /**< Weight of spatial NR result in final output for static pixels. */
    float32_t blendMotion[CAM_ENGINE_ISO_STRENGTH_NUM]; /**< Weight of spatial NR result in final output for 100% moving pixels. */
    float32_t blendSlope[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< Merge slope. Larger values mean more NLM image weight. */
} CamEngine3Dnr3AutoConfig_t;

/*****************************************************************************/
/**
 * @brief   This function config the tdnr3 fe be curve
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   modelA              noise model parameters, calibrated according to gain
 * @param   modelB              noise model parameters, calibrated according to gain
 * @param   fixCurveStart
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineUpdate3Dnr3Curve(
    CamEngineHandle_t hCamEngine,
    float64_t *noiseModelA,
    float64_t *noiseModelB,
    uint8_t fixCurveStart
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                input the 3Dnr3 run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3SetMode
(
    CamEngineHandle_t hCamEngine,
    const CamEngine3Dnr3Mode_t  mode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   This function get 3Dnr3 mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                get the 3Dnr3 run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3Mode_t*  pMode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 work mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                input the 3Dnr3 work mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3SetWorkMode
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3WorkMode_t mode           /**< work mode */
);

/*****************************************************************************/
/**
 * @brief   This function get 3Dnr3 mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                get the 3Dnr3 work mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GetWorkMode
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3WorkMode_t *mode           /**< work-mode */
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 motion config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             pointer to config 
 *          motionDilateEn      enable/disable dilation op of motion mask
 *          motionErodeEn       enable/disable erosion op of motion mask
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3MotionSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngine3Dnr3Motion_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function get 3Dnr3 motion config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             pointer to config 
 *          motionDilateEn      enable/disable dilation op of motion mask
 *          motionErodeEn       enable/disable erosion op of motion mask
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3MotionGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngine3Dnr3Motion_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 noise model config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pNoiseModel         input the 3Dnr3 noise model
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineg3Dnr3NoiseModelSetConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3NoiseModelConfig_t *pNoiseModel
);

/*****************************************************************************/
/**
 * @brief   This function get 3Dnr3 noise model config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pNoiseModel         get the 3Dnr3 noise model
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineg3Dnr3NoiseModelGetConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3NoiseModelConfig_t *pNoiseModel
);

/*****************************************************************************/
/**
 * @brief   This function enable 3Dnr3 gamma febe
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GammaFeBeEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function set 3Dnr3 gamma febe disable
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GammaFeBeDisable
(
    CamEngineHandle_t hCamEngine
);


/*****************************************************************************/
/**
 * @brief   This function enable the 3Dnr3
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3Enable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the 3Dnr3
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3Disable
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   This function config the 3Dnr3
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of 3Dnr3
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3SetConfig
(
    CamEngineHandle_t         hCamEngine,
    CamEngine3Dnr3Config_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the 3Dnr3 status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of 3Dnr3
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngine3Dnr3Config_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function set the auto config of 3Dnr3
 * 
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            3Dnr3 auto cfg
 * @param   sensorMode          sensor mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3SetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3AutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the auto config of 3Dnr3
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            3Dnr3 auto cfg
 * @param   sensorMode          sensor mode

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngine3Dnr3AutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the status of 3Dnr3
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pSensorMode         sensor mode
 * @param   pMode               running mode
 * @param   pCfg                3Dnr3 status

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3GetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t *pRunning,
    CamEngineSensorMode_t *pSensorMode,
    CamEngine3Dnr3Mode_t *pMode,
    CamEngine3Dnr3Status_t *pCfg
);

/*****************************************************************************/
/**
 * @brief   This function software reset 3Dnr3
 *
 * @param   hCamEngine          handle to the CamEngine instance

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3Dnr3SwReset
(
    CamEngineHandle_t hCamEngine
);
#ifdef __cplusplus
}
#endif




/* @} cam_engine_3dnr3_api */


#endif /* CAM_ENGINE_3DNR_V3_1_API_H */

