/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond AE_V3
 *
 * @defgroup cam_engine_ae_api CamEngine AE V3 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_AE_API_H
#define CAM_ENGINE_AE_API_H

#include <common/return_codes.h>
#include <3ai/aei_lib_interface.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_AE_CONFIG_NUM      8
#define CAM_ENGINE_AE_ROI_WINDOWS_MAX 25
#define CAM_ENGINE_EXPV2_GRID_SIZE    32

typedef uint8_t CamEngineAev2Static_t[CAM_ENGINE_EXPV2_GRID_SIZE * CAM_ENGINE_EXPV2_GRID_SIZE * 16];

/*****************************************************************************/
/**
 *
 * @brief   The AE state.
 *
*****************************************************************************/
typedef enum CamEngineAeState_s {
    CAMENGINE_AE_STATE_INVALID       = 0,
    CAMENGINE_AE_STATE_INITIALIZED   = 1,
    CAMENGINE_AE_STATE_STOPPED       = 2,
    CAMENGINE_AE_STATE_RUNNING       = 3,
    CAMENGINE_AE_STATE_LOCKED        = 4,
    CAMENGINE_AE_STATE_MAX
} CamEngineAeState_t;

/*****************************************************************************/
/**
 *
 * @brief   Parameters of AE sem mode set and get functions.
 *
*****************************************************************************/
typedef enum CamEngineAeSemMode_e {
    CAMENGINE_AE_SCENE_EVALUATION_FIX        = 0,
    CAMENGINE_AE_SCENE_EVALUATION_ADAPTIVE   = 1,
    CAMENGINE_AE_SCENE_METADATA              = 2,
    CAMENGINE_AE_SCENE_EVALUATION_MAX
} CamEngineAeSemMode_t;

/*****************************************************************************/
/**
 *
 * @brief   Parameters of AE FlickeMode set and get functions.
 *
*****************************************************************************/
typedef enum CamEngineAeFlickerMode_e {
    CAMENGINE_AE_FLICKER_OFF              = 0,
    CAMENGINE_AE_FLICKER_50HZ             = 1,
    CAMENGINE_AE_FLICKER_60HZ             = 2,
    CAMENGINE_AE_FLICKER_CUSTOMER_DEFINE  = 3,
    CAMENGINE_AE_FLICKER_MAX
} CamEngineAeFlickerPeroid_t;

/*****************************************************************************/
/**
 *          CamEngineAeFlickerMode_t
 *
 * @brief   CamEngine AE FlickeMode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct CamEngineAeFlickerMode_s {
    CamEngineAeFlickerPeroid_t  flickerPeriod;
    float                       userDefinedPeriodus;
} CamEngineAeFlickerMode_t;

/*****************************************************************************/
/**
 *
 * @brief   Parameters of AE all mode set and get functions.
 *
*****************************************************************************/
typedef struct CamEngineAeMode_s {
    CamEngineAeSemMode_t     semMode;
    CamEngineAeFlickerMode_t antiBandingMode;                                
} CamEngineAeMode_t;

/*****************************************************************************/
/**
 * @brief   AE ROI window parameters.
 *
*****************************************************************************/
typedef struct CamEngineAeRoiWindow_s {
    uint16_t fx;
    uint16_t fy;
    uint16_t fw;
    uint16_t fh;
    float    weight;
} CamEngineAeRoiWindow_t;

/*****************************************************************************/
/**
 *
 * @brief   Parameters of AE ROI set and get functions.
 *
*****************************************************************************/
typedef struct CamEngineAeRoi_s {
    uint8_t                 aeRoiNum;
    float                   aeRoiWeight;
    CamEngineAeRoiWindow_t  aeRoiWindow[CAM_ENGINE_AE_ROI_WINDOWS_MAX];
} CamEngineAeRoi_t;

/*****************************************************************************/
/**
 *
 * @brief   Lib configurations which lib calculation is needed.
 *
*****************************************************************************/
typedef struct CamEngineAeConfig_s {
    float     setPoint;
    float     tolerance;
    float     dampOver;
    float     dampUnder;
    float     dampUnderRatio;
    float     dampUnderGain;
    float     dampOverRatio;
    float     dampOverGain;
    float     motionFilter;
    float     motionThreshold;
    float     targetFilter;
    float     lowlightLinearRepress[CAM_ENGINE_AE_CONFIG_NUM];
    float     lowlightLinearGain[CAM_ENGINE_AE_CONFIG_NUM];
    uint32_t  lowlightLinearLevel;
    float     lowlightHdrRepress[CAM_ENGINE_AE_CONFIG_NUM];
    float     lowlightHdrGain[CAM_ENGINE_AE_CONFIG_NUM];
    uint32_t  lowlightHdrLevel;
    float     wdrContrastMin;
    float     wdrContrastMax;

    float     expv2WindowWeight[CAM_ENGINE_EXPV2_GRID_SIZE * CAM_ENGINE_EXPV2_GRID_SIZE];
    bool_t    frameCalEnable;
    bool_t    autoHdrEnable;
} CamEngineAeConfig_t;

/*****************************************************************************/
/**
 *
 *
 * @brief   Registers an AE module handle.
 *
 * @param   hCamEngine     A handle to the CamEngine module
 * @param   pAeiLib        The structure of AE lib function pointer
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/

RESULT CamEngineAeRegister
(
		CamEngineHandle_t  hCamEngine,
// #ifndef VENDOR_3A
// 		int 			   ispID,
// #endif
#ifdef ES_3A
		V_ISP_AE_EXP_FUNC_S  *pAeiLib
#else
		AeiLib_t		   *pAeiLib
#endif

);

RESULT CamEngineAeUnRegister
(
    CamEngineHandle_t  hCamEngine
);


// #ifndef VENDOR_3A
// RESULT CamEngineAeInitEs
// (
//     CamEngineHandle_t  hCamEngine
// );

// RESULT CamEngineAeExitEs
// (
//     CamEngineHandle_t  hCamEngine
// );
// #endif


/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the regions of interest (ROI) for AE calculation.
 *
 * @param   hCamEngine    A handle to the CamEngine module
 * @param   pAeRoi        A pointer to the number of ROIs
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeSetRoi
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAeRoi_t   *pAeRoi
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the ROIs for AE calculation.
 *
 * @param   hCamEngine      A handle to the CamEngine module
 * @param   pAeRoi          A pointer to the number of ROIs
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeGetRoi
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAeRoi_t   *pAeRoi
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets AE calculation mode.
 *
 * @param   hCamEngine      A handle to the CamEngine module
 * @param   pAeMode         A pointer to the AE calculation mode
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeSetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAeMode_t  *pAeMode
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets AE calculation mode.
 *
 * @param   hCamEngine         A handle to the CamEngine module
 * @param   pAeMode            A pointer to the AE calculation mode
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeGetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAeMode_t  *pAeMode
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets configurations for AE calculation.
 *
 * @param   hCamEngine           A handle to the CamEngine module
 * @param   pAeConfig            A pointer to the AE calculation configurations

 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeSetConfig
(
    CamEngineHandle_t    hCamEngine,
    CamEngineAeConfig_t  *pAeConfig
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets configurations for AE calculation
 *
 * @param   hCamEngine          A handle to the CamEngine module
 * @param   pAeConfig           A pointer to the AE calculation configurations
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeGetConfig
(
    CamEngineHandle_t    hCamEngine,
    CamEngineAeConfig_t  *pAeConfig
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the data for AE calculation.
 *
 * @param   hCamEngine          A handle to the CamEngine module
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeStart
(
    CamEngineHandle_t  hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Stops AE calculation.
 *
 * @param   hCamEngine          A handle to the CamEngine module
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeStop
(
    CamEngineHandle_t  hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the status of AE calculation.
 *
 * @param   hCamEngine          A handle to the CamEngine module
 * @param   pAeState            The status of AE calculation
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAeGetStatus
(
    CamEngineHandle_t   hCamEngine,
    CamEngineAeState_t  *pAeState
);

/* @} cam_engine_ae_v3 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif

