/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @cond AFM_V1
 * 
 * @defgroup cam_engine_afmv1_api CamEngine AFM V1 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_AFM_V1_API_H
#define CAM_ENGINE_AFM_V1_API_H

#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @enum    CamEngineIspAfmWindowId_t
 *
 * @brief   The autofocus measuring windows.
 *
 *****************************************************************************/
typedef enum CamEngineIspAfmWindowId_e
{
    CAMENGINE_ISP_AFM_WINDOW_INVALID  = 0,    /**< (For internal use only) Lower border */
    CAMENGINE_ISP_AFM_WINDOW_A        = 1,    /**< Window A (1st window) */
    CAMENGINE_ISP_AFM_WINDOW_B        = 2,    /**< Window B (2nd window) */
    CAMENGINE_ISP_AFM_WINDOW_C        = 3,    /**< Window C (3rd window) */
    CAMENGINE_ISP_AFM_WINDOW_MAX,             /**< (For internal use only) Upper border */
} CamEngineIspAfmWindowId_t;

/******************************************************************************/
/**
 *
 * @brief   The statistic values of AFM v1.
 *
 *****************************************************************************/
typedef struct CamEngineAfmMeasuringResult_s
{
    uint32_t    sharpnessA;         /**< Sharpness of window A */
    uint32_t    sharpnessB;         /**< Sharpness of window B */
    uint32_t    sharpnessC;         /**< Sharpness of window C */

    uint32_t    luminanceA;         /**< Luminance of window A */
    uint32_t    luminanceB;         /**< Luminance of window B */
    uint32_t    luminanceC;         /**< Luminance of window C */

    uint32_t    pixelCntA;          /**< Pixel counts of window A */
    uint32_t    pixelCntB;          /**< Pixel counts of window B */
    uint32_t    pixelCntC;          /**< Pixel counts of window C */
} CamEngineAfmMeasuringResult_t;

/*****************************************************************************/
/**
 *
 *
 * @brief   Enables auto focus (AF) measurement.
 *
 * @param   hCamEngine       A handle to the CamEngine module.
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Disables AF measurement.
 *
 * @param   hCamEngine       A handle to the CamEngine Module.
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Returns the enable status of the auto focus module (AFM).
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   IsEnable            The AFM enable status
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineAfmIsEnable
(
    CamEngineHandle_t hCamEngine,
    bool_t            *IsEnable
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the AF measurement threshold.
 *
 * @param   hCamEngine       A handle to the CamEngine module.
 * @param   threshold        The threshold value.
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmSetThreshold
(
    CamEngineHandle_t hCamEngine,
    const uint32_t    threshold
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the AF measurement threshold.
 *
 * @param   hCamEngine       A handle to the CamEngine module.
 * @param   pThreshold       A pointer to the threshold value.
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmGetThreshold
(
    CamEngineHandle_t hCamEngine,
    uint32_t          *pThreshold
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the configurations of AF calculation.
 *
 * @param   hCamEngine   A handle to the CamEngine module
 * @param   AfmWinId     The ID of an AF window
 * @param   pWindow      The window size
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmSetMeasureWindow
(
    CamEngineHandle_t         hCamEngine,
    CamEngineIspAfmWindowId_t AfmWinId,
    CamEngineWindow_t         *pWindow
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the configurations of AF calculation.
 *
 * @param   hCamEngine   A handle to the CamEngine Module
 * @param   AfmWinId     The ID of an AF window
 * @param   pWindow      The window size
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmGetMeasureWindow
(
    CamEngineHandle_t         hCamEngine,
    CamEngineIspAfmWindowId_t AfmWinId,
    CamEngineWindow_t         *pWindow
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets AF statistics from cameric driver.
 *
 * @param   hCamEngine          A handle to the CamEngine Module
 * @param   hAfmv1Statistic     The pointer to AFM v1 statistic value
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfmGetStatistic
(
    CamEngineHandle_t             hCamEngine,
    CamEngineAfmMeasuringResult_t *hAfmv1Statistic
);

/* @} cam_engine_afm_v1 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif 

