/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file cam_engine_api.h
 *
 * @brief
 *   Interface description of the CamEngine.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_api CamEngine API Definitions
 * @{
 */

#ifndef CAM_ENGINE_API_H
#define CAM_ENGINE_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/picture_buffer.h>

//FIXME
#include <common/mipi.h>
#include <isi/isi.h>

#include <cameric_drv/cameric_drv_api.h>
#include <cameric_drv/cameric_isp_drv_api.h>
#include <cameric_drv/cameric_mi_drv_api.h>
#include <binder/binder_hal_api.h>
#ifdef ISP_GCMONO
#include <cameric_drv/cameric_isp_gcmono_drv_api.h>
#endif

#include <cam_calibdb/cam_calibdb_api.h>

#include "cam_engine_common.h"
#if defined ISP_CCM_0407 || defined ISP_CCM_0507
#include "cam_engine/cam_engine_ccm_api.h"
#endif
#include "cam_engine/cam_engine_raw_crop_api.h"
#include <ebase/offline_trace.h>

#if !defined(ISP_VI200_V2_2) && !defined(ISP_VI200_V2_1) && !defined(ISP_VI200_V2) && !defined(ISP_VI200)
#include <video_in/video_in_api.h>
#include "cam_engine/cam_engine_video_in_api.h"
#endif
#include <common/system_module.h>


#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   Input source type of the ISP pipeline.
 *
 *****************************************************************************/
typedef enum CamEngineInputType_e
{
    CAM_ENGINE_INPUT_TYPE_INVALID          = 0,                /**< (For internal evaluation only) Lower border  */
    CAM_ENGINE_SENSOR_INPUT                = 1,                /**< Data from the sensor port to ISP */
    CAM_ENGINE_IMAGE_INPUT                 = 2,                /**< Data from the DMA read port to ISP */
    CAM_ENGINE_INPUT_TYPE_MAX                                  /**< (For internal evaluation only) Upper border */
} CamEngineInputType_t;

/******************************************************************************/
/**
 * @brief   CamEngine working mode.
 *
 *****************************************************************************/
typedef enum CamEngineWorkMode_e
{
    CAM_ENGINE_WORK_MODE_INVALID          = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_STREAM_MODE                = 1,                /**< Sensor direct link mode */
    CAM_ENGINE_RDMA_MODE                  = 2,                /**< Read image data by DMA  */
    CAM_ENGINE_MCM_MODE                   = 3,                /**< MCM mode */
    CAM_ENGINE_BATCH_MODE                 = 4,
    CAM_ENGINE_WORK_MODE_MAX                                  /**< (For internal evaluation only) Upper border */
} CamEngineWorkMode_t;

/******************************************************************************/
/**
 * @brief   CamEngine input configurations.
 *
 *****************************************************************************/
typedef struct CamEngineInputConfig_s
{
    CamEngineInputType_t           inputType;              /**< The input source type */
} CamEngineInputConfig_t;

/******************************************************************************/
/**
 * @brief   The MCM port ID.
 *
 *****************************************************************************/
typedef enum CamEngineMcmPortId_e {
    CAM_ENGINE_MCM_PORT_INVALID          = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_MCM_PORT_0                = 1,                /**< ID 0 */
    CAM_ENGINE_MCM_PORT_1                = 2,                /**< ID 1 */
    CAM_ENGINE_MCM_PORT_2                = 3,                /**< ID 2 */
    CAM_ENGINE_MCM_PORT_3                = 4,                /**< ID 3 */
    CAM_ENGINE_MCM_PORT_MAX                                  /**< (For internal evaluation only) Upper border */
} CamEngineMcmPortId_t;

/******************************************************************************/
/**
 * @brief   The MCM port ID for CamEngine stream mode.
 *
 *****************************************************************************/
typedef CamEngineMcmPortId_t CamEngineStreamPortId_t;

/******************************************************************************/
/**
 * @brief   MCM operation mode.
 *
 *****************************************************************************/
typedef enum CamEngineMcmOp_e {
    CAM_ENGINE_MCM_OP_INVALID            = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_MCM_OP_SW                 = 1,                /**< MCM Software schedule */
    CAM_ENGINE_MCM_OP_HW                 = 2,                /**< MCM hardware schedule */
    CAM_ENGINE_MCM_OP_MAX                                    /**< (For internal evaluation only) Upper border */
}CamEngineMcmOp_t;

/******************************************************************************/
/**
 * @brief   enum type for the camEngine mcm selection
 *
 *****************************************************************************/
typedef enum CamEngineMcmSelection_e {
    CAM_ENGINE_MCM_SEL_INVALID    = -1,               /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_MCM_SEL_RDMA       = 0,                /**< MCM RDMA */
    CAM_ENGINE_MCM_SEL_RDCD       = 1,                /**< MCM RDCD */
    CAM_ENGINE_MCM_SEL_MAX                            /**< upper border (only for an internal evaluation) */
}CamEngineMcmSelection_t;

/******************************************************************************/
/**
 * @brief   MCM mode
 *
 *****************************************************************************/
typedef struct CamEngineMcmMode_s {
    CamEngineMcmPortId_t    portId;                     /**< MCM port ID */
    CamEngineMcmOp_t        mcmOp;                      /**< MCM operation mode  */
    CamEngineMcmSelection_t mcmSel;                     /**< MCM selection mode  */
} CamEngineMcmMode_t;

/******************************************************************************/
/**
 * @brief   Batch mode
 *
 *****************************************************************************/
typedef struct CamEngineBatchMode_s {
      CamEngineStreamPortId_t  portId;                       /**< MCM port ID */
      uint8_t        batchNumber;                      /**< MCM operation mode  */
} CamEngineBatchMode_t;


/******************************************************************************/
/**
 * @brief   MCM stream mode
 *
 *****************************************************************************/
 typedef struct CamEngineStreamMode_s {
    CamEngineStreamPortId_t  portId;       /**< Stream mode port ID */
}CamEngineStreamMode_t;

/******************************************************************************/
/**
 * @brief   The joint mode of an image
 *
 *****************************************************************************/
typedef enum CamEngineJointMode_e {
    CAM_ENGINE_JOINT_MODE_INVALID          = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_JOINT_MODE_2X1               = 1,                /**< The image is made up of 2x1 tiles  */
    CAM_ENGINE_JOINT_MODE_4X3              = 2,                /**< The image is made up of 4x3 tiles */
    CAM_ENGINE_JOINT_MODE_USER              = 3,                /**< The image is made up of user-defined tiles */
    CAM_ENGINE_JOINT_MODE_MAX                                  /**< (For internal evaluation only) Upper border */
}CamEngineJointMode_t;

/******************************************************************************/
/**
 * @brief   The operator to split an image into tiles
 *
 *****************************************************************************/
typedef enum CamEngineTileOp_e {
    CAM_ENGINE_TILE_OP_GENERAL          = 0,                /**< Not tile mode, just general */
    CAM_ENGINE_TILE_OP_SW               = 1,                /**< The software splits the image into tiles  */
    CAM_ENGINE_TILE_OP_HW               = 2,                /**< The hardware splits the image into tiles. This mode is available only when the hardware supports mp_ctrl.  */
    CAM_ENGINE_TILE_OP_MAX                                  /**< (For internal evaluation only) Upper border */
}CamEngineTileOp_t;

/******************************************************************************/
/**
 * @brief   Tile mode
 *
 *****************************************************************************/
typedef struct CamEngineTileModeCfg_s {
      CamEngineJointMode_t      jointMode;                    /**< The joint mode of an image  */
      CamEngineTileOp_t        tileOp;                     /**< The operator to split an image  */
      uint8_t                  xPices;
      uint8_t                  yPices;
} CamEngineTileModeCfg_t;

/******************************************************************************/
/**
 * @brief   ISP output type
 *
 *****************************************************************************/
typedef enum CamEngineOutputType_e {
    CAM_ENGINE_OUTPUT_TYPE_INVALID              = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_OUTPUT_TYPE_MEMORY               = 1,                /**< MI output */
    CAM_ENGINE_OUTPUT_TYPE_STREAM               = 2,                /**< output by data wire direct link next IP */
    CAM_ENGINE_OUTPUT_TYPE_BOTH                 = 3,                /**< Both MI output and HW output*/
    CAM_ENGINE_OUTPUT_TYPE_MAX                  ,                  /**< (For internal evaluation only) Upper border */
}CamEngineOutputType_t;

/******************************************************************************/
/**
 * @brief   ISP output type configuration
 *
 *****************************************************************************/
typedef struct CamEngineOutputConfig_s {
        CamEngineOutputType_t  outputType;                    /**< ISP output type */
} CamEngineOutputConfig_t;

/******************************************************************************/
/**
 * @brief   The switch priority
 *
 *****************************************************************************/
typedef enum CamEngineSwitchPriority_e {
    CAM_ENGINE_SWITCH_PRIORITY_INVALID              = 0,                /**< (For internal evaluation only) Lower border */
    CAM_ENGINE_SWITCH_PRIORITY_0                = 1,                    /**< Proiority 0*/
    CAM_ENGINE_SWITCH_PRIORITY_1                = 2,                    /**< Proiority 1*/
    CAM_ENGINE_SWITCH_PRIORITY_2                = 3,                    /**< Proiority 2*/
    CAM_ENGINE_SWITCH_PRIORITY_3                = 4,                    /**< Proiority 3*/
    CAM_ENGINE_SWITCH_PRIORITY_4                = 5,                    /**< Proiority 4*/
    CAM_ENGINE_SWITCH_PRIORITY_5                = 6,                    /**< Proiority 5*/
    CAM_ENGINE_SWITCH_PRIORITY_6                = 7,                    /**< Proiority 6*/
    CAM_ENGINE_SWITCH_PRIORITY_7                = 8,                    /**< Proiority 7*/
    CAM_ENGINE_SWITCH_PRIORITY_8                = 9,                    /**< Proiority 8*/
    CAM_ENGINE_SWITCH_PRIORITY_9                = 10,                    /**< Proiority 9*/
    CAM_ENGINE_SWITCH_PRIORITY_10                = 11,                    /**< Proiority 10*/
    CAM_ENGINE_SWITCH_PRIORITY_11                = 12,                    /**< Proiority 11*/
    CAM_ENGINE_SWITCH_PRIORITY_12                = 13,                    /**< Proiority 12*/
    CAM_ENGINE_SWITCH_PRIORITY_13                = 14,                    /**< Proiority 13*/
    CAM_ENGINE_SWITCH_PRIORITY_14                = 15,                    /**< Proiority 14*/
    CAM_ENGINE_SWITCH_PRIORITY_15                = 16,                    /**< Proiority 15*/
    CAM_ENGINE_SWITCH_PRIORITY_MAX                  ,                   /**< (For internal evaluation only) Upper border */
} CamEngineSwitchPriority_t;

/******************************************************************************/
/**
 * @brief   Switch control sequence
 *
 *****************************************************************************/
typedef struct CamEngineSwichCtrlCfg_s{
        CamEngineSwitchPriority_t  priority;                  /**< priority parameter */
        void *                 hCascade;                      /**< Cascade ctx handle */
} CamEngineSwichCtrlCfg_t;

/******************************************************************************/
/**
 * @brief   CamEngine mode configuration
 *
 *****************************************************************************/
typedef union CamEngineModeConfig_s {
      CamEngineStreamMode_t  stream;             /**< Stream mode configuration */
  //    CamEngineRdmaMode_t  rdma;
      CamEngineMcmMode_t  mcm;                   /**< MCM mode configuration */
  	  CamEngineBatchMode_t batch;
} CamEngineModeConfig_t;


/******************************************************************************/
/**
 * @brief   CamEngine working configuration
 *
 *****************************************************************************/
typedef struct CamEngineWorkConfig_s {
      CamEngineWorkMode_t  mode;                  /**< CamEngine working mode */
      CamEngineModeConfig_t  modeCfg;             /**< CamEngine mode configuration */
} CamEngineWorkConfig_t;


/******************************************************************************/
/**
 * @brief   Configurations of a CamEngine instance.
 *
 *****************************************************************************/
typedef struct CamEngineInstanceConfig_s
{
    uint32_t                    maxPendingCommands; /**< Number of commands that can be queued and thus be pending at a time. */
    bool_t                   isSystem3D;            /**< System capable for 3D */

    CamEngineCompletionCb_t     cbCompletion;       /**< Callback function for command completion. */
    void                        *pUserCbCtx;        /**< User context passed on to completion & Afps callbacks. */

    HalMemHandle_t              hMemHal;             /**< HAL memory handle (hardware abstraction layer). */

    CamEngineHandle_t           hCamEngine;         /**< Handle to CamEngine context, set by @ref CamEngineInit if successful, undefined otherwise. */
    bool hdr;
    uint32_t                            hwId;                 /**< Hardware ID */
    uint32_t                            instanceId;           /**< Virtual instance ID */
    CamEngineInputConfig_t              inputCfg;             /**< Input configurations */
    CamEngineWorkConfig_t               workCfg;             /**< Working mode configurations */
    CamEngineOutputConfig_t             outputCfg;             /**< Output configurations */
    CamEngineSwichCtrlCfg_t             swCtrlCfg;             /**< Switch control configurations */
    CamEngineTileModeCfg_t              tileCfg;              /**< Tile mode configurations */
    CamEngineSensorMode_t               sensorMode;
    uint8_t                             hVideoHwId;
} CamEngineInstanceConfig_t;

/******************************************************************************/
/**
 * @brief   Configurations of a CamEngine sensor.
 *
 *****************************************************************************/
typedef struct CamEngineConfigSensor_s
{
    /* sensor interface */
    IsiSensorHandle_t               hSensor;                /**< The main sensor handle (measuring running on this sensor pipe) */
    IsiSensorHandle_t               hSubSensor;             /**< The subordinate sensor handle */

    CamCalibDbHandle_t              hCamCalibDb;            /**< A handle to calibration data base address */

    /* input interface */
    CamerIcInterfaceSelect_t        ifSelect;

    /* input acquisition */
    CamerIcIspSampleEdge_t          sampleEdge;
    CamerIcIspPolarity_t            hSyncPol;
    CamerIcIspPolarity_t            vSyncPol;
    CamerIcIspBayerPattern_t        bayerPattern;
    CamerIcIspColorSubsampling_t    subSampling;
    CamerIcIspCCIRSequence_t        seqCCIR;
    CamerIcIspFieldSelection_t      fieldSelection;
    CamerIcIspInputSelection_t      inputSelection;
#ifdef ISP_DVP_PINMAPPING
            CamerIcIspDvpPinMappig_t        dvpPinMapping;
#endif
#ifdef ISP_DPF_RAW
            CamerIcIspDpfRaw_t              dpf_mode;
#endif
#ifdef ISP_MI_FIFO_DEPTH_NANO
            CamerIcMiMpOutputFifoDepth_t    mi_mp_fifo_depth;
#endif
#ifdef ISP_MI_ALIGN_NANO
            CamerIcMiMpOutputLsbAlign_t     mi_mp_lsb_align;
#endif
#ifdef ISP_MI_BYTESWAP
            CamerIcMiMpOutputByteSwap_t     mi_mp_byte_swap;
#endif
#ifdef ISP_GCMONO
            CamerIcIspGcMonoContext_t       gcmono_ctx;
#endif
            /* isp configuration */
            CamerIcIspMode_t                mode;
            CamEngineWindow_t               acqWindow;
            CamEngineWindow_t               outWindow;
            CamEngineWindow_t               isWindow;

            MipiDataType_t                  mipiMode;

            bool_t                          enableTestpattern;

            CamEngineFlickerPeriod_t        flickerPeriod;
            bool_t                          enableAfps;
            char szSensorName[256];
            char szCsiName[256];
            int64_t csiFormat;
            uint16_t csiPad;
            uint32_t stitchingMode;
            CamerIcISPCompandCurveConfig_t compress_curve;
            CamerIcISPCompandCurveConfig_t expand_curve;
            uint32_t work_mode;
            uint8_t instance_id;
} CamEngineConfigSensor_t;

/******************************************************************************/
/**
 * @brief   Configurations of a CamEngine acq.
 *
 *****************************************************************************/
typedef struct CamEngineConfigAcq_s
{
    /* input acquisition */
    CamerIcIspSampleEdge_t          sampleEdge;
    CamerIcIspPolarity_t            hSyncPol;
    CamerIcIspPolarity_t            vSyncPol;
    CamerIcIspBayerPattern_t        bayerPattern;
    CamerIcIspColorSubsampling_t    subSampling;
    CamerIcIspCCIRSequence_t        seqCCIR;
    CamerIcIspFieldSelection_t      fieldSelection;
    CamerIcIspInputSelection_t      inputSelection;
    CamEngineWindow_t               acqWindow;
    CamEngineWindow_t               outWindow;
    CamEngineWindow_t               isWindow;
    uint32_t                        stitchingMode;

} CamEngineConfigAcq_t;

/******************************************************************************/
/**
 * @brief   Configurations of a CamEngine image.
 *
 *****************************************************************************/
typedef struct CamEngineConfigImage_s
{
    /* image information */
    PicBufType_t                    type;
    PicBufLayout_t                  layout;

    uint8_t                         *pBuffer;
    uint16_t                        width;
    uint16_t                        height;
    uint32_t                        is_lsb_align;

    CamEngineWbGains_t              *pWbGains;
#if defined ISP_CCM_0407 || defined ISP_CCM_0507
    CamEngineCcmConfig_t            *pCcmConfig;
#endif
    CamEngineBlackLevel_t           *pBlvl;

#ifdef ISP_DVP_PINMAPPING
            CamerIcIspDvpPinMappig_t        dvpPinMapping; //for test bench enabling
#endif
#ifdef ISP_DPF_RAW
            CamerIcIspDpfRaw_t              dpf_mode;
#endif
#ifdef ISP_MI_FIFO_DEPTH_NANO
            CamerIcMiMpOutputFifoDepth_t    mi_mp_fifo_depth;
#endif
#ifdef ISP_MI_ALIGN_NANO
            CamerIcMiMpOutputLsbAlign_t     mi_mp_lsb_align;
#endif
#ifdef ISP_MI_BYTESWAP
            CamerIcMiMpOutputByteSwap_t     mi_mp_byte_swap;
#endif
#ifdef ISP_GCMONO
            CamerIcIspGcMonoContext_t       gcmono_ctx;
#endif
    uint32_t                        tpg_enabled;

    /* add raw image support */
    CamerIcIspFieldSelection_t      fieldSelection;
    CamerIcIspInputSelection_t      inputSelection;
    CamerIcISPCompandCurveConfig_t compress_curve;
    CamerIcISPCompandCurveConfig_t expand_curve;
    char                            align;  // raw data align mode

    /* input interface */
    CamerIcInterfaceSelect_t        ifSelect;
    uint32_t stitchingMode;
    CamerIcIspPolarity_t            hSyncPol;
    CamerIcIspPolarity_t            vSyncPol;
} CamEngineConfigImage_t;

/******************************************************************************/
/**
 * @brief   Configuration structure of the CamEngine.
 *
 *****************************************************************************/
typedef struct CamEngineConfig_s
{
    CamEngineConfigType_t type;

    uint8_t               bitWidth;
    CamEngineSensorMode_t sensorMode;
    union CamEngineConfigData_u
    {
        CamEngineConfigSensor_t sensor;
        CamEngineConfigImage_t  image;
    } data;

} CamEngineConfig_t;

/******************************************************************************/
/**
 * @brief   Configuration of the CamEngine buffer management.
 *
 *****************************************************************************/

typedef struct CamEngineBufMgmt_s {
    bool_t                  enable;
    CamEnginePathType_t     chainId;
    BufMgmtHandle_t         bufMgmtHandle;
} CamEngineBufMgmt_t;

/******************************************************************************/
/**
 * @brief   Configuration of the CamEngine buffer management config.
 *
 *****************************************************************************/
typedef struct CamEngineBufMgmtConfig_s {
    CamEngineBufMgmt_t    bufMgmt;
    CamEngineBufMgmt_t    inputBufMgmt;
} CamEngineBufMgmtConfig_t;


/******************************************************************************/
/**
 * @brief   path streaming cfg
 *
 *****************************************************************************/
typedef struct CamEnginePathStatus_s {
    bool_t pathEnable[CAM_ENGINE_PATH_MAX];
} CamEnginePathStatus_t;

/******************************************************************************/
/**
 * @brief   path crop windows cfg
 *
 *****************************************************************************/
typedef struct CamEnginePathCropWindowCfg_s {
    CamEngineWindow_t  cropWindow[(uint8_t)CAM_ENGINE_PATH_SELF2_BP + 1U];
} CamEnginePathCropWindowCfg_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to identify the cam engine output path
 *
 *****************************************************************************/

#define CAMENGINE_MI_SEPERATE_PATH_INVALID   -1    /**< lower border (only for an internal evaluation) */
#define CAMENGINE_MI_SEPERATE_PATH_MAIN      0x1U   /**< main path index */
#define CAMENGINE_MI_SEPERATE_PATH_SELF      0x2U   /**< self path index */
#define CAMENGINE_MI_SEPERATE_PATH_SELF2_BP  0x4U   /**< self path index */
#define CAMENGINE_MI_SEPERATE_PATH_MAIN_RAW  0x8U   /**< duplicate main path raw output, recommend use when main paht both output YUV and Raw case */
#ifdef ISP_MI_HDR
    #define CAMENGINE_MI_SEPERATE_PATH_HDR_RAW 0x10U  /**< retiming write HDR raw path index */
#endif
#ifdef ISP_MI_PP_WRITE
    #define CAMENGINE_MI_SEPERATE_PATH_POST_PROCESS  0x20U  /**< pp path index */
#endif
#define CAMENGINE_MI_SEPERATE_PATH_MAX 0x21U

// #ifndef VENDOR_3A//xiaoy add:if it does not use vendor 3a,it should malloc camEngine before init
// RESULT CamEngineCreateEs
// (
//     CamEngineHandle_t *phCamEngine
// );
// #endif


/*****************************************************************************/
/**
 * @brief   Creates and initializes a CamEngine instance.
 *
 * @param   pConfig     Instance configuration structure.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_INVALID_PARM    Invalid configuration
 * @retval  RET_OUTOFRANGE      A configuration parameter is out of range
 * @retval  RET_WRONG_HANDLE    Invalid HAL handle
 * @retval  RET_OUTOFMEM        Available memory is not enough
 *
 *****************************************************************************/
RESULT CamEngineInit
(
    CamEngineInstanceConfig_t *pConfig
);


/*****************************************************************************/
/**
 * @brief   Shutdowns and destroys a CamEngine instance.
 *
 * @param   hCamEngine Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state to shutdown
 *
 *****************************************************************************/
RESULT CamEngineShutDown
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Starts a CamEngine instance. \n The CamEngine is configured and ready for streaming but streaming is not yet started.
 * Configuration settings of the auto and image processing algorithms may be accessed and altered.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pConfig     Pointer to the configuration structure.
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state to shutdown
 *
 *****************************************************************************/
RESULT CamEngineStart
(
    CamEngineHandle_t   hCamEngine,
    CamEngineConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Stops a CamEngine instance.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineStop
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Starts streaming.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   frames          Number of frames to capture. frames = 0 means continuously.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineStartStreaming
(
    CamEngineHandle_t   hCamEngine,
    uint32_t            frames
);


/*****************************************************************************/
/**
 * @brief   Stops streaming.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineStopStreaming
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Loads an image.
 *
 * @param   uint32_t      Command type.
 * @param   void*         Pointer to the CamEngine instance context.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineLoadimageCallBack
(
    uint32_t          cmd,
    void	     *pUserCtx
);


/*****************************************************************************/
/**
 * @brief   Registers a buffer callback. \n Only one buffer callback may be registered at a time.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   fpCallback      Buffer callback to register.
 * @param   pUserContext    User context to pass on to callback.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NULL_POINTER    callback is null pointer
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineRegisterBufferCb
(
    CamEngineHandle_t   hCamEngine,
    CamEngineBufferCb_t fpCallback,
    void*               pBufferCbCtx
);


/*****************************************************************************/
/**
 * @brief   Unregisters the buffer callback.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineDeRegisterBufferCb
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Returns the current buffer callback.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineGetBufferCb
(
    CamEngineHandle_t    hCamEngine,
    CamEngineBufferCb_t* fpCallback,
    void**               ppBufferCbCtx
);


/*****************************************************************************/
/**
 * @brief   Returns the state of the CamEngine.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 *
 * @return The state of the CamEngine.
 *
 *****************************************************************************/
CamEngineState_t CamEngineGetState
(
    CamEngineHandle_t   hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Sets the configuration of main and self path.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   pConfigMain     Configuration structure for main path.
 * @param   pConfigSelf     Configuration structure for self path.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineSetPathConfig
(
    CamEngineHandle_t               hCamEngine,
    CamEnginePathType_t             path,
    const CamEnginePathConfig_t     *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets the configuration of buffer management
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   pConfig         Structure of buffer management configurations.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineSetBufMgmtConfig
(
    CamEngineHandle_t               hCamEngine,
    CamEngineBufMgmtConfig_t       *pConfig
);


/*****************************************************************************/
/**
 * @brief   Sets the configuration of the main- and self-path.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   pConfigMain     Structure of main-path configurations.
 * @param   pConfigSelf     Structure of self-path configurations.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineSetCalibDb
(
    CamEngineHandle_t   hCamEngine,
    CamCalibDbHandle_t  hCamCalibDb

);


/*****************************************************************************/
/**
 * @brief   Sets the ECM configurations.
 *
 * @param   hCamEngine         A handle to the CamEngine instance.
 * @param   locks              Requested locks, locks = 0xFF, means all.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
RESULT CamEngineSearchAndLock
(
    CamEngineHandle_t   hCamEngine,
    CamEngineLockType_t locks
);


/*****************************************************************************/
/**
 * @brief   Sets the configurations of the main and self path.
 *
 * @param   hCamEngine      Handle to the CamEngine instance.
 * @param   locks           Requested locks, locks = 0xFF, means all.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEngineUnlock
(
    CamEngineHandle_t   hCamEngine,
    CamEngineLockType_t locks
);


/*****************************************************************************/
/**
 * @brief   Runs the auto algorithms and locks the settings of the requested locks, e.g. autofocus, auto exposure control
            and auto white balance.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pConfig     Configuration structure.
 *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
uint32_t CamEngineUpdateIspResolution
(
    CamEngineHandle_t   hCamEngine,
    const CamEngineConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   The function sets path enable
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pPathEnable Path enable
 *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEnginePathEnable
(
    CamEngineHandle_t hCamEngine,
    uint8_t pathEnable
);

/*****************************************************************************/
/**
 * @brief   Cancels the requested locks if locked.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pPathEnable Path enable
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_STATE     The instance is in a wrong state
 *
 *****************************************************************************/
RESULT CamEnginePathDisable
(
    CamEngineHandle_t hCamEngine,
    uint8_t pathDisable
);

/*****************************************************************************/
/**
 * @brief   The function gets the path status
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pPathStatus Path status
 *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGetPathStatus
(
    CamEngineHandle_t hCamEngine,
    CamEnginePathStatus_t *pPathStatus
);


/*****************************************************************************/
/**
 * @brief   Updates the DMA input resolution and configures MI data. \n
 *          This function is avaliable when CamEngine is in the running state.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pPathCropWindow Path crop window
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineSetPathCropWindows
(
    CamEngineHandle_t hCamEngine,
    CamEnginePathCropWindowCfg_t *pPathCropWindow
);

/*****************************************************************************/
/**
 * @brief   The function gets the path crop window
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pPathCropWindow Path crop window
 *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGetPathCropWindows
(
    CamEngineHandle_t hCamEngine,
    CamEnginePathCropWindowCfg_t *pPathCropWindow
);


/*****************************************************************************/
/**
 * @brief   The function set the AcqConfig
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   pConfig AcqConfig
 *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineSetupAcqConfig
(
    CamEngineHandle_t  hCamEngine,
    const CamEngineConfigAcq_t   *pConfig
);

uint32_t CamEngineGetHwResources(void);

// uint32_t CamEngineGetIspFeatrue();

// uint32_t CamEngineGetFeatureVersion();

/*****************************************************************************/
/**
 * @brief   This function is to init hdr mode.
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 * @param   enableHDR   enable hdr.
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/

RESULT CamEngineInitHdrMode(CamEngineHandle_t hCamEngine, bool_t enableHDR);

/*****************************************************************************/
/**
 * @brief   This function is to fast stop isp
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineFastStop
(
    CamEngineHandle_t   hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function is to fast start isp
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineFastStart
(
    CamEngineHandle_t   hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function is to reset isp
 *
 * @param   hCamEngine  Handle to the CamEngine instance.
 *
 * @return  The result of the function call.
 * @return  RET_PENDING         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineHwReset
(
    CamEngineHandle_t   hCamEngine
);

RESULT CamEngineSetEnqueueBufCb
(
    CamEngineHandle_t hCamEngine,
    IspSystemEnqueueBuf_t func,
    void                  *pUserContext,
    void                  *pModuleContext
);

RESULT CamEngineSetDequeueBufCb
(
    CamEngineHandle_t hCamEngine,
    IspSystemDequeueBuf_t func,
    void                  *pUserContext,
    void                  *pModuleContext
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_api */


#endif /* CAM_ENGINE_API_H */

