/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @cond AWB_V1
 * 
 * @defgroup cam_engine_awbv3_api Cam Engine AWB V3 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_AWB_API_H
#define CAM_ENGINE_AWB_API_H

#include <common/return_codes.h>
#include <3ai/awbi_lib_interface.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_AWB_ROI_WINDOWS_MAX      25
#define CAM_ENGINE_AWB_COLOR_TEM_WEIGHT_NUM 7

/*****************************************************************************/
/**
 *          CamEngineAwbTemWeight_t
 *
 * @brief   Awb color temperature weight of every lighting
 *
 */
/*****************************************************************************/
typedef float CamEngineAwbTemWeight_t[CAM_ENGINE_AWB_COLOR_TEM_WEIGHT_NUM];

/*****************************************************************************/
/**
 *          CamEngineAwbState_t
 *
 * @brief   The camengine AWB state
 *
 */
/*****************************************************************************/
typedef enum CamEngineAwbState_e {
    CAMENGINE_AWB_STATE_INVALID       = 0,
    CAMENGINE_AWB_STATE_INITIALIZED   = 1,
    CAMENGINE_AWB_STATE_STOPPED       = 2,
    CAMENGINE_AWB_STATE_RUNNING       = 3,
    CAMENGINE_AWB_STATE_LOCKED        = 4,
    CAMENGINE_AWB_STATE_MAX
} CamEngineAwbState_t;

/*****************************************************************************/
/**
 *          CamEngineAwbModeType_t
 *
 * @brief   CamEngine AWB Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef enum CamEngineAwbModeType_e {
    CAMENGINE_MODE           = 0, 
    CAMENGINE_METEDATA_MODE  = 1,
} CamEngineAwbModeType_t;

/*****************************************************************************/
/**
 *          CamEngineAeRoiWindow_t
 *
 * @brief   CamEngine Awb Roi window parameters
 *
 */
/*****************************************************************************/
typedef struct CamAwbRoiWindow_s { 
    uint16_t fx;    
    uint16_t fy;    
    uint16_t fw;   
    uint16_t fh;  
    float weight;
} CamEngineAwbRoiWindow_t;

/*****************************************************************************/
/**
 *          CamEngineAwbRoi_t
 *
 * @brief   CamEngine Awb Roi set and get function's parameters
 *
 */
/*****************************************************************************/
typedef struct CamEngineAwbRoi_s { 
    uint8_t                  awbRoiNum;
    float                    awbRoiWeight;
    CamEngineAwbRoiWindow_t  awbRoiWindow[CAM_ENGINE_AWB_ROI_WINDOWS_MAX];
} CamEngineAwbRoi_t;

/*****************************************************************************/
/**
 *          CamEngineAwbConfig_t
 *
 * @brief   CamEngine AWB lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct CamAwbConfig_s {
    bool  useDamping;
    float roiWeight;
} CamEngineAwbConfig_t;

/*****************************************************************************/
/**
 *          CamEngineAwbRegister
 *
 * @brief   Regiter CamEngine AWB module handle
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 * @param   pAeiLib        Awb lib function pointer's structe
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbRegister
(
    CamEngineHandle_t  hCamEngine,
// #ifndef VENDOR_3A
// 			int 			   ispID,
// #endif
#ifdef ES_3A
			V_ISP_AWB_EXP_FUNC_S *pAwbiLib
#else
			AwbiLib_t		   *pAwbiLib
#endif
);

// #ifndef VENDOR_3A
// RESULT CamEngineAwbInitEs
// (
//     CamEngineHandle_t  hCamEngine
// );

// RESULT CamEngineAwbExitEs
// (
//     CamEngineHandle_t  hCamEngine
// );
// #endif


/*****************************************************************************/
/**
 *          CamEngineAwbUnRegister
 *
 * @brief   unRegiter CamEngine AWB module's handle
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbUnRegister
(
    CamEngineHandle_t  hCamEngine
);

/*****************************************************************************/
/**
 *          CamEngineAwbSetRoi
 *
 * @brief   Set CamEngine AWB calculation's ROI area parameters 
 *
 * @param   hCamEngine      Camegine AWB mdoule's handle parameters
 * @param   pCamAwbRoi      Pointer to Awb regions of interest
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbSetRoi
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAwbRoi_t  *pAwbRoi
);

/*****************************************************************************/
/**
 *          CamEngineAwbGetRoi
 *
 * @brief   Get CamEngine AWB calculation's ROI parameters
 *
 * @param   hCamEngine       Camegine AWB mdoule's handle parameters
 * @param   pCamAwbRoi       Pointer to Awb regions of interest
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbGetRoi
(
    CamEngineHandle_t   hCamEngine,
    CamEngineAwbRoi_t   *pAwbRoi
);

/*****************************************************************************/
/**
 *          CamEngineAwbSetMode
 *
 * @brief   Set CamEngine AWB calculation's mode parameters 
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 * @param   pAwbMode    AWB Working mode type
 *
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbSetMode
(
    CamEngineHandle_t       hCamEngine,
    CamEngineAwbModeType_t  *pAwbMode
);

/*****************************************************************************/
/**
 *          CamEngineAwbGetMode
 *
 * @brief   Get CamEngine AWB calculation's mode parameters 
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 * @param   pAwbMode    AWB Working mode type
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbGetMode
(
    CamEngineHandle_t       hCamEngine,
    CamEngineAwbModeType_t  *pAwbMode
);

/*****************************************************************************/
/**
 *          CamEngineAwbSetConfig
 *
 * @brief   Set CamEngine AWB calculation's Config parameters
 *
 * @param   hCamEngine       Camegine AWB mdoule's handle parameters
 * @param   pAwbConfig    The pointer to AWB config parameters

 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbSetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineAwbConfig_t  *pAwbConfig
);

/*****************************************************************************/
/**
 *          CamEngineAwbGetConfig
 *
 * @brief   Get CamEngine AWB calculation's Config parameters
 *
 * @param   hCamEngine       Camegine AWB mdoule's handle parameters
 * @param   pAwbConfig    The pointer to AWB config parameters
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineAwbConfig_t  *pAwbConfig
);

/*****************************************************************************/
/**
 *          CamEngineAwbGetColorTempWeight
 *
 * @brief   Get CamEngine AWB temperature weight of every lighting
 *
 * @param   hCamEngine       Camegine AWB mdoule's handle parameters
 * @param   pAwbTemWeight    The pointer to AWB temperature weight of every lighting
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbGetColorTempWeight
(
    CamEngineHandle_t        hCamEngine,
    CamEngineAwbTemWeight_t  *pAwbTemWeight
);

/*****************************************************************************/
/**
 *          CamEngineAwbStart
 *
 * @brief   Start CamEngine AWB calculation
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 *
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbStart
(
    CamEngineHandle_t  hCamEngine
);

/*****************************************************************************/
/**
 *          CamEngineAwbStop
 *
 * @brief   Stop CamEngine AWB calculation
 *
 * @param   hCamEngine     Camegine AWB mdoule's handle parameters
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbStop
(
    CamEngineHandle_t  hCamEngine
);

/*****************************************************************************/
/**
 *          CamEngineAwbGetStatus
 *
 * @brief   Get CamEngine AWB calculation's status parameters
 *
 * @param   hCamEngine           Camegine AWB mdoule's handle parameters
 * @param   pAwbState   The camengine Awb status
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 success
 * @retval  ret = others fail
 *
 *****************************************************************************/
RESULT CamEngineAwbGetStatus
(
    CamEngineHandle_t    hCamEngine,
    CamEngineAwbState_t  *pAwbState
);

#ifdef __cplusplus
}
#endif

#endif 

