/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file cam_engine_common.h
 *
 * @brief
 *   Common definitions of the CamEngine.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_common CamEngine Common Definitions
 * @{
 *
 */
#ifndef CAM_ENGINE_COMMON_H
#define CAM_ENGINE_COMMON_H
#include <common/ispcore_defs.h>
#include <cameric_drv/cameric_mi_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   Handle to a CamEngine instance.
 *
 *****************************************************************************/
typedef struct CamEngineContext_s *CamEngineHandle_t;


/*****************************************************************************/
/**
 * @brief   Commands for the CamEngine
 *
 *****************************************************************************/
typedef enum CamEngineCmdId_e
{
    CAM_ENGINE_CMD_INVALID                  = 0,    /**< (For initialization only) Invalid command */
    CAM_ENGINE_CMD_START                    = 1,    /**< Starts a CamEngine instance */
    CAM_ENGINE_CMD_STOP                     = 2,    /**< Stops a CamEngine instance */
    CAM_ENGINE_CMD_SHUTDOWN                 = 3,    /**< Shutdowns a stopped CamEngine instance */

    CAM_ENGINE_CMD_START_STREAMING          = 4,    /**< Starts streaming */
    CAM_ENGINE_CMD_STOP_STREAMING           = 5,    /**< Stops streaming */

    CAM_ENGINE_CMD_ACQUIRE_LOCK             = 6,    /**< Locks the auto algorithms */
    CAM_ENGINE_CMD_RELEASE_LOCK             = 7,    /**< Releases locks of the auto algorithms */

    CAM_ENGINE_CMD_INTERNAL_BASE            = 1000,                                 /**< Base for internal commands */
    CAM_ENGINE_CMD_AAA_LOCKED               = (CAM_ENGINE_CMD_INTERNAL_BASE + 0),   /**< Selected auto-algorithms are locked now */
    CAM_ENGINE_CMD_HW_STREAMING_FINISHED    = (CAM_ENGINE_CMD_INTERNAL_BASE + 20),  /**< Send by CamerIc hardware, if ISP is disabled (streaming finished) */
    CAM_ENGINE_CMD_HW_DMA_FINISHED          = (CAM_ENGINE_CMD_INTERNAL_BASE + 21),  /**< Send by CamerIc hardware, if DMA transfer is completed */
    CAM_ENGINE_CMD_HW_JPE_DATA_ENCODED      = (CAM_ENGINE_CMD_INTERNAL_BASE + 22),  /**< Send by CamerIc hardware, if JPE data is encoded */
    CAM_ENGINE_CMD_HW_DMA_TRANSFER          = (CAM_ENGINE_CMD_INTERNAL_BASE + 23),  /**< DMA transfers the loaded picture again while the CamEngine is streaming */
    CAM_ENGINE_CMD_DROP_FRAME               = (CAM_ENGINE_CMD_INTERNAL_BASE + 24),  /**< Isp output frame dirty ,need to drop this frame */

    CAM_ENGINE_CMD_MAX
} CamEngineCmdId_t;


/*****************************************************************************/
/**
 * @brief States of the CamEngine.
 *
 *****************************************************************************/
typedef enum CamEngineState_e
{
    eCamEngineStateInvalid    = 0x0000,   /**< Invalid state because no CamEngine instance exists. */
    eCamEngineStateInitialize = 0x0001,   /**< FSM is initialized. */
    eCamEngineStateRunning    = 0x0002,   /**< FSM is running. */
    eCamEngineStateStreaming  = 0x0003,   /**< FSM is streaming. */
    CAM_ENGINE_STATE_MAX
} CamEngineState_t;


/*****************************************************************************/
/**
 * @brief Processing paths of the CamEngine.
 *
 *****************************************************************************/
typedef enum CamEnginePathType_e
{
    CAM_ENGINE_PATH_INVALID = -1,
    CAM_ENGINE_PATH_MAIN    = 0,
    CAM_ENGINE_PATH_SELF    = 1,
    CAM_ENGINE_PATH_SELF2_BP   = 2,
    CAM_ENGINE_PATH_MAIN_RAW,                /**< Duplicate main path output. \n It is recommended to use when main-path both outputs YUV and RAW cases >*/
#ifdef ISP_MI_HDR
    CAM_ENGINE_PATH_HDR_RAW,
#endif
#ifdef ISP_MI_PP_WRITE
    CAM_ENGINE_PATH_POST_PROCESS   ,
#endif
#ifdef ISP_MI_HDR
    CAM_ENGINE_PATH_HDR,
#endif
    CAM_ENGINE_RDMA_PATH,
    CAM_ENGINE_PATH_META,
    CAM_ENGINE_PATH_MAX
} CamEnginePathType_t;

typedef enum CamEngineWbWorkMode_e {
    CAM_ENGINE_WB_WORK_MODE = 0,             /*r, gr, gb, b*/
    CAM_ENGINE_WB_STITCHING_WORK_MODE,       /*r, gr, gb, b*/
    CAM_ENGINE_WB_RGBIR_WORK_MODE,           /*r, g, b*/
    CAM_ENGINE_WB_WORK_MODE_MAX
} CamEngineWbWorkMode_t;


/*****************************************************************************/
/**
 * @brief Video input path configuration types of the CamEngine.
 *
 *****************************************************************************/
typedef enum CamEngineConfigType_e
{
    CAM_ENGINE_CONFIG_INVALID = 0,
    CAM_ENGINE_CONFIG_SENSOR  = 1,
    CAM_ENGINE_CONFIG_IMAGE   = 2,
    CAM_ENGINE_CONFIG_TPG     = 3,
    CAM_ENGINE_CONFIG_USER    = 4,
    CAM_ENGINE_CONFIG_MAX     = 5
} CamEngineConfigType_t;

/*****************************************************************************/
/**
 *  @brief The callback function that signals the completion of a command. \n It can require application interaction.
 * For the ID of the completed command, see @ref CamEngineCmdId_t.
 *
 *****************************************************************************/
typedef void (* CamEngineCompletionCb_t)
(
    CamEngineCmdId_t    cmdId,          /**< Command ID of the notifying event */
    RESULT              result,         /**< Result of the executed command */
    const void          *pUserCbCtx     /**< User data pointer that was passed on creation (see @ref CamEngineInstanceConfig_t) */
);

/*****************************************************************************/
/**
 *  @brief The callback function to indicates a buffer is full and requires application processing.
 *
 *  For information about output paths, see @ref CamEnginePathType_t.
 *
 *****************************************************************************/
typedef void (*CamEngineBufferCb_t)
(
    CamEnginePathType_t path,           /**< Output path of the media buffer */
    MediaBuffer_t       *pMediaBuffer,  /**< Full media buffer */
    void                *pBufferCbCtx   /**< User data pointer that was passed on registering the callback (see @ref CamEngineRegisterBufferCb) */
);

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the data output mode on a given path
 *
 * @note    This enumeration type is used to specify the organization of the image data
 *          for each pixel, whether in planar format or packed format
 *
 *****************************************************************************/
typedef enum CamEngineMiDataMode_e
{
    CAM_ENGINE_MI_DATAMODE_INVALID         = 0,    /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_MI_DATAMODE_DISABLED        = 1,    /**< disables the path */
    CAM_ENGINE_MI_DATAMODE_JPEG            = 2,    /**< data output format is JPEG (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_YUV444          = 3,    /**< data output format is YUV444 */
    CAM_ENGINE_MI_DATAMODE_YUV422          = 4,    /**< data output format is YUV422 */
    CAM_ENGINE_MI_DATAMODE_YUV420          = 5,    /**< data output format is YUV420 */
    CAM_ENGINE_MI_DATAMODE_YUV400          = 6,    /**< data output format is YUV400 */
    CAM_ENGINE_MI_DATAMODE_RGB888          = 7,    /**< data output format is RGB888 (only valid for selpath) */
    CAM_ENGINE_MI_DATAMODE_RGB666          = 8,    /**< data output format is RGB666 (only valid for selpath) */
    CAM_ENGINE_MI_DATAMODE_RGB565          = 9,    /**< data output format is RGB565 (only valid for selpath) */
    CAM_ENGINE_MI_DATAMODE_RAW8            = 10,   /**< data output format is RAW8 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW12           = 11,   /**< data output format is RAW12 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_DPCC            = 12,   /**< path dumps out the current measured defect pixel table */
    CAM_ENGINE_MI_DATAMODE_RAW10           = 13,   /**< data output format is RAW10 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW14           = 14,   /**< data output format is RAW14 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW16           = 15,   /**< data output format is RAW16 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW20           = 16,   /**< data output format is RAW20 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW20_COMPRESS  = 17,   /**< data output format is RAW20 compressed to RAW16 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW24           = 18,   /**< data output format is RAW24 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_RAW24_COMPRESS  = 19,   /**< data output format is RAW24 compressed to RAW16 (only valid for mainpath) */
    CAM_ENGINE_MI_DATAMODE_META            = 20,   /**< data output format is META (only valid for metapath) */
    CAM_ENGINE_MI_DATAMODE_MAX                     /**< upper border (only for an internal evaluation) */
} CamEngineMiDataMode_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the data storage layout
 *
 * @note    This enumeration type is used to specify the organization of the
 *          image data for each pixel, whether in planar format or packed
 *          format.
 *
 *****************************************************************************/
typedef enum CamEngineMiDataLayout_e
{
    CAM_ENGINE_MI_DATASTORAGE_INVALID      = 0,    /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_MI_DATASTORAGE_PLANAR       = 1,    /**< Y values for all pixels are put together, as well as U and V,
                                                     like: YYYYYY......, UUUUUUU......., VVVVVV...... */
    CAM_ENGINE_MI_DATASTORAGE_SEMIPLANAR   = 2,    /**< YUV values are packed together as: YYYY......, UVUVUVUV...... */
    CAM_ENGINE_MI_DATASTORAGE_INTERLEAVED  = 3,    /**< YUV values are packed together as: UYVY UYVY...... */
    CAM_ENGINE_MI_DATASTORAGE_MAX                  /**< upper border (only for an internal evaluation) */
} CamEngineMiDataLayout_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the data storage align mode.
 *
 * @note    This enumeration type is used to specify the align mode of the
 *          image data for each pixel
 *
 *****************************************************************************/
typedef enum CamEngineMiDataAlignMode_e {
	CAM_ENGINE_MI_PIXEL_ALIGN_INVALID = -1,	/**< lower border (only for an internal evaluation) */
	CAM_ENGINE_MI_PIXEL_UN_ALIGN = 0,		/**< The value of pixel is not align. */
	CAM_ENGINE_MI_PIXEL_ALIGN_128BIT = 1,	/**< The value of pixel raw is align with 128 bit.*/
	CAM_ENGINE_MI_PIXEL_ALIGN_DOUBLE_WORD = 1, /**< The value of yuv pixel is double word align.*/
	CAM_ENGINE_MI_PIXEL_ALIGN_WORD = 2,  /**< The value of yuv pixel is  word align.*/
	CAM_ENGINE_MI_PIXEL_ALIGN_16BIT = 2, /**< The value of raw pixel is align with 16bit.*/
	CAM_ENGINE_MI_PIXEL_ALIGN_MAX	  /**< upper border (only for an internal evaluation) */
} CamEngineMiDataAlignMode_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the data storage yuv bit width.
 *
 * @note    This enumeration type is used to specify the yuv bit width of the
 *          image data for each pixel
 *
 *****************************************************************************/
typedef enum CamEngineMiDataYuvBitWidth_e {
	CAM_ENGINE_MI_PIXEL_YUV_BIT_INVALID = -1, /**< lower border (only for an internal evaluation) */
	CAM_ENGINE_MI_PIXEL_YUV_8_BIT = 0,		/**< The value of yuv pixel is 8 bit. */
	CAM_ENGINE_MI_PIXEL_YUV_10_BIT = 1,	/**< The value of yuv pixel is 10 bit.*/
	CAM_ENGINE_MI_PIXEL_YUV_12_BIT = 2,	/**< The value of yuv pixel is 12 bit.*/
	CAM_ENGINE_MI_PIXEL_YUV_BIT_MAX		/**< upper border (only for an internal evaluation) */
} CamEngineMiDataYuvBitWidth_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the swap control.
 *
 * @note    This enumeration type is used to specify the swap control.
 *
 *****************************************************************************/
typedef enum CamEngineMiSwap_e {
    CAM_ENGINE_MI_SWAP_INVALID = -1,       /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_MI_NO_SWAP = 0,             /**< The value of no swap. */
    CAM_ENGINE_MI_SWAP_BYTES = 1,          /**< The value of swap bytes. */
    CAM_ENGINE_MI_SWAP_WORDS = 2,          /**< The value of swap words. */
    CAM_ENGINE_MI_SWAP_DOUBLE_WORDS = 4,   /**< The value of swap double words. */
    CAM_ENGINE_MI_SWAP_FOUR_WORDS = 8,     /**< The value of swap four words. */
    CAM_ENGINE_MI_SWAP_MAX                 /**< upper border (only for an internal evaluation) */
} CamEngineMiDataSwap_t;

/*****************************************************************************/
/**
 * @brief   Union type to specify the raw and yuv swap control.
 *
 * @note    This union type is used to specify the raw and yuv swap control.
 *
 *****************************************************************************/
typedef union CamEngineMiSwap_u {
    struct {
        CamEngineMiDataSwap_t y;
        CamEngineMiDataSwap_t u;
        CamEngineMiDataSwap_t v;
    } yuvSwap;
    CamEngineMiDataSwap_t rawSwap;
} CamEngineMiSwap_t;

/*****************************************************************************/
/**
 * @brief   Enumeration type to specify the order of YUV or RGB channel
 *          Right now only surpport RGB888 format
 *
 *****************************************************************************/
typedef enum CamEngineMiYuvOrder_e
{
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_INVALID = -1,
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_YUV = 0,  /** 0: YUV or RGB */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_YVU = 1,  /** 1: YVU or RBG */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_UYV = 2,  /** 2: UYV or GRB */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_VYU = 3,  /** 3: VYU or BRG */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_UVY = 4,  /** 4: UVY or GBR */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_VUY = 5,  /** 5: VUY or BGR */
    CAM_ENGINE_MI_PIC_CHANNEL_ORDER_MAX = 6,
} CamEngineMiYuvOrder_t;

/*****************************************************************************/
/**
 * @brief   Configuration structure of the output path.
 *
 *****************************************************************************/
typedef struct CamEnginePathConfig_s
{
    uint16_t                   width;
    uint16_t                   height;
    CamEngineMiDataMode_t      mode;
    CamEngineMiDataLayout_t    layout;
    CamEngineMiDataAlignMode_t alignMode;
    CamEngineMiDataYuvBitWidth_t yuvBit;
#if defined(ISP_MIV3) || defined(ISP_MIV2)
    CamEngineMiSwap_t          swap;
    CamEngineMiYuvOrder_t      yuvOrder; // the order of YUV or RGB channel
    uint8_t                    alpha;  /**< alpha value of ARGB format , range 0~255 for ARGB 8-8-8-8, 0~3 for ARGB 10-10-10-2*/
#endif
} CamEnginePathConfig_t;


/*****************************************************************************/
/**
 * @brief   Structure to define a window.
 *
 *****************************************************************************/
typedef struct CamEngineWindow_s
{
    uint16_t hOffset;
    uint16_t vOffset;
    uint16_t width;
    uint16_t height;
} CamEngineWindow_t;


/*****************************************************************************/
/**
 * @brief   Structure to define a vector.
 *
 *****************************************************************************/
typedef struct CamEngineVector_s
{
    int16_t   x;
    int16_t   y;
} CamEngineVector_t;

/*****************************************************************************/
/**
 * @brief   Structure for the black level of the four color components.
 *
 *****************************************************************************/
typedef struct CamEngineBlackLevel_s
{
    uint16_t Red;
    uint16_t GreenR;
    uint16_t GreenB;
    uint16_t Blue;
} CamEngineBlackLevel_t;

enum {
    CAM_ENGINE_MODULE_INIT = 0,
    CAM_ENGINE_MODULE_EE,
    CAM_ENGINE_MODULE_AHDR,
    CAM_ENGINE_MODULE_TDNR,
    CAM_ENGINE_MODULE_2DNRV5,
    CAM_ENGINE_MODULE_WDR4,
    CAM_ENGINE_MODULE_CA,
    CAM_ENGINE_MODULE_DCI,
    CAM_ENGINE_MODULE_CPROC,
    CAM_ENGINE_MODULE_GAMMA64,
    CAM_ENGINE_MODULE_DPF,
    CAM_ENGINE_MODULE_DGAIN,
    CAM_ENGINE_MODULE_DMSC2,
};

enum {
    CAM_ENGINE_MODE_OFF= 0,
    CAM_ENGINE_MODE_MANUAL,
    CAM_ENGINE_MODE_AUTO,
};


typedef enum CamEngineSensorMode_e {
    CAM_ENGINE_SENSOR_MODE_LINEAR = 0,   /**<  linear mode */
    CAM_ENGINE_SENSOR_MODE_HDR,          /**<  hdr mode */
    CAM_ENGINE_SENSOR_MODE_NATIVE_HDR,   /**<  native hdr mode */
    CAM_ENGINE_SENSOR_MODE_MAX
} CamEngineSensorMode_t;

#define CAM_ENGINE_VSM_H_SIZE_MIN   64
#define CAM_ENGINE_VSM_V_SIZE_MIN   64
#define CAM_ENGINE_VSM_H_SIZE_MAX   1920
#define CAM_ENGINE_VSM_V_SIZE_MAX   1088
#define CAM_ENGINE_ISO_STRENGTH_NUM 20

#ifdef __cplusplus
}
#endif


/* @} cam_engine_common */


#endif /* CAM_ENGINE_COMMON_H */

