/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    CPROC
 * 
 * @file    cam_engine_cproc_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine Color Processing.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_cproc_api CamEngine CPROC Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_CPROC_API_H
#define CAM_ENGINE_CPROC_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief   Color processing (CPROC) mode configurations.
 *
 *****************************************************************************/
typedef enum CamEngineCprocMode_e
{
    CAM_ENGINE_CPROC_MODE_INVALID = 0,    /**< (For initialization only) Invalid mode  */
    CAM_ENGINE_CPROC_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_CPROC_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_CPROC_MODE_MAX
} CamEngineCprocMode_t;

/******************************************************************************/
/**
 * @brief   CPROC manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineCprocConfig_s
{
    float32_t bright;
    float32_t contrast;
    float32_t hue;
    float32_t saturation;
#ifdef ES_API
    bool_t bCtModeEn;//0:contrast affect brightness
#endif	
}CamEngineCprocConfig_t;

/******************************************************************************/
/**
 * @brief   CPROC auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineCprocAutoConfig_s
{
    uint8_t autoLevel;
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];
    float32_t contrast[CAM_ENGINE_ISO_STRENGTH_NUM];
    float32_t bright[CAM_ENGINE_ISO_STRENGTH_NUM];
    float32_t saturation[CAM_ENGINE_ISO_STRENGTH_NUM];
    float32_t hue[CAM_ENGINE_ISO_STRENGTH_NUM];
}CamEngineCprocAutoConfig_t;

/*****************************************************************************/
/**
 * @brief   Sets the CPROC mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input CPROC running mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocSetMode
(
    CamEngineHandle_t hCamEngine,
	const CamEngineCprocMode_t  mode           /**< CPROC running mode */
);

/*****************************************************************************/
/**
 * @brief   Gets CPROC mode
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained CPROC running mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocGetMode
(
    CamEngineHandle_t hCamEngine,
	CamEngineCprocMode_t *mode           /**< CPROC running mode */
);


/*****************************************************************************/
/**
 * @brief   Enables the CPROC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the CPROC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the CPROC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             CPROC configurations.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCprocSetConfig
(
    CamEngineHandle_t        hCamEngine,
    const CamEngineCprocConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the CPROC status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of CPROC
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCprocGetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineCprocConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets the CPROC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            CPROC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineCprocAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the auto configurations of CPROC
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            CPROC auto configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineCprocAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the status of CPROC
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   sensorMode          The sensor mode
 * @param   pMode               The running mode
 * @param   pCfg                CPROC configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCprocGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    const CamEngineSensorMode_t* sensorMode,
    CamEngineCprocMode_t*  pMode,
    CamEngineCprocConfig_t* pCfg
);


#ifdef __cplusplus
}
#endif

/* @} cam_engine_cproc_api */

#endif /* CAM_ENGINE_CPROC_API_H */

