/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    DG
 *
 * @file    cam_engine_dg_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine DG.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_dg_api CamEngine DG Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_DG_API_H
#define CAM_ENGINE_DG_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_DIGITAL_GAIN_MAX 255.998f
#define CAM_ENGINE_DIGITAL_GAIN_MIN 0.0f

/******************************************************************************/
/**
 * @brief    Digital gain (DG) manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDgConfig_s
{
	float32_t digitalGainB;    /**< Blue DG */
    float32_t digitalGainGb;   /**< Green blue DG */
    float32_t digitalGainGr;   /**< Green red DG */
    float32_t digitalGainR;    /**< Red DG */
} CamEngineDgConfig_t;

/*****************************************************************************/
/**
 * @brief   Enables the digital gain (DG) module.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDgEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the DG module.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDgDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Sets the gain value of the DG module.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Digital gain value
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDgSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineDgConfig_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DG status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Digital gain value
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDgGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineDgConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the DG status
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pConfig             Digital gain value

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDgGetStatus
(
    CamEngineHandle_t    hCamEngine,
    bool_t               *pRunning,
    CamEngineDgConfig_t   *pConfig
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_dg_api */


#endif /* CAM_ENGINE_DG_API_H */

