/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    DPCC
 *
 * @file    cam_engine_dpcc_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine DPCC Definitions
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_dpcc CamEngine DPCC Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_DPCC_API
#define CAM_ENGINE_DPCC_API

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/ispcore_defs.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_DPCC_DEFECT_PIXEL_NUM          2048  // max number of defected pixels
#define CAM_ENGINE_DPCC_CHANNEL_NUM               2
#define CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM 3
#define CAM_ENGINE_DPCC_METHODS_NUM_MAX           8191
#define CAM_ENGINE_DPCC_MODE_NUM                  6

/*****************************************************************************/
/**
 * @brief   DPCC mode.
 *
 *****************************************************************************/
typedef enum CamEngineDpccMode_e
{
    CAM_ENGINE_DPCC_MODE_INVALID = 0,    /**< (For initialization only) Invalid mode */
    CAM_ENGINE_DPCC_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_DPCC_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_DPCC_MODE_MAX
} CamEngineDpccMode_t;

/******************************************************************************/
/**
 * @brief   DPCC manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDpccConfig_s {
    bool_t bptEnable;       /**< Whether to enable bad pixel table. \n 0: disable. \n 1: enable. */
    uint16_t bptNum;        /**< The number of bad pixel tables. */
    uint8_t bptOutMode;     /**< The output mode of bad pixel table. */
    uint16_t bptPosX[CAM_ENGINE_DPCC_DEFECT_PIXEL_NUM]; /**< Bad pixel table X position */
    uint16_t bptPosY[CAM_ENGINE_DPCC_DEFECT_PIXEL_NUM]; /**< Bad pixel table Y position */
    uint8_t lineMadFac[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]: The three thresholds of the Line_mad method used by R and B channels. \n */
                                                                                        /**< [1][0~2]: The three thresholds of the Line_mad method used by the G channel. */
    uint8_t lineThresh[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]: The three thresholds of the Line_thresh method used by R and B channels. \n */
                                                                                        /**< [1][0~2]: The three thresholds of the Line_thresh method used by the G channel. */
    uint16_t methodsSet[CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM]; /**< [0]: Whether the first set of thresholds of the five bad point determination methods is enabled. \n */
                                                                    /**< [1]: Whether the second set of thresholds of the five bad point determination methods is enabled. \n */
                                                                    /**< [2]: Whether the third set of thresholds of the five bad point determination methods is enabled. */
    uint8_t outMode; /**<Interpolation mode for correction unit. range:[0, 15] */
    uint8_t pgFac[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]: The three thresholds of the peak Gradient check method used by R and B channels. \n */
                                                                                            /**< [1][0~2]: The three thresholds of the peak Gradient check method used by the G channel. */
    uint8_t rgFac[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]: The three thresholds of the Rank neighbor difference method used by R and B channels. \n */
                                                                                            /**< [1][0~2]: The three thresholds of the Rank neighbor difference method used by the G channel. */
    uint8_t rndOffs[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM]; /**< [0][0~2]: The three offsets of the Rand Neighbor Difference method used by R and B channels. \n */
                                                                                             /**< [1][0~2]: The three offsets of the Rand Neighbor Difference method used by the G channel. */
    uint8_t rndThresh[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]: The three thresholds of the Rand Neighbor Difference method used by R and B channels. \n */
                                                                                                /**< [1][0~2]: The three thresholds of the Rand Neighbor Difference method used by the G channel. */
    uint8_t roLimits[CAM_ENGINE_DPCC_CHANNEL_NUM][CAM_ENGINE_DPCC_METHOD_PARAMETER_TYPE_NUM];   /**< [0][0~2]: The three thresholds of the Rand Order method used by the R and B channels. \n */
                                                                                      /**< [1][0~2]: The three thresholds of the Rand Order method used by the G channel. */
    uint8_t setUse; /**< DPCC methods set usage for detection */
} CamEngineDpccConfig_t;

/******************************************************************************/
/**
 * @brief   DPCC auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDpccAutoConfig_s {
    uint8_t autoModeSelect;
} CamEngineDpccAutoConfig_t;

/*****************************************************************************/
/**
 * @brief   Enables DPCC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpccEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DPCC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpccDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DPCC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DPCC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpccSetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineDpccConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DPCC status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DPCC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpccGetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineDpccConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Sets the mode of DPCC.
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The set DPCC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccSetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineDpccMode_t mode           /**< The DPCC running mode */
);

/*****************************************************************************/
/**
 * @brief   Get the mode of DPCC.
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pMode               The obtained DPCC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccGetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineDpccMode_t *pMode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   Sets the DPCC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The input DPCC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDpccAutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the DPCC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The obtained DPCC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDpccAutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the DPCC status
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pCfg                DPCC status

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineDpccMode_t *pMode,
    CamEngineDpccConfig_t *pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_dpcc_api */


#endif /* CAM_ENGINE_DPCC_API */

