/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    EE_V2
 *
 * @file    cam_engine_ee_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine EE V2 Definitions.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_ee_api CamEngine EE V2 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_EE_API_H
#define CAM_ENGINE_EE_API_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <common/return_codes.h>
#include "cam_engine_common.h"
#include <cameric_drv/cameric_isp_eev2_drv_api.h>

#define CAM_ENGINE_CURVE_SIZE 65  //bin number of ca/dci curve

/*****************************************************************************/
/**
 * @brief   EE mode configurations.
 *
 *****************************************************************************/
typedef enum CamEngineEeMode_e
{
    CAM_ENGINE_EE_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_EE_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_EE_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_EE_MODE_MAX
} CamEngineEeMode_t;

/*****************************************************************************/
/**
 * @brief   EE CA mode configurations.
 *
 *****************************************************************************/
typedef enum CamEngineEeCaMode_e
{
    CAM_ENGINE_EE_CA_MODE0  = 0,    /**< Y mode> */
    CAM_ENGINE_EE_CA_MODE1  = 1,    /**< Saturation mode> */
    CAM_ENGINE_EE_CA_MODE2  = 2,    /**< Y mode and saturation mode> */
} CamEngineEeCaMode_t;

/*****************************************************************************/
/**
 * @brief   EE dci mode configurations.
 *
 *****************************************************************************/
typedef enum CamEngineEeDciMode_s
{
    CAM_ENGINE_EE_DCI_MODE_DCI_CURVE = 0,
    CAM_ENGINE_EE_DCI_MODE_CURVE_2SEC = 1,
    CAM_ENGINE_EE_DCI_MODE_CURVE_3SEC = 2,
    CAM_ENGINE_EE_DCI_MODE_CURVE_MAX
} CamEngineEeDciMode_t;

/******************************************************************************/
/**
 * @brief   EE manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineEeConfig_s
{
    uint32_t edgeGain;      /**< Gain for detected edge */
    uint8_t srcStrength;    /**< Y channel input image weight */
    uint8_t strength;       /**< Weight to merge the EE result back to the input image */
    uint16_t uvGain;        /**< EE UV gain */
    uint32_t yDownGain;     /**< Selected at flatten area */
    uint32_t yUpGain;       /**< Selected at edge/texture area */
} CamEngineEeConfig_t;

/******************************************************************************/
/**
 * @brief   EE CA manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineEeCaConfig_s
{
    uint16_t            lutChromaTbl[CAM_ENGINE_CURVE_SIZE];    /**< EE CA curve */
    CamEngineEeCaMode_t caMode; /**< EE CA mode */
} CamEngineEeCaConfig_t;

/******************************************************************************/
/**
 * @brief   EE DCI manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineEeDciConfig_s
{
    uint16_t lutLumaTbl[CAM_ENGINE_CURVE_SIZE]; /**< EE DCI curve */
} CamEngineEeDciConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine EE DCI curve configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineEeDciCurveConfig_s
{
    float32_t degamma;
    CamEngineEeDciMode_t dciMode;
    float32_t lowlumaIndex;
    float32_t middlelumaIndex;
    float32_t highlumaIndex;
    uint16_t inflctStartY;
    uint16_t inflct1X;
    uint16_t inflct1Y;
    uint16_t inflct2X;
    uint16_t inflct2Y;
    uint16_t inflctStopY;
} CamEngineEeDciCurveConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine EE DCI manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineEeDciStatusConfig_s
{
    CamEngineEeDciCurveConfig_t dciCurveConfig;
    CamEngineEeDciConfig_t dciConfig; /**< EE DCI curve */
} CamEngineEeDciStatusConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine EE curve dx manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineCurveDx_s
{
    uint16_t lutXTbl[ CAM_ENGINE_CURVE_SIZE ];
    uint16_t lutShiftTbl[ CAM_ENGINE_CURVE_SIZE ];
} CamEngineCurveDx_t;

/******************************************************************************/
/**
 * @brief   EE status configuration structures.
 *
 *****************************************************************************/
typedef struct CamEngineEeStatus_s
{
    CamEngineEeConfig_t          eeConfig;
    bool_t                       eeCaEn;
    CamEngineEeCaConfig_t        caConfig;
    bool_t                       eeDciEn;
    CamEngineEeDciStatusConfig_t dciStatusConfig;
    bool_t                       eeCurveEnable;
} CamEngineEeStatusConfig_t;

/******************************************************************************/
/**
 * @brief   EE auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineEeAutoConfig_s {
    uint8_t autoLevel;                                   /**< EE auto configuration level */
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< EE gain */
    uint8_t eeStrength[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Weight to merge EE result back to input image */
    uint8_t eeSrcStrength[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< Y channel input image weight */
    uint32_t yUpGain[CAM_ENGINE_ISO_STRENGTH_NUM];       /**< Selected at edge/texture area */
    uint32_t eeYDownGain[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< Selected at flatten area */
    uint16_t eeUvGain[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< EE UV gain */
    uint32_t eeEdgeGain[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Gain for detected edge */
    bool_t caEnable[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< EE CA enable value */
    uint16_t lutChromaTbl[CAM_ENGINE_ISO_STRENGTH_NUM][CAM_ENGINE_CURVE_SIZE]; /**< EE color adjust curve  */
    bool_t dciEnable[CAM_ENGINE_ISO_STRENGTH_NUM]; /**< EE DCI enable value */
    uint16_t dciCurve[CAM_ENGINE_ISO_STRENGTH_NUM][CAM_ENGINE_CURVE_SIZE]; /**< EE dynamic contrast improve curve */
} CamEngineEeAutoConfig_t;

/*****************************************************************************/
/**
 * @brief   Set the EE mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input EE mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineEeSetMode
(
    CamEngineHandle_t hCamEngine,
    const CamEngineEeMode_t  mode           /**< The EE running mode */
);

/*****************************************************************************/
/**
 * @brief   Gets the EE mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained EE mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineEeGetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineEeMode_t*  mode           /**< The EE running mode */
);

/*****************************************************************************/
/**
 * @brief   Enables EE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the EE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function config the EE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Configurations of EE
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeSetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineEeConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the EE status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Configurations of EE
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeGetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineEeConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables EE curve.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCurveEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the EE curve.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCurveDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the curve dx.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pCurveDx            The configurations of the curve dx
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeSetCurveDx
(
    CamEngineHandle_t       hCamEngine,
    CamEngineCurveDx_t     *pCurveDx
);


/*****************************************************************************/
/**
 * @brief   Enables EE CA.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCaEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the EE CA.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCaDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the EE CA.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the EE CA
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCaSetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineEeCaConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the EE CA status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the EE CA
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeCaGetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineEeCaConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables the EE DCI.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeDciEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the EE DCI.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeDciDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the EE DCI
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the EE DCI
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeDciSetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineEeDciConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the EE DCI status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the EE DCI
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineEeDciGetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineEeDciConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function config the ee dci curve
 *
 * @param   hCamEngine          handle to the CamEngine instance
 * @param   pConfig             configuration of ee dci
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_CONFIG    image effects isn't configured
 * @retval  RET_NOTAVAILABLE    module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineEeDciCurveSetConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineEeDciCurveConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function get the ee dci curve config
 *
 * @param   hCamEngine          handle to the CamEngine instance
 * @param   pConfig             configuration of ee dci
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_NOTAVAILABLE    module not available by driver or hardware
 *
 *****************************************************************************/
RESULT CamEngineEeDciCurveGetConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineEeDciCurveConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets EE auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            EE auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineEeSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineEeAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the auto configurations of EE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            EE auto configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineEeGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineEeAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the status of EE.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pStatus             The status configurations of EE

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineEeGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t *pRunning,
    CamEngineEeMode_t *pMode,
    CamEngineEeStatusConfig_t *pStatusConfig
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_ee_api */


#endif /* CAM_ENGINE_EE_API_H */

