/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file  cam_engine_expv2_api.h
 *
 * @brief Interface description of the CamEngine expv2.
 *****************************************************************************/

#ifndef CAM_ENGINE_EXPV2_API_H
#define CAM_ENGINE_EXPV2_API_H

#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_AE_CONFIG_NUM 8
#define CAMENGINE_AAA_EXP_GRID_SIZE 32
#define CAMENGINE_PIXEL_CHANNEL 4
#define CAMENGINE_EXPV2_WIN_MIN_HEIGHT (4 * 32 )
#define CAMENGINE_EXPV2_WIN_MIN_WIDTH (14 * 32 )
#define CAMENGINE_EXPV2_WIN_WIDTH_GAP  2
#define CAMENGINE_EXPV2_WIN_HEIGHT_GAP 1

typedef uint32_t CamEngineExpV2Static_t[CAMENGINE_AAA_EXP_GRID_SIZE * CAMENGINE_AAA_EXP_GRID_SIZE * CAMENGINE_PIXEL_CHANNEL];

typedef enum CamEngineExpV2Sel_e {
    CAMENGINE_EXPV2_INPUT_SEL_DEGAMMA   = 0,
    CAMENGINE_EXPV2_INPUT_SEL_AWBGAIN   = 1,
    CAMENGINE_EXPV2_INPUT_SEL_WDR3      = 2,
    CAMENGINE_EXPV2_INPUT_SEL_LSC       = 3,
    CAMENGINE_EXPV2_STATE_MAX
} CamEngineExpV2Sel_t;

typedef enum CamEngineExpv2StaticType_e
{
    CAMENGINE_EXPV2_8BIT_DATA      = 0,
    CAMENGINE_EXPV2_16BIT_DATA     = 1,
    CAMENGINE_EXPV2_24BIT_DATA     = 2,
} CamEngineExpv2StaticType_t;

typedef struct CamEngineExpv2Window_s
{
    uint16_t hOffset;
    uint16_t vOffset;
    uint16_t hSize;
    uint16_t vSize;
} CamEngineExpv2Window_t;

typedef struct CamEngineExpv2Weight_s
{
    uint8_t rWeight;
    uint8_t grWeight;
    uint8_t gbWeight;
    uint8_t bWeight;
} CamEngineExpv2Weight_t;

/*****************************************************************************/
/**
 * @brief   This function enable expv2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineExpv2Enable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the expv2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineExpv2Disable
(
    CamEngineHandle_t       hCamEngine
);


/*****************************************************************************/
/**
 * @brief   This function get expv2 enable status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   IsEnable            get the expv2 enable status
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineExpv2IsEnable
(
    CamEngineHandle_t       hCamEngine,
    bool_t                  *IsEnable
);


/*****************************************************************************/
/**
 * @brief   This function set expv2 measuring
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   window              set the expv2 window config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineExpv2SetMeasuringWindow
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpv2Window_t *windows
);

/*****************************************************************************/
/**
 * @brief   This function get expv2 measuring
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   window              get the expv2 window config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineExpv2GetMeasuringWindow
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpv2Window_t *windows
);

/*****************************************************************************/
/**
 * @brief   This function set expv2 measuring
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   window              set the expv2 window config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineExpv2SetWeight
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpv2Weight_t *pWeight
);

/*****************************************************************************/
/**
 * @brief   This function get expv2 measuring
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   window              get the expv2 window config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineExpv2GetWeight
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpv2Weight_t *pWeight,
    CamEngineExpV2Sel_t    *pInputSelect
);

/*****************************************************************************/
/**
 *          CamEngineExpV2GetStatistics()
 *
 * @brief return the aev2 statistics
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamEngineExpV2GetStatistics
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpV2Static_t *pAev2Static,
    CamEngineExpv2StaticType_t *pExpv2StatisticType
);

/*****************************************************************************/
/**
 *          CamEngineExpV2SetRatio()
 *
 * @brief set the aev2 size ratio
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamEngineExpV2SetSizeRatio
(
    CamEngineHandle_t hCamEngine,
    float32_t ratio
);

/*****************************************************************************/
/**
 *          CamEngineExpV2GetRatio()
 *
 * @brief get the aev2 size ratio
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamEngineExpV2GetSizeRatio
(
    CamEngineHandle_t hCamEngine,
    float32_t *ratio
);

/*****************************************************************************/
/**
 *          CamEngineExpV2SetInputSel()
 *
 * @brief set the aev2 input sel
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamEngineExpV2SetInputSel
(
    CamEngineHandle_t hCamEngine,
    CamEngineExpV2Sel_t inputSelect
);



#ifdef __cplusplus
}
#endif

#endif 

