/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    GC_V2
 *
 * @file    cam_engine_gamma_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine GC V2 Definitions.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_gamma_api CamEngine GC V2 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_GAMMA_API_H
#define CAM_ENGINE_GAMMA_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#include <cameric_drv/cameric_isp_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_GC_CURVE_SIZE 64 // bin number of gamma curve

/*****************************************************************************/
/**
 * @brief   GC mode configurations.
 *
 *****************************************************************************/
typedef enum CamEngineGcMode_e
{
    CAM_ENGINE_GC_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_GC_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_GC_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_GC_MODE_MAX
} CamEngineGcMode_t;

/******************************************************************************/
/**
 * @brief   GC manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineGcConfig_s
{
	uint16_t curve[CAM_ENGINE_GC_CURVE_SIZE]; /**< Gamma curve */
    bool_t standard;                          /**< True: standard_val. \n False: curve. */
    float32_t standardVal;                    /**< Generate uniform curve with gamma formula */
    bool_t userCurveX;
    uint32_t CurvePx[CAM_ENGINE_GC_CURVE_SIZE];
} CamEngineGcConfig_t;

/******************************************************************************/
/**
 * @brief   GC auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineGcAutoConfig_s
{
    uint8_t autoLevel;  /**< GC auto configuration level */
    float32_t gain[CAM_ENGINE_ISO_STRENGTH_NUM];  /**< GC gain */
    uint16_t curve[CAM_ENGINE_ISO_STRENGTH_NUM][CAM_ENGINE_GC_CURVE_SIZE]; /**< Gamma curve */
} CamEngineGcAutoConfig_t;

/*****************************************************************************/
/**
 * @brief   Sets the GC mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input GC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcSetMode
(
    CamEngineHandle_t hCamEngine,
	const CamEngineGcMode_t  mode           /**< The GC running mode. */
);

/*****************************************************************************/
/**
 * @brief   Gets the gc mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained GC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcGetMode
(
    CamEngineHandle_t  hCamEngine,
	CamEngineGcMode_t* mode           /**< The GC running mode. */
);


/*****************************************************************************/
/**
 * @brief   Enables the GC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the GC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcDisable
(
    CamEngineHandle_t hCamEngine
);



/*****************************************************************************/
/**
 * @brief   Configures the GC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             GC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGcSetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineGcConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the GC status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             GC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGcGetConfig
(
    CamEngineHandle_t   hCamEngine,
    CamEngineGcConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets the GC auto configurations.
 *
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            GC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineGcAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the GC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            GC auto configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineGcAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the GC status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pSensorMode         The sensor mode
 * @param   pMode               The running mode
 * @param   pCfg                GC configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineGcGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    const CamEngineSensorMode_t* pSensorMode,
    CamEngineGcMode_t* pMode,
    CamEngineGcConfig_t* pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_gamma_api */


#endif /* CAM_ENGINE_GAMMA_API_H */

