/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @cond HIST
 *
 * @defgroup cam_engine_hist_api Cam Engine HIST Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_HIST_API_H
#define CAM_ENGINE_HIST_API_H

#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


#ifdef ISP_HIST256
#define CAM_ENGINE_ISP_HIST_NUM_BINS        256  /**< number of bins */
#else
#define CAM_ENGINE_ISP_HIST_NUM_BINS        16  /**< number of bins */
#endif
#define CAM_ENGINE_ISP_HIST_GRID_ITEMS      25  /**< number of grid sub windows */

#define CAM_ENGINE_HIST256_WEIGHT_MAX       16 
#define CAM_ENGINE_HIST256_WEIGHT_MIN       0 

#define CAM_ENGINE_HIST256_COFF_MAX         1.992
#define CAM_ENGINE_HIST256_COFF_MIN         0 

/*****************************************************************************/
/**
 * @brief   Enumeration type to configure CamEngine ISP histogram measuring mode.
 *
 *****************************************************************************/
typedef enum CamEnginespHistMode_e
{
    CAM_ENGINE_ISP_HIST256_MODE_INVALID       = -1,    /**< (For internal use only) Lower border */
    CAM_ENGINE_ISP_HIST256_MODE_DISABLE       = 0,    /**< Disabled, no measurements */
    CAM_ENGINE_ISP_HIST256_MODE_RGB_COMBINED  = 1,    /**< RGB combined histogram */
    CAM_ENGINE_ISP_HIST256_MODE_R             = 2,    /**< R histogram */
    CAM_ENGINE_ISP_HIST256_MODE_G             = 3,    /**< G histogram */
    CAM_ENGINE_ISP_HIST256_MODE_B             = 4,    /**< B histogram */
    CAM_ENGINE_ISP_HIST256_MODE_Y             = 5,    /**< luminance histogram */
    CAM_ENGINE_ISP_HIST256_MODE_MAX,     				/**< upper border (only for an internal evaluation) */
} CamEngineHistMode_t;


/*****************************************************************************/
/**
 * @brief   Enumeration type to configure CamEngine ISP histogram measuring mode.
 *
 *****************************************************************************/
typedef struct CamEngineHistWindow_s
{
	uint16_t offsetX;
	uint16_t offsetY;
	uint16_t width;
	uint16_t height;
} CamEngineHistWindow_t;


/*****************************************************************************/
/**
 * @brief   the block hist's weight
 *
 *****************************************************************************/
typedef struct CamEngineHistWeight_s
{
    uint8_t weights[CAM_ENGINE_ISP_HIST_GRID_ITEMS];
} CamEngineHistWeights_t;


/*****************************************************************************/
/**
 * @brief   hist 256 output statistics
 *
 *****************************************************************************/
typedef struct CamEngineHistBins_s
{
    uint32_t Bins[CAM_ENGINE_ISP_HIST_NUM_BINS];
} CamEngineHistBins_t;


/******************************************************************************/
/**
 * @brief   CamEngineHist256Config_t
 *
 *****************************************************************************/
typedef struct CamEngineHistConfig_s
{
    CamEngineHistMode_t     hist256Mode;
    CamEngineHistWindow_t   hist256window;
    CamEngineHistWeights_t  hist256Weight;
} CamEngineHistConfig_t;


/*****************************************************************************/
/**
 * @brief   This function enable hist.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHistEnable
(
    CamEngineHandle_t    hCamEngine
);


/*****************************************************************************/
/**
 * @brief   This function disable hist.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHistDisable
(
    CamEngineHandle_t    hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Get CamEngine ISP histogram measurement module status.
 *
 * @param   hCamEngine      handle to the CamEngine instance
 * @param   pIsEnabled
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamEngineHistIsEnabled
(
    CamEngineHandle_t  hCamEngine,
    bool_t             *pIsEnabled
);


/*****************************************************************************/
/**
 * @brief   This functions configures the measurement mode of the CamEngine
 *          ISP histogram measurement module.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measurement mode (see @ref CamEnginespHistMode_e)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineHistSetConfig
(
    CamEngineHandle_t     hCamEngine,
    const CamEngineHistConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This functions get the measurement mode of the CamEngine
 *          ISP histogram measurement module.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measurement mode (see @ref CamEnginespHistMode_e)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineHistGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineHistConfig_t *pConfig
);


/*****************************************************************************/
/**
 * @brief   This functions get the hist bin
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pHistogram          hits bin
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamEngineHistGetStatistic
(
    CamEngineHandle_t    hCamEngine,
    CamEngineHistBins_t  *pHistogram
);
#ifdef __cplusplus
}
#endif

#endif 

