/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file    cam_engine_isp_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine ISP.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_isp_api CamEngine ISP Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_ISP_API_H
#define CAM_ENGINE_ISP_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#define CAM_ENGINE_TEM_WEIGHT_NUM 7

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   YUV range types.
 *
 *****************************************************************************/
typedef enum CamEngineYuvRangeType_e {
    CAM_ENGINE_YUV_RANGE_INVALID       = 0,
    CAM_ENGINE_YUV_LIMIT_RANGE,
    CAM_ENGINE_YUV_FULL_RANGE
}CamEngineYuvRangeType_t;

/*****************************************************************************/
/**
 * @brief   Color processing (CPROC) YUV range types.
 *
 *****************************************************************************/
typedef struct CamEngineCprocYuvRange_s {
    CamEngineYuvRangeType_t chromaOut;
    CamEngineYuvRangeType_t lumaIn;
    CamEngineYuvRangeType_t lumaOut;
}CamEngineYuvRange_t;

/*****************************************************************************/
/**
 * @brief   Enables or Disables bypass MCM.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   enable              The enabling status of MCM. \n 0: Disabled. \n 1: Enabled.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineBypassMcm
(
    CamEngineHandle_t hCamEngine,
    bool               enable
);

typedef enum CamEngineExposureFrameIndex_e {

    CAM_ENGINE_EXPOSURE_LINEAR_FRAME       = 0,
    CAM_ENGINE_EXPOSURE_LONG_FRAME         = 0,
    CAM_ENGINE_EXPOSURE_SHORT_FRAME,
    CAM_ENGINE_EXPOSURE_VERY_SHORT_FRAME,
    CAM_ENGINE_EXPOSURE_EXTRA_SHORT_FRAME,
    CAM_ENGINE_EXPOSURE_FIFTH_SHORT_FRAME,
    CAM_ENGINE_EXPOSURE_COMBINED_FRAME,
    CAM_ENGINE_EXPOSURE_FRAME_MAX
}CamEngineExposureFrameIndex_t;

typedef enum CamEngineRawColorCh_e {
    CAM_ENGINE_RAW_CHANNEL_RED     = 0,  /**< Red channel */
    CAM_ENGINE_RAW_CHANNEL_GREENR  = 1,  /**< GreenR channel */
    CAM_ENGINE_RAW_CHANNEL_GREENB  = 2,  /**< GreenB channel */
    CAM_ENGINE_RAW_CHANNEL_BLUE    = 3,  /**< Blue channel */
    CAM_ENGINE_RAW_CHANNEL_NUM
} CamEngineRawColorCh_t;

/******************************************************************************/
/**
 * @brief   Cam Engine ae mode select.
 *
 *****************************************************************************/
typedef enum CamEngineAeWorkMode_e
{
	CAM_ENGINE_AE_WORK_MODE_VSI_CTRL      = 0,   /**< Choose VSI interal ae control*/
	CAM_ENGINE_AE_WORK_MODE_METADATA_CTRL = 1,   /**< Choose metadata ae control*/
	CAM_ENGINE_AE_MODE_MAX
} CamEngineAeWorkMode_t;

/******************************************************************************/
/**
 * @brief   Cam Engine awb mode select.
 *
 *****************************************************************************/
typedef enum CamEngineAwbWorkMode_e
{
    CAM_ENGINE_AWB_WORK_MODE_VSI_CTRL      = 0,   /**< Choose VSI interal awb control*/
    CAM_ENGINE_AWB_WORK_MODE_METADATA_CTRL = 1,   /**< Choose metadata awb control*/
	CAM_ENGINE_AWB_WORK_MODE_MAX
} CamEngineAwbWorkMode_t;

/*****************************************************************************/
/**
 * @brief   Cam Engine integer range information structure.
 *
 *****************************************************************************/
typedef struct CamEngineIntegerRange_s {
    float32_t  max;         /**< Maximum value*/
    float32_t  min;         /**< Minimum value*/
    float32_t  step;        /**< Step value */
} CamEngineIntegerRange_t;

/******************************************************************************/
/**
 * @brief   Image matadata configurations.
 *
 *****************************************************************************/
typedef struct CamEnginePerFrameMetadataCfg_s {
	float32_t aecCurGain[CAM_ENGINE_EXPOSURE_FRAME_MAX];
    float32_t aecCurIntTime[CAM_ENGINE_EXPOSURE_FRAME_MAX];
    CamEngineIntegerRange_t intergerRange[CAM_ENGINE_EXPOSURE_FRAME_MAX]; 
    uint32_t frameNum;
    float32_t colorWeight[CAM_ENGINE_TEM_WEIGHT_NUM];
    float32_t cct;
    float32_t wbGain[CAM_ENGINE_EXPOSURE_FRAME_MAX][CAM_ENGINE_RAW_CHANNEL_NUM];
    CamEngineAeWorkMode_t modeAe;
    CamEngineAwbWorkMode_t modeAwb;
} CamEnginePerFrameMetadataCfg_t;

/*****************************************************************************/
/**
 * @brief   3a statistics source mode.
 *
 *****************************************************************************/
typedef enum CamEngine3aStatisticSel_e
{
    CAM_ENGINE_3A_STATISTIC_CCM_MODE_INVALID = -1,    /**< invalid mode (only for initialization) */
    CAM_ENGINE_3A_STATISTIC_AFTER_CCM = 0,    /**< get 3a statistic after coclor corection */
    CAM_ENGINE_3A_STATISTIC_BEFORE_CCM = 1,    /**< get 3a statistic before coclor corection */
    CAM_ENGINE_3A_STATISTIC_CCM_MODE_MAX,
} CamEngine3aStatisticSel_t;

#ifdef ES_API
RESULT CamEngineSetYuvRangeConfigEs
(
    CamEngineHandle_t     hCamEngine,
    const bool_t          bLimitedRangeEn
);
#endif

/*****************************************************************************/
/**
 * @brief   This function config the yuv range
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the CPROC YUV range
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineSetYuvRangeConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineYuvRange_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables Nr Reloc.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineIspNrRelocEnable
(
    CamEngineHandle_t     hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables Nr Reloc.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineIspNrRelocDisable
(
    CamEngineHandle_t     hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function select 3a statistics source before or after color correction
 *
 * @param   hCamEngine          handle to the CamEngine instance
 * @param   staSel           3a statistics source mode

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngine3aStatisticSel
(
    CamEngineHandle_t hCamEngine,
    CamEngine3aStatisticSel_t staSel
);

/*****************************************************************************/
/**
 * @brief   This function enables the dpf raw output.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfRawOutputEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the DPF RAW output.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfRawOutputDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function enables the DPCC raw output.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccRawOutputEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the DPCC RAW output.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpccRawOutputDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Returns the YUV range status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The configurations of the YUV range
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGetYuvRangeConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineYuvRange_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets the image matadata when loading an image.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Image matadata
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineSetPerFrameMetadata
(
    CamEngineHandle_t           hCamEngine,
    CamEnginePerFrameMetadataCfg_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Deregisters parse metadata callback
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   func                The callback function
 * @param   pUserContext        A pointer to the user context
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_FAILURE         Common error occured
 * @retval  RET_BUSY            Already a callback registered
 * @retval  RET_WRONG_HANDLE    Invalid handle
 * @retval  RET_INVALID_PARM    Invalid parameter
 * @retval  RET_WRONG_STATE     The driver is in a wrong state to register an event callback
 *
 *****************************************************************************/
RESULT CamEngineDeRegisterParseMetadataCb
(
    CamEngineHandle_t hCamEngine
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_isp_api */


#endif /* CAM_ENGINE_ISP_API_H */

