/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    LSC_V3
 *
 * @file    cam_engine_lsc_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine LSC.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_lsc_api CamEngine LSCV3 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_LSC_API_H
#define CAM_ENGINE_LSC_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


#define CAM_ENGINE_LSC_H_BLOCK_NUM 32  /**< Maximum horizontal block size of LSC map */
#define CAM_ENGINE_LSC_V_BLOCK_NUM 32  /**< Maximum vertical block size of LSC map */
#define CAM_ENGINE_LSC_H_POINT_NUM (CAM_ENGINE_LSC_H_BLOCK_NUM + 1)  /**< Maximum horizontal size of LSC map */
#define CAM_ENGINE_LSC_V_POINT_NUM (CAM_ENGINE_LSC_H_BLOCK_NUM + 1)  /**< Maximum vertical size of LSC map */
#define CAM_ENGINE_RAW_CHANNEL_NUM 4

/*****************************************************************************/
/**
 * @brief   LSC inter mode.
 *
 *****************************************************************************/
typedef enum CamEngineLscInterMode_e
{
    CAM_ENGINE_LSC_GAIN_INTER                    = 0,
    CAM_ENGINE_LSC_COLOR_TEMP_WEIGHT_INTER       = 1,
    CAM_ENGINE_LSC_GAIN_COLOR_TEMP_WEIGHT_INTER  = 2,
    CAM_ENGINE_LSC_INTER_MAX
} CamEngineLscInterMode_t;

/*****************************************************************************/
/**
 * @brief   LSC mode.
 *
 *****************************************************************************/
typedef enum CamEngineLscMode_e
{
    CAM_ENGINE_LSC_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_LSC_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_LSC_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_LSC_MODE_MAX
} CamEngineLscMode_t;

/*****************************************************************************/
/**
 * @brief   LSC configurations.
 *
 *****************************************************************************/
typedef struct CamEnginelscConfig_s
{
	uint16_t matrix[CAM_ENGINE_RAW_CHANNEL_NUM][CAM_ENGINE_LSC_V_POINT_NUM][CAM_ENGINE_LSC_H_POINT_NUM];    /**< 4 channels (R, Gr, Gb, B), 33*33 grids */
	uint16_t xSize[CAM_ENGINE_LSC_H_BLOCK_NUM]; /**< LSC V3 int array for x axis size */
	uint16_t ySize[CAM_ENGINE_LSC_V_BLOCK_NUM/2];    /**< LSC V3 int array for y axis size */
} CamEngineLscConfig_t;

/*****************************************************************************/
/**
 * @brief   LSC auto configurations.
 *
 *****************************************************************************/
typedef struct CamEnginelscAutoConfig_s
{
    uint8_t autoLevel;
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];
    float32_t damping;
    float32_t strength[CAM_ENGINE_ISO_STRENGTH_NUM];
    CamEngineLscInterMode_t interMode;
} CamEngineLscAutoConfig_t;


/*****************************************************************************/
/**
 * @brief   Enables LSC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineLscEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the LSC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineLscDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Sets LSC configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The input LSC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscSetConfig
(
    CamEngineHandle_t hCamEngine,
	CamEngineLscConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets LSC configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The obtained LSC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscGetConfig
(
    CamEngineHandle_t hCamEngine,
	CamEngineLscConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the LSC mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained LSC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscGetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineLscMode_t *mode           /**< The LSC running mode. */
);

/*****************************************************************************/
/**
 * @brief   Sets the LSC mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input LSC mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscSetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineLscMode_t mode           /**< The LSC running mode. */
);

/*****************************************************************************/
/**
 * @brief   Sets LSC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The input LSC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineLscAutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets LSC auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The obtained LSC auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineLscAutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets the status of LSC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The LSC running status
 * @param   pLscCfg             LSC configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineLscGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t *pRunning,
    CamEngineLscMode_t *pMode,
    CamEngineLscConfig_t *pLscCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_lsc_api */


#endif /* CAM_ENGINE_LSC_API_H */

