/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    WDR_V4_1
 *
 * @file    cam_engine_wdr4_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine WDR v4 Definitions.
 *
 *****************************************************************************/

#ifndef CAM_ENGINE_WDR4_API_H
#define CAM_ENGINE_WDR4_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/ispcore_defs.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_WDR_GAMMA_DOWN_CURVE_SIZE 20   //bin number of gamma_down curve
#define CAM_ENGINE_WDR_GAMMA_PRE_CURVE_SIZE 20   //bin number of gamma_pre curve
#define CAM_ENGINE_WDR_GAMMA_UP_CURVE_SIZE 20   //bin number of gamma_up curve
#define CAM_ENGINE_WDR_ENTROPY_LUT_SIZE 20   //bin number of entropy lut


/*****************************************************************************/
/**
 * @brief   Wdr4 mode.
 *
 *****************************************************************************/
typedef enum CamEngineWdr4Mode_e
{
    CAM_ENGINE_WDR4_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_WDR4_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_WDR4_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_WDR4_MODE_MAX
} CamEngineWdr4Mode_t;

/******************************************************************************/
/**
 * @brief   Cam Engine WDR V4 auto configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineWdr4AutoConfig_s
{
    uint8_t autoLevel;                                      /**< WDR V4 auto configuration level */
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];               /**< WDR V4 gain */
    uint8_t strength[CAM_ENGINE_ISO_STRENGTH_NUM];          /**< WDR V4 strength */
    uint8_t highStrength[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< WDR V4 high strength */
    uint8_t lowStrength[CAM_ENGINE_ISO_STRENGTH_NUM];       /**< WDR V4 low strength */
    uint8_t globalStrength[CAM_ENGINE_ISO_STRENGTH_NUM];    /**< WDR V4 global strength */
    int16_t contrast[CAM_ENGINE_ISO_STRENGTH_NUM];          /**< WDR V4 contrast */
    uint8_t flatStrength[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< WDR V4 flat area strength */
    uint8_t flatThr[CAM_ENGINE_ISO_STRENGTH_NUM];           /**< WDR V4 flat area threshold */
    uint16_t entropySlope[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< WDR V4 factor for all pixel based on image entropy */
    uint16_t entropyBase[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< WDR V4 factor for all pixel based on image base */
}CamEngineWdr4AutoConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine WDR V4 manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineWdr4Config_s
{
    int16_t contrast;       /**< WDR V4 contrast */
    uint16_t entropyBase;   /**< WDR V4 factor for all pixel based on image entropy */
    uint16_t entropySlope;  /**< WDR V4 factor for all pixel based on image entropy */
    uint8_t flatStrength;   /**< WDR V4 flat area strength */
    uint8_t flatThr;        /**< WDR V4 flat area threshold */
    uint8_t globalStrength; /**< WDR V4 global strength */
    uint8_t highStrength;   /**< WDR V4 high strength */
    uint8_t lowStrength;    /**< WDR V4 low strength */
    uint8_t strength;       /**< WDR V4 strength */
} CamEngineWdr4Config_t;


/******************************************************************************/
/**
 * @brief   Cam Device WDR V4 gamma manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineWdr4GammaConfig_s
{
    uint32_t entropy[CAM_ENGINE_WDR_ENTROPY_LUT_SIZE];          /**< WDR V4 factor for all pixel based on image entropy */
    uint32_t gammaDown[CAM_ENGINE_WDR_GAMMA_DOWN_CURVE_SIZE];   /**< WDR V4 knee points of gamma down */
    uint32_t gammaPre[CAM_ENGINE_WDR_GAMMA_PRE_CURVE_SIZE];     /**< WDR V4 knee points of gamma pre */
    uint32_t gammaUp[CAM_ENGINE_WDR_GAMMA_UP_CURVE_SIZE];        /**< WDR V4 knee points of gamma up */
} CamEngineWdr4GammaConfig_t;


/*****************************************************************************/
/**
 * @brief   This function set Wdr4 mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                input the Wdr4 run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4SetMode
(
    CamEngineHandle_t hCamEngine,
	const CamEngineWdr4Mode_t  mode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   This function get Wdr4 mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                get the Wdr4 run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4GetMode
(
    CamEngineHandle_t hCamEngine,
	CamEngineWdr4Mode_t*  mode           /**< run-mode */
);


/*****************************************************************************/
/**
 * @brief   This function set Wdr4 gamma config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             input the Wdr4 gamma config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4GammaSetConfig
(
    CamEngineHandle_t hCamEngine,
	CamEngineWdr4GammaConfig_t* pConfig
);

/*****************************************************************************/
/**
 * @brief   This function get Wdr4 gamma config
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             get the Wdr4 gamma config
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4GammaGetConfig
(
    CamEngineHandle_t hCamEngine,
	CamEngineWdr4GammaConfig_t* pConfig
);

/*****************************************************************************/
/**
 * @brief   This function enable wdr4.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWdr4Enable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the wdr4.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWdr4Disable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function config the wdr4.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of wdr4
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWdr4SetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineWdr4Config_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the wdr4 status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of wdr4
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineWdr4GetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineWdr4Config_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function set the auto config of wdr4
 * 
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            wdr4 auto cfg
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4SetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineWdr4AutoConfig_t *pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the auto config of wdr4
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            wdr4 auto cfg

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4GetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineWdr4AutoConfig_t *pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the status of wdr4
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pSensorMode         sensor mode
 * @param   pMode               running mode
 * @param   pWdr4Cfg            wdr4 config
 * @param   pWdr4GammaCfg       wdr4 gamma config

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineWdr4GetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineWdr4Mode_t *pMode,
    CamEngineWdr4Config_t *pWdr4Cfg,
    CamEngineWdr4GammaConfig_t *pWdr4GammaCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_wdr4_api */


#endif /* CAM_ENGINE_WDR4_API_H */

