/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @cond    cameric_isp_ccm
 *
 * @file 	cameric_isp_ccm_drv_api.h
 *
 * @brief	This file contains the CamerIc ISP ccm driver API definitions
 *
 *****************************************************************************/
#ifndef CAMERIC_ISP_CCM_DRV_API_H
#define CAMERIC_ISP_CCM_DRV_API_H

#include <ebase/types.h>
#include <ebase/trace.h>
#include <common/return_codes.h>
#include "cameric_drv_api.h"

/******************************************************************************/
/**
 * @brief   A structure to represent a 3x3 matrix.
 *
 *          The nine values are laid out as follows (zero based index):
 *
 *               | 0 1 2 | \n
 *               | 3 4 5 | \n
 *               | 6 7 8 | \n
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcXTalkMatrix_s
{
    uint16_t    Coeff[CAMERIC_CC_MATRIX_SIZE];               /**< array of 3x3 float values */
} CamerIcXTalkMatrix_t;


/******************************************************************************/
/**
 * @brief   A structure/tupple to represent offset values for three (R,G,B)
 *          channels.
 *
 * @note    The offset values are represented as 2's complement integer.
 *          Number ranging from -2048 (0x800) to 2047 (0x7FF).
 *          0 is represented as 0x000.
 *
 *****************************************************************************/
typedef struct CamerIcXTalkOffset_s
{
    uint16_t Red;                       /**< offset value for the red channel */
    uint16_t Green;                     /**< offset value for the green channel */
    uint16_t Blue;                      /**< offset value for the blue channel */
} CamerIcXTalkOffset_t;

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions enable the ccm.
 *          there is no enable of ccm in hardware, enable in software
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCcmEnable
(
    CamerIcDrvHandle_t          handle
);
/* @endcond */


/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions disable the ccm.
 *          there is no disable of ccm in hardware, set the default value when disable      
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCcmDisable
(
    CamerIcDrvHandle_t          handle
);
/* @endcond */



/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function returns the currently configured cross-talk 
 *          coefficients.
 *
 * @param   handle              CamerIc driver handle
 * @param   pCTalkCoefficients  Cross-talk coefficients
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read the cross
 *                              talk matrix (maybe the driver is not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 * @endif
 *
 *****************************************************************************/
extern RESULT CamerIcIspGetCrossTalkCoefficients
(
    CamerIcDrvHandle_t  handle,
    CamerIcXTalkMatrix_t  *pCTalkCoefficients
);
/* @endcond */



/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions sets the cross-talk coefficients.
 *
 * @param   handle              CamerIc driver handle
 * @param   pCTalkCoefficients  Cross-talk coefficients
 *
 * @return                      Return the result of the function call.
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read the cross
 *                              talk matrix (maybe the driver is not initialized)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetCrossTalkCoefficients
(
    CamerIcDrvHandle_t          handle,
    const CamerIcXTalkMatrix_t    *pCTalkCoefficients
);
/* @endcond */



/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function returns the currently configured cross-talk 
 *          offsets.
 *
 * @param   handle              CamerIc driver handle
 * @param   pCrossTalkOffset    Cross-talk offsets
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read the cross
 *                              talk matrix (maybe the driver is not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 * @endif
 *
 *****************************************************************************/
extern RESULT CamerIcIspGetCrossTalkOffset
(
    CamerIcDrvHandle_t          handle,
    CamerIcXTalkOffset_t        *pCrossTalkOffset
);
/* @endcond */



/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions sets the cross-talk offsets.
 *
 * @param   handle              CamerIc driver handle
 * @param   pCrossTalkOffset    Cross-talk offsets
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to set the cross
 *                              talk offsets (maybe the driver is already
 *                              running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetCrossTalkOffset
(
    CamerIcDrvHandle_t          handle,
    const CamerIcXTalkOffset_t  *pCrossTalkOffset
);

#endif /* CAMERIC_ISP_CCM_DRV_API_H */
