/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */
#ifndef CAMERIC_ISP_COMPAND_DRV_API_H
#define CAMERIC_ISP_COMPAND_DRV_API_H

/**
 * @file    cameric_isp_compand_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP compand driver API definitions.
 *
 *****************************************************************************/
/**
 * @cond cameric_isp_compand
 *
 * @defgroup cameric_isp_compand_drv_api CamerIC ISP compand Driver API definitions
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#if defined ISP_CMPD_24BIT
#define COMPAND_BLS_BIT_WIDTH 24

#define COMPAND_BLS_IN_BIT_MAX 24
#define COMPAND_BLS_IN_BIT_MIN 8

#define COMPAND_BLS_VAL_MAX 0x00FFFFFFU
#define COMPAND_BLS_VAL_MIN 0x00000000U

#define COMPAND_CURVE_Y_MAX 0x00FFFFFFU
#define COMPAND_CURVE_Y_MIN 0x00000000U

#define COMPAND_EXPAND_IN_BIT_MAX 24
#define COMPAND_EXPAND_IN_BIT_MIN 12

#elif defined ISP_CMPD_20BIT
#define COMPAND_BLS_BIT_WIDTH 20

#define COMPAND_BLS_IN_BIT_MAX 20
#define COMPAND_BLS_IN_BIT_MIN 8

#define COMPAND_BLS_VAL_MAX 0x000FFFFFU
#define COMPAND_BLS_VAL_MIN 0x00000000U

#define COMPAND_CURVE_Y_MAX 0x000FFFFFU
#define COMPAND_CURVE_Y_MIN 0x00000000U

#define COMPAND_EXPAND_IN_BIT_MAX 20
#define COMPAND_EXPAND_IN_BIT_MIN 12

#endif

#define COMPAND_BLS_CHANNEL_NUM 4

#define COMPAND_CURVE_PX_MAX 0x1F
#define COMPAND_CURVE_PX_MIN 0

#define COMPAND_CURVE_BIN_NUM  64

typedef struct CamerIcIspCmpdCurve_s
{
    uint8_t curvePx[ COMPAND_CURVE_BIN_NUM ];
    uint32_t curveY[ COMPAND_CURVE_BIN_NUM ];
} CamerIcIspCmpdCurve_t;

typedef struct CamerIcIspCmpdExpandCfg_s
{
    bool_t updateCurve;
    uint8_t inBit;
    CamerIcIspCmpdCurve_t expandCurve;
} CamerIcIspCmpdExpandCfg_t;

typedef struct CamerIcIspCmpdCompressCfg_s
{
    bool_t updateCurve;
    uint8_t inBit;
    CamerIcIspCmpdCurve_t compressCurve;
} CamerIcIspCmpdCompressCfg_t;

typedef struct CamerIcIspCmpdBlsCfg_s
{
    uint32_t bls[COMPAND_BLS_CHANNEL_NUM];
    uint8_t inBit;
} CamerIcIspCmpdBlsCfg_t;

/*****************************************************************************/
/**
*          CamerIcIspCmpdExpandSetConfig()
*
* @brief   Set compand expand config
*
* @param   handle          CamerIc driver handle
* @param   pConfig         configuration of cmpd expand
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdExpandSetConfig
(
    CamerIcDrvHandle_t              handle,
    const CamerIcIspCmpdExpandCfg_t *pConfig
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdExpandGetConfig()
*
* @brief   Get compand expand config
*
* @param   handle          CamerIc driver handle
* @param   pConfig         configuration of cmpd expand
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdExpandGetConfig
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspCmpdExpandCfg_t       *pConfig
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdCompressSetConfig()
*
* @brief   Set compand compress config
* @param   handle          CamerIc driver handle
* @param   pConfig         configuration of cmpd compress
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdCompressSetConfig
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspCmpdCompressCfg_t     *pConfig
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdCompressGetConfig()
*
* @brief   Get compand compress config
*
* @param   handle          CamerIc driver handle
* @param   pConfig         configuration of cmpd compress
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdCompressGetConfig
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspCmpdCompressCfg_t     *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function sets the fixed black level substraction values
 *          for the color channels.
 *
 * @note    The assignment of the values for A, B, C and D to R, G, G and B
 *          pattern depends on the position and offset of the input acquisition
 *          window.
 *
 *          The position [0,0] is always assigned to A, correlatively [0,1]
 *          is assigned to B, [1,0] to C and [1,1] to D.
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             black-level substraction value
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_CONFIG    a bayer-pattern was not configured
 * @retval  RET_OUTOFRANGE      substraction value is to large
 *
 *****************************************************************************/
RESULT CamerIcIspCmpdBlsSetBlackLevel
(
    CamerIcDrvHandle_t  handle,
    const CamerIcIspCmpdBlsCfg_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function reads out the current configured fixed black level
 *          substraction values.
 *
 * @note    The assignment of the values for A, B, C and D to R, G, G and B
 *          pattern depends on the position and offset of the input acquisition
 *          window.
 *
 *          The position [0,0] is always assigned to A, correlatively [0,1]
 *          is assigned to B, [1,0] to C and [1,1] to D.
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             black-level substraction value
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
RESULT CamerIcIspCmpdBlsGetBlackLevel
(
    CamerIcDrvHandle_t  handle,
    CamerIcIspCmpdBlsCfg_t *pConfig
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdBlsIsActivated()
*
* @brief   Get compand bls module status
*
* @param   handle          CamerIc driver handle
* @param   pIsEnabled      Enabled or not
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdBlsIsActivated
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdBlsEnable()
*
* @brief   Enable compand bls module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdBlsEnable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdBlsDisable()
*
* @brief   Disable compand bls module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdBlsDisable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdExpandEnable()
*
* @brief   Enable compand expand module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdExpandEnable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdExpandDisable()
*
* @brief   Disable compand expand module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdExpandDisable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdExpandIsActivated()
*
* @brief   Get compand expand module status
*
* @param   handle          CamerIc driver handle
* @param   pIsEnabled      Enabled or not
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdExpandIsActivated
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdCompressEnable()
*
* @brief   Enable compand compresss module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdCompressEnable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdCompressDisable()
*
* @brief   Disable compand compress module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdCompressDisable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
*          CamerIcIspCmpdCompressIsActivated()
*
* @brief   Get compand compress module status
*
* @param   handle          CamerIc driver handle
* @param   pIsEnabled      Enabled or not
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspCmpdCompressIsActivated
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

#ifdef __cplusplus
}
#endif



#endif /* CAMERIC_ISP_COMPAND_DRV_API_H */
