/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMERIC_ISP_TPG_DRV_API_H
#define CAMERIC_ISP_TPG_DRV_API_H

/**
 * @file    cameric_isp_tpg_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP tpg driver API definitions.
 *
 *****************************************************************************/
/**
 * @cond cameric_isp_tpg
 *
 * @defgroup cameric_isp_tpg_drv_api CamerIC ISP tpg Driver API definitions
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   Enumeration of TPG picture type.
 *
 *****************************************************************************/
typedef enum CamerIcISPTpgImageType_e
{
    CAMERIC_ISP_TPG_IMAGE_TYPE_3X3COLORBLOCK    = 0,    /**< 3 x 3 color block */
    CAMERIC_ISP_TPG_IMAGE_TYPE_COLORBAR         = 1,    /**< color bar */
    CAMERIC_ISP_TPG_IMAGE_TYPE_GRAYBAR          = 2,    /**< gray bar */
    CAMERIC_ISP_TPG_IMAGE_TYPE_GRID             = 3,    /**< highlighted grid */
    CAMERIC_ISP_TPG_IMAGE_TYPE_RANDOM           = 4,    /**< random data */

    CAMERIC_ISP_TPG_IMAGE_TYPE_MAX
} CamerIcISPTpgImageType_t;
/* @endcond */


/******************************************************************************/
/**
 * @brief   Enumeration type to configure the bayer pattern in the TPG
 *
 *****************************************************************************/
typedef enum CamerIcIspTpgBayerPattern_e
{
    CAMERIC_ISP_TPG_BAYER_PATTERN_RGGB      = 0,        /**< 1st line: RGRG... , 2nd line GBGB... , etc. */
    CAMERIC_ISP_TPG_BAYER_PATTERN_GRBG      = 1,        /**< 1st line: GRGR... , 2nd line BGBG... , etc. */
    CAMERIC_ISP_TPG_BAYER_PATTERN_GBRG      = 2,        /**< 1st line: GBGB... , 2nd line RGRG... , etc. */
    CAMERIC_ISP_TPG_BAYER_PATTERN_BGGR      = 3,        /**< 2st line: BGBG... , 2nd line GRGR... , etc. */

    CAMERIC_ISP_TPG_BAYER_PATTERN_MAX                   /**< upper border (only for an internal evaluation) */
} CamerIcIspTpgBayerPattern_t;
/* @endcond */


/******************************************************************************/
/**
 * @brief   Enumeration of data width of TPG.
 *
 *****************************************************************************/
typedef enum CamerIcIspTpgColorDepth_e
{
    CAMERIC_ISP_TPG_8BIT            = 0,        /**< 8  Bit output */
    CAMERIC_ISP_TPG_10BIT           = 1,        /**< 10 Bit output */
    CAMERIC_ISP_TPG_12BIT           = 2,        /**< 12 Bit output */

} CamerIcIspTpgColorDepth_t;
/* @endcond */


/******************************************************************************/
/**
 * @brief   Enumeration of TPG resolution.
 *
 *****************************************************************************/
typedef enum CamerIcIspTpgResolution_e
{
    CAMERIC_ISP_TPG_1080P           = 0,        /**< 1920 x 1080  */
    CAMERIC_ISP_TPG_720P            = 1,        /**< 1280 x 720   */
    CAMERIC_ISP_TPG_4K              = 2,        /**< 3840 x 2160  */
    CAMERIC_ISP_TPG_USER_DEFINED    = 3,        /**< user defined */

} CamerIcIspTpgResolution_t;
/* @endcond */

/******************************************************************************/
/**
 * @brief   Stucture of TPG user defined mode.
 *
 *****************************************************************************/
typedef struct CamerIcIspTpgUserDefineMode_s
{
    struct
    {
        uint16_t    total;
        uint16_t    fp;
        uint16_t    syncHeaderWidth;
        uint16_t    bp;
        uint16_t    act;
    }H,V;

} CamerIcIspTpgUserDefineMode_t;
/* @endcond */

/******************************************************************************/
/**
 * @brief   Stucture of TPG configuration.
 *
 *****************************************************************************/
typedef struct CamerIcIspTpgConfig_s
{
    CamerIcISPTpgImageType_t        imageType;
    CamerIcIspTpgBayerPattern_t     bayerPattern;
    CamerIcIspTpgColorDepth_t       colorDepth;
    CamerIcIspTpgResolution_t       resolution;

    uint16_t                        pixleGap;       // image 0 (3x3 block)
    uint16_t                        lineGap;        // image 0 (3x3 block)
    uint16_t                        gapStandard;    // image 1, 2, 3.
    uint32_t                        randomSeed;     // image 4
    uint32_t                        frameNum;

    CamerIcIspTpgUserDefineMode_t   userMode;

} CamerIcIspTpgConfig_t;


/*****************************************************************************/
/**
 *          CamerIcIspTpgEnable()
 *
 * @brief   Enable TPG Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspTpgEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 *          CamerIcIspTpgDisable()
 *
 * @brief   Disable TPG Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspTpgDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 *          CamerIcIspTpgConfigMode()
 *
 * @brief   Config TPG mode
 *
 * @param   handle          CamerIc driver handle
 * @param   pTpgConfig      pointer to the tpg module config context
 *                          @ref CamerIcIspTpgConfig_t
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_WRONG_HANDLE
 *
 *****************************************************************************/
RESULT CamerIcIspTpgConfig
(
    CamerIcDrvHandle_t      handle,
    const CamerIcIspTpgConfig_t  *pTpgConfig

);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input acquisition module.
 *
 * @param   handle              CamerIC driver handle
 * @param   tpg_enable          tpg enable flag
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspSetTpgBypass
(
    CamerIcDrvHandle_t                  handle,
    uint32_t                            tpg_enable
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_tpg_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_TPG_DRV_API_H */

