/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_WB_DRV_API_H
#define CAMERIC_ISP_WB_DRV_API_H

/**
 * @cond    cameric_isp_wb
 *
 * @file    cameric_isp_wb_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP AWB driver API definitions.
 *
 *****************************************************************************/
/**
 * @defgroup cameric_isp_awb_drv_api CamerIC ISP AWB driver API definitions
 * @{
 *
 * The AWB module is able to measure the occurrence of near-white pixels
 * in an image processed by the MARVIN. To be able to measure the desired
 * pixels the AWB module supports to configure the measurement window.
 *
 * Further on the component gains could be adjusted to control the white point.
 * The Automatic White Balancing driver serves as an abstraction layer, so the
 * application does not need to know which bit has to be set where in the
 * registers of the AWB module.
 *
 * The mean value measurement is done by hardware and the gain update has to
 * be done by software of the host processor.
 *
 * The following figure shows the position of the AWB module in the CamerIC ISP
 * pipeline.
 *
 * @image html awb.png "Overview of the CamerIC ISP AWB driver" width=\textwidth
 * @image latex awb.png "Overview of the CamerIC ISP AWB driver" width=\textwidth
 *
 */

#include <ebase/types.h>
#include <common/return_codes.h>
// #include "isi/isi.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   Initialize CamerIc ISP WB driver context
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspWbInit
(
    CamerIcDrvHandle_t handle
);
/*****************************************************************************/
/**
 *          CamerIcIspWbRelease()
 *
 * @brief   Release/Free CamerIc ISP Wb driver context
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspWbRelease
(
    CamerIcDrvHandle_t handle
);
/*****************************************************************************/
/**
 * @brief   This function enables the CamerIc ISP WB function
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         ISP AWB module successfully enabled
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspWbEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc ISP WB measuring module
 *          (bypassing of the white balance measuring module).
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         ISP AWB module successfully disabled
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspWbDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This functions reads out the currently configured white balance
 *          gains.
 *
 * @param   handle              CamerIc driver handle
 * @param   pGains              Pointer to store the white balance gains
 *                              (@ref CamerIcGains_t)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read WB gains
 *                              from CamerIC (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspWbGetGains
(
    CamerIcDrvHandle_t  handle,
    CamerIcGains_t      *pGains
);



/*****************************************************************************/
/**
 * @brief   This functions sets the white balance gains.
 *
 * @param   handle              CamerIc driver handle
 * @param   pGains              Pointer to white balance gains to set
 *                              (@ref CamerIcGains_t)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read WB gains
 *                              from CamerIC (maybe driver not initialized)
 *
 *****************************************************************************/
extern RESULT CamerIcIspWbSetGains
(
    CamerIcDrvHandle_t      handle,
    const CamerIcGains_t    *pGains
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_wb_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_WB_DRV_API_H */

