/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

#ifndef CAMERIC_MI_RDMA_DRV_API_H
#define CAMERIC_MI_RDMA_DRV_API_H

/**
 * @file cameric_mi_rdma_drv_api.h
 *
 * @brief   This file contains the CamerIC MI RDMA driver API definitions.
 *
 *****************************************************************************/
/**
 * @cond cameric_mi
 *
 * @defgroup cameric_mi_rdma_drv_api CamerIc MI Driver API definitions
 * @{
 *
 * on the DMA read port path could be used to read an image from the system
 * memory.The Memory Interface Driver serves as an abstraction layer.
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   Enumeration of the RDMA input path.
 *
 * @note    This is a list of all RDMA input path supported by CamerIC
 *          driver system.
 *
 *****************************************************************************/
typedef enum CamerIcMiRdmaPath_e
{
    CAMERIC_MI_RDMA_PATH_INVALID                 = 0x0000,                               /**< invalid path (only for an internal evaluation) */
    CAMERIC_MI_MCM_RDMA                          = 0x0001,                               /**< mcm RDMA path*/
    CAMERIC_MI_RDMA_PATH_MAX                                                            /**< upper border(only for an internal evaluation)*/
} CamerIcMiRdmaPath_t;

/*****************************************************************************/
/**
 * @brief  create the RDMA interrupt process thread
 *
 * @param   handle    cameric drv handle
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamericMiCreateRdmaThread
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief  destroy the RDMA interrupt process thread
 *
 * @param   handle    cameric drv handle
 * @return                    Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamericMiDestroyRdmaThread
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function registers a Request-Callback at the CamerIC Memory
 *          Interface Module. A request callback is called if the driver
 *          needs an interaction from the application layer (i.e. a new input 
 *          image buffer filed, please also see @ref CamerIcRequestId_e).
 *
 * @param   handle              CamerIc driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT CamerIcMiRdmaRegisterRequestCb
(
    CamerIcDrvHandle_t      handle,
    CamerIcRequestFunc_t    func,
    void                     *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Request-Callback
 *          at CamerIC Memory Interface Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              request callback
 *
 *****************************************************************************/
extern RESULT CamerIcMiRdmaDeRegisterRequestCb
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 * @brief   This functions registers an Event-Callback at CamerIC Memory
 *          Interface Module. An event callback is called if the driver
 *          needs to inform the application layer about an asynchronous event
 *          or an error situation (i.e. please also see @ref CamerIcEventId_e).
 *
 * @param   handle              CamerIc driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              event callback
 *
 *****************************************************************************/
extern RESULT CamerIcMiRdmaRegisterEventCb
(
    CamerIcDrvHandle_t  handle,
    CamerIcEventFunc_t  func,
    void                 *pUserContext
);


/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Event-Callback
 *          at CamerIC Memory Interface Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              event callback
 *
 *****************************************************************************/
extern RESULT CamerIcMiRdmaDeRegisterEventCb
(
    CamerIcDrvHandle_t handle
);


#ifdef __cplusplus
}
#endif

/* @} cameric_mi_rdma_drv_api */

/* @endcond */

#endif /* CAMERIC_MI_RDMA_DRV_API_H */

