/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef __ISPCORE_DEFS_H__
#define __ISPCORE_DEFS_H__

#include <ebase/types.h>

/*****************************************************************************/
/**
 * @brief Lock types for the auto algorithms. Can be OR combined.
 *
 *****************************************************************************/
typedef enum CamEngineLockType_e {
    CAM_ENGINE_LOCK_NO      = 0x00,
    CAM_ENGINE_LOCK_AF      = 0x01,
    CAM_ENGINE_LOCK_AEC     = 0x02,
    CAM_ENGINE_LOCK_AWB     = 0x04,

    CAM_ENGINE_LOCK_ALL     = (CAM_ENGINE_LOCK_AF | CAM_ENGINE_LOCK_AEC | CAM_ENGINE_LOCK_AWB)
} CamEngineLockType_t;

/*****************************************************************************/
/**
 * @brief Flicker period types for the AEC algorithm.
 *
 *****************************************************************************/
typedef enum CamEngineFlickerPeriod_e
{
    CAM_ENGINE_FLICKER_OFF   = 0x00,
    CAM_ENGINE_FLICKER_100HZ = 0x01,
    CAM_ENGINE_FLICKER_120HZ = 0x02
} CamEngineFlickerPeriod_t;


/**
 * @brief   Enumeration type to input data format select.
 *
 */
typedef enum CamerIcIspEeInputDataFormatSelect_e
{
    CAMERIC_ISP_EE_INPUT_DATA_FORMAT_SELECT_INVALID   = 0,
    CAMERIC_ISP_EE_INPUT_DATA_FORMAT_SELECT_RGB       = 1,
    CAMERIC_ISP_EE_INPUT_DATA_FORMAT_SELECT_YUV       = 2,
    CAMERIC_ISP_EE_INPUT_DATA_FORMAT_SELECT_MAX
} CamerIcIspEeInputDataFormatSelect_t;

/*****************************************************************************/
/**
 * @brief   This macro defines the size of lens shading grad table in 16 Bit
 * 			words.
 *
 *****************************************************************************/
#define CAM_DEV_GRAD_TBL_SIZE        8


/*****************************************************************************/
/**
 * @brief   This macro defines the size of lens shading data table in 16 Bit
 * 			words.
 *
 *****************************************************************************/
#define CAM_DEV_DATA_TBL_SIZE      289


/*****************************************************************************/
/**
 * @brief   Auto-Focus-Control search algorithm.
 *
 *****************************************************************************/
typedef enum CamEngineAfSearchAlgorithm_e {
    CAM_ENGINE_AUTOFOCUS_SEARCH_ALGORITHM_INVALID           = 0,    /**< invalid search algorithm */
    CAM_ENGINE_AUTOFOCUS_SEARCH_ALGORITHM_FULL_RANGE        = 1,    /**< full range */
    CAM_ENGINE_AUTOFOCUS_SEARCH_ALGORITHM_ADAPTIVE_RANGE    = 2,    /**< adaptive range */
    CAM_ENGINE_AUTOFOCUS_SEARCH_ALGORITHM_HILL_CLIMBING     = 3,    /**< hill climbing */
    CAM_ENGINE_AUTOFOCUS_SEARCH_ALGORITHM_MAX
} CamEngineAfSearchAlgorithm_t;


/*****************************************************************************/
/**
 * @brief   Auto-Exposure-Control scene evaluation mode.
 *
 *****************************************************************************/
typedef enum CamEngineAecSemMode_e {
    CAM_ENGINE_AEC_SCENE_EVALUATION_INVALID       = 0,    /**< invalid mode (only for initialization) */
    CAM_ENGINE_AEC_SCENE_EVALUATION_DISABLED      = 1,    /**< scene evaluation disabled (fix setpoint) */
    CAM_ENGINE_AEC_SCENE_EVALUATION_FIX           = 2,    /**< scene evaluation fix (static ROI) */
    CAM_ENGINE_AEC_SCENE_EVALUATION_ADAPTIVE      = 3,    /**< scene evaluation adaptive (adaptive ROI) */
    CAM_ENGINE_AEC_SCENE_EVALUATION_MAX
} CamEngineAecSemMode_t;


/*****************************************************************************/
/**
 * @brief   Auto-Exposure-Control histogram.
 *
 *****************************************************************************/
#ifdef ISP_HIST256
#define CAM_ENGINE_AEC_HIST_NUM_BINS           256  /**< number of bins */
#else
#define CAM_ENGINE_AEC_HIST_NUM_BINS           16  /**< number of bins */
#endif
typedef uint32_t CamEngineAecHistBins_t[CAM_ENGINE_AEC_HIST_NUM_BINS];


/*****************************************************************************/
/**
 * @brief   Auto-Exposure-Control luminance grid.
 *
 *****************************************************************************/
#define CAM_ENGINE_AEC_EXP_GRID_ITEMS          25  /**< number of grid items (see @ref CamerIcMeanLuma_t) */
typedef uint8_t CamEngineAecMeanLuma_t[CAM_ENGINE_AEC_EXP_GRID_ITEMS];


/*****************************************************************************/
/**
 * @brief   Auto-White-Balance mode.
 *
 *****************************************************************************/
typedef enum CamEngineAwbMode_e {
    CAM_ENGINE_AWB_MODE_INVALID = 0,    /**< invalid mode (only for initialization) */
    CAM_ENGINE_AWB_MODE_MANUAL  = 1,    /**< manual mode */
    CAM_ENGINE_AWB_MODE_AUTO    = 2,    /**< run auto mode */
    CAM_ENGINE_AWB_MODE_MAX
} CamEngineAwbMode_t;


/*****************************************************************************/
/**
 * @brief   Generic structure for the cross talk matrix of the four color
 *          components.
 *
 *****************************************************************************/
typedef struct CamEngineCcMatrix_s
{
    float Coeff[9U];
} CamEngineCcMatrix_t;

/*****************************************************************************/
/**
 * @brief   Generic structure for the cross talk offset of the four color
 *          components.
 *
 *****************************************************************************/
typedef struct CamEngineCcOffset_s
{
    int16_t Red;
    int16_t Green;
    int16_t Blue;
} CamEngineCcOffset_t;


/*****************************************************************************/
/**
 * @brief   Generic structure for the white balance gains of the four color
 *          components.
 *
 *****************************************************************************/
typedef struct CamEngineWbGains_s
{
    float Red;
    float GreenR;
    float GreenB;
    float Blue;
} CamEngineWbGains_t;


/******************************************************************************/
/**
 * @brief   Enumeration type to configure the super impose working mode.
 *
 *****************************************************************************/
typedef enum CamEngineSimpMode_e
{
    CAM_ENGINE_SIMP_MODE_INVALID            = 0,        /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_SIMP_MODE_OVERLAY            = 1,        /**< overlay mode */
    CAM_ENGINE_SIMP_MODE_KEYCOLORING        = 2,        /**< keycoloring mode */
    CAM_ENGINE_SIMP_MODE_MAX                            /**< upper border (only for an internal evaluation) */
} CamEngineSimpMode_t;



/******************************************************************************/
/**
 * @brief   Structure to configure the super impose module
 *
 * @note    This structure needs to be converted to driver structure
 *
 *****************************************************************************/
typedef struct CamEngineSimpConfig_s
{
    CamEngineSimpMode_t     Mode;

    union SimpModeConfig_u
    {
        struct Overlay_s
        {
            uint32_t OffsetX;
            uint32_t OffsetY;
        } Overlay;

        struct KeyColoring_s
        {
            uint8_t Y;
            uint8_t Cb;
            uint8_t Cr;
        } KeyColoring;
    } SimpModeConfig;

    PicBufMetaData_t      *pPicBuffer;

} CamEngineSimpConfig_t;

#endif  // __ISPCORE_DEFS_H__
