/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef DPF_H
#define DPF_H

/**
 * @file dpf.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup ADPF Auto denoising pre-filter module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"

#include <cameric_drv/cameric_drv_api.h>


#ifdef __cplusplus
extern "C"
{
#endif

#define DPF_MAX_NLF_COEFFS             17 // bin number of noise level curve
#define DPF_MAX_SPATIAL_COEFFS         6

#define DPF_GAINS_MAX                  15.995f
#define DPF_SENSOR_GAIN_MAX            1.0f
#define DPF_STRENGTH_MAX               128.0f
#define DPF_STRENGTH_MIN               0.251f

#define DPF_NF_GAIN_RED_DEFAULT        1.0f
#define DPF_NF_GAIN_GREEN_R_DEFAULT    1.0f
#define DPF_NF_GAIN_GREEN_B_DEFAULT    1.0f
#define DPF_NF_GAIN_BLUE_DEFAULT       1.0f

#define DPF_FSENSOR_GAIN_MAX 1000.0f
#define DPF_FSENSOR_GAIN_MIN 1.0f
#define DPF_GRADIENT_MAX    128.0f
#define DPF_GRADIENT_MIN    0.1f
#define DPF_OFFSET_MAX      128.0f
#define DPF_OFFSET_MIN      0.0f
#define DPF_MIN_MAX         128.0f
#define DPF_MIN_MIN         1.0f
#define DPF_DIV_MAX         65.0f
#define DPF_DIV_MIN         1.0f
#define DPF_SIGMA_G_MAX     128.0f
#define DPF_SIGMA_G_MIN     1.0f
#define DPF_SIGMA_RB_MAX    128.0f
#define DPF_SIGMA_RB_MIN    1.0f
#define DPF_NOISE_CURVE_MAX 4095

/*****************************************************************************/
/**
 *          ADpfHandle_t
 *
 * @brief   ADPF Module instance handle
 *
 *****************************************************************************/
typedef void* ADpfHandle_t;         /**< handle to ADPF context */

/*****************************************************************************/
/**
 * @brief   This type defines the supported gain usage modes in the DPF
 *          preprocessing stage.
 */
/*****************************************************************************/
typedef enum ADpfGainUsage_e
{
    DPF_GAIN_USAGE_INVALID       = 0,   /**< lower border (only for an internal evaluation) */
    DPF_GAIN_USAGE_DISABLED      = 1,   /**< don't use any gains in preprocessing stage */
    DPF_GAIN_USAGE_NF_GAINS      = 2,   /**< use only the noise function gains  from registers DPF_NF_GAIN_R, ... */
    DPF_GAIN_USAGE_LSC_GAINS     = 3,   /**< use only the gains from LSC module */
    DPF_GAIN_USAGE_NF_LSC_GAINS  = 4,   /**< use the moise function gains and the gains from LSC module */
    DPF_GAIN_USAGE_AWB_GAINS     = 5,   /**< use only the gains from AWB module */
    DPF_GAIN_USAGE_AWB_LSC_GAINS = 6,   /**< use the gains from AWB and LSC module */
    DPF_GAIN_USAGE_MAX                  /**< upper border (only for an internal evaluation) */
} ADpfGainUsage_t;

/*****************************************************************************/
/**
 * @brief   This type defines the supported filter kernel sizes for the red
 *          and blue channels.
 */
/*****************************************************************************/
typedef enum ADpfRedBlueFilterSize_e
{
    DPF_RB_FILTERSIZE_INVALID   = 0,    /**< lower border (only for an internal evaluation) */
    DPF_RB_FILTERSIZE_9x9       = 1,    /**< red and blue filter kernel size 9x9 (means 5x5 active pixel) */
    DPF_RB_FILTERSIZE_13x9      = 2,    /**< red and blue filter kernel size 13x9 (means 7x5 active pixel) */
    DPF_RB_FILTERSIZE_MAX               /**< upper border (only for an internal evaluation) */
} ADpfRedBlueFilterSize_t;

/*****************************************************************************/
/**
 * @brief   A structure/tupple to represent gain values for four (R,Gr,Gb,B)
 *          channels.
 *
 * @note    The gain values are represented as float32_t numbers.
 */
/*****************************************************************************/
typedef struct ADpfGains_s
{
    float32_t   fRed;                               /**< gain value for the red channel */
    float32_t   fGreenR;                            /**< gain value for the green channel in red lines */
    float32_t   fGreenB;                            /**< gain value for the green channel in blue lines */
    float32_t   fBlue;                              /**< gain value for the blue channel */
} ADpfGains_t;

/*****************************************************************************/
/**
 * @brief   This type defines the supported scaling of x axis.
 */
/*****************************************************************************/
typedef enum ADpfNoiseLevelLookUpScale_s
{
    NLL_SCALE_INVALID       = 0,        /**< lower border (only for an internal evaluation) */
    NLL_SCALE_LINEAR        = 1,        /**< use a linear scaling */
    NLL_SCALE_LOGARITHMIC   = 2,        /**< use a logarithmic scaling */
    NLL_SCALE_MAX                       /**< upper border (only for an internal evaluation) */
} ADpfNoiseLevelLookUpScale_t;

/*****************************************************************************/
/**
 * @brief
 */
/*****************************************************************************/
typedef struct ADpfNoiseLevelLookUp_s
{
    float32_t                        nllCoeff[DPF_MAX_NLF_COEFFS];           /**< Noise-Level-Lookup coefficients */
    ADpfNoiseLevelLookUpScale_t      xScale;                                 /**< type of x-axis (logarithmic or linear type) */
} ADpfNoiseLevelLookUp_t;

/*****************************************************************************/
/**
 *          ADpfAutoParamTable_t
 *
 * @brief   dpf Module parameters table structure
 *
 *****************************************************************************/
typedef struct ADpfAutoParamTable_s
{
    uint8_t autoLevel;
    float32_t gain[AUTO_NUM];
    float32_t gradient[AUTO_NUM];
    float32_t offset[AUTO_NUM];
    float32_t min[AUTO_NUM];
    float32_t div[AUTO_NUM];
    float32_t sigmaG[AUTO_NUM];
    float32_t sigmaRb[AUTO_NUM];
    float32_t noiseCurve[AUTO_NUM][DPF_MAX_NLF_COEFFS];
} ADpfAutoParamTable_t;


/*****************************************************************************/
/**
 *          ADpfInstanceConfig_t
 *
 * @brief   ADPF Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ADpfInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    ADpfHandle_t            hADpf;              /**< handle returned by ADpfInit() */
} ADpfInstanceConfig_t;

/*****************************************************************************/
/**
 *          ADpfConfig_t
 *
 * @brief   ADPF Module configuration structure
 *
 *****************************************************************************/
typedef struct ADpfConfig_s
{
    ADpfGains_t             nFGains;                    /**< noise function gains */

    ADpfNoiseLevelLookUp_t  nLevelLookup;               /**< Noise-Level-Lookup coefficients */
    float32_t                   fSensorGain;                /**< initial sensor gain */
    float32_t                   sigmaGreen;                 /**< sigma value for green pixel */
    float32_t                   sigmaRedBlue;               /**< sigma value for red/blue pixel */
    float32_t                   fGradient;                  /**< gradient value for dynamic strength calculation */
    float32_t                   fOffset;                    /**< offset value for dynamic strength calculation */
    float32_t                   fMin;                       /**< upper bound for dynamic strength calculation */
    float32_t                   fDiv;                       /**< division factor for dynamic strength calculation */
} ADpfConfig_t;

/*****************************************************************************/
/**
 *          ADpfSetMode()
 *
 * @brief   The function set dpf mode
 *
 * @param   handle  Handle to dpf
 * @param   mode    dpf mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpfSetMode
(
    ADpfHandle_t        handle,
    const ARunMode_t    mode
);

/*****************************************************************************/
/**
 *          ADpfGetMode()
 *
 * @brief   The function get dpf mode
 *
 * @param   handle  Handle to dpf
 * @param   mode    dpf mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpfGetMode
(
    ADpfHandle_t       handle,
    ARunMode_t         *mode
);

/*****************************************************************************/
/**
 *          ADpfInit()
 *
 * @brief   This function initializes the Auto denoising pre-filter module
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpfInit
(
    ADpfInstanceConfig_t *pInstConfig
);



/*****************************************************************************/
/**
 *          ADpfRelease()
 *
 * @brief   The function releases/frees the Auto denoising pre-filter module
 *
 * @param   handle  Handle to ADPFM
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADpfRelease
(
    ADpfHandle_t handle
);

/*****************************************************************************/
/**
 *          ADpfEnable()
 *
 * @brief   The function starts the Auto denoising pre-filter module
 *
 * @param   handle  Handle to ADPFM
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADpfEnable
(
    ADpfHandle_t handle
);



/*****************************************************************************/
/**
 *          ADpfDisable()
 *
 * @brief   The function stops the Auto denoising pre-filter module
 *
 * @param   handle  Handle to ADPFM
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADpfDisable
(
    ADpfHandle_t handle
);


/*****************************************************************************/
/**
 *          ADpfConfigure()
 *
 * @brief   This function configures the Auto denoising pre-filter module
 *
 * @param   handle  Handle to ADPFM
 * @param   pConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADpfSetConfigure
(
    ADpfHandle_t handle,
    ADpfConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the current configuration.
 *
 * @param   handle      ADPF instance handle
 * @param   pConfig     reference of configuration structure to be filled with
 *                      the current configuration
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_NULL_POINTER
 *
 *****************************************************************************/
RESULT ADpfGetConfigure
(
    ADpfHandle_t handle,
    ADpfConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ADpfGetStatus()
 *
 * @brief   The function returns the status of the Auto denoising pre-filter
 *          module
 *
 * @param   handle  Handle to ADPFM
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADpfGetStatus
(
    ADpfHandle_t handle,
    bool_t       *pRunning,
    ARunMode_t   *pMode,
    ADpfConfig_t *pCurrentParam
);

/*****************************************************************************/
/**
 *          ADemosaic2ProcessFrame()
 *
 * @brief   dpf process frame
 *
 * @param   handle  Handle to dpf
 *          gain    current sensorgain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ADpfProcessFrame
(
    ADpfHandle_t    handle,
    const float32_t     gain
);

/*****************************************************************************/
/**
 *          ADpfSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to dpf
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADpfSetAutoParamTable
(
    ADpfHandle_t handle,
    ADpfAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ADpfGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to DPF
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ADpfGetAutoParamTable
(
    ADpfHandle_t handle,
    ADpfAutoParamTable_t *pParamTable
);

#ifdef __cplusplus
}
#endif


/* @} ADPF */


#endif /* ADPF_H*/
