/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef GCV2_H
#define GCV2_H

/**
 * @file gcv2.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup RGBGAMMA module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define GC_CURVE_SIZE 64


#define AGC_CURVE_MAX 1023  // curve max value
#define AGC_CURVE_PX_MAX 12  // curve px max value
#define AGC_STANDARD_VAL_MAX 4.0f  // standard val max value
#define AGC_STANDARD_VAL_MIN 1.0f  // standard val min value

/*****************************************************************************/
/**
 *          AGcHandle_t
 *
 * @brief   gc Module instance handle
 *
 *****************************************************************************/
typedef void *AGcHandle_t;         /**< handle to gc context */

/*****************************************************************************/
/**
 *          AGcInstanceConfig_t
 *
 * @brief   gc Module instance configuration structure
 *
 *****************************************************************************/
typedef struct AGcInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */
    CamerIcDrvHandle_t      hSubCamerIc;        /**< handle to gc cameric drivder*/

    AGcHandle_t             hAGc;            /**< handle returned by AGccInit() */
} AGcInstanceConfig_t;


/*****************************************************************************/
/**
 *          AGcConfig_t
 *
 * @brief   Gc Module configuration structure
 *
 *****************************************************************************/
typedef struct AGcConfig_s
{
    uint32_t curve[GC_CURVE_SIZE];
    bool_t standard;
    float32_t standardVal;
    bool_t userCurveX;
    uint32_t CurvePx[GC_CURVE_SIZE];
} AGcConfig_t;

/*****************************************************************************/
/**
 *          AGcAutoParamTable_t
 *
 * @brief   Gc Module parameters table structure
 *
 *****************************************************************************/
typedef struct AGcAutoParamTable_s
{
    uint8_t  autoLevel;
    float32_t gain[AUTO_NUM];
    uint16_t curve[AUTO_NUM][GC_CURVE_SIZE];
} AGcAutoParamTable_t;


/*****************************************************************************/
/**
 *          AGcInit()
 *
 * @brief   This function initializes the gc
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AGcInit
(
    AGcInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          AGcDeinit()
 *
 * @brief   The function deinit the gc
 *
 * @param   handle  Handle to Gc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcDeinit
(
    AGcHandle_t handle
);


/*****************************************************************************/
/**
 *          AGcSetConfigure()
 *
 * @brief   This function configures the gc module
 *
 * @param   handle   Handle to Gc 
 * @param   pConfig  gc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AGcSetConfigure
(
    AGcHandle_t handle,
    AGcConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          AGcGetConfigure()
 *
 * @brief   This function get the gc module configures
 *
 * @param   handle  Handle to gc
 * @param   pConfig gc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AGcGetConfigure
(
    AGcHandle_t handle,
    AGcConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AGcSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to gc
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AGcSetAutoParamTable
(
    AGcHandle_t         handle,
    AGcAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          AGcGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to Gc
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AGcGetAutoParamTable
(
    AGcHandle_t handle,
    AGcAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          AGcSetMode()
 *
 * @brief   The function set gc mode
 *
 * @param   handle  Handle to gc
 * @param   mode    gc mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcSetMode
(
    AGcHandle_t 		handle,
    const ARunMode_t	mode
);

/*****************************************************************************/
/**
 *          AGcGetMode()
 *
 * @brief   The function get gc mode
 *
 * @param   handle  Handle to gc
 * @param   mode    gc mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcGetMode
(
    AGcHandle_t 		handle,
    ARunMode_t	        *mode
);

/*****************************************************************************/
/**
 *          AGcEnable()
 *
 * @brief   enable gc
 *
 * @param   handle  Handle to gc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcEnable
(
    AGcHandle_t 		handle
);

/*****************************************************************************/
/**
 *          AGcDisable()
 *
 * @brief   disable gc
 *
 * @param   handle  Handle to gc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcDisable
(
    AGcHandle_t 		handle
);


/*****************************************************************************/
/**
 *          AGcGetStatus()
 *
 * @brief   The function returns the status of the rgbgamma
 *          module
 *
 * @param   handle  Handle to gc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcGetStatus
(
    AGcHandle_t 		handle,
    bool_t       		*pRunning,
    ARunMode_t	        *pMode,
    AGcConfig_t 	    *pCurrentParam
);



/*****************************************************************************/
/**
 *          AGcProcessFrame()
 *
 * @brief   gc process frame
 *
 * @param   handle  Handle to gc
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AGcProcessFrame
(
    AGcHandle_t    	    handle,
    const float32_t		    fGain
);


#ifdef __cplusplus
}
#endif


/* @} Gc */


#endif /* GCV2_H*/
