/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file hal_vi_irq_notifier.h
 *
 * @brief   videon in irq notifier
 *
 *          This header file exports the irq notifier thread api to distribute irq event
 * @note 
 *
 *****************************************************************************/

#ifndef __HAL_VI_IRQ_NOTIFIER_H__
#define __HAL_VI_IRQ_NOTIFIER_H__

#include <stdio.h>
#include <ebase/trace.h>
#include <common/align.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "hal_api.h"  // this includes 'hal_altera_pci.h' as well

/******************************************************************************
 * local type definitions
 *****************************************************************************/

/******************************************************************************
 * Hal video in irq notifier command.
 *****************************************************************************/
typedef enum HalVideoInIrqNotifierCmdId_e
{
    HAL_VIDEO_IN_NOTIFIER_INVALID                   = -1,    /**< invalid command (only for initialization) */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH0_IRQ    = 0,    /**< register video in mcm path0 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH1_IRQ,              /**< register video in mcm path1 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH2_IRQ,              /**< register video in mcm path2 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH3_IRQ,              /**< register video in mcm path3 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH4_IRQ,              /**< register video in mcm path4 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH5_IRQ,              /**< register video in mcm path5 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH6_IRQ,              /**< register video in mcm path6 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH7_IRQ,              /**< register video in mcm path7 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH8_IRQ,              /**< register video in mcm path8 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH9_IRQ,              /**< register video in mcm path9 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH10_IRQ,              /**< register video in mcm path10 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH11_IRQ,              /**< register video in mcm path11 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH12_IRQ,              /**< register video in mcm path12 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH13_IRQ,              /**< register video in mcm path13 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH14_IRQ,              /**< register video in mcm path14 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_MCM_PATH15_IRQ,              /**< register video in mcm path15 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_ADAPTOR_0_IRQ,              /**< register video in adaptor 0 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_ADAPTOR_1_IRQ,              /**< register video in adaptor 1 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_ADAPTOR_2_IRQ,              /**< register video in adaptor 2 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_ADAPTOR_3_IRQ,              /**< register video in adaptor 3 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_LV1_IRQ,                /**< register video in fusa lv1 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_ECC_IRQ,                /**< register video in fusa ecc irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_PARITY1_IRQ,            /**< register video in fusa parity1 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_PARITY2_IRQ,            /**< register video in fusa parity2 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_ADAP_PIXCNT_IRQ,        /**< register video in fusa adap pixcnt irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_RDC_PIXCNT_IRQ,         /**< register video in fusa rdc pixcnt irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_RDC_BIST1_IRQ,          /**< register video in fusa bist1 irq command */
    HAL_VIDEO_IN_NOTIFIER_REGISTER_FUSA_RDC_BIST2_IRQ,          /**< register video in fusa bist2 irq command */

    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH0_IRQ,              /**< unregister video in mcm path0 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH1_IRQ,              /**< unregister video in mcm path1 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH2_IRQ,              /**< unregister video in mcm path2 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH3_IRQ,              /**< unregister video in mcm path3 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH4_IRQ,              /**< unregister video in mcm path4 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH5_IRQ,              /**< unregister video in mcm path5 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH6_IRQ,              /**< unregister video in mcm path6 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH7_IRQ,              /**< unregister video in mcm path7 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH8_IRQ,              /**< unregister video in mcm path8 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH9_IRQ,              /**< unregister video in mcm path9 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH10_IRQ,              /**< unregister video in mcm path10 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH11_IRQ,              /**< unregister video in mcm path11 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH12_IRQ,              /**< unregister video in mcm path12 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH13_IRQ,              /**< unregister video in mcm path13 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH14_IRQ,              /**< unregister video in mcm path14 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_MCM_PATH15_IRQ,              /**< unregister video in mcm path15 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_ADAPTOR_0_IRQ,              /**< unregister video in adaptor 0 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_ADAPTOR_1_IRQ,              /**< unregister video in adaptor 1 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_ADAPTOR_2_IRQ,              /**< unregister video in adaptor 2 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_ADAPTOR_3_IRQ,              /**< unregister video in adaptor 3 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_LV1_IRQ,                /**< unregister video in fusa lv1 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_ECC_IRQ,                /**< unregister video in fusa ecc irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_PARITY1_IRQ,            /**< unregister video in fusa parity1 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_PARITY2_IRQ,            /**< unregister video in fusa parity2 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_ADAP_PIXCNT_IRQ,        /**< unregister video in fusa adap pixcnt irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_RDC_PIXCNT_IRQ,         /**< unregister video in fusa rdc pixcnt irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_RDC_BIST1_IRQ,          /**< unregister video in fusa bist1 irq command */
    HAL_VIDEO_IN_NOTIFIER_UNREGISTER_FUSA_RDC_BIST2_IRQ,          /**< unregister video in fusa bist2 irq command */
    HAL_VIDEO_IN_NOTIFIER_MAX

} HalVideoInIrqNotifierCmdId_t;


#define IRQ_MAX       28        //need to be greater than the isp and video in irq number 16+8(fusa)
typedef struct IrqNotifierContext_s
{

//    IrqContentQueue_t   head;
    osThread    isr_thread;
    osEvent      isr_exit_event;
    osMutex     access_lock;

    int32_t      device_file;
    IrqContext_t  irqSrcCtx[IRQ_MAX];
    int           ispId;

} IrqNotifierContext_t;

typedef void* IrqNotifierHandle_t;

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * createHalIrqNotifierThd()
 *****************************************************************************/
RESULT createViHalIrqNotifierThd
(
    IrqNotifierContext_t * pIrqNotifier
);

/******************************************************************************
 * destroyHalIrqNotifierThd()
 *****************************************************************************/
RESULT destroyViHalIrqNotifierThd
(
    IrqNotifierContext_t * pIrqNotifier
);

/******************************************************************************
 * HalIrqNotifierSendCommand
 *****************************************************************************/
RESULT ViHalIrqNotifierSendCommand
(
    IrqNotifierContext_t     *pIrqNotifier,
    HalIrqNotifierCmd_t    *pCommand
);

#ifdef __cplusplus
}
#endif

#endif /* __HAL_VI_IRQ_NOTIFIER_H__ */