/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef LSC_H
#define LSC_H

/**
 * @file lsc.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup lsc module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>
#include "cam_calibdb/cam_calibdb_api.h"

#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 *          ALscHandle_t
 *
 * @brief   lsc Module instance handle
 *
 *****************************************************************************/
typedef void *ALscHandle_t;         /**< handle to Lsc context */

/*****************************************************************************/
/**
 *          ALscInstanceConfig_t
 *
 * @brief   lsc Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ALscInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */
    ALscHandle_t          hALsc;            /**< handle returned by ALscInit() */
} ALscInstanceConfig_t;


#define ALSC_RAW_CHANNEL_NUM 4
#define ALSC_H_BLOCK_NUM 32
#define ALSC_V_BLOCK_NUM 32
#define ALSC_V_POINT_NUM (ALSC_H_BLOCK_NUM+1)
#define ALSC_H_POINT_NUM (ALSC_H_BLOCK_NUM+1)
#define ALSC_COLOR_TEM_WEIGHT_NUM 7

typedef enum ALscInterMode_e
{
    ALSC_GAIN_INTER                    = 0,
    ALSC_COLOR_TEMP_WEIGHT_INTER       = 1,
    ALSC_GAIN_COLOR_TEMP_WEIGHT_INTER  = 2,
    ALSC_INTER_MAX
} ALscInterMode_t;

/*****************************************************************************/
/**
 *          ALscConfig_t
 *
 * @brief   lsc Module configuration structure
 *
 *****************************************************************************/
typedef struct ALscConfig_s
{
	uint16_t matrix[ALSC_RAW_CHANNEL_NUM][ALSC_V_POINT_NUM][ALSC_H_POINT_NUM];
	uint16_t xSize[ALSC_H_BLOCK_NUM];
	uint16_t ySize[ALSC_V_BLOCK_NUM/2];
} ALscConfig_t;

/*****************************************************************************/
/**
 *          AlscWbColorTemWeight_t
 *
 * @brief   Awb color temperature weight of every lighting
 *
 */
/*****************************************************************************/
typedef float32_t AlscWbColorTemWeight_t[ALSC_COLOR_TEM_WEIGHT_NUM];


/*****************************************************************************/
/**
 *          ALscAutoConfig_t
 *
 * @brief   lsc Module auto configuration structure
 *
 *****************************************************************************/
typedef struct ALscAutoConfig_s
{
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    float32_t damping;
    float32_t strength[AUTO_NUM];
    ALscInterMode_t interMode;
} ALscAutoConfig_t;


/*****************************************************************************/
/**
 *          ALscInit()
 *
 * @brief   This function initializes the lsc
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ALscInit
(
    ALscInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          ALscDeinit()
 *
 * @brief   The function deinit the lsc
 *
 * @param   handle  Handle to Lsc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ALscDeinit
(
    ALscHandle_t handle
);


/*****************************************************************************/
/**
 *          ALscSetConfigure()
 *
 * @brief   This function configures the lsc module
 *
 * @param   handle   Handle to lsc 
 * @param   pConfig  lsc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscSetConfigure
(
    ALscHandle_t handle,
    ALscConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ALscGetConfigure()
 *
 * @brief   This function get the lsc module configures
 *
 * @param   handle  Handle to lsc
 * @param   pConfig lsc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscGetConfigure
(
    ALscHandle_t handle,
    ALscConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ALscSetMode()
 *
 * @brief   This function configures the lsc mode
 *
 * @param   handle   Handle to lsc 
 * @param   mode     lsc mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscSetMode
(
    ALscHandle_t handle,
    const ARunMode_t mode
);


/*****************************************************************************/
/**
 *          ALscGetConfigure()
 *
 * @brief   This function get the lsc mode
 *
 * @param   handle  Handle to lsc
 * @param   mode     lsc mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscGetMode
(
    ALscHandle_t handle,
    ARunMode_t *mode
);

/*****************************************************************************/
/**
 *          ALscSetAutoConfigure()
 *
 * @brief   This function auto configures the lsc module
 *
 * @param   handle   Handle to lsc 
 * @param   pConfig  lsc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscSetAutoConfigure
(
    ALscHandle_t handle,
    ALscAutoConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ALscGetAutoConfigure()
 *
 * @brief   This function get the lsc module auto configures
 *
 * @param   handle  Handle to lsc
 * @param   pConfig lsc config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ALscGetAutoConfigure
(
    ALscHandle_t handle,
    ALscAutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          ALscEnable()
 *
 * @brief   enable lsc
 *
 * @param   handle  Handle to lsc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ALscEnable
(
    ALscHandle_t handle,
    CamCalibDbHandle_t hCamCalibDb
);

/*****************************************************************************/
/**
 *          ALscDisable()
 *
 * @brief   disable lsc
 *
 * @param   handle  Handle to lsc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ALscDisable
(
    ALscHandle_t 		handle
);


/*****************************************************************************/
/**
 *          ALscGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to lsc
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ALscGetStatus
(
    ALscHandle_t 		handle,
    bool_t       		*pRunning,
    ARunMode_t          *pMode,
    ALscConfig_t 	    *pCurrentParam
);

/*****************************************************************************/
/**
 *          ALscProcessFrame()
 *
 * @brief   lsc process frame
 *
 * @param   handle  Handle to lsc
 *          gain    current sensor-gain
 *          wbTemWeight wb tem weight 
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ALscProcessFrame
(
    ALscHandle_t handle,
    AlscWbColorTemWeight_t wbTemWeight,
    const float32_t	fGain,
    bool_t lscWbWeightEnable
);

#ifdef __cplusplus
}
#endif


/* @} Lsc */


#endif /* LSC_H*/
