/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/*****************************************************************************
 * @file AWB_ctrl_api.h
 *
 * @brief Include all AWB control function and parameters
 *
 *****************************************************************************/

#ifndef __AWB_CTRL_API_H__
#define __AWB_CTRL_API_H__

#include "3ai/awbi_api.h"
#include "aaa_ctrl_common.h"
#include "cam_calibdb/cam_calibdb_api.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef struct AwbCtrlContext_t *AwbCtrlHandle_t;

#define AWB_CONFIG_CCM_NUM              9
#define AWB_CONFIG_CCM_OFFSET_NUM       3
#define AWB_CONFIG_FADE_NUM             16
#define AWB_CONFIG_GAINDIST_NUM         16    
#define AWB_CONFIG_FILTER_NUM           50
#define AWB_CONFIG_COMPONENTGAIN_NUM    4
#define AWB_CONFIG_PROFILE_NAME         20
#define AWB_CONFIG_ILLUPROFILE_NUM      10
#define AWB_CTRL_COLOR_TEM_WEIGHT_NUM   7
#define AWB_MAX_ILLUMINATION_PROFILES   32L
#define AWB_CTRL_WB_NUM                 4       


/*****************************************************************************/
/**
 *          AwbTemWeight_t
 *
 * @brief   Awb color temperature weight of every lighting
 *
 */
/*****************************************************************************/
typedef float32_t AwbTemWeight_t[AWB_CTRL_COLOR_TEM_WEIGHT_NUM];

/*****************************************************************************/
/**
 *          AwbCtrlState_t
 *
 * @brief   Config structure to create a new sensor instance
 *
 */
/*****************************************************************************/
typedef enum AwbCtrlState_e {
    AWB_CTRL_STATE_INVALID          = 0,
    AWB_CTRL_STATE_INITIALIZED      = 1,
    AWB_CTRL_STATE_STOPPED          = 2,
    AWB_CTRL_STATE_RUNNING          = 3,
    AWB_CTRL_STATE_RUNNING_SETTING  = 4,
    AWB_CTRL_STATE_LOCK             = 5,
    AWB_CTRL_STATE_MAX
} AwbCtrlState_t;


/*****************************************************************************/
/**
 *          AwbLibInstanceConfig_s
 *
 * @brief   Config structure to create a new Awb lib instance
 *
 */
/*****************************************************************************/
typedef struct AwbCtrInstanceConfig_s {
    uint8_t             ispId;
#ifndef ES_3A	
    AwbiLib_t           *pAwbiLib;           /**< Awb Lib struct interface */
#else
	V_ISP_AWB_EXP_FUNC_S *pAwbiLib;
#endif
    CamerIcDrvHandle_t  hCamerIc;
    IsiSensorHandle_t   hSensor;
    AwbCtrlHandle_t     hAwbCtrlHandle;      /**< Awb ctrl handle which create by AWB_ctrl_create */
    bool_t              awbImageMode;
} AwbCtrInstanceConfig_t;


/*****************************************************************************/
/**
 *          AwbCtrlMode_t
 *
 * @brief   I3AI Awb Mode set and get function's parameters
 *
 */
/*****************************************************************************/
typedef enum AwbCtrlMode_e {
    AWB_CTRL_INVALID          = 0,
    AWB_CTRL_MODE             = 1,
    AWB_CTRL_MODE_CC_OFFSET   = 2,
    AWB_CTRL_MODE_CC_MATRIX   = 3,
    AWB_CTRL_MODE_LSC         = 4,
    AWB_CTRL_MODE_USE_DAMPING = 5,
    AWB_CTRL_MODE_ILLUM_INDEX = 6,
} AwbCtrlMode_t;


/*****************************************************************************/
/**
 *          AwbCtrlImageModeConfig_t
 *
 * @brief   AWB config paramter which used in Image raw input mode
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlImageModeConfig_s {
    float32_t  aecCurIntTime;
    float32_t  aecCurGain;
} AwbCtrlImageModeConfig_t;


/*****************************************************************************/
/**
 *          AwbCtrlConfig_t
 *
 * @brief   AWB lib configuration which lib calculation need
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlConfig_s {
    bool_t                       useDamping;

    //performance optimization
    CtrlPerformanceOptiMode_t    performanceOptiMode;   

    //awb lock paramerters
    float32_t                    awbEnterLockThreshold;
    float32_t                    awbEnterUnlockThreshold;   
} AwbCtrlConfig_t;


/*****************************************************************************/
/**
 *          AwbCenterLine_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlCenterLine_s {
    float32_t rg;
    float32_t bg;
    float32_t d;
} AwbCtrlCenterLine_t;


/*****************************************************************************/
/**
 *          AwbCtCurve_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlCtCurve_s {
    float32_t ct_a;
    float32_t ct_b;
    float32_t ct_c;
} AwbCtrlCtCurve_t;


/*****************************************************************************/
/**
 *          AwbFadeParam_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlFadeParam_s {
    float32_t fade[AWB_CONFIG_FADE_NUM];
    float32_t gainDist[AWB_CONFIG_GAINDIST_NUM];
    int32_t   fadePoint;
} AwbCtrlFadeParam_t;


/*****************************************************************************/
/**
 *          AwbIlluProfile_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlIlluProfile_s {
    char      name[AWB_CONFIG_PROFILE_NAME];
    uint8_t   id;
    int32_t   doorType;
    int32_t   awbType;
    float32_t ccmCoeff[AWB_CONFIG_CCM_NUM];
    float32_t ccmOffset[AWB_CONFIG_CCM_OFFSET_NUM];
    float32_t componentGain[AWB_CONFIG_COMPONENTGAIN_NUM];
} AwbCtrlIlluProfile_t;


/*****************************************************************************/
/**
 *          AwbCtrlExpPrior_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlExpPrior_s{
    float32_t iirAdd;
    float32_t iirSub;
    float32_t iirThreshold;
    float32_t iirMin;
    float32_t iirMax;
    float32_t iirCoefInit;
    int32_t   filterItemNumber;
} AwbCtrlExpPrior_t;


/*****************************************************************************/
/**
 *          AwbCtrlCalibData_t
 *
 * @brief   AWB ctrol's sensor parameters which need to setting
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlCalibData_s {
    float32_t             rgProjIndoorMin;
    float32_t             rgProjMax;
    float32_t             kFactor;
    AwbCtrlCenterLine_t   centerLine;
    AwbCtrlCtCurve_t      ctcurve;
    AwbCtrlFadeParam_t    fade1, fade2;
    AwbCtrlIlluProfile_t  illuProfiles[AWB_CONFIG_ILLUPROFILE_NUM];
    AwbCtrlExpPrior_t     iir;
    int32_t               illuProfileNumber;
} AwbCtrlCalibData_t;


/*****************************************************************************/
/**
 *          AwbCtrlCalibConfig_t
 *
 * @brief   Calibration configuration parameters
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlCalibConfig_s {
    uint16_t      width;              /**< picture width */
    uint16_t      height;             /**< picture height */
} AwbCtrlCalibConfig_t;

/*****************************************************************************/
/**
 *          AwbCtrlMetaDataConfig_t
 *
 * @brief   Awb meta data value input data
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlMetaDataConfig_s {
    float32_t    metaDataCCt;
    float32_t    metaDataWbGain[AWB_CONFIG_COMPONENTGAIN_NUM];
} AwbCtrlMetaDataConfig_t;


/*****************************************************************************/
/**
 *          AwbCtrlMetaDataCalResult_t
 *
 * @brief   Awb meta data calculating value
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlMetaDataCalResult_s {
    float32_t    metaDataCcMatrix[AWB_CONFIG_CCM_NUM];
    float32_t    metaDataCcOffset[AWB_CONFIG_CCM_OFFSET_NUM];
    float32_t    metaDataColorTemWeight[AWB_CONFIG_ILLUPROFILE_NUM];
} AwbCtrlMetaDataCalResult_t;

/*****************************************************************************/
/**
 *          AwbCtrlResult_t
 *
 * @brief   awb calculation result set to customer
 *
 */
/*****************************************************************************/
typedef struct AwbCtrlResult_s {
    bool            awbTemWeightValid;
    bool            awbCctValid;
    bool            awbGainValid;
    float32_t       awbGain[AWB_CTRL_WB_NUM];
    float32_t       awbTemWeight[AWB_CTRL_COLOR_TEM_WEIGHT_NUM];
} AwbCtrlResult_t;


/*****************************************************************************/
/**
 *          AwbCtrlRegister
 *
 * @brief   register AWB lib and get the Awb lib's handle
 *
 * @param   pAwbCtrConfig      Ctrl AWB function's parameters
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AwbCtrlRegister
( 
    AwbCtrInstanceConfig_t  *pAwbCtrConfig
);

// #ifndef VENDOR_3A
// RESULT AwbCtrlInitEs
// (
//     AwbCtrlHandle_t handle,
//     AwbCtrInstanceConfig_t  *pAwbCtrConfig
// );

// RESULT AwbCtrlExitEs
// (
//     AwbCtrlHandle_t handle
// );
// #endif


/*****************************************************************************/
/**
 *          AwbCtrlUnRegister
 *
 * @brief   unregister Awb lib and release the AWB lib's handle
 *
 * @param   handle     Handle to the AWB lib intance
 * @param   ispId      Isp id 
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlUnRegister
( 
    AwbCtrlHandle_t  handle,
    uint8_t          ispId
);


/*****************************************************************************/
/**
 *          AwbCtrlSetRoi
 *
 * @brief   Set the AWB calculation's ROI area
 *
 * @param   handle         Handle to the AWB lib intance
 * @param   pAwbCtrlRoi    Pointer AWB regions of interest
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlSetRoi
( 
    AwbCtrlHandle_t     handle,
    const AaaCtrlRoi_t  *pAwbCtrlRoi 
);


/*****************************************************************************/
/**
 *          AwbCtrlGetRoi
 *
 * @brief   Get the AWB calculation's ROI area
 *
 * @param   handle         Handle to the AWB lib intance
 * @param   pAwbCtrlRoi    Pointer AWB regions of interest
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetRoi
( 
    AwbCtrlHandle_t  handle,
    AaaCtrlRoi_t     *pAwbCtrlRoi
);


/*****************************************************************************/
/**
 *          AwbCtrlSetMode
 *
 * @brief   Set AWB's working mode
 *
 * @param   handle         Handle to the AWB lib intance
 * @param   pAwbCtrlMode   The pointer to Working mode type
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlSetMode
( 
    AwbCtrlHandle_t      handle,
    const AwbCtrlMode_t  *pAwbCtrlMode
);


/*****************************************************************************/
/**
 *          AwbCtrlGetMode
 *
 * @brief   Get AWB's working mode
 *
 * @param   handle         Handle to the AWB lib intance
 * @param   pAwbCtrlMode   The pointer to Working mode type
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetMode
( 
    AwbCtrlHandle_t  handle,
    AwbCtrlMode_t    *pAwbCtrlMode
);


/*****************************************************************************/
/**
 *          AwbCtrlImageModeSetConfig
 *
 * @brief   Set Awb's image input mode configuration parameters
 *
 * @param   handle               Handle to the AWB lib intance
 * @param   pAwbImageModeConfig  The pointer to awb image mode config
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlImageModeSetConfig
( 
    AwbCtrlHandle_t           handle,
    AwbCtrlImageModeConfig_t  *pAwbImageModeConfig
);


/*****************************************************************************/
/**
 *          AwbCtrlSetConfig
 *
 * @brief   Set Awb's calibration or tunning parameters
 *
 * @param   handle           Handle to the AWB lib intance
 * @param   pAwbCtrlConfig   The pointer to awb config
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlSetConfig
( 
    AwbCtrlHandle_t        handle,
    const AwbCtrlConfig_t  *pAwbCtrlConfig
);


/*****************************************************************************/
/**
 *          AwbCtrlGetConfig
 *
 * @brief   Get Awb's calibration or tunning parameters
 *  
 * @param   handle              Handle to the AWB lib intance
 * @param   pAwbCtrlConfig      The config type to set
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetConfig
( 
    AwbCtrlHandle_t  handle,
    AwbCtrlConfig_t  *pAwbCtrlConfig
);


/*****************************************************************************/
/**
 *          RESULT AwbCtrlGetCalibData
 *
 * @brief   get the calibration data from calibrate data base handle
 *
 * @param   handle             Handle to the AWB lib intance
 * @param   hCamCalibDb        Calibration data handle
 * @param   pAwbCalibConfig    Calibration data configuration parmeters
 * @param   pAwbCtrlCalibData  Calibration data 
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetCalibData
(
    AwbCtrlHandle_t       handle,
    CamCalibDbHandle_t    hCamCalibDb,
    AwbCtrlCalibConfig_t  *pAwbCalibConfig,
    AwbCtrlCalibData_t    *pAwbCtrlCalibData
);


/*****************************************************************************/
/**
 *          RESULT AwbCtrlSetCalibData
 *
 * @brief   running AWB lib module to get the result
 *
 * @param   handle             Handle to the AWB lib intance
 * @param   pAwbCtrlCalibData  Calibration data 
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlSetCalibData
( 
    AwbCtrlHandle_t           handle,
    const AwbCtrlCalibData_t  *pAwbCtrlCalibData 
);


/*****************************************************************************/
/**
 *          AwbCtrlGetResult
 *
 * @brief   AWB get result
 *
 * @param   handle          Handle to the AWB lib intance
 * @param   pAwbResult      awb result 
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetResult
(
    AwbCtrlHandle_t  handle,
    AwbCtrlResult_t  *pAwbResult
);


/*****************************************************************************/
/**
 *          AwbCtrlGetStatus
 *
 * @brief   Get Awb's status
 *  
 * @param   handle           Handle to the AWB lib intance
 * @param   pAwbCtrlState    The awb ctrl's state
 * 
 * @return  Return the result of the unction call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetStatus
( 
    AwbCtrlHandle_t  handle,
    AwbCtrlState_t   *pAwbCtrlState
);


/*****************************************************************************/
/**
 *          AwbCtrlStart
 *
 * @brief   modify the Ctrl Awb's state to begin Awb module
 *
 * @param   handle           Handle to the AWB lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlStart
( 
    AwbCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          AwbCtrlStop
 *
 * @brief   modify the Ctrl AWB's state to stop AWB module
 *
 * @param   handle          Handle to the AWB lib intance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlStop
( 
    AwbCtrlHandle_t  handle
);


/*****************************************************************************/
/**
 *          AwbCtrlGetStatisticExpV2
 *
 * @brief   Getting expv2 statistic value from callback function
 *
 * @param   handle               Handle to the Awb lib intance
 * @param   statisticExpV2       The pointer to expv2 type's statistic value
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlGetStatisticExpV2
( 
    AwbCtrlHandle_t     handle,
    CamericAev2Static_t *statisticExpV2
);


/*****************************************************************************/
/**
 *          AwbCtrlMetaDataExcute
 *
 * @brief   AWB metadata excute 
 *
 * @param   handle                 Handle to the Awb lib intance
 * @param   pAwbCtrlMetaData       The pointer to awb metadata input value
 * @param   pAwbCtrlMetaDataResult The pointer to awb metadata result
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlMetaDataExcute
( 
    AwbCtrlHandle_t            handle,
    AwbCtrlMetaDataConfig_t    *pAwbCtrlMetaData,
    AwbCtrlMetaDataCalResult_t *pAwbCtrlMetaDataResult
);

/*****************************************************************************/
/**
 *          AwbCtrlExcute
 *
 * @brief   running AWB lib module to get the result
 *
 * @param   handle           Handle to the AWB lib intance
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 *
 *****************************************************************************/
RESULT AwbCtrlExcute
( 
    AwbCtrlHandle_t  handle
);

#ifdef __cplusplus
}
#endif

#endif
