/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @file binder_hal_ops.h
 *
 * @brief   binder Layer
 *
 *          Encapsulates and abstracts services from different hardware platforms.
 *
 *****************************************************************************/

#ifndef BINDER_HAL_OPS_H
#define BINDER_HAL_OPS_H

#include <ebase/types.h>
#include <ebase/dct_assert.h>
//#include <ebase/trace.h>

#include <common/return_codes.h>
#include <oslayer/oslayer.h>

#include <hal/hal_common.h>
#include <virtual_hal/virtual_hal_api.h>

/*****************************************************************************/
/**
 * @brief   work mode
 *
 *****************************************************************************/
typedef enum BinderIspWorkMode_e {
    BINDER_ISP_WORK_MODE_NORMAL_MODE = 0,    //normal hal mode
    BINDER_ISP_WORK_MODE_MCM_MODE,           //MCM mode.
    BINDER_ISP_WORK_MODE_BATCH_MODE,         //Batch mode.
    BINDER_ISP_WORK_MODE_TYPE_MAX
} BinderIspWorkMode_t;


/******************************************************************************/
/**
 * @brief   enum type for the switch sequence mode
 *
 *****************************************************************************/
typedef enum BinderSwitchSeqMode_e {
    BINDER_SWITCH_SEQ_MODE_INVALID              = 0,                /**< lower border (only for an internal evaluation) */
    BINDER_SWITCH_SEQ_BATCH_MODE                = 1,                /**< batch mode */
    BINDER_SWITCH_SEQ_PRIORITY                  = 2,                /**< priority */
    BINDER_SWITCH_SEQ_MODE_MAX                  ,                   /**< upper border (only for an internal evaluation) */
}BinderSwitchSeqMode_t;

/*****************************************************************************/
/**
 * @brief   Device priority
 *
 *****************************************************************************/
typedef enum BinderMcmPriority_e {
    BINDER_MCM_PRIORITY_INVALID = 0,
    BINDER_MCM_PRIORITY_0  = 1, /**<the lower number, the highest priority. */
    BINDER_MCM_PRIORITY_1  = 2,
    BINDER_MCM_PRIORITY_2  = 3,
    BINDER_MCM_PRIORITY_3  = 4,
    BINDER_MCM_PRIORITY_4  = 5,
    BINDER_MCM_PRIORITY_5  = 6, /**<the lower number, the highest priority. */
    BINDER_MCM_PRIORITY_6  = 7,
    BINDER_MCM_PRIORITY_7  = 8,
    BINDER_MCM_PRIORITY_8  = 9,
    BINDER_MCM_PRIORITY_9  = 10,
    BINDER_MCM_PRIORITY_10 = 11, /**<the lower number, the highest priority. */
    BINDER_MCM_PRIORITY_11 = 12,
    BINDER_MCM_PRIORITY_12 = 13,
    BINDER_MCM_PRIORITY_13 = 14,
    BINDER_MCM_PRIORITY_14 = 15,
    BINDER_MCM_PRIORITY_15 = 16,
    BINDER_MCM_PRIORITY_MAX
} BinderMcmPriority_t;

typedef enum BinderMcmOpt_e {
    BINDER_MCM_OPT_INVALID      = -1,
    BINDER_MCM_OPT_SW           = 0,
    BINDER_MCM_OPT_HW           = 1,
    BINDER_MCM_OPT_MAX
} BinderMcmOpt_t;

/*****************************************************************************/
/**
 * @brief   Select the input data entry.
 *
 *****************************************************************************/
typedef enum BinderMcmSelection_e {
    BINDER_MCM_SEL_INVALID     = -1,
    BINDER_MCM_SEL_RDMA        = 0,
    BINDER_MCM_SEL_RDCD        = 1,
    BINDER_MCM_SEL_MAX
} BinderMcmSelection_t;

/*****************************************************************************/
/**
 * @brief   work mode config struct
 *
 * 
 *****************************************************************************/
typedef struct BinderWorkModeCfg_e {
    BinderIspWorkMode_t workMode;
    BinderMcmPriority_t priority;
    BinderSwitchSeqMode_t seqMode;
    BinderMcmOpt_t      mcmOpt;
    BinderMcmSelection_t mcmSel;
} BinderWorkModeCfg_t;

/******************************************************************************
 * inline implementations of API
 *****************************************************************************/

typedef enum BindOpsType_e {
	BIND_HAL_OPS_GENERAL = 0,
	BIND_HAL_OPS_TILE_MODE,		// sw calculate and split the input large image to two right half and left half image by mcm reading.
	BIND_HAL_OPS_TILE_MODE_MP_CTRL, // hw mp_ctrl module will split the input large image to two right half and left half image.
	BIND_HAL_OPS_TYPE_MAX
} BindOpsType;

/******************************************************************************/
/**
 * @brief   joint the pices of image way
 *
 *****************************************************************************/
typedef enum BinderJointWay_e {
    BINDER_JOINT_MODE_INVALID              = 0,                /**< lower border (only for an internal evaluation) */
    BINDER_JOINT_2X1_PICES                 = 1,                /**< 2x1 */
    BINDER_JOINT_4X3_PICES                 = 2,                /**< 4x3 */
    BINDER_JOINT_USER_DEF                  = 3,                /**< user defines pices in x axis and y axis */
    BINDER_JOINT_MODE_MAX                  ,                   /**< upper border (only for an internal evaluation) */
}BinderJointMode_t;

typedef struct BinderTileModeCfg_s
{
    BindOpsType    binderOpsType;
    BinderJointMode_t  jointMode;
    uint8_t xPicesNum;    /**< x aixs image pices num*/
    uint8_t yPicesNum;    /**< y aixs image pices num*/
    HalHandle_t            memHalHandle; /**<memory hal handle*/
    void *                 hCascade; /**< Cascade ctx handle */
    uint8_t                batchFrameCount; /**< Frame count of one batch, batchFrameCount <= 32. */
} BinderTileCfg_t;
// #ifdef __cplusplus
// extern "C"
// {
// #endif

typedef struct HalIrqCtx_s BinderIrqCtx_t;

typedef struct BinderHalApiOps_s {
/*****************************************************************************/
/**
 * @brief   open the low level hal
 * @return  handle of low level context; NULL on failure
 *
 *****************************************************************************/
void *(*open)
(
    uint8_t ispId,
    const BinderTileCfg_t *pTileCfg,
    BinderWorkModeCfg_t *pModeCfg
);

/*****************************************************************************/
/**
 * @brief   close the low level hal
 * @param   Handle   Handle to session as returned by @ref Open.
 * @return  Result of operation.
 *
 *****************************************************************************/
RESULT (*close)
(
    void* Handle
);

/*****************************************************************************/
/**
 * @brief   writes a value to the given address
 * @param   Handle   Handle to binder session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeRegister)
(
    void* Handle,
    uint32_t addr,
    uint32_t val
);


/*****************************************************************************/
/**
 * @brief   reads a value from a given address
 * @param   Handle   Handle to Binder session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*readRegister)
(
    void* Handle,
    uint32_t addr
);

/*****************************************************************************/
/**
 * @brief   writes a value to the given address
 * @param   Handle   Handle to binder session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   value       Value to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeIdRegister)
(
    void* Handle,
    uint8_t ispId,
    uint32_t addr,
    uint32_t val
);

/*****************************************************************************/
/**
 * @brief   writes values to the given address
 * @param   Handle      Handle to binder session as returned by @ref Open.
 * @param   reg_address Address of register to write.
 * @param   pVal        Values to write into register.
 * @return  none
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
void (*writeBatchRegisters)
(
    void *Handle,
    uint32_t addr,
    uint32_t *pVal
);


/*****************************************************************************/
/**
 * @brief   reads a value from a given address
 * @param   Handle   Handle to Binder session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*readIdRegister)
(
    void* Handle,
    uint8_t ispId,
    uint32_t addr
);


/*****************************************************************************/
/**
 * @brief   reads a value from a given address for SRAM
 * @param   Handle  Handle to Binder session as returned by @ref Open.
 * @param   ispId   ISP device ID.
 * @param   addr    Address of register to read.
 * @return  Register value.
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
uint32_t (*readIdTbl)
(
    void* Handle,
    uint8_t ispId,
    uint32_t addr
);


/*****************************************************************************/
/**
 * @brief   
 * @param   Handle   Handle to Binder session as returned by @ref Open.
 * @param   reg_address Address of register to read.
 * @return  result   RET_NULL_POINTER RET_SUCCESS
 *
 * @note    It is required to pass in the full address (base address + offset).
 *****************************************************************************/
RESULT (*ioctl)
(
    void* Handle,
    uint32_t cmd, 
    void *args
);
/*****************************************************************************/
/**
 *          connectIrq()
 *
 *  @brief  Register interrupt service routine with system software.
 *
 *  @param  Handle to binder session as returned by @ref Open
 *  @param  Reference of Binder irq context structure that represent this connection
 *  @param  Interrupt object @ref osInterrupt
 *  @param  Number of the interrupt source, set to 0 if not needed
 *  @param  First interrupt routine, (first level handler) set to NULL if not needed
 *  @param  Second interrupt routine (second level handler)
 *  @param  Context provided when the interrupt routines are called
 *
 *  @warning Add platform specific code to connect to your local interrupt source
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         ISR registered successfully
 *  @retval RET_FAILURE et al.  ISR not registered
 *
 *****************************************************************************/
RESULT (*connectIrq)
(
    void* Handle,
    BinderIrqCtx_t *pIrqCtx,
    uint32_t    int_src,
    osIsrFunc   IsrFunction,
    osDpcFunc   DpcFunction,
    void*       pContext
);

/*****************************************************************************/
/**
 *          disconnectIrq()
 *
 *  @brief  Deregister interrupt service routine from system software.
 *  @param  Handle to binder session as returned by @ref Open
 *  @param  Reference of binder irq context structure that represent this connection
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         ISR deregistered successfully
 *  @retval RET_FAILURE et al.  ISR not properly deregistered
 *
 *****************************************************************************/
RESULT (*disconnectIrq)
(
    void* Handle,
    BinderIrqCtx_t *pIrqCtx
);

/*****************************************************************************/
/**
 *          getHalHandler()
 *
 *  @brief get hal level handler.
 *  @param  Handle to binder session as returned by @ref Open
 *
 *  @return      hanlHandler. NULL on failure
 *
 *****************************************************************************/
HalHandle_t (*getHalHandler)
(
    void* Handle
);


/*****************************************************************************/
/**
 *          getVirtualHalHandler()
 *
 *  @brief get virtual hal level handler.
 *  @param  Handle to binder session as returned by @ref Open
 *
 *  @return      hanlHandler. NULL on failure
 *
 *****************************************************************************/
VirtualHandle_t (*getVirtualHalHandler)
(
    void* Handle
);
/*****************************************************************************/
/**
 * @brief   tell HAL about another user of low level driver
 *  @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note
 *****************************************************************************/
RESULT (*addRef)(
    void* Handle
);
/*****************************************************************************/
/**
 * @brief   tell HAL about gone user of low level driver
 * @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*delRef)(
    void* Handle
);

/*****************************************************************************/
/**
 *          *eventGenerator()
 *
 *  @brief  Cmodel specific interface: Hal ISR and Event generator
 *
 *  @param
 *
 *  @return                     Status of operation
 *  @retval RET_SUCCESS         Input set successfully
 *  @retval RET_FAILURE et al.  Input set fails
 *
 *****************************************************************************/
RESULT (*eventGenerator)
(
    void* Handle,
    HalIrqCtx_t *pIrqCtx,
    HalIsrSrc_t isr_src,
    uint32_t misValue
);

/*****************************************************************************/
/**
 * @brief  get the isp work count
 * @param  Handle to binder session as returned by @ref Open
 * @param  pIspCnt  pointer to uint32_t variable.
 * @return  Result of operation.
 *****************************************************************************/
RESULT (*getIspCnt)(
    void*     Handle,
    uint8_t* pIspCnt
);


/*****************************************************************************/
/**
 * @brief  get the isp 3dnr ref frame buffer size
 * @param  Handle to binder session as returned by @ref Open
 * @param  pRefBufSize to refSize variable
 * @return  Result of operation.
 *****************************************************************************/
RESULT (*getRefBufSize)(
    void*      Handle,
    uint32_t   *pRefBufSize
);
/*****************************************************************************/
/**
 * @brief   set the ic dev pointer member.It is used for cmodel.
 * @param  Handle to binder session as returned by @ref Open
 * @param  pIcdev  pointer to ic_dev object.
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*setIcdev)(
    void* Handle,
    void* pIcdev
);
#ifdef HAL_CMODEL
/*****************************************************************************/
/**
 * @brief   cmodel Buffer transfer handler
 * @param  Handle to binder session as returned by @ref Open
 * @param  trans_ctrl contrl value.
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*bufTransfer)(
    void* Handle,
     uint32_t trans_ctrl
);

/*****************************************************************************/
/**
 * @brief   Cmodel Shd register update process
 * @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*shdRegisterUpdate)(
    void* Handle,
    uint8_t path
);

/*****************************************************************************/
/**
 * @brief   cfg dma buffer addr ,format and size
 * @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*cfgDma)(
    void* Handle,
    HalCmodelBufConfig_t *pPicBuffer
);

/*****************************************************************************/
/**
 * @brief   cfg mi buffer addr ,format and size
 * @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*cfgMi)(
    void* Handle,
     uint32_t path,
    HalCmodelBufConfig_t *pPicBuffer
);

/*****************************************************************************/
/**
 * @brief   run cmodel piplien to process
 * @param  Handle to binder session as returned by @ref Open
 * @return  Result of operation.
 *
 * @note    If the internal ref count is zero, the HAL will be closed as well.
 *****************************************************************************/
RESULT (*ispPipeline)(
    void* Handle
);
#endif
} BinderHalApiOps_t;


// #ifdef __cplusplus
// extern "C"
// }
// #endif
#endif
