/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CAMDEV_AE_API_H
#define CAMDEV_AE_API_H

#include "cam_device_common.h"
#include <stdint.h>

/**
 * @cond AE_V3
 * 
 * @defgroup cam_device_ae_v3 CamDevice AE V3 Definitions
 * @{
 *
 */

#define CAMDEV_AE_HIST_NUM_BINS           256       /**< Bin numbers of AE histogram */
#define CAMDEV_AE_EXP_GRID_ITEMS          (32*32)   /**< Number of grid items (see @ref CamerIcMeanLuma_t) */


#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief    CamDevice AE status.
 *
 *****************************************************************************/
typedef enum CamDeviceAeState_e {
    CAMDEV_AE_STATE_INVALID       = 0,    /**< Invalid AE status */
    CAMDEV_AE_STATE_INITIALIZED   = 1,    /**< AE initialized */
    CAMDEV_AE_STATE_STOPPED       = 2,    /**< AE stopped */
    CAMDEV_AE_STATE_RUNNING       = 3,    /**< AE running */
    CAMDEV_AE_STATE_LOCKED        = 4,    /**< AE locked */
    CAMDEV_AE_STATE_MAX
} CamDeviceAeState_t;


/*****************************************************************************/
/**
 * @brief   CamDevice AE scene evaluation mode parameters.
 *
 *****************************************************************************/
typedef enum CamDeviceAeSemMode_t
{
    CAMDEV_AE_SCENE_EVALUATION_FIX        = 0,    /**< Fixed evaluation */
    CAMDEV_AE_SCENE_EVALUATION_ADAPTIVE   = 1,    /**< Adaptive evaluation */
    CAMDEV_AE_SCENE_METADATA              = 2,    /**< metadata mode */
    CAMDEV_AE_SCENE_EVALUATION_MAX
} CamDeviceAeSemMode_t;

/*****************************************************************************/
/**
 * @brief    CamDevice AE Flicker period.
 *
 *****************************************************************************/
typedef enum CamDeviceAeFlickerPeriod_e {
    CAMDEV_AE_FLICKER_PERIOD_OFF          = 0,    /**< Flicker period off */
    CAMDEV_AE_FLICKER_PERIOD_50Hz         = 1,    /**< Flicker period 50Hz */
    CAMDEV_AE_FLICKER_PERIOD_60Hz         = 2,    /**< Flicker period 60Hz */
    CAMDEV_AE_FLICKER_PERIOD_USER_DEFINED = 3,  /**< User-defined Flicker period */
    CAMDEV_AE_FLICKER_PERIOD_MAX
} CamDeviceAeFlickerPeriod_t;

/*****************************************************************************/
/**
 * @brief   CamDevice AE Flicker Mode parameters.
 *
 *****************************************************************************/
typedef struct CamDeviceAeFlickerMode_s{
    CamDeviceAeFlickerPeriod_t flickerPeriod;    /**< Flicker period instance */
    float32_t userDefinedPeriodus; /**< User defined period which is only valid in user define mode. */
}CamDeviceAeFlickerMode_t;


/******************************************************************************/
/**
 * @brief   CamDevice AE configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceAeConfig_s {
    float32_t setPoint;             /**< Expected mean luma. */
    float32_t tolerance;            /**< Tolerance of stable range. */
    float32_t dampOver;             /**< Clip edge to start interpolate K. */
    float32_t dampUnder;            /**< Clip edge to start interpolate K. */

    float32_t dampUnderRatio;               /**< Clip edge to start under interpolate K. */
    float32_t dampUnderGain;                /**< Under interpolate gain of K, damp = damp^K. */
    float32_t dampOverRatio;                /**< Clip edge to start over interpolate K. */
    float32_t dampOverGain;                 /**< Over interpolate gain of K, damp = damp^K. */

    float32_t motionFilter;                 /**< Temporal average filter, resulting in a smooth exposure change. */
    float32_t motionThreshold;              /**< Motion filter threshold . */
    float32_t targetFilter;                 /**< Temporal average filter, resulting in a smoothadaptive AE target change. */

    float32_t lowlightLinearRepress[8];     /**< Repress the sensor exposure in lowlight scene and interpolate by sensor gain;
                                             Interpolate as Y axis. */
    float32_t lowlightLinearGain[8];        /**< Gain array; Interpolate as X axis. */
    uint32_t lowlightLinearLevel;       /**< Gain array number. */
    float32_t lowlightHdrRepress[8];        /**< Repress the sensor exposure in a lowlight scene and interpolate by sensor gain;
                                             Interpolate as Y axis. */
    float32_t lowlightHdrGain[8];           /**< Gain array; Interpolate as X axis. */
    uint32_t lowlightHdrLevel;          /**< Gain array number. */

    float32_t wdrContrastMin;               /**< Minimumdiff of (background – foreground), map to [0, 1] as backlight ratio. */
    float32_t wdrContrastMax;               /**< Maximumdiff of (background – foreground), map to [0, 1] as backlight ratio. */
    bool_t    frameCalEnable;           /** Frame calculating mode selection */
    bool_t    autoHdrEnable;            /** Hdr auto adjust mode enable */
    float32_t expv2WindowWeight[CAMDEV_AE_EXP_GRID_ITEMS]; /** The weight of EXPV2 every block */
} CamDeviceAeConfig_t;


/*****************************************************************************/
/**
 * @brief   CamDevice AE mode parameters.
 *
 *****************************************************************************/
typedef struct CamDeviceAeMode_s{
    CamDeviceAeSemMode_t     semMode;    /**< AE scene evaluation mode */
    CamDeviceAeFlickerMode_t antiFlickerMode;    /**< AE flicker mode */
} CamDeviceAeMode_t;


/*****************************************************************************/
/**
 * @brief    CamDevice AE histogram grid.
 *
 *****************************************************************************/
typedef uint32_t CamDeviceAeHistBins_t[CAMDEV_AE_HIST_NUM_BINS];

/*****************************************************************************/
/**
 * @brief    CamDevice AE luminance grid.
 *
 *****************************************************************************/
typedef uint8_t CamDeviceAeMeanLuma_t[CAMDEV_AE_EXP_GRID_ITEMS];

/*****************************************************************************/
/**
 * @brief    CamDevice AE object region grid.
 *
 *****************************************************************************/
typedef uint8_t CamDeviceAeObjectRegion_t[CAMDEV_AE_EXP_GRID_ITEMS];


/*****************************************************************************/
/**
 * @brief   This function registers the AE library.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pAeLibHandle        Pointer to the AE library
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceRegisterAeLib
(
    CamDeviceHandle_t   hCamDevice,
//  #ifndef VENDOR_3A
//     int                 ispID,
// #endif   
    void                *pAeLibHandle
);

/*****************************************************************************/
/**
 * @brief   This function unregisters the AE library.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceUnRegisterAeLib
(

	CamDeviceHandle_t hCamDevice
);

// #ifndef VENDOR_3A
// RESULT VsiCamDeviceAeInitEs
// (
//     CamDeviceHandle_t hCamDevice
// );

// RESULT VsiCamDeviceAeExitEs
// (
//     CamDeviceHandle_t hCamDevice
// );
// #endif


/*****************************************************************************/
/**
 * @brief   This function sets AE configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to the AE configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeSetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDeviceAeConfig_t       *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets AE configuration parameters.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to the AE configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetConfig
(
    CamDeviceHandle_t         hCamDevice,
    CamDeviceAeConfig_t      *pConfig
);

/*****************************************************************************/
/**
 *
 * @brief   This function sets AE mode parameters.
 *
 * @param   hCamDevice            Handle to the CamDevice instance
 * @param   pAeMode               Pointer  to AE mode
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeSetMode
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeMode_t       *pAeMode
);

/*****************************************************************************/
/**
 *
 * @brief   This function gets AE mode parameters.
 *
 * @param   hCamDevice            Handle to the CamDevice instance
 * @param   pAeMode               Pointer  to AE mode
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetMode
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeMode_t      *pAeMode
);

/*****************************************************************************/
/**
 * @brief   This function sets AE ROI.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRoi                Pointer to the AE ROI configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeSetRoi
(
    CamDeviceHandle_t      hCamDevice,
    CamDeviceRoi_t         *pRoi
);

/*****************************************************************************/
/**
 * @brief   This function gets AE ROI.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pRoi                Pointer to the AE ROI configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetRoi
(
    CamDeviceHandle_t      hCamDevice,
    CamDeviceRoi_t         *pRoi
);

/*****************************************************************************/
/**
 * @brief   This function enables AE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_INVALID_PARM    Invalid configuration
 * @retval  RET_OUTOFRANGE      A configuration parameter is out of range
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeEnable
(
    CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function disables AE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeDisable
(
    CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function resets AE.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeReset
(
    CamDeviceHandle_t hCamDevice
);


/*****************************************************************************/
/**
 * @brief   This function gets the AE status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 * @param   pAeStatus           Pointer to the AE status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetStatus
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeState_t      *pAeStatus
);


/*****************************************************************************/
/**
 * @brief   This function gets the current AE histogram.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 * @param   pHistogram          Pointer to the histogram bins
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetHistogram
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeHistBins_t   *pHistogram
);

/*****************************************************************************/
/**
 * @brief   This function returns the current AE luminance grid.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 * @param   pLuma               Pointer to the luminance grid
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetLuminance
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeMeanLuma_t   *pLuma
);

/*****************************************************************************/
/**
 * @brief   This function gets the current AE control object region.
 *
 * @param   hCamDevice          Handle to the CamDevice instance.
 * @param   pObjectRegion       Pointer to the object region
 *
 * @retval  RET_SUCCESS         Operation succeeded
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetObjectRegion
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceAeObjectRegion_t  *pObjectRegion
);

/*****************************************************************************/
/**
 * @brief   This function gets AE version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to the AE version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceAeGetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);


/* @} cam_device_ae_v3 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif   // CAMDEV_AE_API_H
