/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond HIST256
 *
 * @defgroup cam_device_hist256 CamDevice HIST256 Definitions
 * @{
 *
 */

#ifndef CAMDEV_HIST256_API_H
#define CAMDEV_HIST256_API_H

#include "cam_device_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMDEV_HIST256_GRID_ITEMS         25  /**< The number of grid sub windows */
#define CAMDEV_HIST256_NUM_BINS           256  /**< number of bins */


/******************************************************************************/
/**
 * @brief   CamDevice HIST256 mode.
 *
 *****************************************************************************/
typedef enum CamDeviceHist256Mode_e {
    CAMDEV_HIST256_MODE_DISABLE      = 0,    /**< Disabled, no measurements  */
    CAMDEV_HIST256_MODE_RGB_COMBINED = 1,    /**< RGB combined histogram */
    CAMDEV_HIST256_MODE_R            = 2,    /**< R histogram */
    CAMDEV_HIST256_MODE_G            = 3,    /**< G histogram */
    CAMDEV_HIST256_MODE_B            = 4,    /**< B histogram */
    CAMDEV_HIST256_MODE_Y            = 5,    /**< Luminance histogram */
    CAMDEV_HIST64_MODE_MAX,
} CamDeviceHist256Mode_t;

/******************************************************************************/
/**
 * @brief   CamDevice HIST256 weight configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceHIST256Weight_s
{
    uint8_t weight[CAMDEV_HIST256_GRID_ITEMS];   /**< HIST256 weights */
} CamDeviceHist256Weight_t;

/******************************************************************************/
/**
 * @brief   CamDevice HIST256 bins.
 *
 *****************************************************************************/
typedef struct CamDeviceHist256Bins_s
{
    uint32_t bins[CAMDEV_HIST256_NUM_BINS];   /**< HIST256 bins */
} CamDeviceHist256Bins_t;

/******************************************************************************/
/**
 * @brief   CamDevice HIST256 configuration.
 *
 *****************************************************************************/
typedef struct CamDeviceHIST256Config_s
{
    CamDeviceHist256Mode_t    mode;    /**< HIST256 mode */
    CamDeviceHist256Weight_t  weight;    /**< HIST256 weight */
    CamDeviceWindow_t        window;    /**< HIST256 window */
} CamDeviceHist256Config_t;

/******************************************************************************/
/**
 * @brief   CamDevice HIST256 status structure.
 *
 *****************************************************************************/
typedef struct CamDeviceHist256Status_s
{
    bool_t  enable;    /**< HIST256 enable status*/
}CamDeviceHist256Status_t;


/*****************************************************************************/
/**
 * @brief   This function sets HIST256 configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to HIST256 configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256SetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceHist256Config_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function gets HIST256 configuration.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pConfig             Pointer to HIST256 configuration
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256GetConfig
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceHist256Config_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function enables HIST256.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256Enable
(
    CamDeviceHandle_t   hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function disables HIST256.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256Disable
(
    CamDeviceHandle_t   hCamDevice
);

/*****************************************************************************/
/**
 * @brief   This function gets HIST256 statistical data.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pHistBin            Pointer to HIST256 bins
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256GetStatistic
(
    CamDeviceHandle_t        hCamDevice,
    CamDeviceHist256Bins_t   *pHistBin
);

/*****************************************************************************/
/**
 * @brief   This function gets HIST256 status.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pStatus             Pointer to HIST256 status
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256GetStatus
(
    CamDeviceHandle_t       hCamDevice,
    CamDeviceHist256Status_t    *pStatus
);

/*****************************************************************************/
/**
 * @brief   This function gets HIST256 version.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 * @param   pVersion            Pointer to HIST256 version
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256GetVersion
(
    CamDeviceHandle_t hCamDevice,
    uint32_t *pVersion
);

/*****************************************************************************/
/**
 * @brief   This function resets HIST256.
 *
 * @param   hCamDevice          Handle to the CamDevice instance
 *
 * @retval  RET_SUCCESS         Operation succeeded
 *
 *****************************************************************************/
RESULT VsiCamDeviceHist256Reset
(
    CamDeviceHandle_t         hCamDevice
);

/* @} cam_device_hist256 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /* CAMDEV_HIST256_API_H */

