/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @cond AF_V3
 *
 * @defgroup cam_engine_af_api CamEngine AF V3 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_AF_API_H
#define CAM_ENGINE_AF_API_H

#include <common/return_codes.h>
#include <3ai/afi_lib_interface.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_AFM_WINDOWNUM    3
#define CAM_AF_FILTERNUM     5
#define CAM_PD_FOCAL_NUM_MAX 48

/*****************************************************************************/
/**
 *
 *
 * @brief   AF states.
 *
 */
/*****************************************************************************/
typedef enum CamEngineAfState_e {
    CAMENGINE_AF_STATE_INVALID       = 0,  /**< Invalid AF status */
    CAMENGINE_AF_STATE_INITIALIZED   = 1,  /**< AF initialized */
    CAMENGINE_AF_STATE_STOPPED       = 2,  /**< AF stopped */
    CAMENGINE_AF_STATE_RUNNING       = 3,  /**< AF running */
    CAMENGINE_AF_STATE_TRACKING      = 4,  /**< AF tracking */
    CAMENGINE_AF_STATE_LOCKED        = 5,  /**< AF locked */
    CAMENGINE_AF_STATE_MAX
} CamEngineAfState_t;

/*****************************************************************************/
/**
 * *
 * @brief   AF modes.
 *
 */
/*****************************************************************************/
typedef enum CamEngineAfMode_e {
    CAMENGINE_AF_MODE_NORMAL  = 0,   /**< AF mode normal */
    CAMENGINE_AF_MODE_MAX
} CamEngineAfMode_t;

/*****************************************************************************/
/**
 *
 *
 * @brief   Configurations of AF lib calculation.
 *
 */
/*****************************************************************************/
typedef struct CamEngineAfConfig_s {
    // CDAF params 
    float    weightWindow[CAM_AFM_WINDOWNUM];  /**< Weight window */
    float    cMotionThreshold;  /**< Range: [0, 1] */
    float    focalStableThreshold;    /**< Stable threshold */
    float    cStableTolerance;      /**< Range: [0, 1] */
    uint8_t  cPointsOfCurve;        /**< Range: [5, 100] */
    float    focalFilter[CAM_AF_FILTERNUM]; /**< Focal filter */
    float    shapFilter[CAM_AF_FILTERNUM]; /**< Shape filter */
	uint16_t maxFocal; /**< Maximum focal point */
    uint16_t minFocal; /**< Minimum focal point*/
    
	// PDAF params
	float    cMseTolerance;  /**< MSE tolerance  */
	float    cPdConfThreshold;  /**< PDAF configuration threshold */
    int      pdFocal[CAM_PD_FOCAL_NUM_MAX]; /**< Phase Detection focal spot */
    int      pdDistance; /**< Phase Detection distance */
    float    pdShiftThreshold; /**< Phase Detection shift stable threshold */
    uint8_t  pdStableCountMax; /**< Maximum focusing to which PDAF locks */
} CamEngineAfConfig_t; 

/*****************************************************************************/
/**
 *
 *
 * @brief   Registers a handle of the AF module.
 *
 * @param   hCamEngine     A handle to the CamEngine module
 * @param   pAfiLib        The structure of the pointer to the AF lib function
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfRegister
(
    CamEngineHandle_t hCamEngine,
    AfiLib_t          *pAfiLib
);

/*****************************************************************************/
/**
 *
 * @brief   Unregisters a handle of the AF module.
 *
 * @param   hCamEngine     A handle to the CamEngine module
 * @param   ispId          ISP ID
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfUnRegister
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the AF calculation mode.
 *
 * @param   hCamEngine      A handle to the CamEngine module
 * @param   pCamAfMode      A pointer to AF calculation mode
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfSetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineAfMode_t *pCamAfMode
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the AF calculation mode.
 *
 * @param   hCamEngine         A handle to the CamEngine module
 * @param   pCamAfMode         A pointer to AF calculation mode
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfGetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineAfMode_t *pCamAfMode
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the configurations of AF calculation.
 *
 * @param   hCamEngine       A handle to the CamEngine module
 * @param   pCamAfConfig     A pointer to AF calculation configurations

 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfSetConfig
(
    CamEngineHandle_t   hCamEngine,
    CamEngineAfConfig_t *pCamAfConfig
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the configurations of AF calculation.
 *
 * @param   hCamEngine       A handle to the CamEngine module
 * @param   pCamAfConfig     A pointer to AF calculation configurations
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfGetConfig
(
    CamEngineHandle_t   hCamEngine,
    CamEngineAfConfig_t *pCamAfConfig
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Sets the index of PDAF ROI. \n You can select from 25 ROIs.
 *
 * @param   hCamEngine      A handle to the CamEngine module
 * @param   roiIndex        The ROI index
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfpdafSetRoiIndex
(
    CamEngineHandle_t hCamEngine,
    uint8_t           *roiIndex
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the index of PDAF ROI. \n You can select from 25 ROIs.
 *
 * @param   hCamEngine      A handle to the CamEngine module
 * @param   roiIndex        The ROI index
 * 
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfpdafGetRoiIndex
(
    CamEngineHandle_t hCamEngine,
    uint8_t           *roiIndex
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the data of AF calculation.
 *
 * @param   hCamEngine    A handle to the CamEngine module
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfStart
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Stop CamEngine AF calculation.
 *
 * @param   hCamEngine    A handle to the CamEngine module
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfStop
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 *
 *
 * @brief   Gets the status of AF calculation.
 *
 * @param   hCamEngine     A handle to the CamEngine module
 * @param   pCamAfState    A pointer to the AF calculation status
 *
 * @return  The result of the function call.
 * @retval  ret = 0 successful
 * @retval  ret = others failed
 *
 *****************************************************************************/
RESULT CamEngineAfGetStatus
(
    CamEngineHandle_t  hCamEngine,
    CamEngineAfState_t *pCamAfState
);

/* @} cam_engine_af_v3 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif

