/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    BLS
 * 
 * @file    cam_engine_bls_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine BLS.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_bls_api CamEngine BLS Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_BLS_API_H
#define CAM_ENGINE_BLS_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_RAW_CHANNEL_NUM 4

#define CAM_ENGINE_BLS_MIN 0

#define CAM_ENGINE_HDR_BLS_MAX 4095
#define CAM_ENGINE_HDR_BLS_MIN 0

#define CAM_ENGINE_BLS_IN_BIT_MAX 24
#define CAM_ENGINE_BLS_IN_BIT_MIN 8

/******************************************************************************/
/**
 * @brief   BLS manual configuration.
 *
 *****************************************************************************/
typedef struct CamEngineBlsConfig_s
{
    uint32_t bls[CAM_ENGINE_RAW_CHANNEL_NUM];  /**< BLS value:
                                                 Raw   : BLS[0]--red,BLS[1]--greenBlue,BLS[2]--greenRed,BLS[3]--blue
                                                 Rgbir : */
} CamEngineBlsConfig_t;

/******************************************************************************/
/**
 * @brief   BLS out configurations.
 *
 *****************************************************************************/
typedef struct CamEngineBlsOutConfig_s
{
    uint32_t bls[CAM_ENGINE_RAW_CHANNEL_NUM];
} CamEngineBlsOutConfig_t;

/******************************************************************************/
/**
 * @brief   BLS auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineAutoBlsConfig_s {
    uint8_t autoLevel;                              /**< The auto configuration level */
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< BLS gains */
    uint32_t bls[CAM_ENGINE_ISO_STRENGTH_NUM][CAM_ENGINE_RAW_CHANNEL_NUM];  /**< The BLS value */
} CamEngineAutoBlsConfig_t;

/*****************************************************************************/
/**
 * @brief   BLS mode.
 *
 *****************************************************************************/
typedef enum CamEngineBlsMode_e
{
    CAM_ENGINE_BLS_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_BLS_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_BLS_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_BLS_MODE_MAX
} CamEngineBlsMode_t;


/*****************************************************************************/
/**
 * @brief   Sets the BLS mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input BLS mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineBlsSetMode
(
    CamEngineHandle_t hCamEngine,
    const CamEngineBlsMode_t  mode           /**< BLS running mode */
);

/*****************************************************************************/
/**
 * @brief   Gets the BLS mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained BLS mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineBlsGetMode
(
    CamEngineHandle_t    hCamEngine,
    CamEngineBlsMode_t*  mode           /**< BLS running mode */
);

/*****************************************************************************/
/**
 * @brief   Configures the BLS.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The BLS configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineBlsSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineBlsConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the BLS status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The BLS configuration
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineBlsGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineBlsConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   Configures the BLS out.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The BLS out configuration
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineBlsOutSetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineBlsOutConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the BLS out status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The BLS out configuration
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineBlsOutGetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineBlsOutConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   Sets the BLS auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            The BLS auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineBlsSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineAutoBlsConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the BLS auto configurations.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            The BLS auto configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineBlsGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineAutoBlsConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the BLS status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pMode               The running mode
 * @param   pCfg                The BLS configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineBlsGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    CamEngineBlsMode_t* pMode,
    CamEngineBlsConfig_t* pCfg
);


#ifdef __cplusplus
}
#endif


/* @} cam_engine_bls_api */


#endif /* CAM_ENGINE_BLS_API_H */

