/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    CCM
 *
 * @file    cam_engine_ccm_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine CCM processing.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_ccm_api CamEngine CCMv11 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_CCM_API_H
#define CAM_ENGINE_CCM_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define ISP_CCM_MASK 0x1000
#define CAM_ENGINE_CCM_MATRIX_NUM 9
#define CAM_ENGINE_CCM_OFFSET_NUM 3

/******************************************************************************/
/**
 * @brief   CCM manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineCcmConfig_s {
    float32_t ccmMatrix[CAM_ENGINE_CCM_MATRIX_NUM];   /**< Color correction matrix coefficient*/
    float32_t ccmOffset[CAM_ENGINE_CCM_OFFSET_NUM];   /**< Color offset coefficient*/
} CamEngineCcmConfig_t;


/*****************************************************************************/
/**
 * @brief   CCM mode.
 *
 *****************************************************************************/
typedef enum CamEngineCcmMode_e
{
    CAM_ENGINE_CCM_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_CCM_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_CCM_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_CCM_MODE_MAX
} CamEngineCcmMode_t;


/*****************************************************************************/
/**
 * @brief   Enables the CCM.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the CCM. \n This functions requires default configurations by algorithm.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmDisable
(
    CamEngineHandle_t       hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Configures the CCM.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The CCM configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmSetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineCcmConfig_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the CCM status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The CCM configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineCcmConfig_t    *pConfig
);


/*****************************************************************************/
/**
 * @brief   set the CCM mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The CCM mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmSetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineCcmMode_t *mode        
);


/*****************************************************************************/
/**
 * @brief   Returns the CCM mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The CCM mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCcmGetMode
(
    CamEngineHandle_t  hCamEngine,
    CamEngineCcmMode_t *mode      
);


/*****************************************************************************/
/**
 * @brief   Gets the CCM status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pMode               the ccm mode
 * @param   pCfg                The CCM configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCcmGetStatus
(
    CamEngineHandle_t     hCamEngine,
    bool_t                *pRunning,
    CamEngineCcmMode_t    *pMode,
    CamEngineCcmConfig_t  *pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_ccm_api */


#endif /* CAM_ENGINE_CCM_API_H */

