/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    CPD
 * 
 * @file    cam_engine_cmpd_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine cmpd prcessing.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_cmpd_api CamEngine CMPD Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_CMPD_API_H
#define CAM_ENGINE_CMPD_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_CPD_CURVE_X_SIZE 64U  // bin number of curve lut x
#define CAM_ENGINE_CPD_CURVE_Y_SIZE 64U  // bin number of curve lut y

#define CAM_ENGINE_CPD_CURVE_Y_MAX ((1U << 20U) - 1U)
#define CAM_ENGINE_CPD_CURVE_Y_MIN 0U

#define CAM_ENGINE_CPD_CURVE_PX_MAX 0x1FU
#define CAM_ENGINE_CPD_CURVE_PX_MIN 0U

/******************************************************************************/
/**
 * @brief   Expanded curve configurations.
 *
 *****************************************************************************/
typedef struct CamEngineCmpdExpandConfig_s
{
    uint8_t expandCurvePx[CAM_ENGINE_CPD_CURVE_X_SIZE];  /**< The X-coordinates of an expanded curve */
    uint32_t expandCurveY[CAM_ENGINE_CPD_CURVE_Y_SIZE];  /**< The Y-coordinates of an expanded curve */
    bool_t expandUseOutYCurve;                           /**< Whether to use the default expanded curve. \n 0: use. \n 1: not use. */
} CamEngineCmpdExpandConfig_t;

/*****************************************************************************/
/**
 * @brief   Enables the CMPD expand.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCmpdExpandEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables the CMPD expand.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCmpdExpandDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the CMPD expand.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Configurations of CMPD expand
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCmpdExpandSetConfig
(
    CamEngineHandle_t               hCamEngine,
    CamEngineCmpdExpandConfig_t     *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the CMPD expand status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Configurations of CMPD expand
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineCmpdExpandGetConfig
(
    CamEngineHandle_t            hCamEngine,
    CamEngineCmpdExpandConfig_t  *pConfig
);


/*****************************************************************************/
/**
 * @brief   Gets the status of CPD compress
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            The running status
 * @param   pCfg                CPD compress configurations

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineCpdExpandGetStatus
(
    CamEngineHandle_t               hCamEngine,
    bool_t                          *pRunning,
    CamEngineCmpdExpandConfig_t     *pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_cmpd_api */


#endif /* CAM_ENGINE_CMPD_API_H */

