/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    DMSC_V2
 * 
 * @file    cam_engine_demosaic2_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine demosaic2.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_demosaic2_api CamEngine DMSC V2 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_DEMOSAIC2_API_H
#define CAM_ENGINE_DEMOSAIC2_API_H

#include <ebase/types.h>
#include <common/return_codes.h>

#include <cameric_drv/cameric_isp_dmsc2_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief   Cam Engine DMSC mode configuration structure.
 *
 *****************************************************************************/
typedef enum CamEngineDmscv2Mode_e
{
    CAM_ENGINE_DMSCV2_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_DMSCV2_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_DMSCV2_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_DMSCV2_MODE_MAX
} CamEngineDmscv2Mode_t;

/******************************************************************************/
/**
 * @brief   DMSC manual configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2Config_s
{
    uint8_t demosaicThr;      /**< Threshold for Bayer demosaicing texture detection. */
    uint16_t dmscDirThrMax;   /**< Maximum threshold for Bayer demosaicing texture detection. */
    uint16_t dmscDirThrMin;   /**< Minimum threshold for Bayer demosaicing texture detection. */

} CamEngineDmsc2Config_t;

/******************************************************************************/
/**
 * @brief   DMSC CAC configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2CacConfig_s
{
    float32_t aBlue;            /**< Parameter A_Blue for radial blue shift calculation. */
    float32_t aRed;             /**< Parameter A_Red for radial blue shift calculation. */
    float32_t bBlue;            /**< Parameter B_Blue for radial blue shift calculation. */
    float32_t bRed;             /**< Parameter B_Red for radial blue shift calculation. */
    float32_t cBlue;            /**< Parameter C_Blue for radial blue shift calculation. */
    float32_t cRed;             /**< Parameter C_Red for radial blue shift calculation. */
    uint16_t centerHOffs;   /**< The red/blue pixel shift in the horizontal direction. */
    uint16_t centerVOffs;   /**< The red/blue pixel shift in the vertical direction. */
} CamEngineDmsc2CacConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC demoire configurations.
 *
 ****************************************************************************/
typedef struct CamEngineDmsc2DemoireConfig_s
{
    uint16_t dmscDemoireR1;         /**< The y-coordinate of the first point on the weight curve of demoire interpolation output results. */
    uint16_t dmscDemoireR2;         /**< The y-coordinate of the second point on the weight curve of demoire interpolation output results. */
    uint16_t demoireEdgeR1;         /**< The y-coordinate of the first point on the weight curve which focuses more on the direction interpolation
    during the process of demoire interpolation. */
    uint16_t demoireEdgeR2;         /**< The y-coordinate of the second point on the weight curve which focuses more on the direction interpolation
    during the process of demoire interpolation. */
    uint16_t demoireEdgeT1;         /**< The x-coordinate of the first point on the weight curve which focuses more on the direction interpolation
    during the process of demoire interpolation. */
    uint8_t demoireEdgeT2Shift;     /**< The x-coordinate of the second point on the weight curve which focuses more on the direction interpolation
    during the process of demoire interpolation. It represents the left shift value.  */
    uint8_t dmscDemoireAreaThr;     /**< Judgment threshold of moire area. */
    uint8_t dmscDemoireSatShrink;   /**< Moire pixel forced saturation. */
    uint8_t dmscDemoireT1;          /**< The x-coordinate of the first point on the weight curve of demoire interpolation output results.
                                         It is the first point in X axis. */
    uint8_t dmscDemoireT2Shift;     /**< The x-coordinate of the second point on the weight curve of demoire interpolation output results.
                                         It represents the left shift value. */
} CamEngineDmsc2DemoireConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC sharpen configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2SharpenConfig_s
{
    uint16_t dmscSharpenClipBlack;      /**< Maximum sharpen level of black edge */
    uint16_t dmscSharpenClipWhite;      /**< Maximum sharpen level of white edge */
    uint16_t dmscSharpenFactorBlack;    /**< Sharpen factor of black edge */
    uint16_t dmscSharpenFactorWhite;    /**< Sharpen factor of white edge */
    uint16_t dmscSharpenR1;             /**< The y-coordinate of the first point on the curve of sharpen level in different texture areas. */
    uint16_t dmscSharpenR2;             /**< The y-coordinate of the second point on the curve of sharpen level in different texture areas. */
    uint16_t dmscSharpenR3;             /**< The y-coordinate of the third point on the curve of sharpen level in different texture areas. */
    uint16_t dmscSharpenT1;             /**< The x-coordinate of the first point on the curve of sharpen level in different texture areas. */
    uint16_t dmscSharpenT3;             /**< The x-coordinate of the third point on the curve of sharpen level in different texture areas. */
    uint8_t dmscSharpenSize;            /**< Green channel sharpen. The larger the value, the more blurred the effects. */
    uint8_t dmscSharpenT2Shift;         /**< The x-coordinate of the second point on the curve of sharpen level in different texture areas. It represents the left shift value. */
    uint8_t dmscSharpenT4Shift;         /**< The x-coordinate of the fourth point on the curve of sharpen level in different texture areas. It represents the left shift value. */
    uint8_t dmscDenoiseStrength;        /**< Green channel denoise. The larger the value, the more blurred the effects. */

} CamEngineDmsc2SharpenConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC sharpenLine configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2SharpenLineConfig_s
{
    uint16_t dmscSharpenLineR1;         /**< The y-coordinate of the first point on the curve of the overall enhancement of the line stroke.
    The wavier the curve, the more texture will be considered as a line and enhanced. */
    uint16_t dmscSharpenLineR2;         /**< The y-coordinate of the second point on the curve of the overall enhancement of the line stroke.
    The wavier the curve, the more texture is considered as a line and enhanced. */
    uint16_t dmscSharpenLineStrength;   /**< The strength of line sharpen. */
    uint16_t dmscSharpenLineThr;        /**< The line detection to get line rate for line sharpen strength control. */
    uint8_t dmscSharpenLineThrShift1;   /**< The x-coordinate of the first point on the curve of the overall enhancement of the line stroke. It represents the left shift value.
    The wavier the curve, the more texture will be considered as a line and enhanced. */
} CamEngineDmsc2SharpenLineConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC skin configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2SkinConfig_s
{
    int16_t dmscSkinCbThrMax;    /**< Maximum threshold of skin color area in Cb color space: [-2047, 2047]*/
    int16_t dmscSkinCbThrMin;    /**< Minimum threshold of skin color area in Cb color space: [-2047, 2047]*/
    int16_t dmscSkinCrThrMax;    /**< Maximum threshold of skin color area in Cr color space: [-2047, 2047]*/
    int16_t dmscSkinCrThrMin;    /**< Minimum threshold of skin color area in Cr color space: [-2047, 2047]*/
    uint16_t dmscSkinYThrMax;    /**< Maximum threshold of skin color area in Y color space: [0, 4095]*/
    uint16_t dmscSkinYThrMin;    /**< Minimum threshold of skin color area in Y color space: [0, 4095]*/
} CamEngineDmsc2SkinConfig_t;

/*****************************************************************************/
/**
 * @brief  DMSC depurple CbCr mode.
 *
 *****************************************************************************/
typedef enum CamEngine2DpulCbCrMode_e {
     CAM_ENGINE_DMSC2_DEPURPLE_CB_CR_MODE_DISABLE = 0,  /**close depurple*/
     CAM_ENGINE_DMSC2_DEPURPLE_CR_CHANNEL,              /**do cr channel depurple*/
     CAM_ENGINE_DMSC2_DEPURPLE_CB_CHANNEL,              /**do cb channel depurple*/
     CAM_ENGINE_DMSC2_DEPURPLE_CB_CR_CHANNLE,           /**do both cb and cr channel depurple*/
     CAM_ENGINE_DMSC2_DEPURPLE_MAX,
}CamEngine2DpulCbCrMode_t;

/******************************************************************************/
/**
 * @brief   DMSC depurple configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2DepurpleConfig_s
{
    CamEngine2DpulCbCrMode_t dmscDepurpleCbcrMode;  /**< Purple fringe and red fringe remove enable */
    uint8_t dmscDepurpleSatShrink;                  /**< Forced saturation compression in the purple fringing area*/
    uint8_t dmscDepurpleThr;                        /**< Judgment threshold of purple fringe area*/
} CamEngineDmsc2DepurpleConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC status configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2StatusConfig_s
{
    bool_t                     dmscv2Running;
    CamEngineDmsc2Config_t     dmscv2Cfg;
    bool_t                     dmscv2CacRunning;
    CamEngineDmsc2CacConfig_t  dmscv2CacCfg;

    bool_t                     dmscv2DemoireRunning;
    CamEngineDmsc2DemoireConfig_t dmscv2DemoireCfg;
    bool_t                     dmscv2SharpenRunning;
    CamEngineDmsc2SharpenConfig_t dmscv2SharpenCfg;
    bool_t                     dmscv2SharpenLineRunning;
    CamEngineDmsc2SharpenLineConfig_t dmscv2SharpenLineCfg;
    bool_t                     dmscv2SkinRunning;
    CamEngineDmsc2SkinConfig_t dmscv2SkinCfg;
    bool_t                     dmscv2DepurpleRunning;
    CamEngineDmsc2DepurpleConfig_t dmscv2DepurpleCfg;
} CamEngineDmsc2StatusConfig_t;

/******************************************************************************/
/**
 * @brief   DMSC auto configurations.
 *
 *****************************************************************************/
typedef struct CamEngineDmsc2AutoConfig_s
{
    uint8_t autoLevel;                                              /**< The auto config level */
    float32_t gains[CAM_ENGINE_ISO_STRENGTH_NUM];                       /**< DMSC gains */
    uint8_t dmscDenoiseStrength[CAM_ENGINE_ISO_STRENGTH_NUM];       /**< Green channel denoise. Larger values mean more blur */

    bool_t dmscSharpenEnable[CAM_ENGINE_ISO_STRENGTH_NUM];          /**< DMSC sharpen enable mask */
    uint16_t dmscSharpenFactorBlack[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< Sharpen factor of black edge */
    uint16_t dmscSharpenFactorWhite[CAM_ENGINE_ISO_STRENGTH_NUM];   /**< Sharpen factor of white edge */
    uint16_t dmscSharpenClipBlack[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Maximum sharpen level of black edge */
    uint16_t dmscSharpenClipWhite[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Maximum sharpen level of white edge */
    uint16_t dmscSharpenT1[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< Curve of sharpen level in different texture areas. It is the first point in X axis. */
    uint8_t dmscSharpenT2Shift[CAM_ENGINE_ISO_STRENGTH_NUM];        /**< Curve of sharpen level in different texture areas. It is the second point in X axis,
                                                                     and represents the left shift value*/
    uint16_t dmscSharpenT3[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< Curve of sharpen level in different texture areas. It is the third point in X axis. */
    uint8_t dmscSharpenT4Shift[CAM_ENGINE_ISO_STRENGTH_NUM];        /**< Curve of sharpen level in different texture areas. It is the fourth point in X axis,
                                                                     and represents the left shift value*/
	uint8_t dmscSharpenSize[CAM_ENGINE_ISO_STRENGTH_NUM];           /**< Green channel sharpen. Larger values mean more sharpening */
    uint16_t dmscSharpenR1[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< Curve of sharpen level in different texture areas. It is the firsy point in Y axis. */
    uint16_t dmscSharpenR2[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< Curve of sharpen level in different texture areas. It is the second point in Y axis. */
    uint16_t dmscSharpenR3[CAM_ENGINE_ISO_STRENGTH_NUM];            /**< Curve of sharpen level in different texture areas. It is the third point in Y axis. */

    bool_t dmscDepurpleEnable[CAM_ENGINE_ISO_STRENGTH_NUM];         /**< DMSC depurple enable mask */
    uint8_t dmscDepurpleSatShrink[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Forced saturation compression in the purple fringing area*/
	uint8_t dmscDepurpleCbcrMode[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< Purple fringe and red fringe remove enable */
    uint8_t dmscDepurpleThr[CAM_ENGINE_ISO_STRENGTH_NUM];           /**< Judgment threshold of purple fringe area*/
} CamEngineDmsc2AutoConfig_t;


/*****************************************************************************/
/**
 * @brief   Sets the mode for DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The input DMSC v2 mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SetMode
(
    CamEngineHandle_t hCamEngine,
	const CamEngineDmscv2Mode_t  mode           /**< DMSC v2 running mode */
);

/*****************************************************************************/
/**
 * @brief   Gets the mode for DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                The obtained DMSC v2 mode.
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDmscv2GetMode
(
    CamEngineHandle_t hCamEngine,
	CamEngineDmscv2Mode_t*  mode           /**< DMSC v2 running mode */
);


/*****************************************************************************/
/**
 * @brief   Enables DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2Enable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2Disable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2Config_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the status of DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2GetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2Config_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 CAC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2CacEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 CAC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2CacDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures the DMSC v2 CAC.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 CAC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2CacSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2CacConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DMSC v2 CAC status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 CAC configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2CacGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2CacConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 demoire.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DemoireEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 demoire.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DemoireDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2 demoire.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 demoire configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DemoireSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2DemoireConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DMSC v2 demoire status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 demoire configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DemoireGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2DemoireConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 sharpen.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 sharpen.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2 sharpen.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 sharpen configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2SharpenConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns DMSC v2 sharpen status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 sharpen configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2SharpenConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 sharpen line.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenLineEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 sharpen line.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenLineDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2 sharpen line.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 sharpen configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenLineSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2SharpenLineConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DMSC v2 sharpen line status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             DMSC v2 sharpen configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SharpenLineGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2SharpenLineConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 skin.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SkinEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 skin.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SkinDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2 skin.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Skin configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SkinSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2SkinConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DMSC v2 skin status.

 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Skin configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2SkinGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2SkinConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Enables DMSC v2 depurple.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DepurpleEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables DMSC v2 depurple.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DepurpleDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Configures DMSC v2 depurple.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Depurple configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DepurpleSetConfig
(
    CamEngineHandle_t       hCamEngine,
    const CamEngineDmsc2DepurpleConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   Returns the DMSC v2 depurple status

 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             Depurple configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDmscv2DepurpleGetConfig
(
    CamEngineHandle_t       hCamEngine,
    CamEngineDmsc2DepurpleConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   Sets the auto configurations of DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            DMSC v2 auto configurations
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDmsc2SetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDmsc2AutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   Gets the auto configurations of DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            DMSC v2 auto configurations.

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDmsc2GetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDmsc2AutoConfig_t* pAutoCfg
);
/*****************************************************************************/
/**
 * @brief   Gets the status of DMSC v2.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pSensorMode         The sensor mode
 * @param   pMode               The running mode
 * @param   pConfig             DMSC v2 status

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDmsc2GetStatus
(
    CamEngineHandle_t               hCamEngine,
    const CamEngineSensorMode_t*    pSensorMode,
    CamEngineDmscv2Mode_t*          pMode,
    CamEngineDmsc2StatusConfig_t*   pConfig
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_demosaic2_api */


#endif /* CAM_ENGINE_DEMOSAIC2_API_H */

