/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    DPF
 *
 * @file    cam_engine_dpf_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine DPF Algorithms.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_dpf_api CamEngine DPF Algorithms API
 * @{
 *
 */

#ifndef CAM_ENGINE_dpf_api_H
#define CAM_ENGINE_dpf_api_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/ispcore_defs.h>

#ifdef __cplusplus
extern "C"
{
#endif

#include "cam_engine_common.h"

#define CAM_ENGINE_DPF_NOISE_CURVE_SIZE 17 // bin number of noise level curve

/*****************************************************************************/
/**
 * @brief   dpf mode.
 *
 *****************************************************************************/
typedef enum CamEngineDpfMode_e
{
    CAM_ENGINE_DPF_MODE_INVALID = 0,    /**< (Only for initialization) Invalid */
    CAM_ENGINE_DPF_MODE_MANUAL  = 1,    /**< Manual mode */
    CAM_ENGINE_DPF_MODE_AUTO    = 2,    /**< Auto mode */
    CAM_ENGINE_DPF_MODE_MAX
} CamEngineDpfMode_t;

/******************************************************************************/
/**
 * @brief   Cam Engine DPF manual configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineDpfConfig_s
{
    float32_t div;       /**< Div. Larger values mean more denoise strength */
    float32_t gain;     /**< gain */
    float32_t gradient; /**< Gradient. Larger values mean more denoise strength */
    float32_t min;      /**< Strength clip */
    float32_t noiseCurve[CAM_ENGINE_DPF_NOISE_CURVE_SIZE];  /**< NUV noise curve. Smaller values mean more denoise strength */
    float32_t offset;   /**< Offset. Larger values mean more denoise strength */
    float32_t sigmaG;   /**< Gaussian sigma for green. Larger values mean more neighbor pixel weight */
    float32_t sigmaRb;  /**< Gaussian sigma for Red blue. Larger values mean more neighbor pixel weight */
} CamEngineDpfConfig_t;

/******************************************************************************/
/**
 * @brief   Cam Engine DPF auto configuration structure.
 *
 *****************************************************************************/
typedef struct CamEngineDpfAutoConfig_s
{
    uint8_t autoLevel;  /**< DPF auto configuration level */
    float32_t gain[CAM_ENGINE_ISO_STRENGTH_NUM];        /**< Sensor gain */
    float32_t gradient[CAM_ENGINE_ISO_STRENGTH_NUM];    /**< Gradient. Larger value means more denoise strength */
    float32_t offset[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< Offset. Larger value means more denoise strength */
    float32_t min[CAM_ENGINE_ISO_STRENGTH_NUM];         /**< Strength clip */
    float32_t div[CAM_ENGINE_ISO_STRENGTH_NUM];         /**< Div. Larger value means more denoise strength */
    float32_t sigmaG[CAM_ENGINE_ISO_STRENGTH_NUM];      /**< Gaussian sigma for green.Larger values mean more neighbor pixel weight */
    float32_t sigmaRb[CAM_ENGINE_ISO_STRENGTH_NUM];     /**< Gaussian sigma for Red blue. Larger values mean more neighbor pixel weight */
    float32_t noiseCurve[CAM_ENGINE_ISO_STRENGTH_NUM][CAM_ENGINE_DPF_NOISE_CURVE_SIZE]; /**< NUV noise curve. Smaller values mean more denoise strength */
} CamEngineDpfAutoConfig_t;

/*****************************************************************************/
/**
 * @brief   This function set dpf mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                input the dpf run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfSetMode
(
    CamEngineHandle_t hCamEngine,
	const CamEngineDpfMode_t  mode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   This function get dpf mode
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                get the dpf run mode
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfGetMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineDpfMode_t*  mode           /**< run-mode */
);

/*****************************************************************************/
/**
 * @brief   This function enable the dpf
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the dpf
 * stats.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfDisable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function config the dpf
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of dpf
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpfSetConfig
(
    CamEngineHandle_t      hCamEngine,
    CamEngineDpfConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the dpf status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of dpf
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineDpfGetConfig
(
    CamEngineHandle_t     hCamEngine,
    CamEngineDpfConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function set the auto config of dpf
 * 
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            dpf auto cfg
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfSetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDpfAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the auto config of dpf
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pAutoCfg            dpf auto cfg

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfGetAutoConfig
(
    CamEngineHandle_t hCamEngine,
    CamEngineDpfAutoConfig_t* pAutoCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the status of dpf
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pSensorMode         sensor mode
 * @param   pMode               running mode
 * @param   pCfg                dpf config

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineDpfGetStatus
(
    CamEngineHandle_t hCamEngine,
    bool_t            *pRunning,
    const CamEngineSensorMode_t* pSensorMode,
    CamEngineDpfMode_t* pMode,
    CamEngineDpfConfig_t* pCfg
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_dpf_api */


#endif /* CAM_ENGINE_dpf_api_H */

