/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @cond    cam_engine_hdr2_v1_api
 *
 * @file    cam_engine_hdr2_v1_api.h
 *
 * @brief
 *
 *   Interface description of the CamEngine Auto Algorithms.
 *
 *****************************************************************************/
/**
 *
 * @defgroup cam_engine_hdr2_v1_api CamEngine Hdr V2.1 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_HDR2_V1_API_H
#define CAM_ENGINE_HDR2_V1_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define CAM_ENGINE_HDR_COLOR_WEIGHT_MAX            255
#define CAM_ENGINE_HDR_COLOR_WEIGHT_MIN            0

#define CAM_ENGINE_HDR_EXTEND_BIT_MAX              8
#define CAM_ENGINE_HDR_EXTEND_BIT_MIN              -1

#define CAM_ENGINE_HDR_RATIO_MAX                   256.0f      
#define CAM_ENGINE_HDR_RATIO_MIN                   1.0f

#define CAM_ENGINE_HDR_TRANS_RANGE_MAX             1.0f     
#define CAM_ENGINE_HDR_TRANS_RANGE_MIN             0.0f

#define CAM_ENGINE_HDR_DIGITAL_GAIN_MAX            256.0f
#define CAM_ENGINE_HDR_DIGITAL_GAIN_MIN            1.0f

#define CAM_ENGINE_HDR_EXTEND_BIT_NUM              2  // extend bit
#define CAM_ENGINE_HDR_RATIO_FRAME_NUM             2  // two set of exposure merge
#define CAM_ENGINE_HDR_COLOR_CHANNEL_NUM           3  // three color channel, r/g/b
#define CAM_ENGINE_HDR_COMB_CURVE_NUM              4  // four hdr combination curve
#define CAM_ENGINE_HDR_TRANS_VALUE_NUM             2  // trans start and end value
#define CAM_ENGINE_HDR_DIGITAL_GAIN_NUM            3  // gain l, gain s, gain vs

#define CAM_ENGINE_HDR_EXP_MEAN_NUM                75
#define CAM_ENGINE_HDR_HIST_NUM                    48

/**
 * select which exposure to output when module bypass
 * 0-L, 1-S, 2-VS
 **/
typedef enum CamEngineHdrBypassSelect_s
{
    CAM_ENGINE_HDR_BYPASS_SELECT_L = 0,   // 0: long frame
    CAM_ENGINE_HDR_BYPASS_SELECT_S,       // 1: short frame
    CAM_ENGINE_HDR_BYPASS_SELECT_VS       // 2: very short frame
} CamEngineHdrBypassSelect_t;

/**
 * sensor type
 **/
typedef enum CamEngineHdrSensorType_s
{
    CAM_ENGINE_SENSOR_HDR_MODE_3DOL_DUAL_DCG         = 0,        /**< dual DCG mode 3x12-bit */
    CAM_ENGINE_SENSOR_HDR_MODE_3DOL                  = 1,        /**< dol3 frame 3x12-bit */
    CAM_ENGINE_SENSOR_HDR_MODE_3DOL_LINEBYLINE       = 2,        /**< 3x12-bit line by line without waiting */
    CAM_ENGINE_SENSOR_HDR_MODE_2DOL_16BIT_COMPRESS   = 3,        /**< 16-bit compressed data + 12-bit RAW */
    CAM_ENGINE_SENSOR_HDR_MODE_2DOL_DUAL_DCG_NOWAIT  = 4,        /**< 2x12-bit dual DCG without waiting */
    CAM_ENGINE_SENSOR_HDR_MODE_2DOL_L_VS             = 5,        /**< dol2 frame or 1 CG+VS sx12-bit RAW */
    CAM_ENGINE_SENSOR_HDR_MODE_2DOL_L_S              = 6,        /**< L+S 2x12-bit RAW */  
    CAM_ENGINE_SENSOR_HDR_MODE_MAX
} CamEngineHdrSensorType_t;

/**
 * base frame
 **/
typedef enum CamEngineHdrBaseFrameType_e
{
    CAM_ENGINE_HDR_BASE_FRAME_S = 0,   // 0: combination base on S frame
    CAM_ENGINE_HDR_BASE_FRAME_L        // 1: combination base on L frame
}CamEngineHdrBaseFrameType_t;

/**
 * bayer pattern
 **/
typedef enum CamEngineHdrBayerPattern_e
{
    CAM_ENGINE_HDR_BAYER_PATTERN_RGRGGBGB      = 0,        /**< 1st line: RGRG... , 2nd line GBGB... , etc. */
    CAM_ENGINE_HDR_BAYER_PATTERN_GRGRBGBG      = 1,        /**< 1st line: GRGR... , 2nd line BGBG... , etc. */
    CAM_ENGINE_HDR_BAYER_PATTERN_GBGBRGRG      = 2,        /**< 1st line: GBGB... , 2nd line RGRG... , etc. */
    CAM_ENGINE_HDR_BAYER_PATTERN_BGBGGRGR      = 3,        /**< 2st line: BGBG... , 2nd line GRGR... , etc. */
    CAM_ENGINE_HDR_BAYER_PATTERN_MAX                       /**< upper border (only for an internal evaluation) */
} CamEngineHdrBayerPattern_t;

/**
 * polarity
 **/
typedef enum CamEngineHdrPolarityType_e
{
    CAM_ENGINE_HDR_POLARITY_TYPE_LEVEL   = 0,         /**< signal from sesnro is level */
    CAM_ENGINE_HDR_POLARITY_TYPE_PULSE   = 1,         /**< signal from sensor is pulse */
    CAM_ENGINE_HDR_POLARITY_TYPE_MAX                  /**< upper border (only for an internal evaluation) */
} CamEngineHdrPolarityType_t;

/**
 * stitching mode
 **/
typedef enum CamEngineHdrStitchingMode_e
{
    CAM_ENGINE_HDR_MODE_LINEAR_COMB                 = 0, // 0: linear combine
    CAM_ENGINE_HDR_MODE_NONLINEAR_L_S               = 1, // 1: nonlinear combine L+S; linear combine LS+VS
    CAM_ENGINE_HDR_MODE_LINEAR_L_S_NONLINEAR_LS_VS  = 2  // 2: linear combine L+S; nonlinear combine LS+VS
}CamEngineHdrStitchingMode_t;


/**
 * properties
 **/
typedef struct CamEngineIspStitchingProperties_s {
    CamEngineHdrPolarityType_t       hSyncPol;
    CamEngineHdrPolarityType_t       vSyncPol;
} CamEngineIspStitchingProperties_t;

/**
 * digital gain
 **/
typedef struct CamEngineIspStitchingDgGain_s {
    float32_t gainR;
    float32_t gainB;
    float32_t gainGr;
    float32_t gainGb;
} CamEngineIspStitchingDgGain_t;

typedef enum CamEngineHdrExpMeasuringMode_e
{
    CAM_ENGINE_HDR_EXP_MEASURING_MODE_INVALID    = 0,    /**< invalid histogram measuring mode   */
    CAM_ENGINE_HDR_EXP_MEASURING_MODE_1          = 1,    /**< Y = (R + G + B) x (85/256)         */
    CAM_ENGINE_HDR_EXP_MEASURING_MODE_2          = 2,    /**< Y = 16 + 0.25R + 0.5G + 0.1094B    */
    CAM_ENGINE_HDR_EXP_MEASURING_MODE_MAX,
} CamEngineHdrExpMeasuringMode_t;

typedef enum CamEngineIspHdrHistMode_e
{
    CAM_ENGINE_ISP_HDR_HIST_MODE_INVALID       = 0,    /**< lower border (only for an internal evaluation) */
    CAM_ENGINE_ISP_HDR_HIST_MODE_RGB_COMBINED  = 1,    /**< RGB combined histogram */
    CAM_ENGINE_ISP_HDR_HIST_MODE_R             = 2,    /**< R histogram */
    CAM_ENGINE_ISP_HDR_HIST_MODE_G             = 3,    /**< G histogram */
    CAM_ENGINE_ISP_HDR_HIST_MODE_B             = 4,    /**< B histogram */
    CAM_ENGINE_ISP_HDR_HIST_MODE_Y             = 5,    /**< luminance histogram */
    CAM_ENGINE_ISP_HDR_HIST_MODE_MAX,     			   /**< upper border (only for an internal evaluation) */
} CamEngineIspHdrHistMode_t;

/**
 * hdr config
 **/
typedef struct CamEngineHdrStitchConfig_s
{
    float32_t                       ratio[CAM_ENGINE_HDR_RATIO_FRAME_NUM];                                       // [L/S, S/VS]
    float32_t                       transRange[CAM_ENGINE_HDR_COMB_CURVE_NUM][CAM_ENGINE_HDR_TRANS_VALUE_NUM];   // [[L+S_ref(L)_start, L+S_ref(L)_end],  
                                                                                                                 //  [LS+VS_ref(LS)_start, LS+VS_ref(LS)_end], 
                                                                                                                 //  [L+S_ref(S)_start, L+S+ref(S)_end], 
                                                                                                                 //  [LS+VS_ref(VS)_start, LS+VS_ref(VS)_end]
    uint8_t                         colorWeight[CAM_ENGINE_HDR_COLOR_CHANNEL_NUM];                               // [R weight, G weight, B weight]
    CamEngineHdrBaseFrameType_t     baseFrameSelection;
    int8_t                          extendBit[CAM_ENGINE_HDR_EXTEND_BIT_NUM];                                   // [SHORT_LS_EXTEND_BIT, SHORT_VS_EXTEND_BIT]
}CamEngineHdrStitchConfig_t;

/**
 * internal config
 **/
typedef struct CamEngineHdrStitchInternalConfig_s
{
    uint32_t                           imageWidth;
    uint32_t                           imageHeight; 
    CamEngineHdrBayerPattern_t         bayerPattern; 
    CamEngineHdrSensorType_t           sensorType;
    CamEngineIspStitchingDgGain_t      dgGain[CAM_ENGINE_HDR_DIGITAL_GAIN_NUM];
    CamEngineIspStitchingProperties_t  properties;
}CamEngineHdrStitchInternalConfig_t;

/**
 * status 
 **/
typedef struct CamEngineHdrStitchStatus_s
{
    bool_t                                  hdrDigitalGainRunning;
    bool_t                                  hdrExpRunning;
    bool_t                                  hdrHistRunning;
    CamEngineHdrStitchConfig_t              hdrConfig; 
}CamEngineHdrStitchStatus_t;

typedef struct CamEngineHdrExpMean_s
{
	uint32_t  hdrExpMean[CAM_ENGINE_HDR_EXP_MEAN_NUM];
} CamEngineHdrExpMean_t;

typedef struct CamEngineHdrHist_s
{
	uint32_t  hdrHist[CAM_ENGINE_HDR_HIST_NUM];
} CamEngineHdrHist_t;

/*****************************************************************************/
/**
 * @brief   This function set the HDR V21 stitching mode
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrSetStitchingMode
(
    CamEngineHandle_t                 hCamEngine,
    const CamEngineHdrStitchingMode_t stitchingMode
);

/*****************************************************************************/
/**
 * @brief   This function get the HDR V21 stitching mode
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrGetStitchingMode
(
    CamEngineHandle_t            hCamEngine,
    CamEngineHdrStitchingMode_t  *stitchingMode
);

/*****************************************************************************/
/**
 * @brief   This function enable HDR V21.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrEnable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the HDR V21.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function enables the HDR V21 digital gain
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrDigitalGainEnable
(
    CamEngineHandle_t hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable HDR V21 digital gain
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrDigitalGainDisable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function config the HDR V21 internal configuration
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             internal configuration of HDR V21
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrSetInternalConfig
(
    CamEngineHandle_t                      hCamEngine,
    CamEngineHdrStitchInternalConfig_t    *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the HDR V21 internal configuration
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             internal configuration of HDR V21
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrGetInternalConfig
(
    CamEngineHandle_t                   hCamEngine,
    CamEngineHdrStitchInternalConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function config the HDR V21.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of HdrV21
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrSetConfig
(
    CamEngineHandle_t           hCamEngine,
    CamEngineHdrStitchConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function returns the HDR V21 Config.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             configuration of hdrv21
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrGetConfig
(
    CamEngineHandle_t           hCamEngine,
    CamEngineHdrStitchConfig_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function disable HDR combine and set the HDR bypass frame, only for debug.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   bypassFrame         Bypass frame select
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineSetBypassSelectEnable
(
    CamEngineHandle_t                       hCamEngine,
    CamEngineHdrBypassSelect_t              bypassFrame
);

/*****************************************************************************/
/**
 * @brief   This function get the HDR bypass frame.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   bypassFrame         Bypass frame select
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineGetBypassSelectEnable
(
    CamEngineHandle_t                       hCamEngine,
    CamEngineHdrBypassSelect_t              *bypassFrame
);
/*****************************************************************************/
/**
 * @brief   This function get the status of HDR V21
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pConfig             HDR V21 config

 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 *
 *****************************************************************************/
RESULT CamEngineHdrGetStatus
(
    CamEngineHandle_t                       hCamEngine,
    bool_t                                  *pRunning,
    CamEngineHdrStitchStatus_t              *pHdrStatus
);

/*****************************************************************************/
/**
 * @brief   This function enable the HDR v21 exp.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpEnable
(
    CamEngineHandle_t           hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the HDR v21 exp.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpDisable
(
    CamEngineHandle_t           hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function sets the HDR v21 exp measuring mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measuring mode
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpSetMeasuringMode
(
    CamEngineHandle_t hCamEngine,
    const CamEngineHdrExpMeasuringMode_t  mode
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR v21 exp measuring mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pMode                measuring mode
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpGetMeasuringMode
(
    CamEngineHandle_t hCamEngine,
    CamEngineHdrExpMeasuringMode_t  *pMode
);

/*****************************************************************************/
/**
 * @brief   This function sets the HDR v21 exp measuring window.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measuring window
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpSetMeasuringWindow
(
    CamEngineHandle_t hCamEngine,
    CamEngineWindow_t  *pWindow
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR v21 exp measuring window.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measuring window
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpGetMeasuringWindow
(
    CamEngineHandle_t hCamEngine,
    CamEngineWindow_t  *pWindow
);


/*****************************************************************************/
/**
 * @brief   This function enable the HDR v21 hist.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistEnable
(
    CamEngineHandle_t           hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function disable the HDR v21 hist.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistDisable
(
    CamEngineHandle_t           hCamEngine
);

/*****************************************************************************/
/**
 * @brief   This function sets the HDR hist mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   mode                measuring mode
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistSetMeasuringMode
(
    CamEngineHandle_t                hCamEngine,
    const CamEngineIspHdrHistMode_t  mode
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR hist mode.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pMode                measuring mode
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistGetMeasuringMode
(
    CamEngineHandle_t          hCamEngine,
    CamEngineIspHdrHistMode_t  *pMode
);

/*****************************************************************************/
/**
 * @brief   This function sets the HDR v21 hist measuring window.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pWindow             measuring window
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistSetMeasuringWindow
(
    CamEngineHandle_t  hCamEngine,
    CamEngineWindow_t  *pWindow
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR v21 hist measuring window.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pWindow             measuring window
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrHistGetMeasuringWindow
(
    CamEngineHandle_t  hCamEngine,
    CamEngineWindow_t  *pWindow
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR v21 exp mean stat.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pExpStat            exp mean stat
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrExpGetExpMeanStat
(
    CamEngineHandle_t hCamEngine,
    CamEngineHdrExpMean_t  *pExpStat
);

/*****************************************************************************/
/**
 * @brief   This function gets the HDR v21 hist stat.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pHist               hdr hist stat
 * 
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHdrGetHist
(
    CamEngineHandle_t hCamEngine,
    CamEngineHdrHist_t *pHist
);

#ifdef __cplusplus
}
#endif


/* @} cam_engine_hdr2_v1_api */


#endif /* CAM_ENGINE_HDR2_V1_API_H */

