/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

/**
 * @cond HIST64
 *
 * @defgroup cam_engine_hist64_api CamEngine HIST64 Definitions
 * @{
 *
 */

#ifndef CAM_ENGINE_HIST64_API_H
#define CAM_ENGINE_HIST64_API_H

#include <common/return_codes.h>
#include "cam_engine_common.h"

#ifdef __cplusplus
extern "C"
{
#endif


#define CAM_ENGINE_ISP_HIST64_NUM_BINS     32  /**< The number of bins */
#define CAM_ENGINE_HIST64_GRID_ITEMS       25  /**< The number of grid sub-windows */

#define CAM_ENGINE_HIST64_WEIGHT_MAX       16 
#define CAM_ENGINE_HIST64_WEIGHT_MIN       0 

#define CAM_ENGINE_HIST64_COFF_MAX         1.992
#define CAM_ENGINE_HIST64_COFF_MIN         0 

/*****************************************************************************/
/**
 * @brief   Histogram measuring modes.
 *
 *****************************************************************************/
typedef enum CamEngineHist64Mode_e
{
    CAM_ENGINE_HIST64_MODE_INVALID       = -1,    /**< (For internal use only) Lower border */
    CAM_ENGINE_HIST64_MODE_DISABLE       = 0,    /**< Disabled, no measurements  */
    CAM_ENGINE_HIST64_MODE_ONE_FROM_YRGB = 1,    /**< Y/R/G/B histogram controlled via coefficients coeff_r/g/b */
    CAM_ENGINE_HIST64_MODE_R             = 2,    /**< R histogram */
    CAM_ENGINE_HIST64_MODE_GR            = 3,    /**< Gr histogram */
    CAM_ENGINE_HIST64_MODE_B             = 4,    /**< B histogram */
    CAM_ENGINE_HIST64_MODE_GB            = 5,    /**< Gb histogram */
    CAM_ENGINE_IST64_MODE_MAX,     				/**< (For internal use only) Upper border */
} CamEngineHist64Mode_t;

/******************************************************************************/
/**
 * @brief   HIST channel.
 *
 *****************************************************************************/
typedef enum CamEngineHist64Channel_e
{
    CAM_ENGINE_HIST64_CHANNEL_INVALID       = -1,   /**< (For internal use only) Lower border */
    CAM_ENGINE_HIST64_CHANNEL_0             = 0,    /* after De-gamma channel */
    CAM_ENGINE_HIST64_CHANNEL_1             = 1,    /* after LSC channel */
    CAM_ENGINE_HIST64_CHANNEL_2             = 2,    /* after AWB GAIN channel */
    CAM_ENGINE_HIST64_CHANNEL_3             = 3,    /* RGB domain after WDR channel */
    CAM_ENGINE_HIST64_CHANNEL_4             = 4,    /* after Demosaic channel */
    CAM_ENGINE_HIST64_CHANNEL_5             = 5,    /* RGB domain after cross talk channel */
    CAM_ENGINE_HIST64_CHANNEL_6             = 6,    /* reserved */
    CAM_ENGINE_HIST64_CHANNEL_7             = 7,    /* RGB domain after gamma out channel */
    CAM_ENGINE_HIST64_CHANNEL_MAX,     				/**< (For internal use only) Upper border*/
} CamEngineHist64Channel_t;

/******************************************************************************/
/**
 * @brief   HIST window.
 *
 *****************************************************************************/
typedef struct CamEngineHist64Window_s
{
	uint16_t offsetX;    /**< Horizontal start offset */
	uint16_t offsetY;    /**< Vertical start offset */
	uint16_t width;      /**< Width */
	uint16_t height;     /**< Height */
} CamEngineHist64Window_t;

/******************************************************************************/
/**
 * @brief   HIST weight.
 *
 *****************************************************************************/
typedef struct CamEngineHist64Weight_s
{
    uint8_t weight[CAM_ENGINE_HIST64_GRID_ITEMS];   /**< hist64 weights */
} CamEngineHist64Weight_t;

/******************************************************************************/
/**
 * @brief   HIST bins.
 *
 *****************************************************************************/
typedef struct CamEngineHist64Bins_s
{
    uint32_t bins[CAM_ENGINE_ISP_HIST64_NUM_BINS];   /**< hist64 bins */
} CamEngineHist64Bins_t;


/******************************************************************************/
/**
 * @brief   CamEngineHist64Config_t
 *
 *****************************************************************************/
typedef struct CamEngineHist64Config_s
{
    CamEngineHist64Mode_t    hist64Mode;
    CamEngineHist64Channel_t hist64Channel;
    CamEngineHist64Weight_t  hist64Weight;
    CamEngineHist64Window_t  hist64window;
    float32_t                rCoeff;
    float32_t                gCoeff;
    float32_t                bCoeff;
} CamEngineHist64Config_t;

/*****************************************************************************/
/**
 * @brief   Enables HIST64.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64Enable
(
    CamEngineHandle_t       hCamEngine
);

/*****************************************************************************/
/**
 * @brief   Disables HIST64.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64Disable
(
    CamEngineHandle_t       hCamEngine
);


/*****************************************************************************/
/**
 * @brief   Gets HIST64 status.
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pIsEnabled          HIST64 status of pointers
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64IsEnabled
(
    CamEngineHandle_t  hCamEngine,
    bool_t             *pIsEnabled
);


/*****************************************************************************/
/**
 * @brief   Sets HIST64 config
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pConfig             The hist64 configuration
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64SetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineHist64Config_t  *pConfig
);


/*****************************************************************************/
/**
 * @brief   Gets HIST64 config
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pChannel            The hist64 configuration
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64GetConfig
(
    CamEngineHandle_t        hCamEngine,
    CamEngineHist64Config_t  *pConfig
);

/*****************************************************************************/
/**
 * @brief   Gets HIST64 statistic
 *
 * @param   hCamEngine          A handle to the CamEngine instance
 * @param   pChannel            The hist64 statistic
 *
 * @return  The result of the function call.
 * @retval  RET_SUCCESS         Successful
 * @retval  RET_WRONG_HANDLE    Invalid instance handle
 * @retval  RET_WRONG_CONFIG    The image effects are not the configured ones.
 * @retval  RET_NOTAVAILABLE    The module is not available for the driver or hardware.
 *
 *****************************************************************************/
RESULT CamEngineHist64GetStatistic
(
    CamEngineHandle_t       hCamEngine,
    CamEngineHist64Bins_t  *pHistogram
);

/* @} cam_engine_hist64 */
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif

