/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_CPROC_DRV_API_H
#define CAMERIC_ISP_CPROC_DRV_API_H

/**
 * @cond    cameric_cproc
 *
 * @file    cameric_cproc_drv_api.h
 *
 * @brief   This file contains the CamerIC CPROC driver API definitions.
 *
 *****************************************************************************/
/**
 * @defgroup cameric_cproc_drv_api CamerIC CPROC Driver API definitions
 * @{
 *
 * The CamerIC color processing module is responsible for color manipulation
 * of the incoming data stream in terms of hue, brightness, contrast and
 * saturation. The pixel luminance is modified to manipulate contrast and
 * brightness while the chrominance values is modified for hue and saturation
 * contrast.
 *
 * @if CAMERIC_LITE
 *
 * @image html cproc_lite.png "Overview of the CamerIC MI driver" width=\textwidth
 * @image latex cproc_lite.png "Overview of the CamerIC MI driver" width=\textwidth
 *
 * @endif
 *
 * @if CAMERIC_FULL
 *
 * @image html cproc.png "Overview of the CamerIC MI driver" width=\textwidth
 * @image latex cproc.png "Overview of the CamerIC MI driver" width=\textwidth
 *
 * @endif
 *
 * Color processing is done in terms of luminance and chrominance adjustment.
 *
 * @arg Contrast processing is done by multiplying the luminance value by the
 * contrast adjustment value defined in CPROC_CONTRAST which is in the range
 * of 0.0 (0x00) and 1.992 (0xFF).
 *
 * @arg Brightness is adjusted by adding a value defined in CPROC_BRIGHTNESS
 * to the luminance value of a pixel. This 8 bit value is a signed number
 * in two’s-complement.
 *
 * @arg Saturation manipulation in chrominance processing in terms of multiplying
 * Cr and Cb with a fixed-point number between 0.0 (0x00) and 1.992 (0xFF). This
 * value is to be programmed in CPROC_SATURATION.
 *
 * @arg Hue processing is a phase shift of the chrominance values between -90
 * (0x80) degree and +87.188 (0x7F) degree to be defined in CPROC_HUE.
 *
 * The color processing module can be configured to produce pixel values
 * according to the BT.601 standard or in full value range. Full value range
 * should be used for JPEG encoding. See register COLOR_PROC_CTRL for details
 * of the possible configurations. The input pixel range is also configurable,
 * because pixels coming from the YCbCr path of the ISP may have both ranges
 * depending on the camera sensor used.
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#ifdef __cplusplus
extern "C"
{
#endif


/******************************************************************************/
/**
 * @brief   Enumeration type to configure the YUV output range in the
 *          CamerIC color processing unit.
 *
 *****************************************************************************/
typedef enum CamerIcIspCprocYuvRange_e {
    CAMERIC_ISP_CPROC_YUV_RANGE_OUT_INVALID       = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_CPROC_YUV_RANGE_OUT_BT601         = 1,    /**< CbCr_out clipping range 16..240 according to ITU-R BT.601 standard */
    CAMERIC_ISP_CPROC_YUV_RANGE_OUT_FULL_RANGE    = 2,    /**< full UV_out clipping range 0..255 */
    CAMERIC_ISP_CPROC_YUV_RANGE_OUT_MAX                   /**< upper border (only for an internal evaluation) */
} CamerIcIspCprocYuvsRange_t;


/******************************************************************************/
/**
 * @brief   Structure to configure the color processing module
 *
 *****************************************************************************/
typedef struct CamerIcIspCprocConfig_s
{
    uint8_t                             contrast;       /**< contrast value to initially set */
    uint32_t                            brightness;     /**< brightness value to initially set */
    uint8_t                             saturation;     /**< saturation value to initially set */
    uint8_t                             hue;            /**< hue value to initially set */
} CamerIcIspCprocConfig_t;


/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC CPROC Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocSetConfig
(
    CamerIcDrvHandle_t         handle,
    const CamerIcIspCprocConfig_t    *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function return configures the CamerIC CPROC Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocGetConfig
(
    CamerIcDrvHandle_t         handle,
    CamerIcIspCprocConfig_t    *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function enables the CamerIC CPROC Module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocEnable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc CPROC module (bypass the color
 *          prcessing module)
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocDisable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 * @brief   This function returns the status of the CamerIC CPROC module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_NULL_POINTER    pIsEnabled is a NULL pointer
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocIsEnabled
(
    CamerIcDrvHandle_t  handle,
    bool_t              *pIsEnabled
);


/*****************************************************************************/
/**
 * @brief   This function sets the ranges in the CamerIc CPROC module.
 *
 * @param   handle              CamerIc driver handle
 * @param   ChromaOut           Color processing chrominance pixel clipping
 *                              range at output
 * @param   LumaOut             Color processing luminance output clipping
 *                              range
 * @param   LumaIn              Color processing luminance input range
 *                              (offset processing)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
 #ifdef ES_API
extern RESULT CamerIcIspCprocSetRangesEs
(
    CamerIcDrvHandle_t                  handle,
    const bool_t                        bLimitedRangeEn
);
#endif
extern RESULT CamerIcIspCprocSetRanges
(
    CamerIcDrvHandle_t                      handle,
    const CamerIcIspCprocYuvsRange_t        ChromaOut,
    const CamerIcIspCprocYuvsRange_t        LumaOut,
    const CamerIcIspCprocYuvsRange_t        LumaIn
);

/*****************************************************************************/
/**
 * @brief   This function gets the ranges in the CamerIc CPROC module.
 *
 * @param   handle              CamerIc driver handle
 * @param   ChromaOut           Color processing chrominance pixel clipping
 *                              range at output
 * @param   LumaOut             Color processing luminance output clipping
 *                              range
 * @param   LumaIn              Color processing luminance input range
 *                              (offset processing)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCprocGetRanges
(
    CamerIcDrvHandle_t                      handle,
    CamerIcIspCprocYuvsRange_t        		*ChromaOut,
    CamerIcIspCprocYuvsRange_t        		*LumaOut,
    CamerIcIspCprocYuvsRange_t        		*LumaIn
);

#ifdef __cplusplus
}
#endif

/* @} cameric_cproc_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_CPROC_DRV_API_H */

