/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @cond    cameric_isp_csm
 *
 * @file 	cameric_isp_csm_drv_api.h
 *
 * @brief	This file contains the CamerIc ISP csm driver API definitions
 *
 *****************************************************************************/
#ifndef CAMERIC_ISP_CSM_DRV_API_H
#define CAMERIC_ISP_CSM_DRV_API_H

#include <ebase/types.h>
#include <ebase/trace.h>
#include <common/return_codes.h>
#include "cameric_isp_drv_api.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions enable the csm.
 *          there is no enable of csm in hardware, enable in software
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCsmEnable
(
    CamerIcDrvHandle_t          handle
);
/* @endcond */


/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions disable the csm.
 *          there is no disable of csm in hardware, set the default value when disable      
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCsmDisable
(
    CamerIcDrvHandle_t          handle
);
/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function sets the color conversion range.
 *
 * @note    The color conversion unit - also called color space matrix -
 *          performs a regular RGB to YCbCr 4:4:4 color space conversion. The
 *          nine coefficients take values between -2 and +1.992.
 *
 *          \arg YCbCr range limited output \n
 *          Y    = c0*R + c1*G + c2*B + 64  \n
 *          Cb   = c3*R + c4*G + c5*B + 512 \n
 *          Cr   = c6*R + c7*G + c8*B + 512 \n
 *
 *          \arg YCbCr full range output    \n
 *          Y    = c0*R + c1*G + c2*B       \n
 *          Cb   = c3*R + c4*G + c5*B + 512 \n
 *          Cr   = c6*R + c7*G + c8*B + 512 \n
 *
 *          \arg default matrix             \n
 *          ( 0x021 0x040 0x00D )       (   \n
 *          ( 0x1ED 0x1DB 0x038 )   =   (   \n
 *          ( 0x038 0x1D1 0x1F7 )       (   \n
 *
 * @param   handle              CamerIc driver handle
 * @param   YConvRange          luminance conversion range
 * @param   CrConvRange         chrominance conversion range
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the color
 *                              conversion matrix (maybe the driver is already
 *                              running)
 * @retval  RET_NOTSUPP         conversion range is not supported
 *
 *****************************************************************************/
#ifdef ES_API
extern RESULT CamerIcIspSetColorConversionRangeEs
(
    CamerIcDrvHandle_t                  handle,
    const bool_t                        bLimitedRangeEn
);
#endif

extern RESULT CamerIcIspSetColorConversionRange
(
    CamerIcDrvHandle_t                  handle,
    const CamerIcColorConversionRange_t YConvRange,
    const CamerIcColorConversionRange_t CrConvRange
);
/* @endcond */


/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function get the color conversion range.
 *
 * @param   handle              CamerIc driver handle
 * @param   YConvRange          luminance conversion range
 * @param   CrConvRange         chrominance conversion range
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the color
 *                              conversion matrix (maybe the driver is already
 *                              running)
 * @retval  RET_NOTSUPP         conversion range is not supported
 *
 *****************************************************************************/
extern RESULT CamerIcIspGetColorConversionRange
(
    CamerIcDrvHandle_t                  handle,
    CamerIcColorConversionRange_t       *YConvRange,
    CamerIcColorConversionRange_t       *CrConvRange
);
/* @endcond */


/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function returns the currently configured color conversion
 *          coefficients.
 *
 * @param   handle              CamerIc driver handle
 * @param   pCConvCoefficients  storage class for color conversion coefficients
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the color
 *                              conversion matrix (maybe the driver is not
 *                              initialized)
 *
 *****************************************************************************/
extern RESULT CamerIcIspGetColorConversionCoefficients
(
    CamerIcDrvHandle_t      handle,
    CamerIc3x3Matrix_t      *pCConvCoefficients
);
/* @endcond */



/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function sets the color conversion coefficients.
 *
 * @param   handle      CamerIc driver handle
 * @param   pCConvCoefficients  Color conversion coefficients to set
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the color
 *                              conversion matrix (maybe the driver is already
 *                              running)
 *
 * @endif
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetColorConversionCoefficients
(
    CamerIcDrvHandle_t              handle,
    const CamerIc3x3Matrix_t        *pCConvCoefficients
);
/* @endcond */

#ifdef __cplusplus
}
#endif

#endif /* CAMERIC_ISP_CSM_DRV_API_H */
