/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

#ifndef CAMERIC_ISP_DMSC2_DRV_API_H
#define CAMERIC_ISP_DMSC2_DRV_API_H

/**
 * @cond    cameric_isp_dmsc2
 *
 * @file 	cameric_isp_dmsc2_drv_api.h
 *
 * @brief	This file contains the CamerIc ISP DMSC2 driver API definitions
 *
 *****************************************************************************/
/**
 * @defgroup cameric_isp_dmsc2_drv_api CamerIc ISP DMSC2 driver API definitions
 * @{
 *
 *Demosaic is a method used in digital image processing to interpolate incomplete
 *color samples to generate complete color samples (because the Bayer pattern looks
 *like a Mosaic, it is called desmosaics). CFA filter is usually used to obtain 
 *Bayer pattern on sensor side, and Bayer pattern needs to be transformed into 
 *complete RGB444(true color) image in later processing. 
 * The Demosaic V20 can be broken-down to the following sub-functions:
 * 1)	Intrepolation
 * 2)	Sharpen
 * 3)	Denoise
 * 4)	Demoire
 * 5)	Depurple(defalse color)
 * 6)	Skin protection
 *
 */

#include <ebase/types.h>
#include <common/return_codes.h>
#include <cameric_drv/cameric_isp_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMERIC_ISP_THRESHOLD_MAX 4095
#define CAMERIC_ISP_DEMOIRE_T2_SHIFT_MAX 8
#define CAMERIC_ISP_DEMOIRE_EDGE_T2_SHIFT_MAX 8

#define CAMERIC_DMSC2_THRESHOLD_MAX 255
#define CAMERIC_DMSC2_THRESHOLD_MIN  0

#define CAMERIC_DMSC2_DIR_THR_MAX 4095
#define CAMERIC_DMSC2_DIR_THR_MIN  0


#define CAMERIC_DEMOIRE_THR_MAX 32
#define CAMERIC_DEMOIRE_THR_MIN  0

#define CAMERIC_DEMOIRE_SHRINK_MAX 32
#define CAMERIC_DEMOIRE_SHRINK_MIN  0

#define CAMERIC_DEMOIRE_R_MAX 256
#define CAMERIC_DEMOIRE_R_MIN  0

#define CAMERIC_DEMOIRE_T2_SHIFT_MAX 8
#define CAMERIC_DEMOIRE_T2_SHIFT_MIN  0

#define CAMERIC_DEMOIRE_T1_MAX 255
#define CAMERIC_DEMOIRE_T1_MIN  0

#define CAMERIC_DEMOIRE_EDGE_T1_MAX 256
#define CAMERIC_DEMOIRE_EDGE_T1_MIN  0

#define CAMERIC_SHARPEN_FACTOR_MAX 511
#define CAMERIC_SHARPEN_FACTOR_MIN  0

#define CAMERIC_SHARPEN_CLIP_MAX 2047
#define CAMERIC_SHARPEN_CLIP_MIN  0

#define CAMERIC_SHARPEN_T_MAX 2047
#define CAMERIC_SHARPEN_T_MIN  0

#define CAMERIC_SHARPEN_T_SHIFT_MAX 11
#define CAMERIC_SHARPEN_T_SHIFT_MIN  0

#define CAMERIC_SHARPEN_R_MAX 256
#define CAMERIC_SHARPEN_R_MIN  0

#define CAMERIC_SHARPEN_SIZE_MAX 16
#define CAMERIC_SHARPEN_SIZE_MIN  0

#define CAMERIC_DENOISE_STR_MAX 32
#define CAMERIC_DENOISE_STR_MIN  0

#define CAMERIC_FILTER_MAX  63
#define CAMERIC_FILTER_MIN  0

#define CAMERIC_SHARPEN_LINE_T_MAX 2047
#define CAMERIC_SHARPEN_LINE_T_MIN  0

#define CAMERIC_SHARPEN_LINE_T_SHIFT_MAX 10
#define CAMERIC_SHARPEN_LINE_T_SHIFT_MIN  0

#define CAMERIC_SHARPEN_LINE_R_MAX 256
#define CAMERIC_SHARPEN_LINE_R_MIN  0

#define CAMERIC_SHARPEN_LINE_STR_MAX 4095
#define CAMERIC_SHARPEN_LINE_STR_MIN  0

#define CAMERIC_SKIN_MAX 4095
#define CAMERIC_SKIN_MIN  0

#define CAMERIC_DEPURPLE_SAT_SHRINK_MAX 8
#define CAMERIC_DEPURPLE_SAT_SHRINK_MIN  0

#define CAMERIC_DEPURPLE_THR_MAX 255
#define CAMERIC_DEPURPLE_THR_MIN  8
/*****************************************************************************/
/**
 * @brief  demosaic2 interplolation configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2Intp_s {
     uint16_t dmscDirThrMin;      /**<[0, 4095] direction threshold in dark area*/
     uint16_t dmscDirThrMax;      /**<[0, 4095] direction threshold in bright area*/
     uint8_t dmscThr;             /**<[0, 255] Interpolate direction threshold on texture, More small, more edge sensitivity*/
} CamerIcIspDmsc2Config_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 demoire configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2Demoire_t {
    uint8_t demoireAreaThr;              /**<[0, 32] Threshold to judge moire area*/
    uint8_t demoireSatShrink;          /**<[0, 32] Saturation reduce strength in moire area, more large, more gray.*/
    uint16_t demoireR2;          /**<[0, 256] Interpolate [r1, r2] if (t1<moire_rate< t2)*/
    uint16_t demoireR1;          /**<[0, 256]Weight to merge demoire image into output image, user r1, if moire rate < t1*/
    uint8_t demoireT2Shift;          /**<[0, 8] T2= t1+2^t2_shift*/
    uint8_t demoireT1;          /**<[0, 255]*/
    uint16_t demoireEdgeR2;          /**<[0, 256]*/
    uint16_t demoireEdgeR1;          /**<[0, 256]*/
    uint8_t demoireEdgeT2Shift;          /**<[0, 8] T2= t1+2^t2_shift*/
    uint16_t demoireEdgeT1;          /**<[0, 256] Weight of Demoire Interpolate by line/details*/
                                     /**<Use r1, if max gradient < t1*/
} CamerIcIspDmsc2Demoire_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 sharpen configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2Sharpen_t {

    uint16_t sharpenFactorBlack;                    /**<[0, 511] Sharpen strength*/
    uint16_t sharpenFactorWhite;                    /**<[0, 511] Sharpen strength*/
    uint16_t sharpenClipBlack;                  /**<[0, 2047] Max Sharpen strength*/
    uint16_t sharpenClipWhite;                  /**<[0, 2047] Max Sharpen strength*/
    uint16_t sharpenT4Shift;                    /**<[0, 11] T4 = t3 + 2^t4_shift*/
    uint16_t sharpenT3;                 /**<[0, 2047]Linear interpolate in [r1, r2, r3, r4]*/
    uint8_t sharpenT2Shift;                 /**<[0, 11] T2 = t1 + 2^t2_shift*/
    uint16_t sharpenT1;                 /**<[0, 2047] */
                                        /**<Threshold of texture value, control to sharpen ratio*/
                                        /**< Use r1, if texture_val< t1*/
    uint16_t sharpenR3;                 /**<[0, 511]*/
    uint16_t sharpenR2;                 /**<[0, 511]*/
    uint16_t sharpenR1;                 /**<[0, 511]*/
    uint8_t  sharpenSize;                   /**<[0, 511]*/
    uint32_t denoise;                   /**<[0, 511]*/
} CamerIcIspDmsc2Sharpen_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 sharpen line configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2SharpenLine_t {
	//uint8_t sharpenLineShift2;  //not used
    uint8_t sharpenLineShift1;       /**<[0, 10]     */
    uint16_t sharpenLineT1;/**<[0, 2047]     */
    uint16_t sharpenLineStrength;/**<[0, 4095] Weight to merge sharpen_line image into output image     */
    uint16_t sharpenLineR2;/**< [0, 256] Linear interpolate in [r1, r2]    */
    uint16_t sharpenLineR1;/**<[0, 256]     */
} CamerIcIspDmsc2SharpenLine_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 3x3 green channel filter kernel,used to calculate 
 * denoise result configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2GreenFilter_t {
    uint8_t filt00;
    uint8_t filt01;
    uint8_t filt02;
    uint8_t filt10;
    uint8_t filt11;
    uint8_t filt12;
    uint8_t filt20;
    uint8_t filt21;
    uint8_t filt22;
} CamerIcIspDmsc2GFilter_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 skin configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2Skin_t {
    uint16_t cbThrMax;    /**<[0, 4095]*/
    uint16_t cbThrMin;/**<[0, 4095]*/
    uint16_t crThrMax;/**<[0, 4095]*/
    uint16_t crThrMin;/**<[0, 4095]*/
    uint16_t yThrMax;/**<[0, 4095]*/
    uint16_t yThrMin;/**<[0, 4095]*/
} CamerIcIspDmsc2Skin_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 depurple cbcr mode enum
 *
 *****************************************************************************/
typedef enum CamerIcIspDmsc2DpulCbCrMode_e {
     CAMERIC_DMSC2_DEPURPLE_CB_CR_MODE_DISABLE = 0,
     CAMERIC_DMSC2_DEPURPLE_CR_CHANNEL,
     CAMERIC_DMSC2_DEPURPLE_CB_CHANNEL,
     CAMERIC_DMSC2_DEPURPLE_CB_CR_CHANNLE,
     CAMERIC_DMSC2_DEPURPLE_MAX,
}CamerIcIspDmsc2DpulCbCrMode_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 depurple configuration structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2Depurple_t {
    CamerIcIspDmsc2DpulCbCrMode_t cbCrMode;  /**<{0, 1, 2, 3} Reduce saturation on cb/cr channel*/
    uint8_t depurpleSatShrink;          /**<[0, 8] Reduce saturation strength*/
    uint8_t depurpleThr;                /**<[0, 255]Threshold to judge purple aera*/
} CamerIcIspDmsc2Depurple_t;

/*****************************************************************************/
/**
 * @brief  demosaic2 color aberration correction structure
 *
 *****************************************************************************/
typedef struct CamerIcIspDmsc2CacConfig_s {
    uint16_t vCountStart; /**<float cac center offset*/
    uint16_t hCountStart;/**<float cac center offset*/
    uint16_t aRed;/**<float  cac poly coefficients*/
    uint16_t aBlue;/**< float cac poly coefficients*/
    uint16_t bRed;/**<float cac poly coefficients*/
    uint16_t bBlue;/**<float cac poly coefficients*/
    uint16_t cRed;/**<float cac poly coefficients*/
    uint16_t cBlue;/**<float cac poly coefficients*/
    uint8_t xNs;/**<float cac poly coefficients*/
    uint8_t xNf;/**<float cac poly coefficients*/
    uint8_t yNs;/**<float cac poly coefficients*/
    uint8_t yNf;/**<float cac poly coefficients*/
} CamerIcIspDmsc2CacConfig_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure the horizontal clip mode
 *
 * @note    Defines the maximum red/blue pixel shift in horizontal direction
 *          At pixel positions, that require a larger displacement, the maximum
 *          shift value is used instead (vector clipping)
 *
 *****************************************************************************/
typedef enum CamerIcIspDmsc2CacHorizontalClipMode_e
{
    CAMERIC_DMSC2_CAC_H_CLIPMODE_INVALID  = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_DMSC2_CAC_H_CLIPMODE_FIX4     = 1,    /**< horizontal vector clipping +/-4 pixel displacement (default) */
    CAMERIC_DMSC2_CAC_H_CLIPMODE_DYN5     = 2,    /**< horizontal vector clipping to +/-4 or +/-5 pixel displacement
                                                 depending on pixel position inside the bayer raster (dynamic
                                                 switching between +/-4 and +/-5) */
    CAMERIC_DMSC2_CAC_H_CLIPMODE_MAX              /**< upper border (only for an internal evaluation) */
} CamerIcIspDmsc2CacHorizontalClipMode_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure the vertical clip mode
 *
 * @note    Defines the maximum red/blue pixel shift in vertical direction
 *
 *****************************************************************************/
typedef enum CamerIcIspDmsc2CacVerticalClipMode_e
{
    CAMERIC_DMSC2_CAC_V_CLIPMODE_INVALID  = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_DMSC2_CAC_V_CLIPMODE_FIX2     = 1,    /**< vertical vector clipping to +/-2 pixel */
    CAMERIC_DMSC2_CAC_V_CLIPMODE_FIX3     = 2,    /**< vertical vector clipping to +/-3 pixel */
    CAMERIC_DMSC2_CAC_V_CLIBMODE_DYN4     = 3,    /**< vertical vector clipping +/-3 or +/-4 pixel displacement
                                                 depending on pixel position inside the bayer raster (dynamic
                                                 switching between +/-3 and +/-4) */
    CAMERIC_DMSC2_CAC_V_CLIPMODE_MAX              /**< upper border (only for an internal evaluation) */
} CamerIcIspDmsc2CacVerticalClipMode_t;

/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic module.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2Enable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic module.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2Disable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP Demosaic interplolation configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2GetCfg
(
    CamerIcDrvHandle_t  handle,
    CamerIcIspDmsc2Config_t *pIntp
);

/*****************************************************************************/
/**
 * @brief   This function config the CamerIc ISP Demosaic interplolation configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SetCfg
(
    CamerIcDrvHandle_t  handle,
    const CamerIcIspDmsc2Config_t *pIntp
);


/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic demoire module.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DemoireEnable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic demoire module.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DemoireDisable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP Demosaic demoire configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DemoireGetCfg
(
    CamerIcDrvHandle_t  handle,
    CamerIcIspDmsc2Demoire_t *pDemoircCfg
);

/*****************************************************************************/
/**
 * @brief   This function config the CamerIc ISP Demosaic demoire configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DemoireSetCfg
(
    CamerIcDrvHandle_t  handle,
    const CamerIcIspDmsc2Demoire_t *pDemoircCfg
);

/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic sharpen.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenEnable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic sharpen.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenDisable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP Demosaic sharpen configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenGetCfg(CamerIcDrvHandle_t handle, CamerIcIspDmsc2Sharpen_t *pSharpenCfg);

/*****************************************************************************/
/**
 * @brief   This function config the CamerIc ISP Demosaic sharpen.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenSetCfg(CamerIcDrvHandle_t handle, const CamerIcIspDmsc2Sharpen_t *pSharpenCfg);

/*****************************************************************************/
/**
 * @brief   This function config the CamerIc ISP Demosaic skin.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SkinSetCfg
(
    CamerIcDrvHandle_t  handle,
    CamerIcIspDmsc2Skin_t *pSkinCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP Demosaic skin configure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SkinGetCfg
(
    CamerIcDrvHandle_t  handle,
    CamerIcIspDmsc2Skin_t *pSkinCfg
);

/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic skin.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SkinEnable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic skin.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SkinDisable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP Demosaic status.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2IsEnabled
(
    CamerIcDrvHandle_t          handle,
    bool_t  *pEnabled

);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP green filter config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2GreenFilterSetCfg
(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2GFilter_t *pGfilter
);

/*****************************************************************************/
/**
 * @brief   This function set the CamerIc ISP green filter config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2GreenFilterGetCfg(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2GFilter_t *pGfilter
) ;

/*****************************************************************************/
/**
 * @brief   This function set the CamerIc ISP SharpenLine config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2SharpenLineSetCfg
(
    CamerIcDrvHandle_t handle, 
    const CamerIcIspDmsc2SharpenLine_t *pSharpenLine
);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP SharpenLine config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2SharpenLineGetCfg
(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2SharpenLine_t *pSharpenLine
);

/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic SharpenLine.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenLineEnable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic SharpenLine.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2SharpenLineDisable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function set the CamerIc ISP depurple config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2DepurpleSetCfg
(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2Depurple_t *pDepurple
);

/*****************************************************************************/
/**
 * @brief   This function get the CamerIc ISP depurple config.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2DepurpleGetCfg
(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2Depurple_t *pDepurple
);

/*****************************************************************************/
/**
 * @brief   This function enable the CamerIc ISP Demosaic depurple.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DepurpleEnable(CamerIcDrvHandle_t handle);

/*****************************************************************************/
/**
 * @brief   This function disable the CamerIc ISP Demosaic depurple.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDmsc2DepurpleDisable(CamerIcDrvHandle_t handle);


/*****************************************************************************/
/**
 * @brief   This function configures the CAC module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             Configuration of the CAC module (@ref
 *                              CamerIcIspDmsc2CacConfig_t)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacSetConfig
(
    CamerIcDrvHandle_t      handle,
    const CamerIcIspDmsc2CacConfig_t   *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function reads out the current configred chromatic aberration
 *          correction configuration.
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             Pointer to store the current configuration
 *                              (@ref  CamerIcIspDmsc2CacConfig_t)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_NULL_POINTER    pLscConfig is a NULL pointer
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacGetConfig
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspDmsc2CacConfig_t           *pConfig
);


/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC ISP CAC module.
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 *          CamerIcIspDmsc2CacDisable()
 *
 * @brief   Disable CAC Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This function
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_NULL_POINTER    pIsEnabled is NULL pointer
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
 * @brief   calculations for square, non-suare pixel mode and normalization
 * values (non-square pixel sensor)
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             Pointer to value to config context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_NULL_POINTER    pIsEnabled is NULL pointer
 *
 *****************************************************************************/
RESULT CamerIcIspDmsc2CacComputeNsNf(
    CamerIcDrvHandle_t handle,
    CamerIcIspDmsc2CacConfig_t * pConfig
);
#ifdef __cplusplus
}
#endif

/* @} cameric_isp_dmsc2_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_DMSC2_DRV_API_H */
