/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */

#ifndef CAMERIC_ISP_DPF_DRV_API_H
#define CAMERIC_ISP_DPF_DRV_API_H

/**
 * @file    cameric_isp_dpf_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP DPF driver API definitions.
 *
 *****************************************************************************/
/**
 * @cond cameric_isp_dpf
 *
 * @defgroup cameric_isp_dpf_drv_api CamerIC ISP DPF Driver API definitions
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/list.h>

#ifdef __cplusplus
extern "C"
{
#endif




#define CAMERIC_DPF_MAX_NLF_COEFFS      17
#define CAMERIC_DPF_MAX_SPATIAL_COEFFS  6

#define SPATIAL_WEIGHT_COEFF_MAX 16
#define SPATIAL_WEIGHT_COEFF_MIN 0
#define STRENGTH_WEIGHT_MAX 255
#define STRENGTH_WEIGHT_MIN 0
#define NLL_COEFF_MAX 1023
#define NLL_COEFF_MIN 0

#define CAMERIC_NF_GAIN_RED_DEFAULT 256
#define CAMERIC_NF_GAIN_GREEN_R_DEFAULT 256
#define CAMERIC_NF_GAIN_GREEN_B_DEFAULT 256
#define CAMERIC_NF_GAIN_BLUE_DEFAULT 256

/*****************************************************************************/
/**
 * @brief   This type defines the supported filter kernel sizes for the red
 *          and blue channels.
 */
/*****************************************************************************/
typedef enum CamerIcDpfRedBlueFilterSize_e
{
    CAMERIC_DPF_RB_FILTERSIZE_INVALID   = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_DPF_RB_FILTERSIZE_9x9       = 1,    /**< red and blue filter kernel size 9x9 (means 5x5 active pixel) */
    CAMERIC_DPF_RB_FILTERSIZE_13x9      = 2,    /**< red and blue filter kernel size 13x9 (means 7x5 active pixel) */
    CAMERIC_DPF_RB_FILTERSIZE_MAX               /**< upper border (only for an internal evaluation) */
} CamerIcDpfRedBlueFilterSize_t;



/*****************************************************************************/
/**
 * @brief   This type defines the supported gain usage modes in the DPF
 *          preprocessing stage.
 */
/*****************************************************************************/
typedef enum CamerIcDpfGainUsage_e
{
    CAMERIC_DPF_GAIN_USAGE_INVALID       = 0,   /**< lower border (only for an internal evaluation) */
    CAMERIC_DPF_GAIN_USAGE_DISABLED      = 1,   /**< don't use any gains in preprocessing stage */
    CAMERIC_DPF_GAIN_USAGE_NF_GAINS      = 2,   /**< use only the noise function gains  from registers DPF_NF_GAIN_R, ... */
    CAMERIC_DPF_GAIN_USAGE_LSC_GAINS     = 3,   /**< use only the gains from LSC module */
    CAMERIC_DPF_GAIN_USAGE_NF_LSC_GAINS  = 4,   /**< use the moise function gains and the gains from LSC module */
    CAMERIC_DPF_GAIN_USAGE_AWB_GAINS     = 5,   /**< use only the gains from AWB module */
    CAMERIC_DPF_GAIN_USAGE_AWB_LSC_GAINS = 6,   /**< use the gains from AWB and LSC module */
    CAMERIC_DPF_GAIN_USAGE_MAX                  /**< upper border (only for an internal evaluation) */
} CamerIcDpfGainUsage_t;



/*****************************************************************************/
/**
 * @brief   This type defines the supported scaling of x axis.
 */
/*****************************************************************************/
typedef enum CamerIcDpfNoiseLevelLookUpScale_e
{
    CAMERIC_NLL_SCALE_INVALID       = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_NLL_SCALE_LINEAR        = 1,        /**< use a linear scaling */
    CAMERIC_NLL_SCALE_LOGARITHMIC   = 2,        /**< use a logarithmic scaling */
    CAMERIC_NLL_SCALE_MAX                       /**< upper border (only for an internal evaluation) */
} CamerIcDpfNoiseLevelLookUpScale_t;



/*****************************************************************************/
/**
 * @brief   This type defines the
 */
/*****************************************************************************/
typedef struct CamerIcDpfInvStrength_s
{
    uint8_t weightR;
    uint8_t weightG;
    uint8_t weightB;
} CamerIcDpfInvStrength_t;



/*****************************************************************************/
/**
 * @brief
 */
/*****************************************************************************/
typedef struct CamerIcDpfNoiseLevelLookUp_s
{
    uint16_t                            nllCoeff[CAMERIC_DPF_MAX_NLF_COEFFS];   /**< Noise-Level-Lookup coefficients */
    CamerIcDpfNoiseLevelLookUpScale_t   xScale;                                 /**< type of x-axis (logarithmic or linear type) */
} CamerIcDpfNoiseLevelLookUp_t;



/*****************************************************************************/
/**
 * @brief   This type defines the supported filter kernel sizes for the red
 *          and blue channels.
 */
/*****************************************************************************/
typedef struct CamerIcDpfSpatial_s
{
    uint8_t weightCoeff[CAMERIC_DPF_MAX_SPATIAL_COEFFS];
} CamerIcDpfSpatial_t;



/*****************************************************************************/
/**
 * @brief   This type defines the configuration structure of the CamerIc
 *          DPF module.
 */
/*****************************************************************************/
typedef struct CamerIcDpfConfig_s
{
    CamerIcDpfGainUsage_t           gainUsage;              /**< which gains shall be used in preprocessing stage of dpf module */

    CamerIcDpfRedBlueFilterSize_t   rBFilterSize;           /**< size of filter kernel for red/blue pixel */

    bool_t                          processRedPixel;        /**< enable filter processing for red pixel */
    bool_t                          processGreenRPixel;     /**< enable filter processing for green pixel in red lines */
    bool_t                          processGreenBPixel;     /**< enable filter processing for green pixel in blue lines */
    bool_t                          processBluePixel;       /**< enable filter processing for blux pixel */

    CamerIcDpfSpatial_t             spatialG;               /**< spatial weights for green pixel */
    CamerIcDpfSpatial_t             spatialRB;              /**< spatial weights for red/blue pixel */
} CamerIcIspDpfConfig_t;



/*****************************************************************************/
/**
 * @brief   This function enables the CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfDisable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 * @brief   This functions returns the current status of the CamerIc ISP
 *          DPF module.
 *
 * @param   handle          CamerIc driver handle.
 * @param   pIsEnabled      DPF status.
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);


/*****************************************************************************/
/**
 * @brief   This function configures the CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfCfg             DPF configuration.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfSetConfig
(
    CamerIcDrvHandle_t          handle,
    const CamerIcIspDpfConfig_t    *pDpfCfg
);

/*****************************************************************************/
/**
 * @brief   This function get the configures of CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfCfg             DPF configuration.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfGetConfig
(
    CamerIcDrvHandle_t       handle,
    CamerIcIspDpfConfig_t    *pDpfCfg
);

/*****************************************************************************/
/**
 * @brief   This function sets the noise function gains to the CamerIc ISP
 *          DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pNfGains            DPF noise funciton gains.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfSetNoiseFunctionGain
(
    CamerIcDrvHandle_t      handle,
    const CamerIcGains_t    *pNfGains
);

/*****************************************************************************/
/**
 * @brief   This function gets the noise function gains to the CamerIc ISP
 *          DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pNfGains            DPF noise funciton gains.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfGetNoiseFunctionGain
(
    CamerIcDrvHandle_t      handle,
    CamerIcGains_t          *pNfGains
);

/*****************************************************************************/
/**
 * @brief   Programs the given inverse strength to the CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfStrength        DPF inverse strength.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfSetStrength
(
    CamerIcDrvHandle_t              handle,
    const CamerIcDpfInvStrength_t   *pDpfStrength
);

/*****************************************************************************/
/**
 * @brief   Get the inverse strength of the CamerIc ISP DPF module.
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfStrength        DPF inverse strength.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfGetStrength
(
    CamerIcDrvHandle_t              handle,
    CamerIcDpfInvStrength_t         *pDpfStrength
);

/*****************************************************************************/
/**
 * @brief   Programs the given Noise-Level-Lookup-Table to the CamerIc
 *          ISP DPF module
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfNll             DPF Noise-Level-Lookup-Table.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfSetNoiseLevelLookUp
(
    CamerIcDrvHandle_t                  handle,
    const CamerIcDpfNoiseLevelLookUp_t  *pDpfNll
);

/*****************************************************************************/
/**
 * @brief   get the Noise-Level-Lookup-Table of the CamerIc
 *          ISP DPF module
 *
 * @param   handle              CamerIc driver handle.
 * @param   pDpfNll             DPF Noise-Level-Lookup-Table.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_NULL_POINTER    NULL pointer detected
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfGetNoiseLevelLookUp
(
    CamerIcDrvHandle_t                  handle,
    CamerIcDpfNoiseLevelLookUp_t        *pDpfNll
);



#ifdef __cplusplus
}
#endif

/* @} cameric_isp_dpf_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_DPF_DRV_API_H */

