/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_DRV_API_H
#define CAMERIC_ISP_DRV_API_H

/**
 * @file    cameric_isp_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP driver API.
 *
 *****************************************************************************/
/**
 * @if CAMERIC_LITE
 *
 * @defgroup cameric_isp_drv_api CamerIC ISP lite driver API definitions
 * @{
 *
 * @image html isp_lite.png "Software ISP-lite driver" width=\textwidth
 * @image latex isp_lite.png "Software ISP-lite driver" width=\textwidth
 *
 * @endif
 *
 * @if CAMERIC_FULL
 *
 * @defgroup cameric_isp_drv_api CamerIC ISP driver API definitions
 * @{
 *
 * @image html isp.png "Software ISP driver" width=\textwidth
 * @image latex isp.png "Software ISP driver" width=\textwidth
 *
 * @endif
  *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   Enumeration type to configure the ISP working mode.
 *
 *****************************************************************************/
typedef enum CamerIcIspMode_e
{
    CAMERIC_ISP_MODE_INVALID                = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_MODE_RAW                    = 1,        /**< RAW picture with BT.601 sync (ISP bypass) */
    CAMERIC_ISP_MODE_656                    = 2,        /**< ITU-R BT.656 (YUV with embedded sync) */
    CAMERIC_ISP_MODE_601                    = 3,        /**< ITU-R BT.601 (YUV input with H and Vsync signals) */
    CAMERIC_ISP_MODE_BAYER_RGB              = 4,        /**< Bayer RGB processing with H and Vsync signals */
    CAMERIC_ISP_MODE_DATA                   = 5,        /**< data mode (ISP bypass, sync signals interpreted as data enable) */
    CAMERIC_ISP_MODE_RGB656                 = 6,        /**< Bayer RGB processing with BT.656 synchronization */
    CAMERIC_ISP_MODE_RAW656                 = 7,        /**< RAW picture with ITU-R BT.656 synchronization (ISP bypass) */
    CAMERIC_ISP_MODE_MAX                                /**< upper border (only for an internal evaluation) */
} CamerIcIspMode_t;
/* @endcond */



/******************************************************************************/
/**
 * @brief   Enumeration type to configure the edge sampling in ISP
 *          input acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspSampleEdge_e
{
    CAMERIC_ISP_SAMPLE_EDGE_INVALID         = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_SAMPLE_EDGE_FALLING         = 1,        /**< sample falling edga */
    CAMERIC_ISP_SAMPLE_EDGE_RISING          = 2,        /**< sample rising edge */
    CAMERIC_ISP_SAMPLE_EDGE_MAX                         /**< upper border (only for an internal evaluation) */
} CamerIcIspSampleEdge_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure the vertical and horizontal
 *          polarity in ISP input acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspPolarity_e
{
    CAMERIC_ISP_POLARITY_INVALID            = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_POLARITY_HIGH               = 1,        /**< high active */
    CAMERIC_ISP_POLARITY_LOW                = 2,        /**< low active */
    CAMERIC_ISP_POLARITY_MAX                            /**< upper border (only for an internal evaluation) */
} CamerIcIspPolarity_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure the bayer pattern in the ISP
 *          input acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspBayerPattern_e
{
    CAMERIC_ISP_BAYER_PATTERN_INVALID       = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_BAYER_PATTERN_RGRGGBGB      = 1,        /**< 1st line: RGRG... , 2nd line GBGB... , etc. */
    CAMERIC_ISP_BAYER_PATTERN_GRGRBGBG      = 2,        /**< 1st line: GRGR... , 2nd line BGBG... , etc. */
    CAMERIC_ISP_BAYER_PATTERN_GBGBRGRG      = 3,        /**< 1st line: GBGB... , 2nd line RGRG... , etc. */
    CAMERIC_ISP_BAYER_PATTERN_BGBGGRGR      = 4,        /**< 2st line: BGBG... , 2nd line GRGR... , etc. */
    CAMERIC_ISP_BAYER_PATTERN_MAX                       /**< upper border (only for an internal evaluation) */
} CamerIcIspBayerPattern_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure the color subsampling mode in the
 *          ISP input acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspColorSubsampling_e
{
    CAMERIC_ISP_CONV422_INVALID             = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_CONV422_COSITED             = 1,        /**< co-sited color subsampling Y0Cb0Cr0 - Y1 */
    CAMERIC_ISP_CONV422_INTERLEAVED         = 2,        /**< interleaved color subsampling Y0Cb0 - Y1Cr1 (not recommended) */
    CAMERIC_ISP_CONV422_NONCOSITED          = 3,        /**< non-cosited color subsampling Y0Cb(0+1)/2 - Y1Cr(0+1)/2 */
    CAMERIC_ISP_CONV422_MAX                             /**< upper border (only for an internal evaluation) */
} CamerIcIspColorSubsampling_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure CCIR sequence in the ISP input
 *          acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspCCIRSequence_e
{
    CAMERIC_ISP_CCIR_SEQUENCE_INVALID       = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_CCIR_SEQUENCE_YCbYCr        = 1,        /**< YCbYCr */
    CAMERIC_ISP_CCIR_SEQUENCE_YCrYCb        = 2,        /**< YCrYCb */
    CAMERIC_ISP_CCIR_SEQUENCE_CbYCrY        = 3,        /**< CbYCrY */
    CAMERIC_ISP_CCIR_SEQUENCE_CrYCbY        = 4,        /**< CrYCbY */
    CAMERIC_ISP_CCIR_SEQUENCE_MAX                       /**< upper border (only for an internal evaluation) */
} CamerIcIspCCIRSequence_t;


/******************************************************************************/
/**
 * @brief   Enumeration type to configure field sampling in the ISP input
 *          acquisition.
 *
 *****************************************************************************/
typedef enum CamerIcIspFieldSelection_e
{
    CAMERIC_ISP_FIELD_SELECTION_INVALID     = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_FIELD_SELECTION_BOTH        = 1,        /**< sample all fields (don't care about fields) */
    CAMERIC_ISP_FIELD_SELECTION_EVEN        = 2,        /**< sample only even fields */
    CAMERIC_ISP_FIELD_SELECTION_ODD         = 3,        /**< sample only odd fields */
    CAMERIC_ISP_FIELD_SELECTION_MAX                     /**< upper border (only for an internal evaluation) */
} CamerIcIspFieldSelection_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to configure camera bus width in the CamerIC.
 *
 *****************************************************************************/
typedef enum CamerIcIspInputSelection_e
{
    CAMERIC_ISP_INPUT_INVALID               = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_INPUT_12BIT                 = 1,        /**< 12 Bit input */
    CAMERIC_ISP_INPUT_10BIT_ZZ              = 2,        /**< 10 Bit input with 2 zereos as LSB's */
    CAMERIC_ISP_INPUT_10BIT_EX              = 3,        /**< 10 Bit input with 2 MSB's as LSB's */
    CAMERIC_ISP_INPUT_8BIT_ZZ               = 4,        /**< 8 Bit input with 4 zeroes as LSB's */
    CAMERIC_ISP_INPUT_8BIT_EX               = 5,        /**< 8 Bit input with 4 MSB's as LSB's */
    CAMERIC_ISP_INPUT_MAX                               /**< upper border (only for an internal evaluation) */
} CamerIcIspInputSelection_t;



/******************************************************************************/
/**
 * @brief   Enumeration type to  configure the input interface selector on
 *          the CamerIC input (if_select).
 *
 *****************************************************************************/
typedef enum CamerIcIspLatencyFifo_e
{
    CAMERIC_ISP_LATENCY_FIFO_INVALID            = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_LATENCY_FIFO_INPUT_FORMATTER    = 1,    /**< use input formatter input for latency fifo */
    CAMERIC_ISP_LATENCY_FIFO_DMA_READ           = 2,    /**< use dma rgb read input for latency fifo */
    CAMERIC_ISP_LATENCY_FIFO_MAX                        /**< upper border (only for an internal evaluation) */
} CamerIcIspLatencyFifo_t;


/******************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   Enumeration type to configure configure the demosaicing bypass.
 *
 *****************************************************************************/
typedef enum CamerIcIspDemosaicBypass_e
{
    CAMERIC_ISP_DEMOSAIC_INVALID            = 0,        /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_DEMOSAIC_NORMAL_OPERATION   = 1,        /**< normal operation for RGB Bayer pattern input */
    CAMERIC_ISP_DEMOSAIC_BYPASS             = 2,        /**< demosaicing bypass for Black&White input data */
    CAMERIC_ISP_DEMOSAIC_MAX                            /**< upper border (only for an internal evaluation) */
} CamerIcIspDemosaicBypass_t;
/* @endcond */



/******************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   Enumeration type to configure configure the color conversion range.
 *
 * @note    The color conversion unit - also called color space matrix -
 *          performs a regular RGB to YCbCr 4:4:4 color space conversion. The
 *          nine coefficients take values between -2 and +1.992.
 *
 *          \arg YCbCr range limited output \n
 *          Y    = c0*R + c1*G + c2*B + 64  \n
 *          Cb   = c3*R + c4*G + c5*B + 512 \n
 *          Cr   = c6*R + c7*G + c8*B + 512 \n
 *
 *          \arg YCbCr full range output    \n
 *          Y    = c0*R + c1*G + c2*B       \n
 *          Cb   = c3*R + c4*G + c5*B + 512 \n
 *          Cr   = c6*R + c7*G + c8*B + 512 \n
 *
 *****************************************************************************/
typedef enum CamerIcColorConversionRange_e
{
    CAMERIC_ISP_CCONV_RANGE_INVALID          = 0,       /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_CCONV_RANGE_LIMITED_RANGE    = 1,       /**< YCbCr range limited output according to ITU-R BT.601 standard */
    CAMERIC_ISP_CCONV_RANGE_FULL_RANGE       = 2,       /**< YCbCr full range output */
    CAMERIC_ISP_CCONV_RANGE_MAX                         /**< upper border (only for an internal evaluation) */
} CamerIcColorConversionRange_t;
/* @endcond */



/******************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   Enumeration type to configure the gamma curve segementation.
 *
 *****************************************************************************/
typedef enum CamerIcIspGammaSegmentationMode_e
{
    CAMERIC_ISP_SEGMENTATION_MODE_INVALID       = 0,    /**< lower border (only for an internal evaluation) */
    CAMERIC_ISP_SEGMENTATION_MODE_LOGARITHMIC   = 1,    /**< logarithmic segmentation from 0 to 4095
                                                             (64,64,64,64,128,128,128,128,256,256,256,512,512,512,512,512) */
    CAMERIC_ISP_SEGMENTATION_MODE_EQUIDISTANT   = 2,    /**< equidistant segmentation from 0 to 4095
                                                             (256, 256, ... ) */
    CAMERIC_ISP_SEGMENTATION_MODE_MAX                   /**< upper border (only for an internal evaluation) */
} CamerIcIspGammaSegmentationMode_t;
/* @endcond */


#define DUMP_HIST_SHIFT 0U
#define DUMP_EXPM_SHIFT 1U
#define DUMP_WBM_SHIFT 2U
#define DUMP_AFM_SHIFT 3U
#define DUMP_VSM_SHIFT  4U
#define DUMP_AEV2_SHIFT 5U
#define DUMP_HIST64_SHIFT  6U
#define DUMP_PDAF_SHIFT 7U
#define DUMP_STA_SHIFT  8U
#define DUMP_GTM_SHIFT  9U
#define DUMP_EXPV3_SHIFT  10U
#define DUMP_AFMV3_SHIFT  11U


typedef enum CamerIcIspDvpPinMappig_s {
    CAMERIC_ISP_DVP_PIN_MAPPING_12TO12 = 0,
    CAMERIC_ISP_DVP_PIN_MAPPING_L10TOH10,
    CAMERIC_ISP_DVP_PIN_MAPPING_L8TOH8,
    CAMERIC_ISP_DVP_PIN_MAPPING_M8TOH8,
    CAMERIC_ISP_DVP_PIN_MAPPING_MAX,
} CamerIcIspDvpPinMappig_t;


/*****************************************************************************/
/**
 * @brief   3a statistics source mode.
 *
 *****************************************************************************/
typedef enum CamerIc3aStatisticSel_s
{
    CAMERIC_3A_STATISTIC_XTALK_MODE_INVALID = -1,    /**< invalid mode (only for initialization) */
    CAMERIC_3A_STATISTIC_AFTER_XTALK = 0,    /**< get 3a statistic after coclor corection */
    CAMERIC_3A_STATISTIC_BEFORE_XTALK = 1,    /**< get 3a statistic before coclor corection */
    CAMERIC_3A_STATISTIC_XTALK_MODE_MAX,
} CamerIc3aStatisticSel_t;

/*****************************************************************************/
/**
 * @brief   This function registers a Request-Callback at the CamerIC ISP
 *          Module. A request callback is called if the driver needs an
 *          interaction from the application layer (i.e. a new data buffer
 *          to fill, please also @ref CamerIcRequestId_e).
 *
 * @param   handle              CamerIC driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    a parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT CamerIcIspRegisterRequestCb
(
    CamerIcDrvHandle_t      handle,
    CamerIcRequestFunc_t    func,
    void                    *pUserContext
);



/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Request-Callback
 *          at CamerIc ISP Module.
 *
 * @param   handle              CamerIC driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              request callback
 *
 *****************************************************************************/
extern RESULT CamerIcIspDeRegisterRequestCb
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This functions registers an Event-Callback at CamerIC ISP
 *          Module. An event callback is called if the driver needs to
 *          inform the application layer about an asynchronous event or
 *          an error situation (see @ref CamerIcEventId_e).
 *
 * @param   handle              CamerIC driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    a parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              event callback (maybe the driver is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspRegisterEventCb
(
    CamerIcDrvHandle_t  handle,
    CamerIcEventFunc_t  func,
    void                *pUserContext
);



/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Event-Callback
 *          at CamerIC ISP Module.
 *
 * @param   handle              CamerIC driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              event callback (maybe the driver is still running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspDeRegisterEventCb
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function will enable CamerIC ISP Module.
 *
 * @param   handle              CamerIC driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  true                operation succeded
 * @retval  false               operation failed
 *
 *****************************************************************************/
extern bool_t CamerIcIsIspEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function configures the ISP working mode.
 *
 * @param   handle              CamerIC driver handle
 * @param   mode                new working mode
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to configure working
 *                              mode (maybe the driver is already running)
 * @retval  RET_NOTSUPP         selected working mode is not supported
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetMode
(
    CamerIcDrvHandle_t      handle,
    const CamerIcIspMode_t  mode
);
/* @endcond */

#if 1 //TODO Review remove to MCM API if need
/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP Mcm write.
 *
 * @param   handle              CamerIC driver handle
 * @param   width               mcm write width
 * @param   height              mcm write height
 * @param   format              mcm write format
 * @param   path                mcm write path
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspCfgMcmWr
(
    CamerIcDrvHandle_t              handle,
    uint16_t width,
    uint16_t height,
    uint16_t  format,
    uint16_t path
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP Mcm g2 channel write .
 *
 * @param   handle              CamerIC driver handle
 * @param   width               mcm g2 channel write width
 * @param   height              mcm g2 channel write height
 * @param   format              mcm g2 channel write format
 * @param   path                mcm g2 channel write path
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspCfgMcmG2Wr
(
    CamerIcDrvHandle_t              handle,
    uint16_t width,
    uint16_t height,
    uint16_t  format,
    uint16_t path
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP Mcm rd format .
 *
 * @param   handle              CamerIC driver handle
 * @param   format               mcm rd format
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspSetMcmRdFmt
(
    CamerIcDrvHandle_t              handle,
    uint16_t  format
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP Mcm bypass sensor switch .
 *
 * @param   handle              CamerIC driver handle
 * @param   bypassSwitch        Bypass switch
                                0000: sensor0 bypass
                                0001: sensor1 bypass
                                …
                                1111: sensor15 bypass if any
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspSetMcmBypassSwitch
(
    CamerIcDrvHandle_t              handle,
    uint8_t  bypassSwitch
);


#endif

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input acquisition module.
 *
 * @param   handle              CamerIC driver handle
 * @param   enable              mcm bypass enable
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 *
 *****************************************************************************/
RESULT CamerIcIspSetMcmBypass
(
    CamerIcDrvHandle_t              handle,
    bool_t                          bypass
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input acquisition module.
 *
 * @param   handle              CamerIC driver handle
 * @param   sampleEdge          sample edge (@ref CamerIcIspSampleEdge_e)
 * @param   hSyncPol            horizontal sync polarity (@ref CamerIcIspPolarity_e)
 * @param   vSyncPol            vertical sync polarity (@ref CamerIcIspPolarity_e)
 * @param   bayerPattern        bayer pattern (@ref CamerIcIspBayerPattern_e)
 * @param   subSampling         color subsampling mode (@ref CamerIcIspColorSubsampling_e)
 * @param   seqCCIR             CCIR output sequence (@ref CamerIcIspCCIRSequence_e)
 * @param   fieldSelection      field sampling (@ref CamerIcIspFieldSelection_e)
 * @param   inputSelection      input format (@ref CamerIcIspInputSelection_e)
 * @param   latencyFifo         latency fifo input (@ref CamerIcIspLatencyFifo_e)
 * @param    stitchingMode     stitching mode
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to configure working
 *                              mode (maybe the driver is already running)
 * @retval  RET_NOTSUPP         a configuration is not supported
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetAcqProperties
(
    CamerIcDrvHandle_t                  handle,
    const CamerIcIspSampleEdge_t        sampleEdge,
    const CamerIcIspPolarity_t          hSyncPol,
    const CamerIcIspPolarity_t          vSyncPol,
    const CamerIcIspBayerPattern_t      bayerPattern,
    const CamerIcIspColorSubsampling_t  subSampling,
    const CamerIcIspCCIRSequence_t      seqCCIR,
    const CamerIcIspFieldSelection_t    fieldSelection,
    const CamerIcIspInputSelection_t    inputSelection,
    const CamerIcIspLatencyFifo_t       latencyFifo,
    const uint32_t stitchingMode
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input Bit mapping for Nano.
 *
 * @param   handle              CamerIC driver handle
 * @param   dvpPinMapping       Bit mapping from LSB to MSB
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the input
 *                              window (maybe the driver is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetAcqPropertiesNano
(
    CamerIcDrvHandle_t               handle,
    const CamerIcIspDvpPinMappig_t   dvpPinMapping
);

/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input resolution.
 *
 * @note    It's possible to crop the image by using a smaller resolution.
 *
 * @param   handle              CamerIC driver handle
 * @param   hOffset             horizontal offset
 * @param   vOffset             vertical offset
 * @param   height              height
 * @param   width               width
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the input
 *                              window (maybe the driver is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetAcqResolution
(
    CamerIcDrvHandle_t  handle,
    const uint16_t      hOffset,
    const uint16_t      vOffset,
    const uint16_t      width,
    const uint16_t      height
);


/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP input bayer pattern.
 *
 * @param   handle              CamerIC driver handle
 * @param   bayerPattern        bayer pattern (@ref CamerIcIspBayerPattern_e)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the input
 *                              window (maybe the driver is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetAcqBayerPattern
(
    CamerIcDrvHandle_t                  handle,
    const CamerIcIspBayerPattern_t      bayerPattern
);



/*****************************************************************************/
/**
 * @brief   This function configures the CamerIC ISP output formatter window.
 *
 * @param   handle              CamerIC driver handle
 * @param   hOffset             horizontal offset
 * @param   vOffset             vertical offset
 * @param   height              height
 * @param   width               width
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the output
 *                              window (maybe the driver is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetOutputFormatterResolution
(
    CamerIcDrvHandle_t  handle,
    const uint16_t      hOffset,
    const uint16_t      vOffset,
    const uint16_t      width,
    const uint16_t      height
);


#if 1 //TODO Review does ISP8200 have this module??
/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function sets CamerIc ISP image stabilization output window.
 *
 * @param   handle              CamerIc driver handle
 * @param   hOffset             horizontal offset
 * @param   vOffset             vertical offset
 * @param   height              height
 * @param   width               width
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to change the image
 *                              stabilization outpu window (maybe the driver
 *                              is already running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspSetImageStabilizationResolution
(
    CamerIcDrvHandle_t  handle,
    const uint16_t      hOffset,
    const uint16_t      vOffset,
    const uint16_t      width,
    const uint16_t      height
);
/* @endcond */
#endif

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions enables or disables the ISP AWB module.
 *
 * @param   handle              CamerIc driver handle
 * @param   enable              BOOL_TRUE enables ISP AWB module,
 *                              BOOL_FALSE bypass the ISP AWB module
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to enable or disable
 *                              AWB module
 *
 *****************************************************************************/
extern RESULT CamerIcIspActivateWB
(
    CamerIcDrvHandle_t  handle,
    const bool_t        enable
);
/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions returns wether the ISP AWB module is enable or not.
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current ISP AWB state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspIsWBActivated
(
    CamerIcDrvHandle_t  handle,
    bool_t              *pIsEnabled
);

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions enable the nr_relocation that noise module input is 20bit.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspNrRelocEnable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions disable the nr_relocation that noise module input is 12bit.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern RESULT CamerIcIspNrRelocDisable
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions get the nr_relocation status.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
extern bool_t CamerIcIspNrRelocIsActived
(
    CamerIcDrvHandle_t  handle
);

/*****************************************************************************/
/**
 * @brief   This functions select 3a statistics source before or after color correction.
 *
 * @param   handle              CamerIc driver handle
 * @param   staSel              3a statistics source mode
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to set the cross
 *                              talk offsets (maybe the driver is already
 *                              running)
 *
 *****************************************************************************/
extern RESULT CamerIcIsp3aStatisticSel
(
    CamerIcDrvHandle_t          handle,
    CamerIc3aStatisticSel_t     staSel
);


/*****************************************************************************/
/**
 * @brief   This function enables the CamerIc ISP DPF raw output.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfRawOutputEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc ISP DPF raw output.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpfRawOutputDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function enables the CamerIc ISP DPCC raw output.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpccRawOutputEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function disables the CamerIc ISP DPCC raw output.
 *
 * @param   handle              CamerIc driver handle.
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 *
 *****************************************************************************/
extern RESULT CamerIcIspDpccRawOutputDisable
(
    CamerIcDrvHandle_t handle
);

/* @endcond */
/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions write isp register.
 *
 * @param   handle              CamerIc driver handle
 * @param   addr                register address
 * @param   value               register value
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/

RESULT CamerIcIspWriteReg
(
    CamerIcDrvHandle_t handle,
    uint32_t           addr,
    uint32_t           value
);
/* @endcond */

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions read isp register.
 *
 * @param   handle              CamerIc driver handle
 * @param   addr                register address
 * @param   pValue               register value pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/
RESULT CamerIcIspReadReg
(
    CamerIcDrvHandle_t handle,
    uint32_t           addr,
    uint32_t           *pValue
);
/* @endcond */

#if 1 //TODO Review Is it need?
/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This function dumps measure to file.
 *
 * @param   handle              CamerIc driver handle
 * @param   dump_name           File name of dump
 * @param   dump_mask           dump mask value


 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read gamma-out
 *                              status
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/

/******************************************************************************
 * CamerIcIspDumpMeasureData()
 *****************************************************************************/
extern RESULT CamerIcIspDumpMeasureData
(
    CamerIcDrvHandle_t                      handle,
    const char *                            dump_name,
    uint32_t                            dump_mask
);

/* @endcond */
#endif

/*****************************************************************************/
/**
 * @cond    CAMERIC_FULL
 *
 * @brief   This functions get sensor frame count.
 *
 * @param   handle              CamerIc driver handle
 * @param   pPicBuffer          All the meta data one needs to know about an image buffer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to read AWB state
 *                              (maybe driver not initialized)
 * @retval  RET_INVALID_PARM    invalid parameter
 *
 *****************************************************************************/

RESULT CamerIcIspGetFrameCnt
(
    CamerIcDrvHandle_t handle,
    PicBufMetaData_t   *pPicBuffer
);
/* @endcond */


#ifdef __cplusplus
}
#endif

/* @} cameric_isp_drv_api */

#endif /* CAMERIC_ISP_DRV_API_H */

