/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2022 */
#ifndef CAMERIC_ISP_EEV2_DRV_API_H
#define CAMERIC_ISP_EEV2_DRV_API_H

#include <ebase/types.h>
#include <common/return_codes.h>
#ifdef __cplusplus
extern "C"
{
#endif

#ifdef ISP_EEV2
/*****************************************************************************/
/**
 * @brief   parameter range check macro.
 *
 *****************************************************************************/
#define CAMERIC_ISP_EE_EDGE_GAIN_MIN 0
#define CAMERIC_ISP_EE_EDGE_GAIN_MAX 65535
#define CAMERIC_ISP_EE_SRC_STRENGTH_MIN 0
#define CAMERIC_ISP_EE_SRC_STRENGTH_MAX 255
#define CAMERIC_ISP_EE_STRENGTH_MIN 0
#define CAMERIC_ISP_EE_STRENGTH_MAX 255
#define CAMERIC_ISP_EE_UV_GAIN_MIN 0
#define CAMERIC_ISP_EE_UV_GAIN_MAX 65535
#define CAMERIC_ISP_EE_Y_DOWN_GAIN_MIN 0
#define CAMERIC_ISP_EE_Y_DOWN_GAIN_MAX 65535
#define CAMERIC_ISP_EE_Y_UP_GAIN_MIN 0
#define CAMERIC_ISP_EE_Y_UP_GAIN_MAX 65535

#define CAMERIC_ISP_CURVE_LUT_TABLE_MIN 0
#define CAMERIC_ISP_CURVE_LUT_TABLE_MAX 1024

#define CURVE_DATA_TABLE_LEN  65

/******************************************************************************/
/**
 * @brief   CamerIC EE configuration
 *
 *****************************************************************************/
typedef enum CamerIcIspEeInDataFmtSel_e
{
    CAMERIC_ISP_EE_IN_DATA_FMT_SEL_RGB       = 0,
    CAMERIC_ISP_EE_IN_DATA_FMT_SEL_YUV       = 1,
    CAMERIC_ISP_EE_IN_DATA_FMT_SEL_MAX
} CamerIcIspEeInDataFmtSel_t;

typedef struct CamerIcIspEeConfig_s
{
	uint8_t  								strength;

	uint16_t 								yUpGain;
	uint16_t 								yDownGain;

	uint16_t 								uvGain;
	uint16_t 								edgeGain;

	uint8_t  								srcStrength;
} CamerIcIspEeConfig_t;

typedef enum CamerIcIspCaMode_e
{
    CAMERIC_ISP_CA_MODE0  = 0,    /**<Y mode> */
    CAMERIC_ISP_CA_MODE1  = 1,    /**<Saturation mode> */
    CAMERIC_ISP_CA_MODE2  = 2,    /**< Y & Saturation mode> */
} CamerIcIspCaMode_t;

typedef struct CamerIcIspCaConfig_s
{
    CamerIcIspCaMode_t caMode;
    unsigned short lutChromaTbl[CURVE_DATA_TABLE_LEN];
} CamerIcIspCaConfig_t;

typedef struct CamerIcIspDciConfig_s
{
    unsigned short lutLumaTbl[CURVE_DATA_TABLE_LEN];
} CamerIcIspDciConfig_t;

typedef struct CamerIcIspCurveDx_s
{
    unsigned short lutXTbl[CURVE_DATA_TABLE_LEN];
    unsigned short lutShiftTbl[CURVE_DATA_TABLE_LEN];
} CamerIcIspCurveDx_t;
/*****************************************************************************/
/**
 * @brief   This function configures the EE module.
 *
 * @param   handle              CamerIc driver handle
 * @param   pConfig             Configuration of the EE module (@ref
 *                              CamerIcIspEeConfig_t)
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_OUTOFRANGE      At least one perameter of out range
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspEeSetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspEeConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC ISP EE module.
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspEeEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 *          CamerIcIspEeDisable()
 *
 * @brief   Disable EE Module
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspEeDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This function
 *
 * @param   handle              CamerIc driver handle
 * @param   pIsEnabled          Pointer to value to store current state
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_NULL_POINTER    pIsEnabled is NULL pointer
 *
 *****************************************************************************/
extern RESULT CamerIcIspEeIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

extern RESULT CamerIcIspEeGetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspEeConfig_t   *pConfig
);

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC curve module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCurveEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This functions disables the CamerIC curve module
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCurveDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC color adjust
 *           module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCaEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This functions disables the CamerIC color adjust
 *            module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspCaDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   Get CamerIC color adjust module status.
 *
 * @param   handle          CamerIc driver handle.
 * @param   pIsEnabled
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspCaIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

extern RESULT CamerIcIspCaSetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspCaConfig_t    *pCaCfg
);

extern RESULT CamerIcIspCaGetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspCaConfig_t    *pCaCfg
);

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC Dci
 *           module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspDciEnable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   This functions disables the CamerIC Dci
 *            module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspDciDisable
(
    CamerIcDrvHandle_t handle
);



/*****************************************************************************/
/**
 * @brief   Get CamerIC Dci module status.
 *
 * @param   handle          CamerIc driver handle.
 * @param   pIsEnabled
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspDciIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

extern RESULT CamerIcIspDciSetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspDciConfig_t   *pDciCfg
);

extern RESULT CamerIcIspDciGetConfig
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspDciConfig_t   *pDciCfg
);

extern RESULT CamerIcIspSetCurveDx
(
    CamerIcDrvHandle_t      handle,
    CamerIcIspCurveDx_t     *pCurveDx
);

#ifdef __cplusplus
}
#endif

/* @} cameric_isp_eev2_drv_api */

/* @endcond */
#endif
#endif /* CAMERIC_ISP_EEV2_DRV_API_H */


