/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_EXPV2_DRV_API_H
#define CAMERIC_ISP_EXPV2_DRV_API_H

/**
 * @cond    cameric_isp_expv2
 *
 * @file    cameric_isp_expv2_drv_api.h
 *
 * @brief   This file contains the CamerIC ISP AE Driver API definitions
 *
 *****************************************************************************/
/**
 * @defgroup cameric_isp_expv2_drv_api CamerIC ISP AE Driver API definitions
 * @{
 *
 * @image html exp.png "Overview of the CamerIC ISP CAC driver" width=\textwidth
 * @image latex exp.png "Overview of the CamerIC ISP CAC driver" width=\textwidth
 *
 * @image html exp_grid.png "Luminance Measurement Grid" width=0.8\textwidth
 * @image latex exp_grid.png "Luminance Measurement Grid" width=0.8\textwidth
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CAMERIC_ISP_EXPV2_GRID_ITEMS  32  /**< number of grid items */
#define CAMERIC_ISP_EXPV2_PIXEL_CHANNEL 4 /**< number of pixel channel */
#define CAMERIC_ISP_EXPV2_DATA_32BIT_EXPAND 4 /**< number of pixel channel */

typedef uint8_t CamericAev2Static_t[CAMERIC_ISP_EXPV2_GRID_ITEMS * CAMERIC_ISP_EXPV2_GRID_ITEMS * CAMERIC_ISP_EXPV2_PIXEL_CHANNEL * CAMERIC_ISP_EXPV2_DATA_32BIT_EXPAND];

typedef uint32_t CamericExpv2Static_t[CAMERIC_ISP_EXPV2_GRID_ITEMS * CAMERIC_ISP_EXPV2_GRID_ITEMS * CAMERIC_ISP_EXPV2_PIXEL_CHANNEL];

typedef enum CamerIcIspExpV2Sel_e
{
    AEV2_INPUT_SEL_DEGAMMA       = 0,
    AEV2_INPUT_SEL_AWBGAIN       = 1,
    AEV2_INPUT_SEL_WDR3          = 2,
    AEV2_INPUT_SEL_LSC           = 3
} CamerIcIspExpV2Sel_t;

typedef enum CamericExpv2StaticType_e
{
    CAMERIC_EXPV2_8BIT_DATA      = 0,
    CAMERIC_EXPV2_16BIT_DATA     = 1,
    CAMERIC_EXPV2_24BIT_DATA     = 2,
} CamericExpv2StaticType_t;

/*****************************************************************************/
/**
 * @brief   This functions registers an Event-Callback at CamerIC ISP histogram
 *          measurement module. An event callback is called if the driver needs
 *          to inform the application layer about an asynchronous event or an
 *          error situation (i.e. please also @see CamerIcEventId_e).
 *
 * @param   handle              CamerIc driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_INVALID_PARM    a parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              event callback (maybe the driver is already
 *                              running)
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2RegisterEventCb
(
    CamerIcDrvHandle_t handle,
    CamerIcEventFunc_t func,
    void               *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Event-Callback
 *          at CamerIc ISP histogram measurement module.
 *
 * @param   handle              CamerIC driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              request callback
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2DeRegisterEventCb
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This functions enables the CamerIC ISP exposure measurement
 *          module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2Enable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This functions disables the CamerIC ISP exposure measurement
 *          module.
 *
 * @param   handle              CamerIc driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2Disable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   Get CamerIC ISP exposure measurement module status.
 *
 * @param   handle          CamerIc driver handle.
 * @param   pIsEnabled
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2IsEnabled
(
    CamerIcDrvHandle_t handle,
    bool_t             *pIsEnabled
);

/*****************************************************************************/
/**
 * @brief   This function sets the position and size of a the measurement
 *          window in the CamerIC ISP exposure measurement module.
 *
 * @note    This function calculates the grid-size by dividing the width and
 *          height by 5.
 *
 * @param   handle              CamerIc driver handle
 * @param   x                   start x position of measuring window
 * @param   y                   start y position of measuring window
 * @param   width               width of measuring window
 * @param   height              height of measuring window
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
extern RESULT CamerIcIspExpV2SetMeasuringWindow
(
    CamerIcDrvHandle_t handle,
    const uint16_t     x,
    const uint16_t     y,
    const uint16_t     width,
    const uint16_t     height
);

/*****************************************************************************/
/**
 * @brief   This function get the position and size of a the measurement
 *          window in the CamerIC ISP exposure measurement module.
 *
 * @note    This function get the calculation value from setting
 *
 * @param   handle              CamerIc driver handle
 * @param   x                   start x position of measuring window
 * @param   y                   start y position of measuring window
 * @param   width               width of measuring window
 * @param   height              height of measuring window
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspExpV2GetMeasuringWindow
(
    CamerIcDrvHandle_t handle,
    uint16_t           *x,
    uint16_t           *y,
    uint16_t           *width,
    uint16_t           *height
);

/*****************************************************************************/
/**
 * @brief   This function sets every channel's weight value
 *
 * @note    This function set the value to hardware
 *
 * @param   handle              CamerIc driver handle
 * @param   rWeight             The weight value of channel R
 * @param   grWeight            The weight value of channel Gr
 * @param   gbWeight            The weight value of channel Gb
 * @param   bWeight             The weight value of channel B
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspExpV2SetWeight
(
    CamerIcDrvHandle_t handle,
    const uint8_t      rWeight,
    const uint8_t      grWeight,
    const uint8_t      gbWeight,
    const uint8_t      bWeight
);

/*****************************************************************************/
/**
 * @brief   This function get every channel's weight value and the inputSelect
 *
 * @note    This function get the value to hardware
 *
 * @param   handle              CamerIc driver handle
 * @param   rWeight             The weight value of channel R
 * @param   grWeight            The weight value of channel Gr
 * @param   gbWeight            The weight value of channel Gb
 * @param   bWeight             The weight value of channel B
 * @param   inputSelect        The exposure value get the isp process step
 * 
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspExpV2GetWeight
(
    CamerIcDrvHandle_t handle,
    uint8_t            *rWeight,
    uint8_t            *grWeight,
    uint8_t            *gbWeight,
    uint8_t            *bWeight,
    uint8_t            *inputSelect
);

RESULT CamerIcIspExpV2SetSizeRatio
(
    CamerIcDrvHandle_t handle,
    float32_t              ratio
);

RESULT CamerIcIspExpV2InputSel
(
    CamerIcDrvHandle_t   handle,
    CamerIcIspExpV2Sel_t inputSelect
);

/*****************************************************************************/
/**
 * @brief   This function set statistic
 *
 * @note    This function set statistic
 *
 * @param   handle              CamerIc driver handle

 * 
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspExpV2SetStatistics
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
 * @brief   This function Get statistic
 *
 * @note    This function Get statistic
 *
 * @param   handle              CamerIc driver handle
 * @param   pExpv2Statistic      The pointer to expv2 statistic value
 * @param   pExpv2StatisticType  The pointer to expv2 statistic type
 * 
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         Configuration successfully applied
 * @retval  RET_INVALID_PARM    invalid window identifier
 * @retval  RET_WRONG_HANDLE    handle is invalid
 *
 *****************************************************************************/
RESULT CamerIcIspExpV2GetStatistics
(
    CamerIcDrvHandle_t       handle,
    CamericExpv2Static_t     *pExpv2Statistic,
    CamericExpv2StaticType_t *pExpv2StatisticType
);
#ifdef __cplusplus
}
#endif

/* @} cameric_isp_exp_drv_api */

/* @endcond */

#endif /* CAMERIC_ISP_EXPV2_DRV_API_H */

