/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

#ifndef CAMERIC_ISP_STITCHING2_V1_DRV_API_H
#define CAMERIC_ISP_STITCHING2_V1_DRV_API_H

/**
* @file    cameric_isp_stitching2_v1_drv_api.h
*
* @brief   This file contains the CamerIC ISP STITCHING driver API definitions.
*
*****************************************************************************/
/**
* @cond cameric_isp_stitching2_v1
*
* @defgroup cameric_isp_stitching2_v1_drv_api CamerIC ISP STITCHING Driver 
* @{
*
*/
#include <ebase/types.h>
#include <common/return_codes.h>
#include "cameric_isp_exp_drv_api.h"
//#include "cameric_isp_hist_drv_api.h"

#define CAMERIC_HDR_EXTEND_BIT_NUM          2  // extend bit
#define CAMERIC_HDR_COLOR_CHANNEL_NUM       3  // three color channel, r/g/b
#define CAMERIC_HDR_COMB_CURVE_NUM          4  // two hdr combination curve
#define CAMERIC_HDR_TRANS_VALUE_NUM         2  // trans start and end value
#define CAMERIC_HDR_RATIO_NUM               2  // ratio greater than 1, smaller than 1
#define CAMERIC_HDR_RATIO_FRAME_NUM         2  // ratio frame L/S, S/VS
#define CAMERIC_HDR_DIGITAL_GAIN_NUM        3  // gain l, gain s, gain vs
#define CAMERIC_HDR_COMPRESS_LUT_NUM        16 // 16 compress lut

/******************************************************************************/
/**
 * @brief   Enumeration type to configure bypass select mode.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingBypassSelectMode_e
{
    CAMERIC_ISP_STITCHING_BYPASS_SELECT_MODE_LONG           = 0,        /**< select long frame */
    CAMERIC_ISP_STITCHING_BYPASS_SELECT_MODE_SHORT          = 1,        /**< select short frame */
    CAMERIC_ISP_STITCHING_BYPASS_SELECT_MODE_VERY_SHORT     = 2,        /**< select very short frame */
    CAMERIC_ISP_STITCHING_BYPASS_SELECT_MODE_MAX                        /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingBypassSelectMode_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure the vertical and horizontal
 *          polarity type in stitching.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingPolarityType_e
{
    CAMERIC_ISP_STITCHING_POLARITY_TYPE_LEVEL              = 0,         /**< signal from sesnro is level */
    CAMERIC_ISP_STITCHING_POLARITY_TYPE_PULSE              = 1,         /**< signal from sensor is pulse */
    CAMERIC_ISP_STITCHING_POLARITY_TYPE_MAX                             /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingPolarityType_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure the bayer pattern in the
 *          stitching.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingBayerPattern_e
{
    CAMERIC_ISP_STITCHING_BAYER_PATTERN_RGRGGBGB      = 0,        /**< 1st line: RGRG... , 2nd line GBGB... , etc. */
    CAMERIC_ISP_STITCHING_BAYER_PATTERN_GRGRBGBG      = 1,        /**< 1st line: GRGR... , 2nd line BGBG... , etc. */
    CAMERIC_ISP_STITCHING_BAYER_PATTERN_GBGBRGRG      = 2,        /**< 1st line: GBGB... , 2nd line RGRG... , etc. */
    CAMERIC_ISP_STITCHING_BAYER_PATTERN_BGBGGRGR      = 3,        /**< 2st line: BGBG... , 2nd line GRGR... , etc. */
    CAMERIC_ISP_STITCHING_BAYER_PATTERN_MAX                       /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingBayerPattern_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure Sensor Type.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingSensorType_e {
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_3DOL_DUAL_DCG         = 0,        /**< dual DCG mode 3x12-bit */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_3DOL                  = 1,        /**< dol3 frame 3x12-bit */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_3DOL_LINEBYLINE       = 2,        /**< 3x12-bit line by line without waiting */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_2DOL_16BIT_COMPRESS   = 3,        /**< 16-bit compressed data + 12-bit RAW */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_2DOL_DUAL_DCG_NOWAIT  = 4,        /**< 2x12-bit dual DCG without waiting */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_2DOL_L_VS             = 5,        /**< dol2 frame or 1 CG+VS sx12-bit RAW */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_2DOL_L_S              = 6,        /**< L+S 2x12-bit RAW */
    CAMERIC_ISP_STITCHING_SENSOR_HDR_MODE_MAX                               /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingSensorType_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to stitich channel path.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingChannel_e
{
    CAMERIC_ISP_STITCHING_CHANNEL_LONG           = 0,        /**< channel long frame */
    CAMERIC_ISP_STITCHING_CHANNEL_SHORT          = 1,        /**< channel short frame */
    CAMERIC_ISP_STITCHING_CHANNEL_VERY_SHORT     = 2,        /**< channel very short frame */
    CAMERIC_ISP_STITCHING_CHANNEL_MODE_MAX                   /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingChannel_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure base frame selection.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingBaseFrameSelection_e
{
    CAMERIC_ISP_STITCHING_BASE_FRAME_SELECTION_SHORT            = 0,        /**< select short frame as base frame */
    CAMERIC_ISP_STITCHING_BASE_FRAME_SELECTION_LONG             = 1,        /**< select long frame as base frame */
    CAMERIC_ISP_STITCHING_BASE_FRAME_SELECTION_MAX                          /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingBaseFrameSelection_t;

/******************************************************************************/
/**
 * @brief   Enumeration type to configure stitching combination mode.
 *
 *****************************************************************************/
typedef enum CamerIcIspStitchingMode_e {
    CAMERIC_ISP_STITCHING_LINEAR_COMBINE            = 0,        /**< linear L+S and LV+VS*/
    CAMERIC_ISP_STITCHING_NONLINEAR_COMBINE         = 1,        /**< nonlinear L+S */
    CAMERIC_ISP_STITCHING_LINEAR_NONLINEAR_COMBINE  = 2,        /**< linear L+S and nonlinear LS+VS */
    CAMERIC_ISP_STITCHING_COMBINATION_MODE_MAX                  /**< upper border (only for an internal evaluation) */
} CamerIcIspStitchingCombineMode_t;

/******************************************************************************/
/**
 * @brief   A structure to represent Stitching Common configuration among different versions.
 *
 * 
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcIspStitchingConfig_s {
    uint16_t                                ratio[CAMERIC_HDR_RATIO_FRAME_NUM][CAMERIC_HDR_RATIO_NUM];             // [L/S, S/VS]
    uint16_t                                transRange[CAMERIC_HDR_COMB_CURVE_NUM][CAMERIC_HDR_TRANS_VALUE_NUM];   // [[L+S_ref(L)_start, L+S_ref(L)_end],  
                                                                                                                   //  [LS+VS_ref(LS)_start, LS+VS_ref(LS)_end], 
                                                                                                                   //  [L+S_ref(S)_start, L+S+ref(S)_end], 
                                                                                                                   //  [LS+VS_ref(VS)_start, LS+VS_ref(VS)_end]
    uint8_t                                 colorWeight[CAMERIC_HDR_COLOR_CHANNEL_NUM];                            // [R weight, G weight, B weight]
    CamerIcIspStitchingBypassSelectMode_t   bypassSelectMode;
    CamerIcIspStitchingBaseFrameSelection_t baseFrameSelection;
    CamerIcIspStitchingCombineMode_t        stitchingMode;
    int8_t                                  extendBit[CAMERIC_HDR_EXTEND_BIT_NUM];                                 // [SHORT_LS_EXTEND_BIT, SHORT_VS_EXTEND_BIT]
} CamerIcIspStitchingConfig_t;

/******************************************************************************/
/**
 * @brief   A structure to represent Stitching Properties.
 *
 * 
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcIspStitchingProperties_s {
    uint16_t                                dummyShortHBlank;
    uint16_t                                dummyVsHBlank;
    CamerIcIspStitchingPolarityType_t       hSyncPol;
    CamerIcIspStitchingPolarityType_t       vSyncPol;
} CamerIcIspStitchingProperties_t;

/******************************************************************************/
/**
 * @brief   A structure to represent Stitching Digital Gain.
 *
 * 
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcIspStitchingDgGain_s {
    uint16_t gainR;
    uint16_t gainB;
    uint16_t gainGr;
    uint16_t gainGb;
} CamerIcIspStitchingDgGain_t;

/******************************************************************************/
/**
 * @brief   A structure to represent Stitching black level.
 *
 * 
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcIspStitchingBls_s {
    uint16_t blsA;
    uint16_t blsB;
    uint16_t blsC;
    uint16_t blsD;
} CamerIcIspStitchingBls_t;

/******************************************************************************/
/**
 * @brief   A structure to represent Stitching compress lut.
 *
 * 
 *
 * @note    The values are represented as fix point numbers.
 *
 *****************************************************************************/
typedef struct CamerIcIspStitchingCompressLut_s {
    uint16_t compressLutStartIdx;
    uint16_t compressLut[CAMERIC_HDR_COMPRESS_LUT_NUM];
} CamerIcIspStitchingCompressLut_t;

#ifdef ISP_HDR_2_SENSORS
/******************************************************************************/
/**
 * @brief   Enumeration type to configure bypass select mode.
 *
 *****************************************************************************/
typedef enum CamerIcIspHdrPortId_e {
    CAMERIC_ISP_STITCHING_PORT_ID_0      = 0,        /**< select sensor 0 */
    CAMERIC_ISP_STITCHING_PORT_ID_1      = 1         /**< select sensor 1 */
} CamerIcIspHdrPortId_t;
#endif

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetCombineMode()
*
* @brief   Set STITCHING Module combine mode
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetCombineMode
(
	CamerIcDrvHandle_t                          handle,
    const CamerIcIspStitchingCombineMode_t      stitchingMode
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetCombineMode()
*
* @brief   Get STITCHING Module combine mode
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetCombineMode
(
	CamerIcDrvHandle_t                             handle,
    CamerIcIspStitchingCombineMode_t               *stitchingMode
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingEnable()
*
* @brief   Enable STITCHING Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingEnable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingDisable()
*
* @brief   Disable STITCHING Module
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingDisable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
 *          CamerIcIspStitchingIsEnabled()
 *
 * @brief   is stitching enabled
 *
 * @param   handle          CamerIc driver handle
 *
 * @return                  Return the result of the function call.
 * @retval                  RET_SUCCESS
 * @retval                  RET_FAILURE
 *
 *****************************************************************************/
RESULT CamerIcIspStitchingIsEnabled
(
    CamerIcDrvHandle_t handle,
    bool_t             *pIsEnabled
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingSetProperties()
*
* @brief   configue stitching properties
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetProperties
(
    CamerIcDrvHandle_t                      handle,
    const CamerIcIspStitchingProperties_t 	*pProperties
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetProperties()
*
* @brief   obtain stitching properties
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetProperties
(
    CamerIcDrvHandle_t                  handle,
    CamerIcIspStitchingProperties_t 	*pProperties
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefL()
*
* @brief   set STITCHING Module linear trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefL
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startLinear,
    const uint16_t          factorMulLinear
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefL()
*
* @brief   Get STITCHING Module linear trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefL
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startLinear,
    uint16_t                *factorMulLinear
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefLS()
*
* @brief   set STITCHING Module LS+VS refLS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefLS
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startNonLinear,
    const uint16_t          factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefLS()
*
* @brief   set STITCHING Module non-linear trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefLS
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startNonLinear,
    uint16_t                *factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefS()
*
* @brief   set STITCHING Module L+S refS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefS
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startNonLinear,
    uint16_t                *factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefS()
*
* @brief   set STITCHING Module L+S refS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefS
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startNonLinear,
    const uint16_t          factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefVS()
*
* @brief   set STITCHING Module LS+VS refVS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefVS
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startNonLinear,
    uint16_t                *factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefVS()
*
* @brief   set STITCHING Module LS+VS refVS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefVS
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startNonLinear,
    const uint16_t          factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefLSVS()
*
* @brief   set STITCHING Module LSVS+E3 refLSVS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefLSVS
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startNonLinear,
    uint16_t                *factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefLSVS()
*
* @brief   set STITCHING Module LSVS+E3 refLSVS trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefLSVS
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startNonLinear,
    const uint16_t          factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetTransRangeRefE3()
*
* @brief   set STITCHING Module LSVS+E3 refE3 trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetTransRangeRefE3
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *startNonLinear,
    uint16_t                *factorMulNonLinear
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetTransRangeRefE3()
*
* @brief   set STITCHING Module LSVS+E3 refE3 trans range
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetTransRangeRefE3
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          startNonLinear,
    const uint16_t          factorMulNonLinear
);



/*****************************************************************************/
/**
*          CamerIcIspStitchingSetLSRatio()
*
* @brief   set STITCHING Module L and S Ratio
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetLSRatio
(
	CamerIcDrvHandle_t 		handle,
	const uint16_t			ratio0,
	const uint16_t			ratio1
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetLSRatio()
*
* @brief   get STITCHING Module L and S Ratio
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetLSRatio
(
	CamerIcDrvHandle_t 	handle,
	uint16_t			*ratio0,
	uint16_t			*ratio1
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingSetVSRatio()
*
* @brief   set STITCHING Module VS Ratio
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetVSRatio
(
	CamerIcDrvHandle_t		handle,
	const uint16_t			ratio0,
	const uint16_t			ratio1
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetVSRatio()
*
* @brief   get STITCHING Module VS Ratio
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetVSRatio
(
	CamerIcDrvHandle_t 	handle,
	uint16_t			*ratio0,
	uint16_t			*ratio1
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetBlackLevel()
*
* @brief   set stitching black level
*
* @param   handle          CamerIc driver handle
* @param   channel         channel
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetBlackLevel
(
    CamerIcDrvHandle_t               handle,
    CamerIcIspStitchingChannel_t     channel,
    const CamerIcIspStitchingBls_t  *bls
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetBlackLevel()
*
* @brief   get stitching black level
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetBlackLevel
(
    CamerIcDrvHandle_t            handle,
    CamerIcIspStitchingChannel_t  channel,
    CamerIcIspStitchingBls_t     *bls
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetBlackLevelOut()
*
* @brief   set stitching black level Out
*
* @param   handle          CamerIc driver handle
* @param   channel         channel
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetBlackLevelOut
(
    CamerIcDrvHandle_t                  handle,
    const CamerIcIspStitchingBls_t  *bls
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetBlackLevelOut()
*
* @brief   get stitching black level Out
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetBlackLevelOut
(
    CamerIcDrvHandle_t               handle,
    CamerIcIspStitchingBls_t     *bls
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingWbEnable()
*
* @brief   configue stitching white blance enable
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingWbEnable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingWbDisable()
*
* @brief   configue stitching white blance disable
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingWbDisable
(
    CamerIcDrvHandle_t handle
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingWbIsEnabled()
*
* @brief   check the status of stitching Bls 
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingWbIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetAwbGain()
*
* @brief   set STITCHING Module awb gain
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetWbGain
(
    CamerIcDrvHandle_t           handle,
    CamerIcIspStitchingChannel_t channel,
    const CamerIcGains_t         *pGains
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetAwbGain()
*
* @brief   get STITCHING Module awb gain
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetWbGain
(
    CamerIcDrvHandle_t           handle,
    CamerIcIspStitchingChannel_t channel,
    CamerIcGains_t               *pGains
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingDgEnable()
*
* @brief   configue stitching Digitial Gain enable
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingDgEnable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingDgDisable()
*
* @brief   configue stitching Digitial Gain disable
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingDgDisable
(
    CamerIcDrvHandle_t handle
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingDgIsEnabled()
*
* @brief   check the status of stitching Digitial Gain 
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingDgIsEnabled
(
    CamerIcDrvHandle_t      handle,
    bool_t                  *pIsEnabled
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetDgGain()
*
* @brief   set STITCHING Module digital gain
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetDgGain
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspStitchingChannel_t channel,
    const CamerIcIspStitchingDgGain_t *pDigiCfg
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetDgGain()
*
* @brief   get STITCHING Module digital gain
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetDgGain
(
    CamerIcDrvHandle_t              handle,
    CamerIcIspStitchingChannel_t channel,
    CamerIcIspStitchingDgGain_t *pDigiCfg
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingSetColorWeight()
*
* @brief   Set STITCHING Module color weight of R, G, B channel
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetColorWeight
(
	CamerIcDrvHandle_t		handle,
	const uint8_t 			weightR,
	const uint8_t 			weightG,
	const uint8_t 			weightB
);


/*****************************************************************************/
/**
*          CamerIcIspStitchingGetColorWeight()
*
* @brief   Get STITCHING Module color weight of R, G, B channel
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetColorWeight
(
    CamerIcDrvHandle_t      handle,
    uint8_t 			    *weightR,
	uint8_t 			    *weightG,
	uint8_t 			    *weightB
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetBaseFrame()
*
* @brief   Set STITCHING Module base frame
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetBaseFrame
(
	CamerIcDrvHandle_t                                  handle,
    const CamerIcIspStitchingBaseFrameSelection_t       baseFrame
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetBaseFrame()
*
* @brief   Get STITCHING Module base frame
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetBaseFrame
(
	CamerIcDrvHandle_t                            handle,
    CamerIcIspStitchingBaseFrameSelection_t       *baseFrame
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetBypassSelect()
*
* @brief   Set STITCHING Module bypass frame
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetBypassSelect
(
	CamerIcDrvHandle_t                                  handle,
    const CamerIcIspStitchingBypassSelectMode_t         bypassSelect
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetBypassSelect()
*
* @brief   Get STITCHING Module bypass frame
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetBypassSelect
(
	CamerIcDrvHandle_t                         handle,
    CamerIcIspStitchingBypassSelectMode_t      *bypassSelect
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetImageSize()
*
* @brief   Set STITCHING Module image size
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetImageSize
(
	CamerIcDrvHandle_t          handle,
    const uint32_t              width,
    const uint32_t              height
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetExtBit()
*
* @brief   Set STITCHING Module external bit
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetExtBit
(
    CamerIcDrvHandle_t      handle,
    const uint8_t      		lsExtBit,
	const uint8_t      		vsExtBit
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetExtBit()
*
* @brief   Get STITCHING Module external bit
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetExtBit
(
    CamerIcDrvHandle_t      handle,
    uint8_t            		*pLsExtBit,
	uint8_t            		*pVsExtBit
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetImageSize()
*
* @brief   Get STITCHING Module image size
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetImageSize
(
	CamerIcDrvHandle_t    handle,
    uint32_t              *width,
    uint32_t              *height
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetBayerPattern()
*
* @brief   Set STITCHING Module sensor bayer pattern
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetBayerPattern
(
	CamerIcDrvHandle_t                           handle,
    const CamerIcIspStitchingBayerPattern_t      bayerPattern
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetBayerPattern()
*
* @brief   Get STITCHING Module sensor bayer pattern
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetBayerPattern
(
	CamerIcDrvHandle_t                          handle,
    CamerIcIspStitchingBayerPattern_t           *bayerPattern
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetSensorType()
*
* @brief   Set STITCHING Module sensor type, related to HW, not suggest the customer to use
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetSensorType
(
	CamerIcDrvHandle_t                           handle,
    const CamerIcIspStitchingSensorType_t        sensorType
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetSensorType()
*
* @brief   Get STITCHING Module sensor type, related to HW, not suggest the customer to use
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetSensorType
(
	CamerIcDrvHandle_t                           handle,
    CamerIcIspStitchingSensorType_t             *sensorType
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetCompressLut()
*
* @brief   Set STITCHING Module Compress Lut
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetCompressLut
(
	CamerIcDrvHandle_t                      handle,
    const CamerIcIspStitchingCompressLut_t  *compressLut
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetCompressLut()
*
* @brief   Get STITCHING Module Compress Lut
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetCompressLut
(
	CamerIcDrvHandle_t                handle,
    CamerIcIspStitchingCompressLut_t  *compressLut
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetVSParams()
*
* @brief   Set STITCHING Module VS parameters
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetVSParams
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          vs_valid_thresh,
    const uint16_t          vs_valid_offset_val
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetVSParams()
*
* @brief   Get STITCHING Module VS parameters
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetVSParams
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *vs_valid_thresh,
    uint16_t                *vs_valid_offset_val
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingSetLParams()
*
* @brief   Set STITCHING Module L parameters
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingSetLParams
(
    CamerIcDrvHandle_t      handle,
    const uint16_t          long_sat_thresh,
    const uint16_t          long_combine_weight
);

/*****************************************************************************/
/**
*          CamerIcIspStitchingGetLParams()
*
* @brief   Set STITCHING Module L parameters
*
* @param   handle          CamerIc driver handle
*
* @return                  Return the result of the function call.
* @retval                  RET_SUCCESS
* @retval                  RET_FAILURE
*
*****************************************************************************/
RESULT CamerIcIspStitchingGetLParams
(
    CamerIcDrvHandle_t      handle,
    uint16_t                *long_sat_thresh,
    uint16_t                *long_combine_weight
);


#ifdef __cplusplus
extern "C"
{
#endif

#endif
