/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef CCM_H
#define CCM_H

/**
 * @file ccm.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup ccm module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>
#include "cam_calibdb/cam_calibdb_api.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define AUTO_CCM_WB_GAIN_NUM               4
#define AUTO_CCM_TEM_LIGHT_NUM             7
#define AUTO_CCM_MATRIX_NUM                9
#define AUTO_CCM_OFFSET_MATRIX_NUM         3
#define AUTO_CCM_CONFIG_ILLUPROFILE_NUM    10L
#define AUTO_CCM_MATRIX_MAX                15.992f
#define AUTO_CCM_MATRIX_MIN                -15.992f

#define AUTO_CCM_OFFSET_MAX                2047.0f
#define AUTO_CCM_OFFSET_MIN                -2047.0f

#define AUTO_CCM_MASK                      0x1000

/*****************************************************************************/
/**
 * @brief   auto CCM input parameters
 */
/*****************************************************************************/
typedef struct AutoCcmInput_s
{
    float32_t    awbTemperatureWeight[AUTO_CCM_TEM_LIGHT_NUM];    
} AutoCcmInput_t;

/*****************************************************************************/
/**
 *          ACcmHandle_t
 *
 * @brief   CCM Module instance handle
 *
 *****************************************************************************/
typedef void *ACcmHandle_t;         /**< handle to CCM context */

/*****************************************************************************/
/**
 *          ACcmInstanceConfig_t
 *
 * @brief   Ccm Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ACcmInstanceConfig_t
{
    CamerIcDrvHandle_t      hCamerIc;       /**< handle to cameric driver */
    ACcmHandle_t            hAccm;          /**< handle returned by ACcmInit() */
    uint8_t                 ispId;          /**< idp id Number */
} ACcmInstanceConfig_t;

/*****************************************************************************/
/**
 *          ACcmConfig_t
 *
 * @brief   CCM configuration
 *
 *****************************************************************************/
typedef struct ACcmConfig_s
{
    float32_t    ccmMatrix[AUTO_CCM_MATRIX_NUM];
    float32_t    ccmOffset[AUTO_CCM_OFFSET_MATRIX_NUM];    
} ACcmConfig_t;


/*****************************************************************************/
/**
 *          ACcmInit()
 *
 * @brief   This function initializes the CCM
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ACcmInit
(
    ACcmInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          ACcmDeinit()
 *
 * @brief   The function deinit the CCM
 *
 * @param   handle  Handle to CCM
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmDeinit
(
    ACcmHandle_t handle
);


/*****************************************************************************/
/**
 *          ACcmSetConfigure()
 *
 * @brief   The CCM function set configuration
 *
 * @param   handle   Handle to CCM
 * @param   pConfig  CCM manual configuration
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmSetConfigure
(
    ACcmHandle_t handle,
    ACcmConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ACcmGetConfigure()
 *
 * @brief   The CCM function get configuration
 *
 * @param   handle   Handle to CCM
 * @param   pConfig  CCM manual configuration
 * 
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmGetConfigure
(
    ACcmHandle_t handle,
    ACcmConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          ACcmSetMode()
 *
 * @brief   This function configures the Ccm mode
 *
 * @param   handle   Handle to Ccm 
 * @param   mode     Ccm mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACcmSetMode
(
    ACcmHandle_t     handle,
    const ARunMode_t mode
);


/*****************************************************************************/
/**
 *          ACcmGetMode()
 *
 * @brief   This function get the Ccm mode
 *
 * @param   handle  Handle to Ccm
 * @param   mode     Ccm mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT ACcmGetMode
(
    ACcmHandle_t handle,
    ARunMode_t   *mode
);


/*****************************************************************************/
/**
 *          ACcmEnable()
 *
 * @brief   enable Ccm
 *
 * @param   handle       Handle to Ccm
 * @param   hCamCalibDb  Calibration data handle
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmEnable
(
    ACcmHandle_t 		handle,
    CamCalibDbHandle_t  hCamCalibDb
);


/*****************************************************************************/
/**
 *          ACcmDisable()
 *
 * @brief   disable Ccm
 *
 * @param   handle  Handle to Ccm
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmDisable
(
    ACcmHandle_t 		handle
);


/*****************************************************************************/
/**
 *          ACcmGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle     Handle to Ccm
 * @param   pRunning   running status
 * @param   pMode      mode
  * @param  pCcmConfig configuration of cc,

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmGetStatus
(
    ACcmHandle_t 		handle,
    bool_t       		*pRunning,
    ARunMode_t          *pMode,
    ACcmConfig_t        *pCcmConfig
);

/*****************************************************************************/
/**
 *          ACcmProcessFrame()
 *
 * @brief   ccm process frame
 *
 * @param   handle         Handle to ccm
 * @param   pAutoCcmInput  CCM auto calculation input parameters
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT ACcmProcessFrame
(
    ACcmHandle_t    handle,
    AutoCcmInput_t  *pAutoCcmInput
);

#ifdef __cplusplus
}
#endif


/* @} Ccm */


#endif /* Ccm_H*/
