/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef DPCC_H
#define DPCC_H

/**
 * @file dpcc.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup Auto DPCC
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"

#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define ADPCC_DEFECT_PIXEL_NUM          2048  // max number of defected pixels
#define ADPCC_CHANNEL_NUM                 2
#define ADPCC_METHOD_PARAMETER_TYPE_NUM   3
#define ADPCC_LINE_MAD_FAC_MAX            63
#define ADPCC_BPT_OUT_MODE_MAX            15
#define ADPCC_PG_FAC_MAX                  63
#define ADPCC_RND_THRESH_MAX              63
#define ADPCC_RG_FAC_MAX                  63
#define ADPCC_RO_LIMITS_MAX               3
#define ADPCC_RND_OFFS_MAX                3
#define ADPCC_SET_USE                     15
#define ADPCC_MODE_NUM                    6

/*****************************************************************************/
/**
 *          ADpccHandle_t
 *
 * @brief   ADPCC Module instance handle
 *
 *****************************************************************************/
typedef struct ADpccContext_t *ADpccHandle_t;         /**< handle to ADpcc context */

/*****************************************************************************/
/**
 *          ADpccInstanceConfig_t
 *
 * @brief   ADpcc Module instance configuration structure
 *
 *****************************************************************************/
typedef struct ADpccInstanceConfig_s 
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */

    ADpccHandle_t            hADpcc;             /**< handle returned by ADpccInit() */
} ADpccInstanceConfig_t;

/*****************************************************************************/
/**
 * @brief
 */
/*****************************************************************************/
typedef struct ADpccConfig_s 
{
    bool_t bptEnable;       /**< Bad pixel table enable */
    uint16_t bptNum;        /**< Bad pixel table number */
    uint8_t bptOutMode;     /**< Bad pixel table output mode */
    uint16_t bptPosX[ADPCC_DEFECT_PIXEL_NUM]; /**< Bad pixel table X position */
    uint16_t bptPosY[ADPCC_DEFECT_PIXEL_NUM]; /**< Bad pixel table Y position */
    uint8_t lineMadFac[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]Represents the three thresholds of Line_mad method used by the RB channel\n */
                                                                                        /**< [1][0~2]Represents the three thresholds of Line_mad method used by the G channel */
    uint8_t lineThresh[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]Represents the three thresholds of Line_thresh method used by the RB channel\n */
                                                                                        /**< [1][0~2]Represents the three thresholds of Line_thresh method used by the G channel */
    uint16_t methodsSet[ADPCC_METHOD_PARAMETER_TYPE_NUM]; /**< [0] Represents the switch whether the first set of thresholds of the five bad point determination methods are enabled\n */
                                                                    /**< [1] Represents the switch whether the second set of thresholds of the five bad point determination methods are enabled\n */
                                                                    /**< [2] Represents the switch whether the third set of thresholds of the five bad point determination methods are enabled */
    uint8_t outMode; /**<Interpolation mode for correction unit. range:[0, 15] */
    uint8_t pgFac[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]Represents the three thresholds of peak Gradient check method used by the RB channel\n */
                                                                                            /**< [1][0~2]Represents the three thresholds of peak Gradient check method used by the G channel */
    uint8_t rgFac[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]Represents the three thresholds of Rank neighbor difference method used by the RB channel\n */
                                                                                            /**< [1][0~2]Represents the three thresholds of Rank neighbor difference method used by the G channel */
    uint8_t rndOffs[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM]; /**< [0][0~2]Represents the three offsets of Rand Neighbor Difference method used by the RB channel\n */
                                                                                             /**< [1][0~2]Represents the three offsets of Rand Neighbor Difference method used by the G channel */
    uint8_t rndThresh[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];  /**< [0][0~2]Represents the three thresholds of Rand Neighbor Difference method used by the RB channel\n */
                                                                                                /**< [1][0~2]Represents the three thresholds of Rand Neighbor Difference method used by the G channel */
    uint8_t roLimits[ADPCC_CHANNEL_NUM][ADPCC_METHOD_PARAMETER_TYPE_NUM];   /**< [0][0~2]Represents the three thresholds of Rand Order method used by the RB channel\n */
                                                                                      /**< [1][0~2]Represents the three thresholds of Rand Order method used by the G channel */
    uint8_t setUse; 
} ADpccConfig_t;

/******************************************************************************/
/**
 * @brief   ADPCC auto configuration structure.
 *
 *****************************************************************************/
typedef struct ADpccAutoParamTable_s 
{
    uint8_t autoModeSelect;
} ADpccAutoParamTable_t;


/*****************************************************************************/
/**
 *          ADpccEnable()
 *
 * @brief   The function enable DPCC
 *
 * @param   handle  Handle to Dpcc
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccEnable
(
    ADpccHandle_t       handle
);

/*****************************************************************************/
/**
 *          ADpccDisable()
 *
 * @brief   The function disable DPCC
 *
 * @param   handle  Handle to Dpcc
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccDisable
(
    ADpccHandle_t       handle
);

/*****************************************************************************/
/**
 *          ADpccInit()
 *
 * @brief   The function init DPCC
 *
 * @param   pInstConfig  
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccInit
(
    ADpccInstanceConfig_t *pInstConfig
);

/*****************************************************************************/
/**
 *          ADpccRelease()
 *
 * @brief   The function release DPCC
 *
 * @param   handle  Handle to Dpcc
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccRelease
(
    ADpccHandle_t handle
);

/*****************************************************************************/
/**
 *          ADpccSetConfig()
 *
 * @brief   The function to set dpcc config
 *
 * @param   handle  Handle to Dpcc
 * @param   pConfig DPCC config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccSetConfig
(
    ADpccHandle_t     handle,
    ADpccConfig_t   *pConfig
);

/*****************************************************************************/
/**
 *          ADpccGetConfig()
 *
 * @brief   The function to get dpcc config
 *
 * @param   handle  Handle to Dpcc
 * @param   pConfig DPCC config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccGetConfig
(
    ADpccHandle_t     handle,
    ADpccConfig_t   *pConfig
);

/*****************************************************************************/
/**
 *          ADpccSetMode()
 *
 * @brief   The function set DPCC mode
 *
 * @param   handle  Handle to Dpcc
 * @param   mode    Dpcc mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccSetMode
(
    ADpccHandle_t        handle,
    ARunMode_t            mode
);

/*****************************************************************************/
/**
 *          ADpccGetMode()
 *
 * @brief   The function get Dpcc mode
 *
 * @param   handle  Handle to Dpcc
 * @param   mode    Dpcc mode
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccGetMode
(
    ADpccHandle_t       handle,
    ARunMode_t         *mode
);

/*****************************************************************************/
/**
 *          ADpccSetAutoConfig()
 *
 * @brief   The function to get Dpcc AutoConfig
 *
 * @param   handle  Handle to Dpcc
 * @param   pParamTable DPCC autoConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccSetAutoParamTable
(
    ADpccHandle_t         handle,
    ADpccAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ADpccGetAutoConfig()
 *
 * @brief   The function to get Dpcc AutoConfig
 *
 * @param   handle  Handle to Dpcc
 * @param   pParamTable DPCC autoConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccGetAutoParamTable
(
    ADpccHandle_t       handle,
    ADpccAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          ADpccProcessFrame()
 *
 * @brief   The function to calculate dpcc Param
 *
 * @param   handle  
 * @param   gain 
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT ADpccProcessFrame
(
    ADpccHandle_t    handle
);
/*****************************************************************************/
/**
 * @brief   This function get the status of Dpcc
 *
 * @param   handle          handle to the CamEngine instance
 * @param   pRunning            running status
 * @param   pCfg                Dpcc config

 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 *
 *****************************************************************************/
RESULT ADpccGetStatus
(
    ADpccHandle_t       handle,
    bool_t              *pRunning,
    ARunMode_t          *pMode,
    ADpccConfig_t       *pCfg
);

#ifdef __cplusplus
}
#endif


/* @} ADpcc */


#endif /* ADpcc_H*/
