/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef EE_H
#define EE_H

/**
 * @file ee.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup EE module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"

#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CURVE_SIZE 65

/*****************************************************************************/
/**
 * @brief   parameter range check macro.
 *
 *****************************************************************************/
#define AEE_EDGE_GAIN_MAX 65535

#define AEE_SRC_STRENGTH_MAX 128
#define AEE_STRENGTH_MAX 128
#define AEE_UV_GAIN_MAX 1024
#define AEE_Y_DOWN_GAIN_MAX 65535
#define AEE_Y_UP_GAIN_MAX 65535

#define ACURVE_LUT_TABLE_MAX 1024
/*****************************************************************************/
/**
 *          AEeHandle_t
 *
 * @brief   EE Module instance handle
 *
 *****************************************************************************/
typedef void *AEeHandle_t;         /**< handle to EE context */

/*****************************************************************************/
/**
 *          AEeInstanceConfig_t
 *
 * @brief   EE Module instance configuration structure
 *
 *****************************************************************************/
typedef struct AEeInstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */
    CamerIcDrvHandle_t      hSubCamerIc;        /**< handle to ee cameric drivder*/

    AEeHandle_t             hAEe;            /**< handle returned by AEeInit() */
} AEeInstanceConfig_t;


/*****************************************************************************/
/**
 *          AEeConfig_t
 *
 * @brief   EE Module configuration structure
 *
 *****************************************************************************/
typedef struct AEeConfig_s
{
    uint32_t edgeGain;
    uint8_t srcStrength;
    uint8_t strength;
    uint16_t uvGain;
    uint32_t yDownGain;
    uint32_t yUpGain;
} AEeConfig_t;


/*****************************************************************************/
/**
 *          AEeAutoParamTable_t
 *
 * @brief   EE Module parameters table structure
 *
 *****************************************************************************/
typedef struct AEeAutoParamTable_s
{
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    uint8_t eeStrength[AUTO_NUM];
    uint8_t eeSrcStrength[AUTO_NUM];
    uint32_t yUpGain[AUTO_NUM];
    uint32_t eeYDownGain[AUTO_NUM];
    uint16_t eeUvGain[AUTO_NUM];
    uint32_t eeEdgeGain[AUTO_NUM];
    bool_t caEnable[AUTO_NUM];
    uint16_t lutChromaTbl[AUTO_NUM][CURVE_SIZE];
    bool_t dciEnable[AUTO_NUM];
    uint16_t dciCurve[AUTO_NUM][CURVE_SIZE];
} AEeAutoParamTable_t;

/*****************************************************************************/
/**
 *          AEeInit()
 *
 * @brief   This function initializes the ee
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AEeInit
(
    AEeInstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          AEeDeinit()
 *
 * @brief   The function deinit the ee
 *
 * @param   handle  Handle to EE
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeDeinit
(
    AEeHandle_t handle
);

/*****************************************************************************/
/**
 *          AEeCurveEnable()
 *
 * @brief   This function enable the ee curve module
 *
 * @param   handle   Handle to ee 
 * @param   pCurveDx  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCurveEnable
(
    AEeHandle_t         handle
);

/*****************************************************************************/
/**
 *          AEeCurveDisable()
 *
 * @brief   This function disable the ee curve module
 *
 * @param   handle   Handle to ee 
 * @param   pCurveDx  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeCurveDisable
(
    AEeHandle_t         handle
);

/*****************************************************************************/
/**
 *          AEeSetConfigure()
 *
 * @brief   This function configures the ee module
 *
 * @param   handle   Handle to EE 
 * @param   pConfig  ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeSetConfigure
(
    AEeHandle_t handle,
    AEeConfig_t *pConfig
);


/*****************************************************************************/
/**
 *          AEeGetConfigure()
 *
 * @brief   This function get the ee module configures
 *
 * @param   handle  Handle to EE
 * @param   pConfig ee config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeGetConfigure
(
    AEeHandle_t handle,
    AEeConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AEeSetAutoParamTable()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle         Handle to EE
 * @param   pParamTable    auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeSetAutoParamTable
(
    AEeHandle_t handle,
    AEeAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          AEeGetAutoParamTable()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle        Handle to EE
 * @param   pParamTable   auto param table
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AEeGetAutoParamTable
(
    AEeHandle_t handle,
    AEeAutoParamTable_t *pParamTable
);

/*****************************************************************************/
/**
 *          AEeSetMode()
 *
 * @brief   The function set ee mode
 *
 * @param   handle  Handle to EE
 * @param   mode    EE mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeSetMode
(
    AEeHandle_t 		handle,
    const ARunMode_t	mode
);

/*****************************************************************************/
/**
 *          AEeGetMode()
 *
 * @brief   The function get ee mode
 *
 * @param   handle  Handle to EE
 * @param   mode    EE mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeGetMode
(
    AEeHandle_t 		handle,
    ARunMode_t	        *mode
);

/*****************************************************************************/
/**
 *          AEeEnable()
 *
 * @brief   enable ee
 *
 * @param   handle  Handle to EE
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeEnable
(
    AEeHandle_t 		handle
);

/*****************************************************************************/
/**
 *          AEeDisable()
 *
 * @brief   disable ee
 *
 * @param   handle  Handle to EE
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeDisable
(
    AEeHandle_t 		handle
);


/*****************************************************************************/
/**
 *          AEeGetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v3
 *          module
 *
 * @param   handle  Handle to EE
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeGetStatus
(
    AEeHandle_t handle,
    bool_t *pRunning,
    ARunMode_t *pMode,
    AEeConfig_t *pConfig
);



/*****************************************************************************/
/**
 *          AEeProcessFrame()
 *
 * @brief   ee process frame
 *
 * @param   handle  Handle to EE
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeProcessFrame
(
    AEeHandle_t    	handle,
    const float32_t	fGain
);

/*****************************************************************************/
/**
 *          AEeGetCurveStatus()
 *
 * @brief   ee curve status
 *
 * @param   handle     Handle to EE
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AEeGetCurveStatus
(
    AEeHandle_t         handle,
    bool_t              *curveEnable
);

#ifdef __cplusplus
}
#endif


/* @} EE */


#endif /* EE_H*/
