/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file    hal_altera_irq_notifier.h
 *
 * @brief   irq notifier of Hardware Abstraction Layer
 *
 *          This header file exports the irq notifier thread api to distribute irq event
 * @note 
 *
 *****************************************************************************/

#ifndef __HAL_ALTERA_IRQ_NOTIFIER_H__
#define __HAL_ALTERA_IRQ_NOTIFIER_H__

#include <stdio.h>
#include <ebase/trace.h>
#include <common/align.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "hal_api.h"  // this includes 'hal_altera_pci.h' as well


/******************************************************************************
 * local type definitions
 *****************************************************************************/

/******************************************************************************
 * Hal isp irq notifier command.
 *****************************************************************************/
typedef enum HalIrqNotifierCmdId_e
{
    HAL_IRQ_NORIFIER_CMD_INVALID                   = 0,    /**< invalid command (only for initialization) */
    HAL_IRQ_NORIFIER_CMD_REGISTER_ISP_IRQ          = 1,    /**< register isp irq command */
    HAL_IRQ_NORIFIER_CMD_REGISTER_JPE_STATUS_IRQ   = 2,    /**< register jpe status irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_JPE_ERROR_IRQ    = 3,    /**< register jpe error  irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI_IRQ           = 4,    /**< register mi irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI1_IRQ           ,    /**< register mi1 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI2_IRQ           ,    /**< register mi2 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI3_IRQ           ,    /**< register mi3 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_RDMA_IRQ           ,    /**< register rdma irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MCM_WR_RAW0_IRQ           ,    /**< register mcm wr raw0 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MCM_WR_RAW1_IRQ           ,    /**< register mcm wr raw1 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MCM_WR_G2_RAW0_IRQ           ,    /**< register mcm wr g2 raw0 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MCM_WR_G2_RAW1_IRQ           ,    /**< register mcm wr g2 raw1 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MIPI_IRQ          ,    /**< register mipi irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_ISP_STITCHING_IRQ ,    /**< register mi stitch irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI_HDR_IRQ       ,    /**< register mi hdr1 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_MI_HDR1_IRQ       ,    /**< register mi hdr1 irq command  */
    HAL_IRQ_NORIFIER_CMD_REGISTER_FE_IRQ            ,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FE_BATCH_MODE_IRQ ,

    HAL_IRQ_NORIFIER_CMD_REGISTER_JDP_IRQ       ,    /**< register mi jdp irq command  */
    
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC1,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC2,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC3,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC4,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC5,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_ECC_IMSC6,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_DUP_IMSC,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_PARITY_IMSC,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_LV1_IMSC1,
    HAL_IRQ_NORIFIER_CMD_REGISTER_RDCD_MI_IRQ,
    HAL_IRQ_NORIFIER_CMD_REGISTER_FUSA_RDCD_IRQ,

    HAL_IRQ_NORIFIER_CMD_UNREGISTER_ISP_IRQ          ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_JPE_STATUS_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_JPE_ERROR_IRQ,

    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI_IRQ ,    /**< unregister mi irq command  */
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI1_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI2_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI3_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_RDMA_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MCM_WR_RAW0_IRQ           ,    /**< unregister mcm wr raw0 irq command  */
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MCM_WR_RAW1_IRQ           ,    /**< unregister mcm wr raw0 irq command  */
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MCM_WR_G2_RAW0_IRQ           ,    /**< unregister mcm wr g2 raw0 irq command  */
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MCM_WR_G2_RAW1_IRQ           ,    /**< unregister mcm wr g2 raw1 irq command  */
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MIPI_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_ISP_STITCHING_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI_HDR_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_MI_HDR1_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FE_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FE_BATCH_MODE_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_JDP_IRQ,                       /**< unregister mi jdp irq command  */

    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC1,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC2,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC3,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC4,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC5,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_ECC_IMSC6,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_DUP_IMSC,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_PARITY_IMSC,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_LV1_IMSC1,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_RDCD_MI_IRQ,
    HAL_IRQ_NORIFIER_CMD_UNREGISTER_FUSA_RDCD_IRQ,
    HAL_IRQ_NORIFIER_CMD_MAX

} HalIrqNotifierCmdId_t;

typedef struct IrqNotifierContext_s
{

//    IrqContentQueue_t   head;
    osThread    isr_thread;
    osEvent      isr_exit_event;
    osMutex     access_lock;

    int32_t      device_file;
    IrqContext_t  irqSrcCtx[eHalIsrMax];
    int           ispId;

} IrqNotifierContext_t;

typedef void* IrqNotifierHandle_t;

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * createHalIrqNotifierThd()
 *****************************************************************************/
RESULT createHalIrqNotifierThd
(
    IrqNotifierContext_t * pIrqNotifier
);

/******************************************************************************
 * destroyHalIrqNotifierThd()
 *****************************************************************************/
RESULT destroyHalIrqNotifierThd
(
    IrqNotifierContext_t * pIrqNotifier
);

/******************************************************************************
 * HalIrqNotifierSendCommand
 *****************************************************************************/
RESULT HalIrqNotifierSendCommand
(
    IrqNotifierContext_t     *pIrqNotifier,
    HalIrqNotifierCmd_t    *pCommand
);

#ifdef __cplusplus
}
#endif

#endif /* __HAL_ALTERA_IRQ_NOTIFIER_H__ */