/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/**
 * @file hal_mipi.h
 *
 * @brief   Definition of HAL MIPI API.
 *
 *****************************************************************************/

#ifndef __HAL_MIPI_H__
#define __HAL_MIPI_H__

#include <common/mipi.h>
#include <common/return_codes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * MipiHalHandle_t
 *****************************************************************************/
typedef void* HalMipiHandle_t;
typedef void* HalDvpHandle_t;

/******************************************************************************
 * HalMipiConfig_t
 *****************************************************************************/
typedef struct HalMipiConfig_s{

    MipiImageRect_t       imageRect; /* Rx device area */
    MipiDataType_t        dataType;
    MipiLaneId_t          laneNum;
    MipiOutItfConfig_t    outItfCfg[MIPI_OUTPUT_ITF_MAX];
} HalMipiConfig_t;

/******************************************************************************
 * HalDvpConfig_t
 *****************************************************************************/
typedef struct HalDvpConfig_s{
  bool_t pwdnPol;  /*0: high, 1: low*/
  bool_t rstPol;   /*0: high, 1: low*/
  bool_t fsinPol;  /*0: high, 1: low*/
  uint32_t bitWidth;
} HalDvpConfig_t;


/******************************************************************************
 * HalMipiContext_t
 *****************************************************************************/
typedef struct HalMipiContext_s {
    uint32_t            refCount;               //!< internal ref count
    uint8_t             devId;
    int32_t             fd;
    osMutex             refMutex;               //!< common short term mutex; e.g. for read-modify-write accesses
} HalMipiContext_t;


/*****************************************************************************/
/**
 * @brief   open the low level driver
 * @param   mipiId    mipi device id
 * @return  handle of driver; NULL on failure
 *
 * @note    Create mipi handle and set internal reference count to 0.
 *****************************************************************************/
HalMipiHandle_t HalMipiDrvOpen(uint8_t mipiId);

/*****************************************************************************/
/**
 * @brief   close the low level driver
 * @param   mipiHalHandle   Handle to HAL session as returned by @ref HalMipiDrvOpen.
 * @return  Result of operation.
 *
 * @note    When reference count is 0, close the mipi device file, and when count
 *          is greater than 0, return a status error code.
 *****************************************************************************/
RESULT HalMipiDrvClose(HalMipiHandle_t mipiHalHandle);

/*****************************************************************************/
/**
 * @brief   initialize mipi device
 * @param   mipiHalHandle    Handle to HAL session as returned by @ref HalMipiDrvOpen.
 * @return  Result of operation.
 *
 * @note    Reset mipi device and set stream off.
 *****************************************************************************/
RESULT HalMipiDrvSystemInit(HalMipiHandle_t mipiHalHandle);

/*****************************************************************************/
/**
 * @brief   configure mipi device
 * @param   mipiHalHandle    Handle to HAL session as returned by @ref HalMipiDrvOpen.
 * @param   mipiConfig       mipi configuration parameters
 * @return  Result of operation.
 *
 * @note    Support raw data type.
 *****************************************************************************/
RESULT HalMipiSetConfig(HalMipiHandle_t mipiHalHandle, HalMipiConfig_t mipiConfig);

/*****************************************************************************/
/**
 * @brief   start mipi data stream
 * @param   mipiHalHandle    Handle to HAL session as returned by @ref HalMipiDrvOpen.
 * @return  Result of operation.
 *
 * @note    Get mipi fd from mipi hal handle.
 *****************************************************************************/
RESULT HalMipiDrvStreamOn(HalMipiHandle_t mipiHalHandle);

/*****************************************************************************/
/**
 * @brief   stop mipi data stream
 * @param   mipiHalHandle    Handle to HAL session as returned by @ref HalMipiDrvOpen.
 * @return  Result of operation.
 *
 * @note    Get mipi fd from mipi hal handle.
 *****************************************************************************/
RESULT HalMipiDrvStreamOff(HalMipiHandle_t mipiHalHandle);

RESULT HalDvpSetConfig(HalDvpHandle_t       HalMipiHandle, HalDvpConfig_t dvpConfig);

RESULT HalDvpSetPower(uint8_t      dvpCtrlId,HalDvpConfig_t dvpConfig,bool_t isPower);

#ifdef __cplusplus
}
#endif

#endif /* __HAL_MIPI_H__ */
