/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @file isp_switch.h
 *
 * @brief
 *   Internal interface of the Switch.
 *
 *****************************************************************************/
/**
 *
 * @defgroup switch  interface
 * @{
 *
 */
#ifndef SWITCH_H
#define SWITCH_H

#include <ebase/types.h>
#include <oslayer/oslayer.h>
#include <common/return_codes.h>
#include <common/list.h>

#include "cascade.h"
#include "priority_queue.h"
#include "isp_switch_api.h"
#include "hal_common.h"
#include "isp_cmd_buffer_api.h"
#include "isp_start_engine_api.h"

#ifdef __cplusplus
extern "C"
{
#endif


/**
 * @brief Generic command type.
 *
 */
typedef struct SwitchCmd_s
{
    SwitchCmdId_t    cmdId;
    void                *pCmdCtx;
} SwitchCmd_t;

#define MAX_STORE_FRAMES 10

/*****************************************************************************/
/**
 * @brief   Switch Device information
 *
 *****************************************************************************/
typedef struct SwitchDevInfo_s {
    void *pDevHandle;
    SwitchDevId_t devID;
    SwitchDevPriority_t priority;  /**<sensor frames priroity*/
    IspCmdBufferHandle_t hIspCmdBuffer;//store the Dev comnbuffer handle.
    CascadeHandle_t hCascade;  /** <store the Cascade Context handle. */
    uint32_t miFrameDoneMask;
    uint32_t miRdmaFrameDoneMask;
    uint32_t ispFrameDoneFlag;
    uint32_t miFrameIrq;

    osQueue *pIrqQueue;

    //cmd buffer
    bool runBufferStatus;
    bool runBufferRequest;

    uint32_t cmdNumMax;
    uint32_t cmdDmaAddr;

    bool     pingpongFlag;

    void *regCmdBuffer;
}SwitchDevInfo_t;

/*****************************************************************************/
/**
 * @brief   Switch Device list
 *
 *****************************************************************************/
typedef struct SwitchDevList_s {
    SwitchDevInfo_t devInfo;
    bool            devState;
}SwitchDevList_t;

typedef struct SwitchIrqContext_s {
    bool            switchIrqState;
    HalIsrSrc_t     irqSrc;
    osMutex         switchIrqLock;  /**< Switch Mutex Lock.*/
    HalIrqCtx_t     switchIrqCtx;
    uint32_t        refCount ;   /**< Switch reference counter.*/
}SwitchIrqContext_t;

//CMD Buffer Common Register
typedef struct SwitchComReg_s{
    //((offset >= 0x1200 && offset <= 0x1294)
    // (offset >= 0x1610 && offset <= 0x1668) ||
    // (offset >= 0x500c && offset <= 0x5060) ||
    // (offset >= 0x5700 && offset <= 0x5894) ||
    // (offset >= 0x5900 && offset <= 0x5924) ||
    // (offset >= 0x5980 && offset <= 0x59A4))

    uint32_t vi_dpcl;//data path control register 0x00000018
    uint32_t mcm_ctrl;//mcm control register 0x00001200
    uint32_t mcm_rd_cfg;//mcm read format register 0x00001280
    uint32_t mcm_retiming0;//Mcm retiming control register 0 0x00001284
    uint32_t mcm_retiming1;//mcm retiming control register 1 0x00001288
    uint32_t miv2_ctrl;//global control register 0x00001300
    uint32_t miv2_mcm_ctrl;//mcm path mi control register 0x00001600
    uint32_t miv2_mcm_fmt;//mcm pixel format control register 0x00001604
    uint32_t miv2_mcm_bus_cfg;//mcm axi bus cfg 0x00001608
    uint32_t miv2_mcm_bus_id;//mcm axi bus id configuration   0x0000160c
    uint32_t miv2_imsc;     //mcm axi bus id configuration   0x000016c0
    uint32_t miv2_mcm_g2_ctrl;//mcm path mi control register  0x00005000
    uint32_t miv2_mcm_g2_fmt;//mcm pixel format control register  0x00005004
    uint32_t miv2_mcm_g2_bus_cfg;//mcm axi bus cfg   0x00005008
}SwitchComReg_t;

typedef enum SwitchMcmOpt_e{
    SWITCH_MCM_INVALID_MODE      = -1,
    SWITCH_MCM_SW_MODE      = 0,   /**< SW MCM MODE*/
    SWITCH_MCM_HW_MODE      = 1,    /**< HW MCM MODE*/
    SWITCH_MCM_MAX_MODE
}SwitchMcmOpt_t;


/**
 * @brief Internal context of the switch
 *
 */
typedef struct SwitchContext_s {
    //SwitchHandle_t   hSwitch;
    SwitchWorkMode_t workMode; /**< Work mode of the Switch */
    SwitchState_t    state;  /**< State of the Switch */
    osEvent     frameEvent;  /**< Swirch thread frameEvent */
    int8_t hwProcessFlag;

    uint32_t maxCommands; /**< Max pending commands in command queue. */
    osQueue commandQueue; /**< Command queue. */
    osQueue HwSchduleQueue; /**< HwSchdule queue. */
    osThread thread;      /**< Switch thread. */
    osEvent  exitEvent;  /**< Swirch thread exitEvent */
    osMutex  switchLock;  /**< Switch Mutex Lock.*/
    uint32_t refCount ;   /**< Switch reference counter.*/
    uint32_t hpID ;       /**< Switch Instance hw pipeline id.*/

    SwitchDevList_t switchDevList[SWITCH_DEVICE_ID_MAX];
    SwitchIrqContext_t switchIrqCtx[eHalIsrMax];

    osMutex  requestLock;  /**< Request Mutex Lock.*/
    PriorityQueue requestQueue;

    osMutex  irqThreadLock;  /**< Irq thread Mutex Lock.*/

    //CMD Buffer Common Register
    osMutex  comRegLock;
    SwitchComReg_t  switchComReg;

    uint32_t currDevID;
    uint32_t nextDevID;
    bool     twoDevFlag;

    uint32_t hdrDevID;
    uint32_t hdr1DevID;

    HalHandle_t hHalLayer;
    HalMemHandle_t hMemHandle;
    IspStartEngineHandle_t hStartEng;
    // uint32_t miFrameDoneMask;
    // uint32_t miRdmaFrameDoneMask;
    // uint32_t ispFrameDoneFlag;
    uint8_t batchFrameCount;        /**< The frame count of one batch.*/

    bool     StlDupTestFlag;
    bool     isStlDupNextFrame;
    SwitchMcmOpt_t  mcmMode;
} SwitchContext_t;

/*****************************************************************************/
/**
 * @brief   Create an instance of Switch
 *
 * @param   pSwitchCtx   Pointer to the context of Switch instance
 *
 * @return              Return the result of the function call.
 * @retval              RET_SUCCESS
 * @retval              RET_VAL2
 *
 *****************************************************************************/
RESULT SwitchCreate
(
    SwitchContext_t  *pSwitchCtx
);


/*****************************************************************************/
/**
 * @brief   Destroy an instance of Switch
 *
 * @param   pSwitchCtx   Pointer to the context of Switch instance
 *
 * @return              Return the result of the function call.
 * @retval              RET_SUCCESS
 * @retval              RET_VAL2
 *
 *****************************************************************************/
RESULT SwitchDestroy
(
    SwitchContext_t  *pSwitchCtx
);

/*****************************************************************************/
/**
 * @brief   This functions sends a command to a Switch instance by putting
 *          a command id into the command-queue of the cam-egine instance.
 *
 * @param   pSwitchCtx   context of the Switch instance
 * @param   pCommand        pointer to command to send
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
RESULT SwitchSendCommand
(
    SwitchContext_t   *pSwitchCtx,
    SwitchCmd_t       *pCommand
);

/*****************************************************************************/
/**
 * @brief   This functions check the Device List Devic ID status. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deinit.
 * If the return value is RET_FAILED, means the DEV List still have some device init.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
RESULT SwitchCheckDevList
(
    SwitchContext_t *pSwitchCtx
);


/*****************************************************************************/
/**
 * @brief   This functions check the Device List Devic ID status. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deinit.
 * If the return value is RET_FAILED, means the DEV List still have some device init.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
SwitchDevId_t SwitchGetDevID
(
    const SwitchContext_t *pSwitchCtx,
    const void *pDevHandle
);

/*****************************************************************************/
/**
 * @brief   This functions Bind the Device instance with Vitural ID. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deinit.
 * If the return value is RET_FAILED, means the DEV List still have some device init.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
RESULT SwitchBindDev
(
    SwitchContext_t *pSwitchCtx,
    void *pDevHandle,
    SwitchDevId_t *devID
);

/*****************************************************************************/
/**
 * @brief   This functions UnBind the Device instance with Vitural ID. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deinit.
 * If the return value is RET_FAILED, means the DEV List still have some device init.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
RESULT SwitchUnbindDev
(
    SwitchContext_t *pSwitchCtx,
    void *pDevHandle
);

/*****************************************************************************/
/**
 * @brief   This functions UnBind the Device instance with Vitural ID. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deinit.
 * If the return value is RET_FAILED, means the DEV List still have some device init.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 * @param   pDevHandle   Device Handle
 * @param   priority     the priority of the dev
 * @param   state        the state of the dev
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
SwitchDevId_t SwitchUpdateDevInfo
(
    SwitchContext_t *pSwitchCtx,
    const void *pDevHandle,
    SwitchDevPriority_t priority,
    bool state
);
/*****************************************************************************/
/**
 * @brief   This functions check the Device List state status. 
 * If the Return value is RET_SUCCESS,means the DEV List all device had deregister.
 * If the return value is RET_FAILED, means the DEV List still have some device register.
 * 
 * @param   pSwitchCtx   context of the Switch instance
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeed
 * @retval  RET_WRONG_HANDLE    pSwitch is NULL or invalid
 * @retval  RET_CANCELED        cam-instance is shutting down
 * @retval  RET_FAILURE         write to command-queue failed
 *
 *****************************************************************************/
RESULT SwitchCheckDevState
(
    SwitchContext_t *pSwitchCtx
);

void SwitchSetState
(
    SwitchContext_t   *pSwitchCtx,
    SwitchState_t     newState
);

int32_t SwitchSchduleIrq(void *pArg);

RESULT SwitchCmdBufferUpdate
(
    SwitchContext_t  *pSwitchCtx,
    SwitchDevInfo_t  *pSwitchDevInfo
);

#ifdef __cplusplus
}
#endif


/* @} switch */

#endif /* __SWITCH_H__ */

