/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @file isp_switch_api.h
 *
 * @brief
 *   Interface description of the Switch.
 *
 *****************************************************************************/
/**
 *
 * @mainpage General Concept
 *
 * The Switch provides the application programming interface (API) to control
 * the switch isp context from C/C++ applications. It abstracts
 * a common camera interface and contains functions to control the usage modes of
 * a typical image capturing device.
 *
 * In order to support isp time division multiplexing, Add switch layer to support
 * swich isp context to support different input sensor.
 *
 * The Switch follows the object model of the camerIC evaluation software. Before usage, the
 * Switch needs to be statically configured by the @ref   function. Then, an instance
 * of the Switch can be started using the @ref SwitchStart function. From now on commands
 * can be send to the Switch. The instance can be stopped using the @ref SwitchStop
 * function and all resources are released with the @ref SwitchShutDown function.
 *
 * @defgroup isp_switch_api Switch API
 * @{
 */

#ifndef SWITCH_API_H
#define SWITCH_API_H

#include <binder/binder_hal_api.h>
#include <common/return_codes.h>
#include <ebase/types.h>
#include "isp_switch_common.h"
#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/**
 * @brief   Handle to a Switch instance.
 *
 *****************************************************************************/
typedef struct SwitchContext_s *SwitchHandle_t;

/*****************************************************************************/
/**
 * @brief   Commands for the Switch
 *
 *****************************************************************************/
typedef enum SwitchCmdId_e {
    SWITCH_CMD_INVALID = 0,	 /**< invalid command (only for initialization) */
    SWITCH_CMD_DESTORY = 1,
    SWITCH_CMD_REGISTER_DEV = 2,	 /**< start a switch instance */
    SWITCH_CMD_DEREGISTER_DEV = 3,	 /**< stop a switch instance */
    SWITCH_CMD_DEVICE_REQUEST = 4, /**< device request the frame start */
    SWITCH_CMD_ISP_FRAME_DONE = 5, /**<isp process frame done, can start next frame process */
    SWITCH_CMD_MAX
} SwitchCmdId_t;

/*****************************************************************************/
/**
 * @brief   Device priority
 *
 *****************************************************************************/
typedef enum SwitchDevPriority_e {
    SWITCH_DEVICE_PRIORITY_INVALID = 0,
    SWITCH_DEVICE_PRIORITY_0  = 1, /**<the lower number, the highest priority. */
    SWITCH_DEVICE_PRIORITY_1  = 2,
    SWITCH_DEVICE_PRIORITY_2  = 3,
    SWITCH_DEVICE_PRIORITY_3  = 4,
    SWITCH_DEVICE_PRIORITY_4  = 5,
    SWITCH_DEVICE_PRIORITY_5  = 6, /**<the lower number, the highest priority. */
    SWITCH_DEVICE_PRIORITY_6  = 7,
    SWITCH_DEVICE_PRIORITY_7  = 8,
    SWITCH_DEVICE_PRIORITY_8  = 9,
    SWITCH_DEVICE_PRIORITY_9  = 10,
    SWITCH_DEVICE_PRIORITY_10 = 11, /**<the lower number, the highest priority. */
    SWITCH_DEVICE_PRIORITY_11 = 12,
    SWITCH_DEVICE_PRIORITY_12 = 13,
    SWITCH_DEVICE_PRIORITY_13 = 14,
    SWITCH_DEVICE_PRIORITY_14 = 15,
    SWITCH_DEVICE_PRIORITY_15 = 16,
    SWITCH_DEVICE_PRIORITY_MAX
} SwitchDevPriority_t;

/*****************************************************************************/
/**
 * @brief   Device ID for the Switch
 *
 *****************************************************************************/
typedef enum SwitchDevId_e {
    SWITCH_DEVICE_INVALID = 255U,
    SWITCH_DEVICE_ID0  = 0U,  /**< vDevice id 1  */
    SWITCH_DEVICE_ID1  = 1U,  /**< vDevice id 2 */
    SWITCH_DEVICE_ID2  = 2U,  /**< vDevice id 3 */
    SWITCH_DEVICE_ID3  = 3U,  /**< vDevice id 4 */
    SWITCH_DEVICE_ID4  = 4U,  /**< vDevice id 5 */
    SWITCH_DEVICE_ID5  = 5U,  /**< vDevice id 6 */
    SWITCH_DEVICE_ID6  = 6U,  /**< vDevice id 7 */
    SWITCH_DEVICE_ID7  = 7U,  /**< vDevice id 8 */
    SWITCH_DEVICE_ID8  = 8U,  /**< vDevice id 9 */
    SWITCH_DEVICE_ID9  = 9U, /**< vDevice id 10 */
    SWITCH_DEVICE_ID10 = 10U, /**< vDevice id 11 */
    SWITCH_DEVICE_ID11 = 11U, /**< vDevice id 12 */
    SWITCH_DEVICE_ID12 = 12U, /**< vDevice id 13 */
    SWITCH_DEVICE_ID13 = 13U, /**< vDevice id 14 */
    SWITCH_DEVICE_ID14 = 14U, /**< vDevice id 15 */
    SWITCH_DEVICE_ID15 = 15U, /**< vDevice id 16 */
    SWITCH_DEVICE_ID_MAX = 16U
} SwitchDevId_t;

/*****************************************************************************/
/**
 * @brief States of the Switch.
 *
 *****************************************************************************/
typedef enum SwitchState_e {
    eSwitchStateInvalid = 0x0000,	 /**< switch state is invalid since Switch instance does not exist. */
    eSwitchStateInitialize = 0x0001, /**< switch is in state initialized. */
    eSwitchStateStandby = 0x0002,	 /**< switch is in state blocking. */
    eSwitchStateRunning = 0x0003,	 /**< switch is in state running. */
    eSwitch_STATE_MAX
} SwitchState_t;

typedef enum SwitchWorkMode_e {
    SWITCH_WORK_MODE_BYPASS = 0,
    SWITCH_WORK_MODE_MCM = 1,
    SWITCH_WORK_MODE_TEST = 2,
    SWITCH_WORK_MODE_FIX = 3,
    SWITCH_WORK_MODE_DUP = 4,
    SWITCH_WORK_MODE_BATCH = 5,
    SWITCH_WORK_MODE_RDCD = 6,
    SWITCH_WORK_MODE_MAX
} SwitchWorkMode_t;

typedef enum SwitchApiMcmOpt_e{
    SWITCH_API_MCM_MODE_INVALID     = -1,
    SWITCH_API_MCM_MODE_SW          = 0,   /**< SW MCM MODE*/
    SWITCH_API_MCM_MODE_HW          = 1,    /**< HW MCM MODE*/
    SWITCH_API_MCM_MODE_MAX
}SwitchApiMcmOpt_t;


/**
 * @brief   Configuration structure of the switch instance.
 *
 */
typedef struct SwitchInstanceConfig_s{
    SwitchHandle_t           hSwitch;           /**< Handle to switch context, set by @ref SwitchInit if successful, undefined otherwise. */
    void *                   hDevHandle;        /**< Handle to Device context. */
    void *                   hMemHandle;        /**< Handle to MEM context. */
    uint8_t                  hpID;              /**< Hardware Pipeline ID. */
    SwitchDevId_t            devID;
    SwitchApiMcmOpt_t        mcmOpt;
    SwitchWorkMode_t         workMode;          /**< Hardware Pipeline ID. */
    uint8_t                  batchFrameCount;   /**< The frame count of one batch. */
} SwitchInstanceConfig_t;

typedef struct SwitchVdInfo_s
{
    SwitchWorkMode_t work_mode;
    uint32_t vdid;
}SwitchVdInfo_t;

typedef struct SwitchCtrl_s {
    SwitchVdInfo_t currVd;
    SwitchVdInfo_t nextVd;
}SwitchCtrl_t;

/*****************************************************************************/
/**
 * @brief   The function creates and initializes a Switch instance.
 *
 * @param   pConfig     Instance configuration structure.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_INVALID_PARM    invalid configuration
 * @retval  RET_OUTOFRANGE      a configuration parameter is out of range
 * @retval  RET_WRONG_HANDLE    invalid HAL handle
 * @retval  RET_OUTOFMEM        not enough memory available
 *
 *****************************************************************************/
RESULT SwitchInit
(
    SwitchInstanceConfig_t *pConfig
);

/*****************************************************************************/
/**
 * @brief   This function deinit and destroys a Switch instance.
 *
 * @param   hSwitch Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state to shutdown
 *
 *****************************************************************************/
RESULT SwitchDeInit
(
    SwitchHandle_t   hSwitch,
    void *hDevHandle
);

/*****************************************************************************/
/**
 * @brief   The function register a device into Switch device list.
 *
 * @param   hSwitch  Handle to the Switch instance. *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state to shutdown
 *
 *****************************************************************************/
RESULT SwitchRegisterDev
(
    SwitchHandle_t   hSwitch,
    const void *hDevHandle,
    SwitchDevPriority_t priority
);

/*****************************************************************************/
/**
 * @brief   The function deregister a device from the Switch device list.
 *
 * @param   hSwitch  Handle to the Switch instance.
 * @param   hDevHandle    device handle
 *
 * @return  Return the result of the function call.
 * @retval  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
RESULT SwitchDeRegisterDev
(
    SwitchHandle_t   hSwitch,
    const void *hDevHandle
);

/*****************************************************************************/
/**
 * @brief   The function update the device cascade into Switch device list.
 *
 * @param   hSwitch  Handle to the Switch instance. *
 * @return  Return the result of the function call.
 * @return  RET_PENDING         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state to shutdown
 *
 *****************************************************************************/
RESULT SwitchUpdateCascade
(
    SwitchHandle_t   hSwitch,
    const void *hDevHandle,
    const void *hCascade  /**<Cascade ctx handle*/
);

/*****************************************************************************/
/**
 * @brief   This function starts streaming. This puts the Switch
 *          into the streaming state.
 *
 * @param   hSwitch      Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
RESULT SwitchSendRequest
(
    SwitchHandle_t   hSwitch,
    const void *hDevHandle 
);

/*****************************************************************************/
/**
 * @brief   This function  send isp frame done command and 
 * recover the Standby status
 *
 * @param   hSwitch      Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
RESULT SwitchFrameDone
(
    SwitchHandle_t   hSwitch,
    void *hDevHandle
);


/*****************************************************************************/
/**
 * @brief   This function get the Switch Handle Instance 
 *
 * @param   hSwitch      Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
SwitchHandle_t SwitchGetHandle
(
    uint8_t   hpID
);

/*****************************************************************************/
/**
 * @brief   This function get the Switch State 
 *
 * @param   hSwitch      Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
SwitchState_t SwitchGetState
(
    SwitchHandle_t   hSwitch
);


RESULT SwitchRegisterIRQ
(
    SwitchHandle_t   hSwitch,
    const void       *hDevHandle,
    HalIsrSrc_t      irqSrc,
    osQueue          *pIrqQueue,
    uint32_t         misRegAddress,
    uint32_t         icrRegAddress
);

RESULT SwitchDeregisterIRQ
(
    SwitchHandle_t   hSwitch,
    const void       *hDevHandle,
    HalIsrSrc_t      irqSrc
);


RESULT SwitchSetCmdBuffHandle
(
    SwitchHandle_t hSwitch,
    const void *hDevHandle,
    void *hIspCmdBuffer
);

void* SwitchGetHalHandle
(
    SwitchHandle_t   hSwitch
);

/*****************************************************************************/
/**
 * @brief   This function get the Switch Handle Instance 
 *
 * @param   hSwitch      Handle to the Switch instance.
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS         function succeed
 * @retval  RET_WRONG_HANDLE    invalid instance handle
 * @retval  RET_WRONG_STATE     instance is in wrong state
 *
 *****************************************************************************/
RESULT SwitchUpdateCommonRegister
(
    SwitchHandle_t   hSwitch,
    uint32_t reg_address, 
    uint32_t value
);

#ifdef __cplusplus
}
#endif

/* @} isp_switch_api */

#endif /* __SWITCH_H__ */

