/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2023 */

/**
 * @file    video_in_api.h.h
 *
 * @brief   This file defines the video in driver API
 *
 *****************************************************************************/
/**
 * @defgroup video_in_api.h video in API definitions
 * @{
 *
 *
 */

#ifndef VIDEO_IN_API_H
#define VIDEO_IN_API_H

#include <ebase/types.h>

#include <common/return_codes.h>
#include <common/picture_buffer.h>

#include <hal/hal_api.h>
#include <binder/binder_hal_api.h>

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @brief   Handle type of the video in software driver.
 *
 *****************************************************************************/
typedef void* VideoInHandle_t;

/******************************************************************************/
/**
 * @brief   enum mcm path id
 *
 *****************************************************************************/
typedef enum VideoInPath_e {
    VIDEO_IN_MCM_PATH_INVALID = -1,
    VIDEO_IN_MCM_PATH0,
    VIDEO_IN_MCM_PATH1,
    VIDEO_IN_MCM_PATH2,
    VIDEO_IN_MCM_PATH3,
    VIDEO_IN_MCM_PATH_ID_MAX
} VideoInPath_t;

/******************************************************************************/
/**
 * @brief   enum video in format
 *
 *****************************************************************************/
typedef enum VideoInMcmFmt_e{
    VIDEO_IN_FMT_INVALID = 0,
    VIDEO_IN_FMT_DIABLED,
    VIDEO_IN_FMT_RAW8 ,
    VIDEO_IN_FMT_RAW10,
    VIDEO_IN_FMT_RAW12,
    VIDEO_IN_FMT_RAW14,
    VIDEO_IN_FMT_RAW16,
    VIDEO_IN_FMT_RAW20_COMPRESS,
    VIDEO_IN_FMT_RAW24,
    VIDEO_IN_FMT_RAW24_COMPRESS,
    VIDEO_IN_FMT_MAX,
}VideoInMcmFmt_t;

/******************************************************************************/
/**
 * @brief   enum video in data align mode
 *
 *****************************************************************************/
typedef enum VideoInMcmAlignMode_e{
    VIDEO_IN_UNALIGN_MODE      = 0    ,
    VIDEO_IN_ALIGN_MODE1           ,
    VIDEO_IN_ALIGN_MODE2           ,
    VIDEO_IN_ALIGN_MODE_MAX,
}VideoInMcmAlignMode_t;

/******************************************************************************/
/**
 * @brief   enum video in data callback path
 *
 *****************************************************************************/
typedef enum VideoInMcmCbPath_e {
    VIDEO_IN_MCM_CB_PATH_INVALID = -1,
    VIDEO_IN_MCM_CB_PATH_IMG          ,       /** <image data package*/
    VIDEO_IN_MCM_CB_PATH_META          ,       /** <metadat data package*/
    VIDEO_IN_MCM_CB_PATH_MAX,
} VideoInMcmCbPath_t;

/******************************************************************************/
/**
 * @brief   struct video in mcm cnfigure
 *
 *****************************************************************************/
typedef struct VideoInMcmPathCfg_e {
    uint16_t                     width; /**< input image width*/
    uint16_t                     height; /**< input image heidht*/
    VideoInMcmFmt_t               format;
    VideoInMcmAlignMode_t         align;
} VideoInMcmPathCfg_t;


/*****************************************************************************/
/**
 * @brief   Enumeration of asynchronous event IDs (callback parameter).
 *
 * @note    This is a list of all asynchronous events supported by video in
 *          driver system.
 *
 *****************************************************************************/
typedef enum VideoInEventId_e
{
    VIDEO_IN_EVENT_INVALID                   = 0x0000,                               /**< invalid event (only for an internal evaluation) */

    /* VideoIn module */
    VIDEO_IN_FULL_EVENT_BASE                  = 0x1000,                           /**< base of events */
    VIDEO_IN_EVENT_FULL_MCM_IMG_BUFFER  ,                                          /**< new frame on mcm wr path available */
    VIDEO_IN_EVENT_FULL_MCM_META_BUFFER  ,                                       /**< new frame on mcm wr meta path available */
    VIDEO_IN_EVENT_FULL_BUFFER_MAX      ,                                         /**<full event max */

    VIDEO_IN_FLUSH_EVENT_BASE                  = 0x2000,                             /**< base of flush events */
    VIDEO_IN_EVENT_FLUSHED_MCM_IMG_BUFFER,                                            /**< delivers a flushed buffer on  on mcm wr path  */
    VIDEO_IN_EVENT_FLUSHED_MCM_META_BUFFER  ,                                       /**< delivers a flushed buffer on  on mcm wr meta0 */

    VIDEO_IN_EVENT_FLUSHED_BUFFER_MAX  ,                                           /**< delivers a flushed buffer event max  */

    VIDEO_IN_DROP_EVENT_BASE                  = 0x3000,                              /**< base of drop events */
    VIDEO_IN_EVENT_DROPPED_MCM_IMG_BUFFER,                                            /**< informs that a frame was dropped/skipped on mcm wr0 path  */
    VIDEO_IN_EVENT_DROPPED_MCM_META_BUFFER  ,                                       /**< informs that a frame was dropped/skipped on mcm wr meta0 path */

    VIDEO_IN_DROP_EVENT_MAX                ,                                         /**< informs that a frame was dropped/skippedon event max  */
} VideoInEventId_t;

/*****************************************************************************/
/**
 * @brief   video in int config
 *
 * @note    This is a struct of configuration for video in init
 *
 *****************************************************************************/
typedef struct VideoInConfig_s
{
    VideoInPath_t           path;
    uint8_t                 hwId;
    HalHandle_t             Halhandle;          /**< isp hal handle */
    VideoInHandle_t         *pVideoInHandle;    /**< Handle to created video control context, set by @ref videoInInit if successfull, undefined otherwise.*/
} VideoInConfig_t;


/*****************************************************************************/
/**
 * @brief   This function init video in driver
 *
 * @param   pConfig             video in config pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT VideoInInit
(
    VideoInConfig_t *pConfig            /**< Reference to configuration structure.*/
);

/*****************************************************************************/
/**
 * @brief   This function release video in driver
 *
 * @param   pConfig             video in config pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT VideoInRelease
(
    VideoInHandle_t videoInHandle       /**< Handle to video in context as returned by @ref videoInInit.*/
);

/*****************************************************************************/
/**
 * @brief   This function set video in path
 *
 * @param   videoInHandle             video in handle
 * @param   path                      video in path
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT VideoInSetPath
(
    VideoInHandle_t videoInHandle,       /**< Handle to video in context as returned by @ref videoInInit.*/
    VideoInPath_t   path
);
/*****************************************************************************/
/**
 * @brief   This function create video in interrupt process thread
 *
 * @param   pConfig             video in config pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT videoInStart
(
    VideoInHandle_t     videoInHandle  /**< Handle to video in context as returned by @ref videoInInit.*/
);

/*****************************************************************************/
/**
 * @brief   This function destroy video in interrupt process thread
 *
 * @param   pConfig             video in config pointer
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT videoInStop
(
    VideoInHandle_t videoInHandle       /**< Handle to video in context as returned by @ref videoInInit.*/
);

/*****************************************************************************/
/**
 * @brief   This function get the video bypass status
 *
 * @param   handle             video in handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT VideoInIsEnabled
(
    VideoInHandle_t handle, 
    bool_t *pEnabled
);

/*****************************************************************************/
/**
 * @brief   This function set whether if bypass video in
 *
 * @param   handle             video in handle
 * @param   bypass             ture: bypass video in , false:enable video in

 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT VideoInBypass
(
    VideoInHandle_t              handle,
    bool_t                       bypass
);

/*****************************************************************************/
/**
 * @brief   This function config mcm write format
 *
 * @param   handle              video in handle
 * @param   pCfg                path config
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
RESULT VideoInSetMcmWrCfg
(
    VideoInHandle_t              handle,
    const VideoInMcmPathCfg_t    *pCfg
);

/******************************************************************************/
/**
 * @brief   Request callback function definition
 *
 * @note    This callback is used to request something from the application
 *          software, e.g. an buffer.
 *
 * @return  RESULT
 *
 *****************************************************************************/
typedef RESULT (* VideoInRequestFunc_t)
(
    VideoInMcmCbPath_t              path,
    void                        **param,
    void                        *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This function registers a Request-Callback at the video in
 *          Interface Module. A request callback is called if the driver
 *          needs an interaction from the application layer .
 *
 * @param   handle              video in driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              request callback
 *
 *****************************************************************************/
extern RESULT VideoInRegisterRequestCb
(
    VideoInHandle_t              handle,
    VideoInRequestFunc_t         func,
    void                         *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Request-Callback
 *          at Interface Module.
 *
 * @param   handle              VideoIn driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              request callback
 *
 *****************************************************************************/
extern RESULT VideoInDeRegisterRequestCb
(
    VideoInHandle_t handle
);

/******************************************************************************/
/**
 * @brief   Event callback
 *
 * @note    This callback is used to signal something to the application
 *          software, e.g. an error or an information.
 *
 * @return  void
 *
 *****************************************************************************/
typedef void (* VideoInEventFunc_t)
(
    const VideoInEventId_t  evtId,
    void                    *param,
    void                    *pUserContext
);

/*****************************************************************************/
/**
 * @brief   This functions registers an Event-Callback at
 *          Interface Module. An event callback is called if the driver
 *          needs to inform the application layer about an asynchronous event
 *          or an error situation (i.e. please also see @ref VideoInEventId_e).
 *
 * @param   handle              VideoIn driver handle
 * @param   func                Callback function
 * @param   pUserContext        User-Context
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_BUSY            already a callback registered
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_INVALID_PARM    given parameter is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to register a
 *                              event callback
 *
 *****************************************************************************/
extern RESULT VideoInRegisterEventCb
(
    VideoInHandle_t  handle,
    VideoInEventFunc_t  func,
    void                 *pUserContext
);


/*****************************************************************************/
/**
 * @brief   This functions deregisters/releases a registered Event-Callback
 *          at CamerIC Memory Interface Module.
 *
 * @param   handle              VideoIn driver handle
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_WRONG_HANDLE    given handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to deregister the
 *                              event callback
 *
 *****************************************************************************/
extern RESULT VideoInDeRegisterEventCb
(
    VideoInHandle_t handle
);

#ifdef __cplusplus
}
#endif

/* @} video_in_api.h */

#endif /* __VIDEO_IN_API_H__ */
