/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @file vin_drv_api.h
 *
 * @brief   Definition of VIN driver API.
 *
 *****************************************************************************/

#ifndef __VIN_DRV_API_H__
#define __VIN_DRV_API_H__

#include <common/mipi.h>

#include <vin_drv/vin_drv_common.h>


#ifdef __cplusplus
extern "C"
{
#endif


/*****************************************************************************/
/**
 * @brief   This function creates and initializesthe MIPI driver context.
 *
 * @param   pConfig             Reference to configuration structure.
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_OUTOFMEM        not enough memory
 *
 * @note    On succes, this function fills in the DrvHandle from the configuration
 *          structure @ref MipiDrvConfig_s. This returned DrvHandle has to be
 *          used for later driver calls and should be store in application context.
 *
 *****************************************************************************/
RESULT VinDrvInit
(
    VinDrvInitConfig_t     *pConfig,
    VinDrvHandle_t         *pVinDrvHandle
);




/*****************************************************************************/
/**
 * @brief   This function releases the MIPI software driver context.
 *
 * @param   MipiDrvHandle       Handle to MIPI driver context as returned by @ref MipiDrvInit.
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to release its
 *                              context
 *
 * @note    Allocated memory by @ref MipiDrvInit is also released
 *
 *****************************************************************************/
RESULT VinDrvRelease
(
    VinDrvHandle_t   vinDrvHandle
);



/*****************************************************************************/
/**
 * @brief   This function configures the MIPI software driver.
 *
 * @param   MipiDrvHandle       Handle to MIPI driver context as returned by @ref MipiDrvInit.
 * @param   pMipiConfig         Refrence to MIPI configuration to use.
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to stop the driver
 *                              (maybe the driver is already running)
 *
 *****************************************************************************/
RESULT VinDrvSetConfig
(
    VinDrvHandle_t       vinDrvHandle,
    VinDrvConfig_t      *pConfig
);


/*****************************************************************************/
/**
 * @brief   This function starts the MIPI software driver.
 *
 * @param   MipiDrvHandle       Handle to MIPI driver context as returned by @ref MipiDrvInit.
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to stop the driver
 *                              (maybe the driver is already running)
 *
 * @note    This function also starts the interrupt handling
 *
 *****************************************************************************/
RESULT VinDrvStart
(
    VinDrvHandle_t   vinDrvHandle
);



/*****************************************************************************/
/**
 * @brief   This function stops the MIPI software driver.
 *
 * @param   MipiDrvHandle       Handle to MIPI driver context as returned by @ref MipiDrvInit.
 *
 * @return                      Return the result of the function call.
 * @retval  RET_SUCCESS         operation succeded
 * @retval  RET_FAILURE         common error occured
 * @retval  RET_WRONG_HANDLE    handle is invalid
 * @retval  RET_WRONG_STATE     driver is in wrong state to stop the driver
 *                              (maybe the driver is already stopped)
 *
 *****************************************************************************/
RESULT VinDrvStop
(
    VinDrvHandle_t   vinDrvHandle
);

extern int CsiOpen(const char* name);
extern void CsiClose(int fd);
extern int CsiSetFormat(int fd, int pad, int width, int height, int format);
extern int CsiSetHdr(int fd, int pad, bool enable);
extern int CsiSetPower(int fd, bool enable);
extern int CsiStreamon(int fd);
extern int CsiStreamoff(int fd);

#ifdef __cplusplus
}
#endif

#endif /* __VIN_DRV_API_H__ */
