/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

/* VeriSilicon 2020 */

/**
 * @file vin_drv_common.h
 *
 * @brief   Definition of VIN driver stuff common to API and internal implementation.
 *
 *****************************************************************************/

#ifndef __VIN_DRV_COMMON_H__
#define __VIN_DRV_COMMON_H__

#include <ebase/types.h>
#include <common/return_codes.h>
#include <common/mipi.h>

#include <hal/hal_api.h>
#include <isi/isi.h>

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/**
 * @brief Handle to VIN  driver context.
 *
 ******************************************************************************/
typedef void* VinDrvHandle_t;

/******************************************************************************/
/**
 * @brief vin driver type enumerations.
 *
 ******************************************************************************/
typedef enum VinDrvType_e{
    VIN_DRV_TYPE_INVALID = 0,
    VIN_DRV_TYPE_MIPI,         /**< MIPI   conform data stream */
    VIN_DRV_TYPE_LVDS,         /**< LVDS   conform data stream */
    VIN_DRV_TYPE_DVP,          /**< DVP    conform data stream */
    VIN_DRV_TYPE_BT601,        /**< BT601  conform data stream */
    VIN_DRV_TYPE_BT656,        /**< BT656  conform data stream */
    VIN_DRV_TYPE_BT1120,       /**< BT656  conform data stream */
	VIN_DRV_TYPE_HISPI,        /**< HISPI  conform data stream */    
    VIN_DRV_TYPE_MAX
}VinDrvType_t;


/******************************************************************************/
/**
 * @brief vin driver signal polarity enumerations.
 *
 ******************************************************************************/
typedef enum VinDrvSignalPolarity_e{
    VIN_DRV_SIG_POL_HIGH_ACIVE = 0,
    VIN_DRV_SIG_POL_LOW_ACIVE  = 1
}VinDrvSignalPolarity_t;


/******************************************************************************/
/**
 * @brief vin driver mipi configuration structure.
 *
 ******************************************************************************/
typedef struct VinDrvMipiConfig_s{

    MipiImageRect_t           imageRect; /* Rx device area */
    MipiDataType_t            dataType;
    MipiLaneId_t              laneNum;
    MipiOutItfConfig_t        outItfCfg[MIPI_OUTPUT_ITF_MAX];
} VinDrvMipiConfig_t;


/******************************************************************************/
/**
 * @brief vin driver dvp configuration structure.
 *
 ******************************************************************************/
typedef struct VinDrvDvpConfig_s{
	VinDrvSignalPolarity_t hSync;
	VinDrvSignalPolarity_t vSync;
	VinDrvSignalPolarity_t pwdn;
	VinDrvSignalPolarity_t rst;
	VinDrvSignalPolarity_t fsin;
    uint32_t bitWidth;
} VinDrvDvpConfig_t;

typedef struct VinDrvVicapConfig_s{
    MipiImageRect_t    imageRect; /* Rx device area */
    MipiDataType_t     dataType;
    VinDrvType_t       vinDrvType;
    bool_t  LaneEnable[MIPI_LANE_MAX];
} VinDrvVicapConfig_t;


/******************************************************************************/
/**
 * @brief   MIPI driver initial configuration structure
 *
 ******************************************************************************/
typedef struct MipiDrvInitConfig_s
{
    uint8_t     mipiId;
} MipiDrvInitConfig_t;

/******************************************************************************/
/**
 * @brief   DVP driver initial configuration structure
 *
 ******************************************************************************/
typedef struct DvpDrvInitConfig_s
{
    uint8_t     dvpCtrlId;
} DvpDrvInitConfig_t;

typedef struct VicapDrvInitConfig_s
{
    uint8_t     vicapId;
} VicapDrvInitConfig_t;


/******************************************************************************/
/**
 * @brief   LVDS driver configuration structure
 *
 ******************************************************************************/
typedef struct LvdsDrvConfig_s {
    int                 fd;
    HalHandle_t         HalHandle;
    IsiSensorHandle_t   IsiSensorHandle;
} LvdsDrvConfig_t;

/******************************************************************************/
/**
 * @brief   DVP driver configuration structure
 *
 ******************************************************************************/
typedef struct DvpDrvConfig_s {
    int                 fd;
    HalHandle_t         HalHandle;
    IsiSensorHandle_t   IsiSensorHandle;

    enum SensorPolarity_e {
        DVP_SENSOR_HPOL_SYNCPOS  = 0,    /**< sync signal pulses high between lines */
        DVP_SENSOR_HPOL_SYNCNEG,         /**< sync signal pulses low between lines */
        DVP_SENSOR_HPOL_REFPOS,          /**< reference signal is high as long as sensor puts out line data */
        DVP_SENSOR_HPOL_REFNEG,          /**< reference signal is low as long as sensor puts out line data */
        DVP_SENSOR_VPOL_POS,
        DVP_SENSOR_VPOL_NEG,
        DVP_SENSOR_EDGE_RISING,
        DVP_SENSOR_EDGE_FALLING,
    } IsiSensorAwbMode_t;

} DvpDrvConfig_t;

/******************************************************************************/
/**
 * @brief   BT driver configuration structure
 *
 ******************************************************************************/
typedef struct BT601DrvConfig_s {
    uint8_t nop;
} BT601DrvConfig_t;

typedef struct BT656DrvConfig_s {
    uint8_t nop;
} BT656DrvConfig_t;

typedef struct BT1120DrvConfig_s {
    uint8_t nop;
} BT1120DrvConfig_t;


/******************************************************************************/
/**
 * @brief video interface driver initial config data structure.
 *
 ******************************************************************************/
typedef struct VinDrvInitConfig_s
{
   VinDrvType_t       type;

   union {
        MipiDrvInitConfig_t mipiCfg;
		DvpDrvInitConfig_t  dvpCfg;
		VicapDrvInitConfig_t vicapCfg;		
   };
} VinDrvInitConfig_t;


/******************************************************************************/
/**
 * @brief   VIN driver configuration structure
 *
 ******************************************************************************/
typedef struct VinDrvConfig_s{
    VinDrvType_t                vinType;

    union {
        VinDrvMipiConfig_t   MipiDrvCfg;
		VinDrvDvpConfig_t    dvpDrvCfg;
		VinDrvVicapConfig_t  VicapDrvCfg;		
    };
}VinDrvConfig_t;



#ifdef __cplusplus
}
#endif

#endif /* __VIN_DRV_COMMON_H__ */
