/******************************************************************************\
|* Copyright (c) <2024> by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")     *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets       *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef WDR4_H
#define WDR4_H

/**
 * @file wdr4.h
 *
 * @brief
 *
 *****************************************************************************/
/**
 * @page module_name_page Module Name
 * Describe here what this module does.
 *
 * For a detailed list of functions and implementation detail refer to:
 * - @ref module_name
 *
 * @defgroup wdr4 module
 * @{
 *
 */
#include <ebase/types.h>
#include <common/return_codes.h>
#include "common.h"
#include <oslayer/oslayer.h>
#include <cameric_drv/cameric_drv_api.h>

#ifdef __cplusplus
extern "C"
{
#endif


#define AWDR4_CONTRAST_MAX 1023
#define AWDR4_CONTRAST_MIN -1023

#define AWDR4_ENTROPY_BASE_MAX 1023
#define AWDR4_ENTROPY_BASE_MIN 0


#define AWDR4_ENTROPY_SLOPE_MAX 1023
#define AWDR4_ENTROPY_SLOPE_MIN 0

#define AWDR4_FLAT_STRENGHTH_MAX 19
#define AWDR4_FLAT_STRENGHTH_MIN 0

#define AWDR4_FLAT_THR_MAX 20
#define AWDR4_FLAT_THR_MIN 0

#define AWDR4_GLOBAL_STRENGHTH_MAX 128
#define AWDR4_GLOBAL_STRENGHTH_MIN 0

#define AWDR4_HIGH_STRENGHTH_MAX 128
#define AWDR4_HIGH_STRENGHTH_MIN 0

#define AWDR4_LOW_STRENGHTH_MAX 255
#define AWDR4_LOW_STRENGHTH_MIN 0

#define AWDR4_STRENGHTH_MAX 128
#define AWDR4_STRENGHTH_MIN 0


/*****************************************************************************/
/**
 *          AWdr4Handle_t
 *
 * @brief   wdr4 Module instance handle
 *
 *****************************************************************************/
typedef void *AWdr4Handle_t;         /**< handle to wdr4 context */

/*****************************************************************************/
/**
 *          AWdr4InstanceConfig_t
 *
 * @brief   wdr4 Module instance configuration structure
 *
 *****************************************************************************/
typedef struct AWdr4InstanceConfig_s
{
    CamerIcDrvHandle_t      hCamerIc;           /**< handle to cameric driver */
    AWdr4Handle_t           hAwdr4;             /**< handle returned by AWdr4Init() */
    bool_t                  isHdr;              /*if the sensor mode is hdr, isHdr is true*/
} AWdr4InstanceConfig_t;


#define AWDR4_GAMMA_DOWN_CURVE_SIZE 20   //bin number of gamma_down curve
#define AWDR4_GAMMA_PRE_CURVE_SIZE 20   //bin number of gamma_pre curve
#define AWDR4_GAMMA_UP_CURVE_SIZE 20   //bin number of gamma_up curve
#define AWDR4_ENTROPY_LUT_SIZE 20   //bin number of entropy lut

/*****************************************************************************/
/**
 *          AWdr4Config_t
 *
 * @brief   wdr4 Module configuration structure
 *
 *****************************************************************************/
typedef struct AWdr4Config_s
{
    int16_t contrast;
    uint16_t entropyBase;
    uint16_t entropySlope;
    uint8_t flatStrength;
    uint8_t flatThr;
    uint8_t globalStrength;
    uint8_t highStrength;
    uint8_t lowStrength;
    uint8_t strength;
} AWdr4Config_t;


/*****************************************************************************/
/**
 *          AWdr4GammaConfig_t
 *
 * @brief   wdr4 Module gamma curve configuration structure
 *
 *****************************************************************************/
typedef struct AWdr4GammaConfig_s
{
    uint32_t entropy[AWDR4_ENTROPY_LUT_SIZE];
    uint32_t gammaDown[AWDR4_GAMMA_DOWN_CURVE_SIZE];
    uint32_t gammaPre[AWDR4_GAMMA_PRE_CURVE_SIZE];
    uint32_t gammaUp[AWDR4_GAMMA_UP_CURVE_SIZE];
} AWdr4GammaConfig_t;


/*****************************************************************************/
/**
 *          AWdr4AutoConfig_t
 *
 * @brief   wdr4 Module auto configuration structure
 *
 *****************************************************************************/
typedef struct AWdr4AutoConfig_s
{
    uint8_t autoLevel;
    float32_t gains[AUTO_NUM];
    uint8_t strength[AUTO_NUM];
    uint8_t highStrength[AUTO_NUM];
    uint8_t lowStrength[AUTO_NUM];
    uint8_t globalStrength[AUTO_NUM];
    int16_t contrast[AUTO_NUM];
    uint8_t flatStrength[AUTO_NUM];
    uint8_t flatThr[AUTO_NUM];
    uint16_t entropySlope[AUTO_NUM];
    uint16_t entropyBase[AUTO_NUM];
} AWdr4AutoConfig_t;


/*****************************************************************************/
/**
 *          AWdr4Init()
 *
 * @brief   This function initializes the wdr4
 *
 * @param   pInstConfig
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_INVALID_PARM
 * @retval  RET_OUTOFMEM
 *
 *****************************************************************************/
RESULT AWdr4Init
(
    AWdr4InstanceConfig_t *pInstConfig
);


/*****************************************************************************/
/**
 *          AWdr4Deinit()
 *
 * @brief   The function deinit the wdr4
 *
 * @param   handle  Handle to wdr4
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4Deinit
(
    AWdr4Handle_t handle
);


/*****************************************************************************/
/**
 *          AWdr4SetMode()
 *
 * @brief   The function set the wdr4 mode
 *
 * @param   handle  Handle to wdr4
 * @param   mode  wdr4 work mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4SetMode
(
    AWdr4Handle_t 		handle,
    const ARunMode_t	mode
);


/*****************************************************************************/
/**
 *          AWdr4GetMode()
 *
 * @brief   The function get the wdr4 mode
 *
 * @param   handle  Handle to wdr4
 * @param   mode  wdr4 work mode
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4GetMode
(
    AWdr4Handle_t handle,
    ARunMode_t  *mode
);

/*****************************************************************************/
/**
 *          AWdr4SetAutoConfigure()
 *
 * @brief   This function set auto mode parameters table
 *
 * @param   handle   Handle to wdr4
 * @param   pConfig  wdr4 auto config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4SetAutoConfigure
(
    AWdr4Handle_t handle,
    AWdr4AutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AWdr4GetAutoConfigure()
 *
 * @brief   This function get auto mode parameters table
 *
 * @param   handle   Handle to wdr4
 * @param   pConfig  wdr4 auto config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4GetAutoConfigure
(
    AWdr4Handle_t handle,
    AWdr4AutoConfig_t *pConfig
);

/*****************************************************************************/
/**
 *          AWdr4SetConfigure()
 *
 * @brief   This function configures the wdr4 module
 *
 * @param   handle   Handle to wdr4 
 * @param   pConfig  wdr4 config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4SetConfigure
(
    AWdr4Handle_t handle,
    AWdr4Config_t *pConfig
);


/*****************************************************************************/
/**
 *          AWdr4GetConfigure()
 *
 * @brief   This function get the wdr4 module configures
 *
 * @param   handle  Handle to wdr4
 * @param   pConfig wdr4 config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4GetConfigure
(
    AWdr4Handle_t handle,
    AWdr4Config_t *pConfig
);

/*****************************************************************************/
/**
 *          AWdr4GammaSetConfigure()
 *
 * @brief   This function configures the wdr4 module
 *
 * @param   handle   Handle to wdr4 
 * @param   pGammaConfig  wdr4 gamma curve config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4GammaSetConfigure
(
    AWdr4Handle_t handle,
    AWdr4GammaConfig_t *pGammaConfig
);

/*****************************************************************************/
/**
 *          AWdr4SetGammaConfigure()
 *
 * @brief   This function set the wdr4 module configures
 *
 * @param   handle  Handle to wdr4
 * @param   pGammaConfig wdr4 gamma curve config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4SetGammaConfigure
(
    AWdr4Handle_t handle,
    AWdr4GammaConfig_t *pGammaConfig
);

/*****************************************************************************/
/**
 *          AWdr4GetGammaConfigure()
 *
 * @brief   This function get the wdr4 module configures
 *
 * @param   handle  Handle to wdr4
 * @param   pGammaConfig wdr4 gamma curve config
 *
 * @return  Returns the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_WRONG_HANDLE
 * @retval  RET_INVALID_PARM
 * @retval  RET_WRONG_STATE
 *
 *****************************************************************************/
RESULT AWdr4GetGammaConfigure
(
    AWdr4Handle_t handle,
    AWdr4GammaConfig_t *pGammaConfig
);

/*****************************************************************************/
/**
 *          AWdr4Enable()
 *
 * @brief   enable wdr4
 *
 * @param   handle  Handle to wdr4
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4Enable
(
    AWdr4Handle_t 		handle
);

/*****************************************************************************/
/**
 *          AWdr4Disable()
 *
 * @brief   disable wdr4
 *
 * @param   handle  Handle to wdr4
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4Disable
(
    AWdr4Handle_t 		handle
);

/*****************************************************************************/
/**
 *          AWdr4GetStatus()
 *
 * @brief   The function returns the status of the Auto wide dynamic range v4
 *          module
 *
 * @param   handle  Handle to wdr4
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4GetStatus
(
    AWdr4Handle_t handle,
    bool_t *pRunning,
    ARunMode_t *pMode,
    AWdr4Config_t *pCurrentParam,
    AWdr4GammaConfig_t *pCurrentGammaParam
);

/*****************************************************************************/
/**
 *          AWdr4ProcessFrame()
 *
 * @brief   wdr4 process frame
 *
 * @param   handle  Handle to wdr4
 *          gain    current sensor-gain
 *
 * @return  Return the result of the function call.
 * @retval  RET_SUCCESS
 * @retval  RET_FAILURE
 *
 *****************************************************************************/
RESULT AWdr4ProcessFrame
(
    AWdr4Handle_t handle,
    const float32_t fGain
);

#ifdef __cplusplus
}
#endif


/* @} Wdr4 */


#endif /* WDR4_H*/
